@@ -12,6 +12,7 @@
 #include "js/Proxy.h"
 
 namespace js {
+struct CompartmentFilter;
 
 /*
  * Helper for Wrapper::New default options.
@@ -40,73 +41,82 @@ class MOZ_STACK_CLASS WrapperOptions : public ProxyOptions {
 
 // Base class for proxy handlers that want to forward all operations to an
 // object stored in the proxy's private slot.
-class JS_FRIEND_API ForwardingProxyHandler : public BaseProxyHandler {
+class JS_PUBLIC_API ForwardingProxyHandler : public BaseProxyHandler {
  public:
   using BaseProxyHandler::BaseProxyHandler;
 
   /* Standard internal methods. */
   virtual bool getOwnPropertyDescriptor(
-      JSContext* cx, HandleObject proxy, HandleId id,
-      MutableHandle<PropertyDescriptor> desc) const override;
-  virtual bool defineProperty(JSContext* cx, HandleObject proxy, HandleId id,
-                              Handle<PropertyDescriptor> desc,
-                              ObjectOpResult& result) const override;
-  virtual bool ownPropertyKeys(JSContext* cx, HandleObject proxy,
-                               MutableHandleIdVector props) const override;
-  virtual bool delete_(JSContext* cx, HandleObject proxy, HandleId id,
-                       ObjectOpResult& result) const override;
-  virtual bool enumerate(JSContext* cx, HandleObject proxy,
-                         MutableHandleIdVector props) const override;
-  virtual bool getPrototype(JSContext* cx, HandleObject proxy,
-                            MutableHandleObject protop) const override;
-  virtual bool setPrototype(JSContext* cx, HandleObject proxy,
-                            HandleObject proto,
-                            ObjectOpResult& result) const override;
+      JSContext* cx, JS::HandleObject proxy, JS::HandleId id,
+      JS::MutableHandle<mozilla::Maybe<JS::PropertyDescriptor>> desc)
+      const override;
+  virtual bool defineProperty(JSContext* cx, JS::HandleObject proxy,
+                              JS::HandleId id,
+                              JS::Handle<JS::PropertyDescriptor> desc,
+                              JS::ObjectOpResult& result) const override;
+  virtual bool ownPropertyKeys(JSContext* cx, JS::HandleObject proxy,
+                               JS::MutableHandleIdVector props) const override;
+  virtual bool delete_(JSContext* cx, JS::HandleObject proxy, JS::HandleId id,
+                       JS::ObjectOpResult& result) const override;
+  virtual bool enumerate(JSContext* cx, JS::HandleObject proxy,
+                         JS::MutableHandleIdVector props) const override;
+  virtual bool getPrototype(JSContext* cx, JS::HandleObject proxy,
+                            JS::MutableHandleObject protop) const override;
+  virtual bool setPrototype(JSContext* cx, JS::HandleObject proxy,
+                            JS::HandleObject proto,
+                            JS::ObjectOpResult& result) const override;
   virtual bool getPrototypeIfOrdinary(
-      JSContext* cx, HandleObject proxy, bool* isOrdinary,
-      MutableHandleObject protop) const override;
-  virtual bool setImmutablePrototype(JSContext* cx, HandleObject proxy,
+      JSContext* cx, JS::HandleObject proxy, bool* isOrdinary,
+      JS::MutableHandleObject protop) const override;
+  virtual bool setImmutablePrototype(JSContext* cx, JS::HandleObject proxy,
                                      bool* succeeded) const override;
-  virtual bool preventExtensions(JSContext* cx, HandleObject proxy,
-                                 ObjectOpResult& result) const override;
-  virtual bool isExtensible(JSContext* cx, HandleObject proxy,
+  virtual bool preventExtensions(JSContext* cx, JS::HandleObject proxy,
+                                 JS::ObjectOpResult& result) const override;
+  virtual bool isExtensible(JSContext* cx, JS::HandleObject proxy,
                             bool* extensible) const override;
-  virtual bool has(JSContext* cx, HandleObject proxy, HandleId id,
+  virtual bool has(JSContext* cx, JS::HandleObject proxy, JS::HandleId id,
                    bool* bp) const override;
-  virtual bool get(JSContext* cx, HandleObject proxy, HandleValue receiver,
-                   HandleId id, MutableHandleValue vp) const override;
-  virtual bool set(JSContext* cx, HandleObject proxy, HandleId id,
-                   HandleValue v, HandleValue receiver,
-                   ObjectOpResult& result) const override;
-  virtual bool call(JSContext* cx, HandleObject proxy,
-                    const CallArgs& args) const override;
-  virtual bool construct(JSContext* cx, HandleObject proxy,
-                         const CallArgs& args) const override;
+  virtual bool get(JSContext* cx, JS::HandleObject proxy,
+                   JS::HandleValue receiver, JS::HandleId id,
+                   JS::MutableHandleValue vp) const override;
+  virtual bool set(JSContext* cx, JS::HandleObject proxy, JS::HandleId id,
+                   JS::HandleValue v, JS::HandleValue receiver,
+                   JS::ObjectOpResult& result) const override;
+  virtual bool call(JSContext* cx, JS::HandleObject proxy,
+                    const JS::CallArgs& args) const override;
+  virtual bool construct(JSContext* cx, JS::HandleObject proxy,
+                         const JS::CallArgs& args) const override;
 
   /* SpiderMonkey extensions. */
-  virtual bool hasOwn(JSContext* cx, HandleObject proxy, HandleId id,
+  virtual bool hasOwn(JSContext* cx, JS::HandleObject proxy, JS::HandleId id,
                       bool* bp) const override;
   virtual bool getOwnEnumerablePropertyKeys(
-      JSContext* cx, HandleObject proxy,
-      MutableHandleIdVector props) const override;
-  virtual bool nativeCall(JSContext* cx, IsAcceptableThis test, NativeImpl impl,
-                          const CallArgs& args) const override;
-  virtual bool hasInstance(JSContext* cx, HandleObject proxy,
-                           MutableHandleValue v, bool* bp) const override;
-  virtual bool getBuiltinClass(JSContext* cx, HandleObject proxy,
+      JSContext* cx, JS::HandleObject proxy,
+      JS::MutableHandleIdVector props) const override;
+  virtual bool nativeCall(JSContext* cx, JS::IsAcceptableThis test,
+                          JS::NativeImpl impl,
+                          const JS::CallArgs& args) const override;
+  virtual bool hasInstance(JSContext* cx, JS::HandleObject proxy,
+                           JS::MutableHandleValue v, bool* bp) const override;
+  virtual bool getBuiltinClass(JSContext* cx, JS::HandleObject proxy,
                                ESClass* cls) const override;
-  virtual bool isArray(JSContext* cx, HandleObject proxy,
+  virtual bool isArray(JSContext* cx, JS::HandleObject proxy,
                        JS::IsArrayAnswer* answer) const override;
   virtual const char* className(JSContext* cx,
-                                HandleObject proxy) const override;
-  virtual JSString* fun_toString(JSContext* cx, HandleObject proxy,
+                                JS::HandleObject proxy) const override;
+  virtual JSString* fun_toString(JSContext* cx, JS::HandleObject proxy,
                                  bool isToSource) const override;
   virtual RegExpShared* regexp_toShared(JSContext* cx,
-                                        HandleObject proxy) const override;
-  virtual bool boxedValue_unbox(JSContext* cx, HandleObject proxy,
-                                MutableHandleValue vp) const override;
+                                        JS::HandleObject proxy) const override;
+  virtual bool boxedValue_unbox(JSContext* cx, JS::HandleObject proxy,
+                                JS::MutableHandleValue vp) const override;
   virtual bool isCallable(JSObject* obj) const override;
   virtual bool isConstructor(JSObject* obj) const override;
+
+  // Use the target object for private fields.
+  virtual bool useProxyExpandoObjectForPrivateFields() const override {
+    return false;
+  }
 };
 
 /*
@@ -122,7 +132,7 @@ class JS_FRIEND_API ForwardingProxyHandler : public BaseProxyHandler {
  * to add an override in CrossCompartmentWrapper. If you don't, you risk
  * compartment mismatches. See bug 945826 comment 0.
  */
-class JS_FRIEND_API Wrapper : public ForwardingProxyHandler {
+class JS_PUBLIC_API Wrapper : public ForwardingProxyHandler {
   unsigned mFlags;
 
  public:
@@ -131,14 +141,14 @@ class JS_FRIEND_API Wrapper : public ForwardingProxyHandler {
       : ForwardingProxyHandler(&family, aHasPrototype, aHasSecurityPolicy),
         mFlags(aFlags) {}
 
-  virtual bool finalizeInBackground(const Value& priv) const override;
+  virtual bool finalizeInBackground(const JS::Value& priv) const override;
 
   /**
    * A hook subclasses can override to implement CheckedUnwrapDynamic
    * behavior.  The JSContext represents the "who is trying to unwrap?" Realm.
    * The JSObject is the wrapper that the caller is trying to unwrap.
    */
-  virtual bool dynamicCheckedUnwrapAllowed(HandleObject obj,
+  virtual bool dynamicCheckedUnwrapAllowed(JS::HandleObject obj,
                                            JSContext* cx) const {
     MOZ_ASSERT(hasSecurityPolicy(), "Why are you asking?");
     return false;
@@ -151,10 +161,6 @@ class JS_FRIEND_API Wrapper : public ForwardingProxyHandler {
   static JSObject* New(JSContext* cx, JSObject* obj, const Wrapper* handler,
                        const WrapperOptions& options = WrapperOptions());
 
-  static JSObject* NewSingleton(
-      JSContext* cx, JSObject* obj, const Wrapper* handler,
-      const WrapperOptions& options = WrapperOptions());
-
   static JSObject* Renew(JSObject* existing, JSObject* obj,
                          const Wrapper* handler);
 
@@ -180,7 +186,7 @@ inline JSObject* WrapperOptions::proto() const {
 }
 
 /* Base class for all cross compartment wrapper handlers. */
-class JS_FRIEND_API CrossCompartmentWrapper : public Wrapper {
+class JS_PUBLIC_API CrossCompartmentWrapper : public Wrapper {
  public:
   explicit constexpr CrossCompartmentWrapper(unsigned aFlags,
                                              bool aHasPrototype = false,
@@ -190,62 +196,66 @@ class JS_FRIEND_API CrossCompartmentWrapper : public Wrapper {
 
   /* Standard internal methods. */
   virtual bool getOwnPropertyDescriptor(
-      JSContext* cx, HandleObject wrapper, HandleId id,
-      MutableHandle<PropertyDescriptor> desc) const override;
-  virtual bool defineProperty(JSContext* cx, HandleObject wrapper, HandleId id,
-                              Handle<PropertyDescriptor> desc,
-                              ObjectOpResult& result) const override;
-  virtual bool ownPropertyKeys(JSContext* cx, HandleObject wrapper,
-                               MutableHandleIdVector props) const override;
-  virtual bool delete_(JSContext* cx, HandleObject wrapper, HandleId id,
-                       ObjectOpResult& result) const override;
-  virtual bool enumerate(JSContext* cx, HandleObject proxy,
-                         MutableHandleIdVector props) const override;
-  virtual bool getPrototype(JSContext* cx, HandleObject proxy,
-                            MutableHandleObject protop) const override;
-  virtual bool setPrototype(JSContext* cx, HandleObject proxy,
-                            HandleObject proto,
-                            ObjectOpResult& result) const override;
+      JSContext* cx, JS::HandleObject wrapper, JS::HandleId id,
+      JS::MutableHandle<mozilla::Maybe<JS::PropertyDescriptor>> desc)
+      const override;
+  virtual bool defineProperty(JSContext* cx, JS::HandleObject wrapper,
+                              JS::HandleId id,
+                              JS::Handle<JS::PropertyDescriptor> desc,
+                              JS::ObjectOpResult& result) const override;
+  virtual bool ownPropertyKeys(JSContext* cx, JS::HandleObject wrapper,
+                               JS::MutableHandleIdVector props) const override;
+  virtual bool delete_(JSContext* cx, JS::HandleObject wrapper, JS::HandleId id,
+                       JS::ObjectOpResult& result) const override;
+  virtual bool enumerate(JSContext* cx, JS::HandleObject proxy,
+                         JS::MutableHandleIdVector props) const override;
+  virtual bool getPrototype(JSContext* cx, JS::HandleObject proxy,
+                            JS::MutableHandleObject protop) const override;
+  virtual bool setPrototype(JSContext* cx, JS::HandleObject proxy,
+                            JS::HandleObject proto,
+                            JS::ObjectOpResult& result) const override;
 
   virtual bool getPrototypeIfOrdinary(
-      JSContext* cx, HandleObject proxy, bool* isOrdinary,
-      MutableHandleObject protop) const override;
-  virtual bool setImmutablePrototype(JSContext* cx, HandleObject proxy,
+      JSContext* cx, JS::HandleObject proxy, bool* isOrdinary,
+      JS::MutableHandleObject protop) const override;
+  virtual bool setImmutablePrototype(JSContext* cx, JS::HandleObject proxy,
                                      bool* succeeded) const override;
-  virtual bool preventExtensions(JSContext* cx, HandleObject wrapper,
-                                 ObjectOpResult& result) const override;
-  virtual bool isExtensible(JSContext* cx, HandleObject wrapper,
+  virtual bool preventExtensions(JSContext* cx, JS::HandleObject wrapper,
+                                 JS::ObjectOpResult& result) const override;
+  virtual bool isExtensible(JSContext* cx, JS::HandleObject wrapper,
                             bool* extensible) const override;
-  virtual bool has(JSContext* cx, HandleObject wrapper, HandleId id,
+  virtual bool has(JSContext* cx, JS::HandleObject wrapper, JS::HandleId id,
                    bool* bp) const override;
-  virtual bool get(JSContext* cx, HandleObject wrapper, HandleValue receiver,
-                   HandleId id, MutableHandleValue vp) const override;
-  virtual bool set(JSContext* cx, HandleObject wrapper, HandleId id,
-                   HandleValue v, HandleValue receiver,
-                   ObjectOpResult& result) const override;
-  virtual bool call(JSContext* cx, HandleObject wrapper,
-                    const CallArgs& args) const override;
-  virtual bool construct(JSContext* cx, HandleObject wrapper,
-                         const CallArgs& args) const override;
+  virtual bool get(JSContext* cx, JS::HandleObject wrapper,
+                   JS::HandleValue receiver, JS::HandleId id,
+                   JS::MutableHandleValue vp) const override;
+  virtual bool set(JSContext* cx, JS::HandleObject wrapper, JS::HandleId id,
+                   JS::HandleValue v, JS::HandleValue receiver,
+                   JS::ObjectOpResult& result) const override;
+  virtual bool call(JSContext* cx, JS::HandleObject wrapper,
+                    const JS::CallArgs& args) const override;
+  virtual bool construct(JSContext* cx, JS::HandleObject wrapper,
+                         const JS::CallArgs& args) const override;
 
   /* SpiderMonkey extensions. */
-  virtual bool hasOwn(JSContext* cx, HandleObject wrapper, HandleId id,
+  virtual bool hasOwn(JSContext* cx, JS::HandleObject wrapper, JS::HandleId id,
                       bool* bp) const override;
   virtual bool getOwnEnumerablePropertyKeys(
-      JSContext* cx, HandleObject wrapper,
-      MutableHandleIdVector props) const override;
-  virtual bool nativeCall(JSContext* cx, IsAcceptableThis test, NativeImpl impl,
-                          const CallArgs& args) const override;
-  virtual bool hasInstance(JSContext* cx, HandleObject wrapper,
-                           MutableHandleValue v, bool* bp) const override;
+      JSContext* cx, JS::HandleObject wrapper,
+      JS::MutableHandleIdVector props) const override;
+  virtual bool nativeCall(JSContext* cx, JS::IsAcceptableThis test,
+                          JS::NativeImpl impl,
+                          const JS::CallArgs& args) const override;
+  virtual bool hasInstance(JSContext* cx, JS::HandleObject wrapper,
+                           JS::MutableHandleValue v, bool* bp) const override;
   virtual const char* className(JSContext* cx,
-                                HandleObject proxy) const override;
-  virtual JSString* fun_toString(JSContext* cx, HandleObject wrapper,
+                                JS::HandleObject proxy) const override;
+  virtual JSString* fun_toString(JSContext* cx, JS::HandleObject wrapper,
                                  bool isToSource) const override;
   virtual RegExpShared* regexp_toShared(JSContext* cx,
-                                        HandleObject proxy) const override;
-  virtual bool boxedValue_unbox(JSContext* cx, HandleObject proxy,
-                                MutableHandleValue vp) const override;
+                                        JS::HandleObject proxy) const override;
+  virtual bool boxedValue_unbox(JSContext* cx, JS::HandleObject proxy,
+                                JS::MutableHandleValue vp) const override;
 
   // Allocate CrossCompartmentWrappers in the nursery.
   virtual bool canNurseryAllocate() const override { return true; }
@@ -254,7 +264,7 @@ class JS_FRIEND_API CrossCompartmentWrapper : public Wrapper {
   static const CrossCompartmentWrapper singletonWithPrototype;
 };
 
-class JS_FRIEND_API OpaqueCrossCompartmentWrapper
+class JS_PUBLIC_API OpaqueCrossCompartmentWrapper
     : public CrossCompartmentWrapper {
  public:
   explicit constexpr OpaqueCrossCompartmentWrapper()
@@ -262,58 +272,61 @@ class JS_FRIEND_API OpaqueCrossCompartmentWrapper
 
   /* Standard internal methods. */
   virtual bool getOwnPropertyDescriptor(
-      JSContext* cx, HandleObject wrapper, HandleId id,
-      MutableHandle<PropertyDescriptor> desc) const override;
-  virtual bool defineProperty(JSContext* cx, HandleObject wrapper, HandleId id,
-                              Handle<PropertyDescriptor> desc,
-                              ObjectOpResult& result) const override;
-  virtual bool ownPropertyKeys(JSContext* cx, HandleObject wrapper,
-                               MutableHandleIdVector props) const override;
-  virtual bool delete_(JSContext* cx, HandleObject wrapper, HandleId id,
-                       ObjectOpResult& result) const override;
-  virtual bool enumerate(JSContext* cx, HandleObject proxy,
-                         MutableHandleIdVector props) const override;
-  virtual bool getPrototype(JSContext* cx, HandleObject wrapper,
-                            MutableHandleObject protop) const override;
-  virtual bool setPrototype(JSContext* cx, HandleObject wrapper,
-                            HandleObject proto,
-                            ObjectOpResult& result) const override;
+      JSContext* cx, JS::HandleObject wrapper, JS::HandleId id,
+      JS::MutableHandle<mozilla::Maybe<JS::PropertyDescriptor>> desc)
+      const override;
+  virtual bool defineProperty(JSContext* cx, JS::HandleObject wrapper,
+                              JS::HandleId id,
+                              JS::Handle<JS::PropertyDescriptor> desc,
+                              JS::ObjectOpResult& result) const override;
+  virtual bool ownPropertyKeys(JSContext* cx, JS::HandleObject wrapper,
+                               JS::MutableHandleIdVector props) const override;
+  virtual bool delete_(JSContext* cx, JS::HandleObject wrapper, JS::HandleId id,
+                       JS::ObjectOpResult& result) const override;
+  virtual bool enumerate(JSContext* cx, JS::HandleObject proxy,
+                         JS::MutableHandleIdVector props) const override;
+  virtual bool getPrototype(JSContext* cx, JS::HandleObject wrapper,
+                            JS::MutableHandleObject protop) const override;
+  virtual bool setPrototype(JSContext* cx, JS::HandleObject wrapper,
+                            JS::HandleObject proto,
+                            JS::ObjectOpResult& result) const override;
   virtual bool getPrototypeIfOrdinary(
-      JSContext* cx, HandleObject wrapper, bool* isOrdinary,
-      MutableHandleObject protop) const override;
-  virtual bool setImmutablePrototype(JSContext* cx, HandleObject wrapper,
+      JSContext* cx, JS::HandleObject wrapper, bool* isOrdinary,
+      JS::MutableHandleObject protop) const override;
+  virtual bool setImmutablePrototype(JSContext* cx, JS::HandleObject wrapper,
                                      bool* succeeded) const override;
-  virtual bool preventExtensions(JSContext* cx, HandleObject wrapper,
-                                 ObjectOpResult& result) const override;
-  virtual bool isExtensible(JSContext* cx, HandleObject wrapper,
+  virtual bool preventExtensions(JSContext* cx, JS::HandleObject wrapper,
+                                 JS::ObjectOpResult& result) const override;
+  virtual bool isExtensible(JSContext* cx, JS::HandleObject wrapper,
                             bool* extensible) const override;
-  virtual bool has(JSContext* cx, HandleObject wrapper, HandleId id,
+  virtual bool has(JSContext* cx, JS::HandleObject wrapper, JS::HandleId id,
                    bool* bp) const override;
-  virtual bool get(JSContext* cx, HandleObject wrapper, HandleValue receiver,
-                   HandleId id, MutableHandleValue vp) const override;
-  virtual bool set(JSContext* cx, HandleObject wrapper, HandleId id,
-                   HandleValue v, HandleValue receiver,
-                   ObjectOpResult& result) const override;
-  virtual bool call(JSContext* cx, HandleObject wrapper,
-                    const CallArgs& args) const override;
-  virtual bool construct(JSContext* cx, HandleObject wrapper,
-                         const CallArgs& args) const override;
+  virtual bool get(JSContext* cx, JS::HandleObject wrapper,
+                   JS::HandleValue receiver, JS::HandleId id,
+                   JS::MutableHandleValue vp) const override;
+  virtual bool set(JSContext* cx, JS::HandleObject wrapper, JS::HandleId id,
+                   JS::HandleValue v, JS::HandleValue receiver,
+                   JS::ObjectOpResult& result) const override;
+  virtual bool call(JSContext* cx, JS::HandleObject wrapper,
+                    const JS::CallArgs& args) const override;
+  virtual bool construct(JSContext* cx, JS::HandleObject wrapper,
+                         const JS::CallArgs& args) const override;
 
   /* SpiderMonkey extensions. */
-  virtual bool hasOwn(JSContext* cx, HandleObject wrapper, HandleId id,
+  virtual bool hasOwn(JSContext* cx, JS::HandleObject wrapper, JS::HandleId id,
                       bool* bp) const override;
   virtual bool getOwnEnumerablePropertyKeys(
-      JSContext* cx, HandleObject wrapper,
-      MutableHandleIdVector props) const override;
-  virtual bool getBuiltinClass(JSContext* cx, HandleObject wrapper,
+      JSContext* cx, JS::HandleObject wrapper,
+      JS::MutableHandleIdVector props) const override;
+  virtual bool getBuiltinClass(JSContext* cx, JS::HandleObject wrapper,
                                ESClass* cls) const override;
-  virtual bool isArray(JSContext* cx, HandleObject obj,
+  virtual bool isArray(JSContext* cx, JS::HandleObject obj,
                        JS::IsArrayAnswer* answer) const override;
-  virtual bool hasInstance(JSContext* cx, HandleObject wrapper,
-                           MutableHandleValue v, bool* bp) const override;
+  virtual bool hasInstance(JSContext* cx, JS::HandleObject wrapper,
+                           JS::MutableHandleValue v, bool* bp) const override;
   virtual const char* className(JSContext* cx,
-                                HandleObject wrapper) const override;
-  virtual JSString* fun_toString(JSContext* cx, HandleObject proxy,
+                                JS::HandleObject wrapper) const override;
+  virtual JSString* fun_toString(JSContext* cx, JS::HandleObject proxy,
                                  bool isToSource) const override;
 
   static const OpaqueCrossCompartmentWrapper singleton;
@@ -329,38 +342,40 @@ class JS_FRIEND_API OpaqueCrossCompartmentWrapper
  * access, relying on derived SecurityWrapper to block access when necessary.
  */
 template <class Base>
-class JS_FRIEND_API SecurityWrapper : public Base {
+class JS_PUBLIC_API SecurityWrapper : public Base {
  public:
   explicit constexpr SecurityWrapper(unsigned flags, bool hasPrototype = false)
       : Base(flags, hasPrototype, /* hasSecurityPolicy = */ true) {}
 
-  virtual bool enter(JSContext* cx, HandleObject wrapper, HandleId id,
+  virtual bool enter(JSContext* cx, JS::HandleObject wrapper, JS::HandleId id,
                      Wrapper::Action act, bool mayThrow,
                      bool* bp) const override;
 
-  virtual bool defineProperty(JSContext* cx, HandleObject wrapper, HandleId id,
-                              Handle<PropertyDescriptor> desc,
-                              ObjectOpResult& result) const override;
-  virtual bool isExtensible(JSContext* cx, HandleObject wrapper,
+  virtual bool defineProperty(JSContext* cx, JS::HandleObject wrapper,
+                              JS::HandleId id,
+                              JS::Handle<JS::PropertyDescriptor> desc,
+                              JS::ObjectOpResult& result) const override;
+  virtual bool isExtensible(JSContext* cx, JS::HandleObject wrapper,
                             bool* extensible) const override;
-  virtual bool preventExtensions(JSContext* cx, HandleObject wrapper,
-                                 ObjectOpResult& result) const override;
-  virtual bool setPrototype(JSContext* cx, HandleObject proxy,
-                            HandleObject proto,
-                            ObjectOpResult& result) const override;
-  virtual bool setImmutablePrototype(JSContext* cx, HandleObject proxy,
+  virtual bool preventExtensions(JSContext* cx, JS::HandleObject wrapper,
+                                 JS::ObjectOpResult& result) const override;
+  virtual bool setPrototype(JSContext* cx, JS::HandleObject proxy,
+                            JS::HandleObject proto,
+                            JS::ObjectOpResult& result) const override;
+  virtual bool setImmutablePrototype(JSContext* cx, JS::HandleObject proxy,
                                      bool* succeeded) const override;
 
-  virtual bool nativeCall(JSContext* cx, IsAcceptableThis test, NativeImpl impl,
-                          const CallArgs& args) const override;
-  virtual bool getBuiltinClass(JSContext* cx, HandleObject wrapper,
+  virtual bool nativeCall(JSContext* cx, JS::IsAcceptableThis test,
+                          JS::NativeImpl impl,
+                          const JS::CallArgs& args) const override;
+  virtual bool getBuiltinClass(JSContext* cx, JS::HandleObject wrapper,
                                ESClass* cls) const override;
-  virtual bool isArray(JSContext* cx, HandleObject wrapper,
+  virtual bool isArray(JSContext* cx, JS::HandleObject wrapper,
                        JS::IsArrayAnswer* answer) const override;
   virtual RegExpShared* regexp_toShared(JSContext* cx,
-                                        HandleObject proxy) const override;
-  virtual bool boxedValue_unbox(JSContext* cx, HandleObject proxy,
-                                MutableHandleValue vp) const override;
+                                        JS::HandleObject proxy) const override;
+  virtual bool boxedValue_unbox(JSContext* cx, JS::HandleObject proxy,
+                                JS::MutableHandleValue vp) const override;
 
   // Allow isCallable and isConstructor. They used to be class-level, and so
   // could not be guarded against.
@@ -376,8 +391,9 @@ class JS_FRIEND_API SecurityWrapper : public Base {
 typedef SecurityWrapper<CrossCompartmentWrapper>
     CrossCompartmentSecurityWrapper;
 
-extern JSObject* TransparentObjectWrapper(JSContext* cx, HandleObject existing,
-                                          HandleObject obj);
+extern JSObject* TransparentObjectWrapper(JSContext* cx,
+                                          JS::HandleObject existing,
+                                          JS::HandleObject obj);
 
 inline bool IsWrapper(const JSObject* obj) {
   return IsProxy(obj) && GetProxyHandler(obj)->family() == &Wrapper::family;
@@ -402,7 +418,7 @@ inline bool IsCrossCompartmentWrapper(const JSObject* obj) {
 // ExposeToActiveJS is called on wrapper targets to allow gray marking
 // assertions to work while an incremental GC is in progress, but this means
 // that this cannot be called from the GC or off the main thread.
-JS_FRIEND_API JSObject* UncheckedUnwrap(JSObject* obj,
+JS_PUBLIC_API JSObject* UncheckedUnwrap(JSObject* obj,
                                         bool stopAtWindowProxy = true,
                                         unsigned* flagsp = nullptr);
 
@@ -422,11 +438,11 @@ JS_FRIEND_API JSObject* UncheckedUnwrap(JSObject* obj,
 // ExposeToActiveJS is called on wrapper targets to allow gray marking
 // assertions to work while an incremental GC is in progress, but this means
 // that this cannot be called from the GC or off the main thread.
-JS_FRIEND_API JSObject* CheckedUnwrapStatic(JSObject* obj);
+JS_PUBLIC_API JSObject* CheckedUnwrapStatic(JSObject* obj);
 
 // Unwrap only the outermost security wrapper, with the same semantics as
 // above. This is the checked version of Wrapper::wrappedObject.
-JS_FRIEND_API JSObject* UnwrapOneCheckedStatic(JSObject* obj);
+JS_PUBLIC_API JSObject* UnwrapOneCheckedStatic(JSObject* obj);
 
 // Given a JSObject, returns that object stripped of wrappers. At each stage,
 // the security wrapper has the opportunity to veto the unwrap. If
@@ -450,12 +466,13 @@ JS_FRIEND_API JSObject* UnwrapOneCheckedStatic(JSObject* obj);
 // plan to use the argument object after they have called this function will
 // need to root it to avoid hazard failures, even though this function doesn't
 // require a Handle.
-JS_FRIEND_API JSObject* CheckedUnwrapDynamic(JSObject* obj, JSContext* cx,
+JS_PUBLIC_API JSObject* CheckedUnwrapDynamic(JSObject* obj, JSContext* cx,
                                              bool stopAtWindowProxy = true);
 
 // Unwrap only the outermost security wrapper, with the same semantics as
 // above. This is the checked version of Wrapper::wrappedObject.
-JS_FRIEND_API JSObject* UnwrapOneCheckedDynamic(HandleObject obj, JSContext* cx,
+JS_PUBLIC_API JSObject* UnwrapOneCheckedDynamic(JS::HandleObject obj,
+                                                JSContext* cx,
                                                 bool stopAtWindowProxy = true);
 
 // Given a JSObject, returns that object stripped of wrappers. This returns the
@@ -463,28 +480,29 @@ JS_FRIEND_API JSObject* UnwrapOneCheckedDynamic(HandleObject obj, JSContext* cx,
 //
 // ExposeToActiveJS is not called on wrapper targets so this can be called from
 // the GC or off the main thread.
-JS_FRIEND_API JSObject* UncheckedUnwrapWithoutExpose(JSObject* obj);
+JS_PUBLIC_API JSObject* UncheckedUnwrapWithoutExpose(JSObject* obj);
 
 void ReportAccessDenied(JSContext* cx);
 
-JS_FRIEND_API void NukeCrossCompartmentWrapper(JSContext* cx,
+JS_PUBLIC_API void NukeCrossCompartmentWrapper(JSContext* cx,
                                                JSObject* wrapper);
 
 // If a cross-compartment wrapper source => target exists, nuke it.
-JS_FRIEND_API void NukeCrossCompartmentWrapperIfExists(JSContext* cx,
+JS_PUBLIC_API void NukeCrossCompartmentWrapperIfExists(JSContext* cx,
                                                        JS::Compartment* source,
                                                        JSObject* target);
 
 void RemapWrapper(JSContext* cx, JSObject* wobj, JSObject* newTarget);
-void RemapDeadWrapper(JSContext* cx, HandleObject wobj, HandleObject newTarget);
+void RemapDeadWrapper(JSContext* cx, JS::HandleObject wobj,
+                      JS::HandleObject newTarget);
 
-JS_FRIEND_API bool RemapAllWrappersForObject(JSContext* cx,
-                                             HandleObject oldTarget,
-                                             HandleObject newTarget);
+JS_PUBLIC_API bool RemapAllWrappersForObject(JSContext* cx,
+                                             JS::HandleObject oldTarget,
+                                             JS::HandleObject newTarget);
 
 // API to recompute all cross-compartment wrappers whose source and target
 // match the given filters.
-JS_FRIEND_API bool RecomputeWrappers(JSContext* cx,
+JS_PUBLIC_API bool RecomputeWrappers(JSContext* cx,
                                      const CompartmentFilter& sourceFilter,
                                      const CompartmentFilter& targetFilter);
 
