@@ -11,13 +11,16 @@
 #include "mozilla/Range.h"
 #include "mozilla/ScopeExit.h"
 
+#include <algorithm>
+
 #include "jsnum.h"
 #include "jstypes.h"
-#include "jsutil.h"
 
 #include "builtin/Array.h"
 #include "builtin/BigInt.h"
-#include "builtin/String.h"
+#include "js/friend/ErrorMessages.h"  // js::GetErrorMessage, JSMSG_*
+#include "js/friend/StackLimits.h"    // js::AutoCheckRecursionLimit
+#include "js/Object.h"                // JS::GetBuiltinClass
 #include "js/PropertySpec.h"
 #include "js/StableStringChars.h"
 #include "util/StringBuffer.h"
@@ -26,6 +29,8 @@
 #include "vm/JSContext.h"
 #include "vm/JSObject.h"
 #include "vm/JSONParser.h"
+#include "vm/PlainObject.h"    // js::PlainObject
+#include "vm/WellKnownAtom.h"  // js_*_str
 
 #include "builtin/Array-inl.h"
 #include "builtin/Boolean-inl.h"
@@ -41,9 +46,6 @@ using mozilla::RangedPtr;
 
 using JS::AutoStableStringChars;
 
-const Class js::JSONClass = {js_JSON_str,
-                             JSCLASS_HAS_CACHED_PROTO(JSProto_JSON)};
-
 /* ES5 15.12.3 Quote.
  * Requires that the destination has enough space allocated for src after
  * escaping (that is, `2 + 6 * (srcEnd - srcBegin)` characters).
@@ -343,7 +345,7 @@ static bool PreprocessValue(JSContext* cx, HandleObject holder, KeyType key,
     RootedObject obj(cx, &vp.get().toObject());
 
     ESClass cls;
-    if (!GetBuiltinClass(cx, obj, &cls)) {
+    if (!JS::GetBuiltinClass(cx, obj, &cls)) {
       return false;
     }
 
@@ -480,11 +482,13 @@ static bool JO(JSContext* cx, HandleObject obj, StringifyContext* scx) {
     RootedValue outputValue(cx);
 #ifdef DEBUG
     if (scx->maybeSafely) {
-      RootedNativeObject nativeObj(cx, &obj->as<NativeObject>());
-      Rooted<PropertyResult> prop(cx);
-      NativeLookupOwnPropertyNoResolve(cx, nativeObj, id, &prop);
-      MOZ_ASSERT(prop && prop.isNativeProperty() &&
-                 prop.shape()->isDataDescriptor());
+      PropertyResult prop;
+      if (!NativeLookupOwnPropertyNoResolve(cx, &obj->as<NativeObject>(), id,
+                                            &prop)) {
+        return false;
+      }
+      MOZ_ASSERT(prop.isNativeProperty() &&
+                 prop.propertyInfo().isDataDescriptor());
     }
 #endif  // DEBUG
     if (!GetProperty(cx, obj, obj, id, &outputValue)) {
@@ -526,6 +530,38 @@ static bool JO(JSContext* cx, HandleObject obj, StringifyContext* scx) {
   return scx->sb.append('}');
 }
 
+// For JSON.stringify and JSON.parse with a reviver function, we need to know
+// the length of an object for which JS::IsArray returned true. This must be
+// either an ArrayObject or a proxy wrapping one.
+static MOZ_ALWAYS_INLINE bool GetLengthPropertyForArray(JSContext* cx,
+                                                        HandleObject obj,
+                                                        uint32_t* lengthp) {
+  if (MOZ_LIKELY(obj->is<ArrayObject>())) {
+    *lengthp = obj->as<ArrayObject>().length();
+    return true;
+  }
+
+  MOZ_ASSERT(obj->is<ProxyObject>());
+
+  uint64_t len = 0;
+  if (!GetLengthProperty(cx, obj, &len)) {
+    return false;
+  }
+
+  // A scripted proxy wrapping an array can return a length value larger than
+  // UINT32_MAX. Stringification will likely report an OOM in this case. Match
+  // other JS engines and report an early error in this case, although
+  // technically this is observable, for example when stringifying with a
+  // replacer function.
+  if (len > UINT32_MAX) {
+    ReportAllocationOverflow(cx);
+    return false;
+  }
+
+  *lengthp = uint32_t(len);
+  return true;
+}
+
 /* ES5 15.12.3 JA. */
 static bool JA(JSContext* cx, HandleObject obj, StringifyContext* scx) {
   /*
@@ -550,7 +586,7 @@ static bool JA(JSContext* cx, HandleObject obj, StringifyContext* scx) {
 
   /* Step 6. */
   uint32_t length;
-  if (!GetLengthProperty(cx, obj, &length)) {
+  if (!GetLengthPropertyForArray(cx, obj, &length)) {
     return false;
   }
 
@@ -636,7 +672,8 @@ static bool Str(JSContext* cx, const Value& v, StringifyContext* scx) {
   /* Step 11 must be handled by the caller. */
   MOZ_ASSERT(!IsFilteredValue(v));
 
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
 
@@ -742,7 +779,7 @@ bool js::Stringify(JSContext* cx, MutableHandleValue vp, JSObject* replacer_,
 
       /* Step 4b(iii)(2-3). */
       uint32_t len;
-      if (!GetLengthProperty(cx, replacer, &len)) {
+      if (!GetLengthPropertyForArray(cx, replacer, &len)) {
         return false;
       }
 
@@ -752,7 +789,7 @@ bool js::Stringify(JSContext* cx, MutableHandleValue vp, JSObject* replacer_,
       // size, the set will naturally resize to accommodate them.
       const uint32_t MaxInitialSize = 32;
       Rooted<GCHashSet<jsid>> idSet(
-          cx, GCHashSet<jsid>(cx, Min(len, MaxInitialSize)));
+          cx, GCHashSet<jsid>(cx, std::min(len, MaxInitialSize)));
 
       /* Step 4b(iii)(4). */
       uint32_t k = 0;
@@ -770,7 +807,12 @@ bool js::Stringify(JSContext* cx, MutableHandleValue vp, JSObject* replacer_,
         }
 
         /* Step 4b(iii)(5)(c-g). */
-        if (!item.isNumber() && !item.isString()) {
+        RootedId id(cx);
+        if (item.isNumber() || item.isString()) {
+          if (!PrimitiveValueToId<CanGC>(cx, item, &id)) {
+            return false;
+          }
+        } else {
           ESClass cls;
           if (!GetClassOfValue(cx, item, &cls)) {
             return false;
@@ -779,11 +821,13 @@ bool js::Stringify(JSContext* cx, MutableHandleValue vp, JSObject* replacer_,
           if (cls != ESClass::String && cls != ESClass::Number) {
             continue;
           }
-        }
 
-        RootedId id(cx);
-        if (!ValueToId<CanGC>(cx, item, &id)) {
-          return false;
+          JSAtom* atom = ToAtom<CanGC>(cx, item);
+          if (!atom) {
+            return false;
+          }
+
+          id.set(AtomToId(atom));
         }
 
         /* Step 4b(iii)(5)(g). */
@@ -805,7 +849,7 @@ bool js::Stringify(JSContext* cx, MutableHandleValue vp, JSObject* replacer_,
     RootedObject spaceObj(cx, &space.toObject());
 
     ESClass cls;
-    if (!GetBuiltinClass(cx, spaceObj, &cls)) {
+    if (!JS::GetBuiltinClass(cx, spaceObj, &cls)) {
       return false;
     }
 
@@ -830,7 +874,7 @@ bool js::Stringify(JSContext* cx, MutableHandleValue vp, JSObject* replacer_,
     /* Step 6. */
     double d;
     MOZ_ALWAYS_TRUE(ToInteger(cx, space, &d));
-    d = Min(10.0, d);
+    d = std::min(10.0, d);
     if (d >= 1 && !gap.appendN(' ', uint32_t(d))) {
       return false;
     }
@@ -840,7 +884,7 @@ bool js::Stringify(JSContext* cx, MutableHandleValue vp, JSObject* replacer_,
     if (!str) {
       return false;
     }
-    size_t len = Min(size_t(10), str->length());
+    size_t len = std::min(size_t(10), str->length());
     if (!gap.appendSubstring(str, 0, len)) {
       return false;
     }
@@ -883,7 +927,8 @@ bool js::Stringify(JSContext* cx, MutableHandleValue vp, JSObject* replacer_,
 /* ES5 15.12.2 Walk. */
 static bool Walk(JSContext* cx, HandleObject holder, HandleId name,
                  HandleValue reviver, MutableHandleValue vp) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
 
@@ -905,7 +950,7 @@ static bool Walk(JSContext* cx, HandleObject holder, HandleId name,
     if (isArray) {
       /* Step 2a(ii). */
       uint32_t length;
-      if (!GetLengthProperty(cx, obj, &length)) {
+      if (!GetLengthPropertyForArray(cx, obj, &length)) {
         return false;
       }
 
@@ -934,8 +979,11 @@ static bool Walk(JSContext* cx, HandleObject holder, HandleId name,
           }
         } else {
           /* Step 2a(iii)(3). The spec deliberately ignores strict failure. */
-          Rooted<PropertyDescriptor> desc(cx);
-          desc.setDataDescriptor(newElement, JSPROP_ENUMERATE);
+          Rooted<PropertyDescriptor> desc(
+              cx, PropertyDescriptor::Data(newElement,
+                                           {JS::PropertyAttribute::Configurable,
+                                            JS::PropertyAttribute::Enumerable,
+                                            JS::PropertyAttribute::Writable}));
           if (!DefineProperty(cx, obj, id, desc, ignored)) {
             return false;
           }
@@ -970,8 +1018,11 @@ static bool Walk(JSContext* cx, HandleObject holder, HandleId name,
           }
         } else {
           /* Step 2b(ii)(3). The spec deliberately ignores strict failure. */
-          Rooted<PropertyDescriptor> desc(cx);
-          desc.setDataDescriptor(newElement, JSPROP_ENUMERATE);
+          Rooted<PropertyDescriptor> desc(
+              cx, PropertyDescriptor::Data(newElement,
+                                           {JS::PropertyAttribute::Configurable,
+                                            JS::PropertyAttribute::Enumerable,
+                                            JS::PropertyAttribute::Writable}));
           if (!DefineProperty(cx, obj, id, desc, ignored)) {
             return false;
           }
@@ -1009,7 +1060,8 @@ bool js::ParseJSONWithReviver(JSContext* cx,
                               const mozilla::Range<const CharT> chars,
                               HandleValue reviver, MutableHandleValue vp) {
   /* 15.12.2 steps 2-3. */
-  Rooted<JSONParser<CharT>> parser(cx, JSONParser<CharT>(cx, chars));
+  Rooted<JSONParser<CharT>> parser(
+      cx, JSONParser<CharT>(cx, chars, JSONParserBase::ParseType::JSONParse));
   if (!parser.parse(vp)) {
     return false;
   }
@@ -1101,30 +1153,21 @@ static const JSFunctionSpec json_static_methods[] = {
     JS_FN("parse", json_parse, 2, 0), JS_FN("stringify", json_stringify, 3, 0),
     JS_FS_END};
 
-JSObject* js::InitJSONClass(JSContext* cx, Handle<GlobalObject*> global) {
+static const JSPropertySpec json_static_properties[] = {
+    JS_STRING_SYM_PS(toStringTag, "JSON", JSPROP_READONLY), JS_PS_END};
+
+static JSObject* CreateJSONObject(JSContext* cx, JSProtoKey key) {
+  Handle<GlobalObject*> global = cx->global();
   RootedObject proto(cx, GlobalObject::getOrCreateObjectPrototype(cx, global));
   if (!proto) {
     return nullptr;
   }
-  RootedObject JSON(
-      cx, NewObjectWithGivenProto(cx, &JSONClass, proto, SingletonObject));
-  if (!JSON) {
-    return nullptr;
-  }
-
-  if (!JS_DefineProperty(cx, global, js_JSON_str, JSON, JSPROP_RESOLVING)) {
-    return nullptr;
-  }
-
-  if (!JS_DefineFunctions(cx, JSON, json_static_methods)) {
-    return nullptr;
-  }
-
-  if (!DefineToStringTag(cx, JSON, cx->names().JSON)) {
-    return nullptr;
-  }
-
-  global->setConstructor(JSProto_JSON, ObjectValue(*JSON));
-
-  return JSON;
+  return NewTenuredObjectWithGivenProto(cx, &JSONClass, proto);
 }
+
+static const ClassSpec JSONClassSpec = {
+    CreateJSONObject, nullptr, json_static_methods, json_static_properties};
+
+const JSClass js::JSONClass = {js_JSON_str,
+                               JSCLASS_HAS_CACHED_PROTO(JSProto_JSON),
+                               JS_NULL_CLASS_OPS, &JSONClassSpec};
