@@ -7,16 +7,14 @@
 #ifndef builtin_intl_PluralRules_h
 #define builtin_intl_PluralRules_h
 
-#include "mozilla/Attributes.h"
-
 #include "builtin/SelfHostingDefines.h"
 #include "js/Class.h"
 #include "js/RootingAPI.h"
 #include "vm/NativeObject.h"
 
-struct UFormattedNumber;
-struct UNumberFormatter;
-struct UPluralRules;
+namespace mozilla::intl {
+class PluralRules;
+}
 
 namespace js {
 
@@ -27,56 +25,29 @@ class PluralRulesObject : public NativeObject {
 
   static constexpr uint32_t INTERNALS_SLOT = 0;
   static constexpr uint32_t UPLURAL_RULES_SLOT = 1;
-  static constexpr uint32_t UNUMBER_FORMATTER_SLOT = 2;
-  static constexpr uint32_t UFORMATTED_NUMBER_SLOT = 3;
-  static constexpr uint32_t SLOT_COUNT = 4;
+  static constexpr uint32_t SLOT_COUNT = 2;
 
   static_assert(INTERNALS_SLOT == INTL_INTERNALS_OBJECT_SLOT,
                 "INTERNALS_SLOT must match self-hosting define for internals "
                 "object slot");
 
-  // Estimated memory use for UNumberFormatter and UFormattedNumber.
-  static constexpr size_t UNumberFormatterEstimatedMemoryUse = 750;
+  // Estimated memory use for UPluralRules (see IcuMemoryUsage).
+  // Includes usage for UNumberFormat since our PluralRules impl
+  // contains a NumberFormat object.
+  static constexpr size_t UPluralRulesEstimatedMemoryUse = 3726;
 
-  // Estimated memory use for UPluralRules.
-  static constexpr size_t UPluralRulesEstimatedMemoryUse = 2976;
-
-  UPluralRules* getPluralRules() const {
+  mozilla::intl::PluralRules* getPluralRules() const {
     const auto& slot = getFixedSlot(UPLURAL_RULES_SLOT);
     if (slot.isUndefined()) {
       return nullptr;
     }
-    return static_cast<UPluralRules*>(slot.toPrivate());
+    return static_cast<mozilla::intl::PluralRules*>(slot.toPrivate());
   }
 
-  void setPluralRules(UPluralRules* pluralRules) {
+  void setPluralRules(mozilla::intl::PluralRules* pluralRules) {
     setFixedSlot(UPLURAL_RULES_SLOT, PrivateValue(pluralRules));
   }
 
-  UNumberFormatter* getNumberFormatter() const {
-    const auto& slot = getFixedSlot(UNUMBER_FORMATTER_SLOT);
-    if (slot.isUndefined()) {
-      return nullptr;
-    }
-    return static_cast<UNumberFormatter*>(slot.toPrivate());
-  }
-
-  void setNumberFormatter(UNumberFormatter* formatter) {
-    setFixedSlot(UNUMBER_FORMATTER_SLOT, PrivateValue(formatter));
-  }
-
-  UFormattedNumber* getFormattedNumber() const {
-    const auto& slot = getFixedSlot(UFORMATTED_NUMBER_SLOT);
-    if (slot.isUndefined()) {
-      return nullptr;
-    }
-    return static_cast<UFormattedNumber*>(slot.toPrivate());
-  }
-
-  void setFormattedNumber(UFormattedNumber* formatted) {
-    setFixedSlot(UFORMATTED_NUMBER_SLOT, PrivateValue(formatted));
-  }
-
  private:
   static const JSClassOps classOps_;
   static const ClassSpec classSpec_;
@@ -93,8 +64,8 @@ class PluralRulesObject : public NativeObject {
  *
  * Usage: rule = intl_SelectPluralRule(pluralRules, x)
  */
-extern MOZ_MUST_USE bool intl_SelectPluralRule(JSContext* cx, unsigned argc,
-                                               JS::Value* vp);
+[[nodiscard]] extern bool intl_SelectPluralRule(JSContext* cx, unsigned argc,
+                                                JS::Value* vp);
 
 /**
  * Returns an array of plural rules categories for a given pluralRules object.
@@ -106,8 +77,8 @@ extern MOZ_MUST_USE bool intl_SelectPluralRule(JSContext* cx, unsigned argc,
  * pluralRules = new Intl.PluralRules('pl', {type: 'cardinal'});
  * intl_getPluralCategories(pluralRules); // ['one', 'few', 'many', 'other']
  */
-extern MOZ_MUST_USE bool intl_GetPluralCategories(JSContext* cx, unsigned argc,
-                                                  JS::Value* vp);
+[[nodiscard]] extern bool intl_GetPluralCategories(JSContext* cx, unsigned argc,
+                                                   JS::Value* vp);
 
 }  // namespace js
 
