@@ -38,7 +38,9 @@
  */
 
 #include "mozilla/HashFunctions.h"
-#include "mozilla/Move.h"
+#include "mozilla/MemoryReporting.h"
+
+#include <utility>
 
 #include "js/HashTable.h"
 
@@ -54,8 +56,8 @@ namespace detail {
 template <class T, class Ops, class AllocPolicy>
 class OrderedHashTable {
  public:
-  typedef typename Ops::KeyType Key;
-  typedef typename Ops::Lookup Lookup;
+  using Key = typename Ops::KeyType;
+  using Lookup = typename Ops::Lookup;
 
   struct Data {
     T element;
@@ -99,7 +101,7 @@ class OrderedHashTable {
   }
 
  public:
-  OrderedHashTable(AllocPolicy& ap, mozilla::HashCodeScrambler hcs)
+  OrderedHashTable(AllocPolicy ap, mozilla::HashCodeScrambler hcs)
       : hashTable(nullptr),
         data(nullptr),
         dataLength(0),
@@ -108,10 +110,10 @@ class OrderedHashTable {
         hashShift(0),
         ranges(nullptr),
         nurseryRanges(nullptr),
-        alloc(ap),
+        alloc(std::move(ap)),
         hcs(hcs) {}
 
-  MOZ_MUST_USE bool init() {
+  [[nodiscard]] bool init() {
     MOZ_ASSERT(!hashTable, "init must be called at most once");
 
     uint32_t buckets = initialBuckets();
@@ -144,10 +146,24 @@ class OrderedHashTable {
 
   ~OrderedHashTable() {
     forEachRange<Range::onTableDestroyed>();
-    alloc.free_(hashTable, hashBuckets());
+    if (hashTable) {
+      // |hashBuckets()| isn't valid when |hashTable| hasn't been created.
+      alloc.free_(hashTable, hashBuckets());
+    }
     freeData(data, dataLength, dataCapacity);
   }
 
+  size_t sizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const {
+    size_t size = 0;
+    if (hashTable) {
+      size += mallocSizeOf(hashTable);
+    }
+    if (data) {
+      size += mallocSizeOf(data);
+    }
+    return size;
+  }
+
   /* Return the number of elements in the table. */
   uint32_t count() const { return liveCount; }
 
@@ -174,7 +190,7 @@ class OrderedHashTable {
    * means the element was not added to the table.
    */
   template <typename ElementInput>
-  MOZ_MUST_USE bool put(ElementInput&& element) {
+  [[nodiscard]] bool put(ElementInput&& element) {
     HashNumber h = prepareHash(Ops::getKey(element));
     if (Data* e = lookup(Ops::getKey(element), h)) {
       e->element = std::forward<ElementInput>(element);
@@ -248,7 +264,7 @@ class OrderedHashTable {
    * particular, those Ranges are still live and will see any entries added
    * after a successful clear().
    */
-  MOZ_MUST_USE bool clear() {
+  [[nodiscard]] bool clear() {
     if (dataLength != 0) {
       Data** oldHashTable = hashTable;
       Data* oldData = data;
@@ -688,7 +704,7 @@ class OrderedHashTable {
    * empty elements in data[0:dataLength]. On allocation failure, this
    * leaves everything as it was and returns false.
    */
-  MOZ_MUST_USE bool rehash(uint32_t newHashShift) {
+  [[nodiscard]] bool rehash(uint32_t newHashShift) {
     // If the size of the table is not changing, rehash in place to avoid
     // allocating memory.
     if (newHashShift == hashShift) {
@@ -777,7 +793,7 @@ class OrderedHashMap {
 
  private:
   struct MapOps : OrderedHashPolicy {
-    typedef Key KeyType;
+    using KeyType = Key;
     static void makeEmpty(Entry* e) {
       OrderedHashPolicy::makeEmpty(const_cast<Key*>(&e->key));
 
@@ -793,21 +809,21 @@ class OrderedHashMap {
   Impl impl;
 
  public:
-  typedef typename Impl::Range Range;
+  using Range = typename Impl::Range;
 
   OrderedHashMap(AllocPolicy ap, mozilla::HashCodeScrambler hcs)
-      : impl(ap, hcs) {}
-  MOZ_MUST_USE bool init() { return impl.init(); }
+      : impl(std::move(ap), hcs) {}
+  [[nodiscard]] bool init() { return impl.init(); }
   uint32_t count() const { return impl.count(); }
   bool has(const Key& key) const { return impl.has(key); }
   Range all() { return impl.all(); }
   const Entry* get(const Key& key) const { return impl.get(key); }
   Entry* get(const Key& key) { return impl.get(key); }
   bool remove(const Key& key, bool* foundp) { return impl.remove(key, foundp); }
-  MOZ_MUST_USE bool clear() { return impl.clear(); }
+  [[nodiscard]] bool clear() { return impl.clear(); }
 
   template <typename V>
-  MOZ_MUST_USE bool put(const Key& key, V&& value) {
+  [[nodiscard]] bool put(const Key& key, V&& value) {
     return impl.put(Entry(key, std::forward<V>(value)));
   }
 
@@ -834,13 +850,20 @@ class OrderedHashMap {
     return Impl::offsetOfDataElement();
   }
   static constexpr size_t sizeofImplData() { return Impl::sizeofData(); }
+
+  size_t sizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const {
+    return impl.sizeOfExcludingThis(mallocSizeOf);
+  }
+  size_t sizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf) const {
+    return mallocSizeOf(this) + sizeOfExcludingThis(mallocSizeOf);
+  }
 };
 
 template <class T, class OrderedHashPolicy, class AllocPolicy>
 class OrderedHashSet {
  private:
   struct SetOps : OrderedHashPolicy {
-    typedef const T KeyType;
+    using KeyType = const T;
     static const T& getKey(const T& v) { return v; }
     static void setKey(const T& e, const T& v) { const_cast<T&>(e) = v; }
   };
@@ -849,19 +872,19 @@ class OrderedHashSet {
   Impl impl;
 
  public:
-  typedef typename Impl::Range Range;
+  using Range = typename Impl::Range;
 
   explicit OrderedHashSet(AllocPolicy ap, mozilla::HashCodeScrambler hcs)
-      : impl(ap, hcs) {}
-  MOZ_MUST_USE bool init() { return impl.init(); }
+      : impl(std::move(ap), hcs) {}
+  [[nodiscard]] bool init() { return impl.init(); }
   uint32_t count() const { return impl.count(); }
   bool has(const T& value) const { return impl.has(value); }
   Range all() { return impl.all(); }
-  MOZ_MUST_USE bool put(const T& value) { return impl.put(value); }
+  [[nodiscard]] bool put(const T& value) { return impl.put(value); }
   bool remove(const T& value, bool* foundp) {
     return impl.remove(value, foundp);
   }
-  MOZ_MUST_USE bool clear() { return impl.clear(); }
+  [[nodiscard]] bool clear() { return impl.clear(); }
 
   HashNumber hash(const T& value) const { return impl.prepareHash(value); }
 
@@ -882,6 +905,13 @@ class OrderedHashSet {
     return Impl::offsetOfDataElement();
   }
   static constexpr size_t sizeofImplData() { return Impl::sizeofData(); }
+
+  size_t sizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const {
+    return impl.sizeOfExcludingThis(mallocSizeOf);
+  }
+  size_t sizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf) const {
+    return mallocSizeOf(this) + sizeOfExcludingThis(mallocSizeOf);
+  }
 };
 
 }  // namespace js
