@@ -22,17 +22,17 @@ namespace frontend {
 // If the ParseNode is actually bad, we crash.
 
 #ifdef DEBUG
-extern MOZ_MUST_USE bool CheckParseTree(JSContext* cx, const LifoAlloc& alloc,
-                                        ParseNode* pn);
+[[nodiscard]] extern bool CheckParseTree(JSContext* cx, const LifoAlloc& alloc,
+                                         ParseNode* pn);
 #else
-inline MOZ_MUST_USE bool CheckParseTree(JSContext* cx, const LifoAlloc& alloc,
-                                        ParseNode* pn) {
+[[nodiscard]] inline bool CheckParseTree(JSContext* cx, const LifoAlloc& alloc,
+                                         ParseNode* pn) {
   return true;
 }
 #endif
 
-inline MOZ_MUST_USE bool CheckParseTree(JSContext* cx, const LifoAlloc& alloc,
-                                        SyntaxParseHandler::Node pn) {
+[[nodiscard]] inline bool CheckParseTree(JSContext* cx, const LifoAlloc& alloc,
+                                         SyntaxParseHandler::Node pn) {
   return true;
 }
 
