@@ -100,7 +100,7 @@ static bool assignImportKind(const Import& import, HandleObject obj,
 static int testWasmFuzz(const uint8_t* buf, size_t size) {
   auto gcGuard = mozilla::MakeScopeExit([&] {
     JS::PrepareForFullGC(gCx);
-    JS::NonIncrementalGC(gCx, GC_NORMAL, JS::GCReason::API);
+    JS::NonIncrementalGC(gCx, JS::GCOptions::Normal, JS::GCReason::API);
   });
 
   const size_t MINIMUM_MODULE_SIZE = 8;
@@ -143,12 +143,14 @@ static int testWasmFuzz(const uint8_t* buf, size_t size) {
       // take into account whether those compilers support particular features
       // that may have been enabled.
       bool enableWasmBaseline = ((optByte & 0xF0) == (1 << 7));
-      bool enableWasmIon =
-          IonPlatformSupport() && ((optByte & 0xF0) == (1 << 6));
-      bool enableWasmCranelift = false;
+      bool enableWasmOptimizing = false;
 #ifdef ENABLE_WASM_CRANELIFT
-      enableWasmCranelift =
+      // Cranelift->Ion transition
+      enableWasmOptimizing =
           CraneliftPlatformSupport() && ((optByte & 0xF0) == (1 << 5));
+#else
+      enableWasmOptimizing =
+          IonPlatformSupport() && ((optByte & 0xF0) == (1 << 6));
 #endif
       bool enableWasmAwaitTier2 = (IonPlatformSupport()
 #ifdef ENABLE_WASM_CRANELIFT
@@ -157,35 +159,37 @@ static int testWasmFuzz(const uint8_t* buf, size_t size) {
                                        ) &&
                                   ((optByte & 0xF) == (1 << 3));
 
-      if (!enableWasmBaseline && !enableWasmIon && !enableWasmCranelift) {
-        // If nothing is selected explicitly, select Ion to test
-        // more platform specific JIT code. However, on some platforms,
-        // e.g. ARM64, we do not have Ion available, so we need to switch
-        // to baseline instead.
-        if (IonPlatformSupport()) {
-          enableWasmIon = true;
+      if (!enableWasmBaseline && !enableWasmOptimizing) {
+        // If nothing is selected explicitly, enable an optimizing compiler to
+        // test more platform specific JIT code. However, on some platforms,
+        // e.g. ARM64 on Windows, we do not have Ion available, so we need to
+        // switch to baseline instead.
+        if (IonPlatformSupport() || CraneliftPlatformSupport()) {
+          enableWasmOptimizing = true;
         } else {
           enableWasmBaseline = true;
         }
       }
 
       if (enableWasmAwaitTier2) {
-        // Tier 2 needs Baseline + {Ion,Cranelift}
+        // Tier 2 needs Baseline + Optimizing
         enableWasmBaseline = true;
 
-        if (!enableWasmIon && !enableWasmCranelift) {
-          enableWasmIon = true;
+        if (!enableWasmOptimizing) {
+          enableWasmOptimizing = true;
         }
       }
 
       JS::ContextOptionsRef(gCx)
           .setWasmBaseline(enableWasmBaseline)
-          .setWasmIon(enableWasmIon)
-          .setTestWasmAwaitTier2(enableWasmAwaitTier2)
 #ifdef ENABLE_WASM_CRANELIFT
-          .setWasmCranelift(enableWasmCranelift)
+          .setWasmCranelift(enableWasmOptimizing)
+          .setWasmIon(false)
+#else
+          .setWasmCranelift(false)
+          .setWasmIon(enableWasmOptimizing)
 #endif
-          ;
+          .setTestWasmAwaitTier2(enableWasmAwaitTier2);
     }
 
     // Expected header for a valid WebAssembly module
@@ -209,8 +213,9 @@ static int testWasmFuzz(const uint8_t* buf, size_t size) {
     currentIndex += moduleLen;
 
     ScriptedCaller scriptedCaller;
+    FeatureOptions options;
     SharedCompileArgs compileArgs =
-        CompileArgs::build(gCx, std::move(scriptedCaller));
+        CompileArgs::build(gCx, std::move(scriptedCaller), options);
     if (!compileArgs) {
       return 0;
     }
@@ -251,6 +256,9 @@ static int testWasmFuzz(const uint8_t* buf, size_t size) {
     size_t currentTableExportId = 0;
     size_t currentMemoryExportId = 0;
     size_t currentGlobalExportId = 0;
+#ifdef ENABLE_WASM_EXCEPTIONS
+    size_t currentEventExportId = 0;
+#endif
 
     for (const Import& import : importVec) {
       // First try to get the namespace object, create one if this is the
@@ -320,6 +328,17 @@ static int testWasmFuzz(const uint8_t* buf, size_t size) {
               return 0;
             }
             break;
+
+#ifdef ENABLE_WASM_EXCEPTIONS
+          case DefinitionKind::Event:
+            // TODO: Pass a dummy defaultValue
+            if (!assignImportKind<WasmExceptionObject>(
+                    import, obj, lastExportsObj, lastExportIds,
+                    &currentEventExportId, exportsLength, nullValue)) {
+              return 0;
+            }
+            break;
+#endif
         }
       }
     }
@@ -420,7 +439,7 @@ static int testWasmFuzz(const uint8_t* buf, size_t size) {
           Rooted<WasmGlobalObject*> global(gCx,
                                            &propObj->as<WasmGlobalObject>());
           if (global->type() != ValType::I64) {
-            global->value(gCx, &lastReturnVal);
+            global->val().get().toJSValue(gCx, &lastReturnVal);
           }
         }
       }
