@@ -9,14 +9,20 @@
 #include "mozilla/DebugOnly.h"
 #include "mozilla/TimeStamp.h"
 
+#include <type_traits>
+
 #include "gc/GCInternals.h"
-#include "gc/GCTrace.h"
+#include "gc/GCLock.h"
+#include "gc/GCProbes.h"
 #include "gc/Nursery.h"
-#include "jit/JitRealm.h"
 #include "threading/CpuCount.h"
+#include "util/Poison.h"
+#include "vm/GetterSetter.h"
 #include "vm/JSContext.h"
+#include "vm/PropMap.h"
 #include "vm/Runtime.h"
 #include "vm/StringType.h"
+#include "vm/TraceLogging.h"
 
 #include "gc/ArenaList-inl.h"
 #include "gc/Heap-inl.h"
@@ -31,8 +37,9 @@ using namespace gc;
 
 template <AllowGC allowGC /* = CanGC */>
 JSObject* js::AllocateObject(JSContext* cx, AllocKind kind,
-                             size_t nDynamicSlots, InitialHeap heap,
-                             const Class* clasp) {
+                             size_t nDynamicSlots, gc::InitialHeap heap,
+                             const JSClass* clasp,
+                             AllocSite* site /* = nullptr */) {
   MOZ_ASSERT(IsObjectAllocKind(kind));
   size_t thingSize = Arena::thingSize(kind);
 
@@ -42,7 +49,10 @@ JSObject* js::AllocateObject(JSContext* cx, AllocKind kind,
       sizeof(JSObject_Slots0) >= MinCellSize,
       "All allocations must be at least the allocator-imposed minimum size.");
 
-  MOZ_ASSERT_IF(nDynamicSlots != 0, clasp->isNative());
+  MOZ_ASSERT_IF(nDynamicSlots != 0, clasp->isNativeObject());
+
+  MOZ_ASSERT_IF(site && site->initialHeap() == TenuredHeap,
+                heap == TenuredHeap);
 
   // We cannot trigger GC or make runtime assertions when nursery allocation
   // is suppressed, either explicitly or because we are off-thread.
@@ -61,8 +71,12 @@ JSObject* js::AllocateObject(JSContext* cx, AllocKind kind,
   }
 
   if (cx->nursery().isEnabled() && heap != TenuredHeap) {
-    JSObject* obj = rt->gc.tryNewNurseryObject<allowGC>(cx, thingSize,
-                                                        nDynamicSlots, clasp);
+    if (!site) {
+      site = cx->zone()->unknownAllocSite();
+    }
+
+    JSObject* obj = rt->gc.tryNewNurseryObject<allowGC>(
+        cx, thingSize, nDynamicSlots, clasp, site);
     if (obj) {
       return obj;
     }
@@ -83,11 +97,13 @@ JSObject* js::AllocateObject(JSContext* cx, AllocKind kind,
 template JSObject* js::AllocateObject<NoGC>(JSContext* cx, gc::AllocKind kind,
                                             size_t nDynamicSlots,
                                             gc::InitialHeap heap,
-                                            const Class* clasp);
+                                            const JSClass* clasp,
+                                            gc::AllocSite* site);
 template JSObject* js::AllocateObject<CanGC>(JSContext* cx, gc::AllocKind kind,
                                              size_t nDynamicSlots,
                                              gc::InitialHeap heap,
-                                             const Class* clasp);
+                                             const JSClass* clasp,
+                                             gc::AllocSite* site);
 
 // Attempt to allocate a new JSObject out of the nursery. If there is not
 // enough room in the nursery or there is an OOM, this method will return
@@ -95,15 +111,16 @@ template JSObject* js::AllocateObject<CanGC>(JSContext* cx, gc::AllocKind kind,
 template <AllowGC allowGC>
 JSObject* GCRuntime::tryNewNurseryObject(JSContext* cx, size_t thingSize,
                                          size_t nDynamicSlots,
-                                         const Class* clasp) {
-  MOZ_RELEASE_ASSERT(!cx->helperThread());
+                                         const JSClass* clasp,
+                                         AllocSite* site) {
+  MOZ_RELEASE_ASSERT(!cx->isHelperThreadContext());
 
   MOZ_ASSERT(cx->isNurseryAllocAllowed());
   MOZ_ASSERT(!cx->isNurseryAllocSuppressed());
   MOZ_ASSERT(!cx->zone()->isAtomsZone());
 
   JSObject* obj =
-      cx->nursery().allocateObject(cx, thingSize, nDynamicSlots, clasp);
+      cx->nursery().allocateObject(site, thingSize, nDynamicSlots, clasp);
   if (obj) {
     return obj;
   }
@@ -113,7 +130,8 @@ JSObject* GCRuntime::tryNewNurseryObject(JSContext* cx, size_t thingSize,
 
     // Exceeding gcMaxBytes while tenuring can disable the Nursery.
     if (cx->nursery().isEnabled()) {
-      return cx->nursery().allocateObject(cx, thingSize, nDynamicSlots, clasp);
+      return cx->nursery().allocateObject(site, thingSize, nDynamicSlots,
+                                          clasp);
     }
   }
   return nullptr;
@@ -123,26 +141,31 @@ template <AllowGC allowGC>
 JSObject* GCRuntime::tryNewTenuredObject(JSContext* cx, AllocKind kind,
                                          size_t thingSize,
                                          size_t nDynamicSlots) {
-  HeapSlot* slots = nullptr;
+  ObjectSlots* slotsHeader = nullptr;
   if (nDynamicSlots) {
-    slots = cx->maybe_pod_malloc<HeapSlot>(nDynamicSlots);
-    if (MOZ_UNLIKELY(!slots)) {
+    HeapSlot* allocation =
+        cx->maybe_pod_malloc<HeapSlot>(ObjectSlots::allocCount(nDynamicSlots));
+    if (MOZ_UNLIKELY(!allocation)) {
       if (allowGC) {
         ReportOutOfMemory(cx);
       }
       return nullptr;
     }
-    Debug_SetSlotRangeToCrashOnTouch(slots, nDynamicSlots);
+
+    slotsHeader = new (allocation) ObjectSlots(nDynamicSlots, 0);
+    Debug_SetSlotRangeToCrashOnTouch(slotsHeader->slots(), nDynamicSlots);
   }
 
   JSObject* obj = tryNewTenuredThing<JSObject, allowGC>(cx, kind, thingSize);
 
   if (obj) {
     if (nDynamicSlots) {
-      static_cast<NativeObject*>(obj)->initSlots(slots);
+      static_cast<NativeObject*>(obj)->initSlots(slotsHeader->slots());
+      AddCellMemory(obj, ObjectSlots::allocSize(nDynamicSlots),
+                    MemoryUse::ObjectSlots);
     }
   } else {
-    js_free(slots);
+    js_free(slotsHeader);
   }
 
   return obj;
@@ -155,11 +178,12 @@ JSString* GCRuntime::tryNewNurseryString(JSContext* cx, size_t thingSize,
                                          AllocKind kind) {
   MOZ_ASSERT(IsNurseryAllocable(kind));
   MOZ_ASSERT(cx->isNurseryAllocAllowed());
-  MOZ_ASSERT(!cx->helperThread());
+  MOZ_ASSERT(!cx->isHelperThreadContext());
   MOZ_ASSERT(!cx->isNurseryAllocSuppressed());
   MOZ_ASSERT(!cx->zone()->isAtomsZone());
 
-  Cell* cell = cx->nursery().allocateString(cx->zone(), thingSize, kind);
+  AllocSite* site = cx->zone()->unknownAllocSite();
+  Cell* cell = cx->nursery().allocateString(site, thingSize);
   if (cell) {
     return static_cast<JSString*>(cell);
   }
@@ -171,7 +195,7 @@ JSString* GCRuntime::tryNewNurseryString(JSContext* cx, size_t thingSize,
     // other heuristics can disable nursery strings for this zone.
     if (cx->nursery().isEnabled() && cx->zone()->allocNurseryStrings) {
       return static_cast<JSString*>(
-          cx->nursery().allocateString(cx->zone(), thingSize, kind));
+          cx->nursery().allocateString(site, thingSize));
     }
   }
   return nullptr;
@@ -179,7 +203,7 @@ JSString* GCRuntime::tryNewNurseryString(JSContext* cx, size_t thingSize,
 
 template <typename StringAllocT, AllowGC allowGC /* = CanGC */>
 StringAllocT* js::AllocateStringImpl(JSContext* cx, InitialHeap heap) {
-  static_assert(mozilla::IsConvertible<StringAllocT*, JSString*>::value,
+  static_assert(std::is_convertible_v<StringAllocT*, JSString*>,
                 "must be JSString derived");
 
   AllocKind kind = MapTypeToFinalizeKind<StringAllocT>::kind;
@@ -204,7 +228,7 @@ StringAllocT* js::AllocateStringImpl(JSContext* cx, InitialHeap heap) {
 
   if (cx->nursery().isEnabled() && heap != TenuredHeap &&
       cx->nursery().canAllocateStrings() && cx->zone()->allocNurseryStrings) {
-    auto str = static_cast<StringAllocT*>(
+    auto* str = static_cast<StringAllocT*>(
         rt->gc.tryNewNurseryString<allowGC>(cx, size, kind));
     if (str) {
       return str;
@@ -223,6 +247,82 @@ StringAllocT* js::AllocateStringImpl(JSContext* cx, InitialHeap heap) {
   return GCRuntime::tryNewTenuredThing<StringAllocT, allowGC>(cx, kind, size);
 }
 
+// Attempt to allocate a new BigInt out of the nursery. If there is not enough
+// room in the nursery or there is an OOM, this method will return nullptr.
+template <AllowGC allowGC>
+JS::BigInt* GCRuntime::tryNewNurseryBigInt(JSContext* cx, size_t thingSize,
+                                           AllocKind kind) {
+  MOZ_ASSERT(IsNurseryAllocable(kind));
+  MOZ_ASSERT(cx->isNurseryAllocAllowed());
+  MOZ_ASSERT(!cx->isHelperThreadContext());
+  MOZ_ASSERT(!cx->isNurseryAllocSuppressed());
+  MOZ_ASSERT(!cx->zone()->isAtomsZone());
+
+  AllocSite* site = cx->zone()->unknownAllocSite();
+  Cell* cell = cx->nursery().allocateBigInt(site, thingSize);
+  if (cell) {
+    return static_cast<JS::BigInt*>(cell);
+  }
+
+  if (allowGC && !cx->suppressGC) {
+    cx->runtime()->gc.minorGC(JS::GCReason::OUT_OF_NURSERY);
+
+    // Exceeding gcMaxBytes while tenuring can disable the Nursery, and
+    // other heuristics can disable nursery BigInts for this zone.
+    if (cx->nursery().isEnabled() && cx->zone()->allocNurseryBigInts) {
+      return static_cast<JS::BigInt*>(
+          cx->nursery().allocateBigInt(site, thingSize));
+    }
+  }
+  return nullptr;
+}
+
+template <AllowGC allowGC /* = CanGC */>
+JS::BigInt* js::AllocateBigInt(JSContext* cx, InitialHeap heap) {
+  AllocKind kind = MapTypeToFinalizeKind<JS::BigInt>::kind;
+  size_t size = sizeof(JS::BigInt);
+  MOZ_ASSERT(size == Arena::thingSize(kind));
+
+  // Off-thread alloc cannot trigger GC or make runtime assertions.
+  if (cx->isNurseryAllocSuppressed()) {
+    JS::BigInt* bi =
+        GCRuntime::tryNewTenuredThing<JS::BigInt, NoGC>(cx, kind, size);
+    if (MOZ_UNLIKELY(allowGC && !bi)) {
+      ReportOutOfMemory(cx);
+    }
+    return bi;
+  }
+
+  JSRuntime* rt = cx->runtime();
+  if (!rt->gc.checkAllocatorState<allowGC>(cx, kind)) {
+    return nullptr;
+  }
+
+  if (cx->nursery().isEnabled() && heap != TenuredHeap &&
+      cx->nursery().canAllocateBigInts() && cx->zone()->allocNurseryBigInts) {
+    auto* bi = static_cast<JS::BigInt*>(
+        rt->gc.tryNewNurseryBigInt<allowGC>(cx, size, kind));
+    if (bi) {
+      return bi;
+    }
+
+    // Our most common non-jit allocation path is NoGC; thus, if we fail the
+    // alloc and cannot GC, we *must* return nullptr here so that the caller
+    // will do a CanGC allocation to clear the nursery. Failing to do so will
+    // cause all allocations on this path to land in Tenured, and we will not
+    // get the benefit of the nursery.
+    if (!allowGC) {
+      return nullptr;
+    }
+  }
+
+  return GCRuntime::tryNewTenuredThing<JS::BigInt, allowGC>(cx, kind, size);
+}
+template JS::BigInt* js::AllocateBigInt<NoGC>(JSContext* cx,
+                                              gc::InitialHeap heap);
+template JS::BigInt* js::AllocateBigInt<CanGC>(JSContext* cx,
+                                               gc::InitialHeap heap);
+
 #define DECL_ALLOCATOR_INSTANCES(allocKind, traceKind, type, sizedType, \
                                  bgfinal, nursery, compact)             \
   template type* js::AllocateStringImpl<type, NoGC>(JSContext * cx,     \
@@ -234,7 +334,7 @@ FOR_EACH_NURSERY_STRING_ALLOCKIND(DECL_ALLOCATOR_INSTANCES)
 
 template <typename T, AllowGC allowGC /* = CanGC */>
 T* js::Allocate(JSContext* cx) {
-  static_assert(!mozilla::IsConvertible<T*, JSObject*>::value,
+  static_assert(!std::is_convertible_v<T*, JSObject*>,
                 "must not be JSObject derived");
   static_assert(
       sizeof(T) >= MinCellSize,
@@ -244,7 +344,7 @@ T* js::Allocate(JSContext* cx) {
   size_t thingSize = sizeof(T);
   MOZ_ASSERT(thingSize == Arena::thingSize(kind));
 
-  if (!cx->helperThread()) {
+  if (!cx->isHelperThreadContext()) {
     if (!cx->runtime()->gc.checkAllocatorState<allowGC>(cx, kind)) {
       return nullptr;
     }
@@ -265,7 +365,7 @@ template <typename T, AllowGC allowGC>
 T* GCRuntime::tryNewTenuredThing(JSContext* cx, AllocKind kind,
                                  size_t thingSize) {
   // Bump allocate in the arena's current free-list span.
-  T* t = reinterpret_cast<T*>(cx->freeLists().allocate(kind));
+  auto* t = reinterpret_cast<T*>(cx->freeLists().allocate(kind));
   if (MOZ_UNLIKELY(!t)) {
     // Get the next available free list and allocate out of it. This may
     // acquire a new arena, which will lock the chunk list. If there are no
@@ -287,7 +387,7 @@ T* GCRuntime::tryNewTenuredThing(JSContext* cx, AllocKind kind,
   }
 
   checkIncrementalZoneState(cx, t);
-  gcTracer.traceTenuredAlloc(t, kind);
+  gcprobes::TenuredAlloc(t, kind);
   // We count this regardless of the profiler's state, assuming that it costs
   // just as much to count it, as to check the profiler's state and decide not
   // to count it.
@@ -300,7 +400,7 @@ void GCRuntime::attemptLastDitchGC(JSContext* cx) {
   // size limit. Try to perform an all-compartments, non-incremental, shrinking
   // GC and wait for it to finish.
 
-  if (cx->helperThread()) {
+  if (cx->isHelperThreadContext()) {
     return;
   }
 
@@ -310,7 +410,7 @@ void GCRuntime::attemptLastDitchGC(JSContext* cx) {
   }
 
   JS::PrepareForFullGC(cx);
-  gc(GC_SHRINK, JS::GCReason::LAST_DITCH);
+  gc(JS::GCOptions::Shrink, JS::GCReason::LAST_DITCH);
   waitBackgroundAllocEnd();
   waitBackgroundFreeEnd();
 
@@ -332,6 +432,8 @@ bool GCRuntime::checkAllocatorState(JSContext* cx, AllocKind kind) {
                     kind == AllocKind::SCOPE);
   MOZ_ASSERT_IF(!cx->zone()->isAtomsZone(),
                 kind != AllocKind::ATOM && kind != AllocKind::FAT_INLINE_ATOM);
+  MOZ_ASSERT_IF(cx->zone()->isSelfHostingZone(),
+                !rt->parentRuntime && !selfHostingZoneFrozen);
   MOZ_ASSERT(!JS::RuntimeHeapIsBusy());
 #endif
 
@@ -353,7 +455,7 @@ bool GCRuntime::checkAllocatorState(JSContext* cx, AllocKind kind) {
   return true;
 }
 
-bool GCRuntime::gcIfNeededAtAllocation(JSContext* cx) {
+inline bool GCRuntime::gcIfNeededAtAllocation(JSContext* cx) {
 #ifdef JS_GC_ZEAL
   if (needZealousGC()) {
     runDebugGC();
@@ -366,15 +468,6 @@ bool GCRuntime::gcIfNeededAtAllocation(JSContext* cx) {
     gcIfRequested();
   }
 
-  // If we have grown past our GC heap threshold while in the middle of
-  // an incremental GC, we're growing faster than we're GCing, so stop
-  // the world and do a full, non-incremental GC right now, if possible.
-  if (isIncrementalGCInProgress() &&
-      cx->zone()->totalBytes() > cx->zone()->threshold.gcTriggerBytes()) {
-    PrepareZoneForGC(cx->zone());
-    gc(GC_NORMAL, JS::GCReason::INCREMENTAL_TOO_SLOW);
-  }
-
   return true;
 }
 
@@ -382,13 +475,13 @@ template <typename T>
 /* static */
 void GCRuntime::checkIncrementalZoneState(JSContext* cx, T* t) {
 #ifdef DEBUG
-  if (cx->helperThread() || !t) {
+  if (cx->isHelperThreadContext() || !t) {
     return;
   }
 
   TenuredCell* cell = &t->asTenured();
   Zone* zone = cell->zone();
-  if (zone->isGCMarking() || zone->isGCSweeping()) {
+  if (zone->isGCMarkingOrSweeping()) {
     MOZ_ASSERT(cell->isMarkedBlack());
   } else {
     MOZ_ASSERT(!cell->isMarkedAny());
@@ -397,30 +490,27 @@ void GCRuntime::checkIncrementalZoneState(JSContext* cx, T* t) {
 }
 
 TenuredCell* js::gc::AllocateCellInGC(Zone* zone, AllocKind thingKind) {
-  void* cell = zone->arenas.allocateFromFreeList(thingKind);
+  TenuredCell* cell = zone->arenas.allocateFromFreeList(thingKind);
   if (!cell) {
     AutoEnterOOMUnsafeRegion oomUnsafe;
     cell = GCRuntime::refillFreeListInGC(zone, thingKind);
     if (!cell) {
-      oomUnsafe.crash(ChunkSize, "Failed not allocate new chunk during GC");
+      oomUnsafe.crash(ChunkSize, "Failed to allocate new chunk during GC");
     }
   }
-  return TenuredCell::fromPointer(cell);
+  return cell;
 }
 
 // ///////////  Arena -> Thing Allocator  //////////////////////////////////////
 
-bool GCRuntime::startBackgroundAllocTaskIfIdle() {
-  AutoLockHelperThreadState helperLock;
-  if (allocTask.isRunningWithLockHeld(helperLock)) {
-    return true;
+void GCRuntime::startBackgroundAllocTaskIfIdle() {
+  AutoLockHelperThreadState lock;
+  if (!allocTask.wasStarted(lock)) {
+    // Join the previous invocation of the task. This will return immediately
+    // if the thread has never been started.
+    allocTask.joinWithLockHeld(lock);
+    allocTask.startWithLockHeld(lock);
   }
-
-  // Join the previous invocation of the task. This will return immediately
-  // if the thread has never been started.
-  allocTask.joinWithLockHeld(helperLock);
-
-  return allocTask.startWithLockHeld(helperLock);
 }
 
 /* static */
@@ -428,7 +518,7 @@ TenuredCell* GCRuntime::refillFreeListFromAnyThread(JSContext* cx,
                                                     AllocKind thingKind) {
   MOZ_ASSERT(cx->freeLists().isEmpty(thingKind));
 
-  if (!cx->helperThread()) {
+  if (!cx->isHelperThreadContext()) {
     return refillFreeListFromMainThread(cx, thingKind);
   }
 
@@ -484,8 +574,7 @@ TenuredCell* ArenaLists::refillFreeListAndAllocate(
     maybeLock.emplace(rt);
   }
 
-  ArenaList& al = arenaLists(thingKind);
-  Arena* arena = al.takeNextArena();
+  Arena* arena = arenaList(thingKind).takeNextArena();
   if (arena) {
     // Empty arenas should be immediately freed.
     MOZ_ASSERT(!arena->isEmpty());
@@ -499,25 +588,32 @@ TenuredCell* ArenaLists::refillFreeListAndAllocate(
     maybeLock.emplace(rt);
   }
 
-  Chunk* chunk = rt->gc.pickChunk(maybeLock.ref());
+  TenuredChunk* chunk = rt->gc.pickChunk(maybeLock.ref());
   if (!chunk) {
     return nullptr;
   }
 
   // Although our chunk should definitely have enough space for another arena,
-  // there are other valid reasons why Chunk::allocateArena() may fail.
+  // there are other valid reasons why TenuredChunk::allocateArena() may fail.
   arena = rt->gc.allocateArena(chunk, zone_, thingKind, checkThresholds,
                                maybeLock.ref());
   if (!arena) {
     return nullptr;
   }
 
-  MOZ_ASSERT(al.isCursorAtEnd());
-  al.insertBeforeCursor(arena);
+  addNewArena(arena, thingKind);
 
   return freeLists.setArenaAndAllocate(arena, thingKind);
 }
 
+inline void ArenaLists::addNewArena(Arena* arena, AllocKind thingKind) {
+  ArenaList& al = zone_->isGCMarking() ? newArenasInMarkPhase(thingKind)
+                                       : arenaList(thingKind);
+
+  MOZ_ASSERT(al.isCursorAtEnd());
+  al.insertBeforeCursor(arena);
+}
+
 inline TenuredCell* FreeLists::setArenaAndAllocate(Arena* arena,
                                                    AllocKind kind) {
 #ifdef DEBUG
@@ -530,7 +626,8 @@ inline TenuredCell* FreeLists::setArenaAndAllocate(Arena* arena,
   FreeSpan* span = arena->getFirstFreeSpan();
   freeLists_[kind] = span;
 
-  if (MOZ_UNLIKELY(arena->zone->wasGCStarted())) {
+  Zone* zone = arena->zone;
+  if (MOZ_UNLIKELY(zone->isGCMarkingOrSweeping())) {
     arena->arenaAllocatedDuringGC();
   }
 
@@ -545,12 +642,10 @@ void Arena::arenaAllocatedDuringGC() {
   // incremental GC will be marked black by pre-marking all free cells in the
   // arena we are about to allocate from.
 
-  if (zone->needsIncrementalBarrier() || zone->isGCSweeping()) {
-    for (ArenaFreeCellIter iter(this); !iter.done(); iter.next()) {
-      TenuredCell* cell = iter.getCell();
-      MOZ_ASSERT(!cell->isMarkedAny());
-      cell->markBlack();
-    }
+  MOZ_ASSERT(zone->isGCMarkingOrSweeping());
+  for (ArenaFreeCellIter cell(this); !cell.done(); cell.next()) {
+    MOZ_ASSERT(!cell->isMarkedAny());
+    cell->markBlack();
   }
 }
 
@@ -574,7 +669,26 @@ void ArenaLists::setParallelAllocEnabled(bool enabled) {
   }
 }
 
-// ///////////  Chunk -> Arena Allocator  //////////////////////////////////////
+void GCRuntime::setParallelUnmarkEnabled(bool enabled) {
+  // This can only be changed on the main thread otherwise we could race.
+  MOZ_ASSERT(CurrentThreadCanAccessRuntime(rt));
+  MOZ_ASSERT(JS::RuntimeHeapIsMajorCollecting());
+  for (GCZonesIter zone(this); !zone.done(); zone.next()) {
+    zone->arenas.setParallelUnmarkEnabled(enabled);
+  }
+}
+
+void ArenaLists::setParallelUnmarkEnabled(bool enabled) {
+  static const ConcurrentUse states[2] = {ConcurrentUse::None,
+                                          ConcurrentUse::ParallelUnmark};
+
+  for (auto kind : AllAllocKinds()) {
+    MOZ_ASSERT(concurrentUse(kind) == states[!enabled]);
+    concurrentUse(kind) = states[enabled];
+  }
+}
+
+// ///////////  TenuredChunk -> Arena Allocator  ///////////////////////////////
 
 bool GCRuntime::wantBackgroundAllocation(const AutoLockGC& lock) const {
   // To minimize memory waste, we do not want to run the background chunk
@@ -585,97 +699,114 @@ bool GCRuntime::wantBackgroundAllocation(const AutoLockGC& lock) const {
          (fullChunks(lock).count() + availableChunks(lock).count()) >= 4;
 }
 
-Arena* GCRuntime::allocateArena(Chunk* chunk, Zone* zone, AllocKind thingKind,
+Arena* GCRuntime::allocateArena(TenuredChunk* chunk, Zone* zone,
+                                AllocKind thingKind,
                                 ShouldCheckThresholds checkThresholds,
                                 const AutoLockGC& lock) {
   MOZ_ASSERT(chunk->hasAvailableArenas());
 
   // Fail the allocation if we are over our heap size limits.
   if ((checkThresholds != ShouldCheckThresholds::DontCheckThresholds) &&
-      (heapSize.gcBytes() >= tunables.gcMaxBytes()))
+      (heapSize.bytes() >= tunables.gcMaxBytes()))
     return nullptr;
 
-  Arena* arena = chunk->allocateArena(rt, zone, thingKind, lock);
-  zone->zoneSize.addGCArena();
+  Arena* arena = chunk->allocateArena(this, zone, thingKind, lock);
+  zone->gcHeapSize.addGCArena();
 
   // Trigger an incremental slice if needed.
   if (checkThresholds != ShouldCheckThresholds::DontCheckThresholds) {
-    maybeAllocTriggerZoneGC(zone);
+    maybeTriggerGCAfterAlloc(zone);
   }
 
   return arena;
 }
 
-Arena* Chunk::allocateArena(JSRuntime* rt, Zone* zone, AllocKind thingKind,
-                            const AutoLockGC& lock) {
-  Arena* arena = info.numArenasFreeCommitted > 0 ? fetchNextFreeArena(rt)
-                                                 : fetchNextDecommittedArena();
+Arena* TenuredChunk::allocateArena(GCRuntime* gc, Zone* zone,
+                                   AllocKind thingKind,
+                                   const AutoLockGC& lock) {
+  if (info.numArenasFreeCommitted == 0) {
+    commitOnePage(gc);
+    MOZ_ASSERT(info.numArenasFreeCommitted == ArenasPerPage);
+  }
+  MOZ_ASSERT(info.numArenasFreeCommitted > 0);
+  Arena* arena = fetchNextFreeArena(gc);
+
   arena->init(zone, thingKind, lock);
-  updateChunkListAfterAlloc(rt, lock);
+  updateChunkListAfterAlloc(gc, lock);
   return arena;
 }
 
-inline void GCRuntime::updateOnFreeArenaAlloc(const ChunkInfo& info) {
+void TenuredChunk::commitOnePage(GCRuntime* gc) {
+  MOZ_ASSERT(info.numArenasFreeCommitted == 0);
+  MOZ_ASSERT(!info.freeArenasHead);
+  MOZ_ASSERT(info.numArenasFree > 0);
+
+  unsigned offset = findDecommittedPageOffset();
+  info.lastDecommittedPageOffset = offset + 1;
+
+  if (DecommitEnabled()) {
+    MarkPagesInUseSoft(pageAddress(offset), PageSize);
+  }
+
+  size_t arenaIndex = offset * ArenasPerPage;
+  decommittedPages[offset] = false;
+  for (size_t i = 0; i < ArenasPerPage; i++) {
+    arenas[arenaIndex + i].setAsNotAllocated();
+  }
+
+  // numArenasFreeCommitted will be updated in addArenasInPageToFreeList.
+  // No need to update numArenasFree, as the arena is still free, it just
+  // changes from decommitted to committed free. Later fetchNextFreeArena should
+  // update the numArenasFree.
+  addArenasInPageToFreeList(gc, offset);
+}
+
+inline void GCRuntime::updateOnFreeArenaAlloc(const TenuredChunkInfo& info) {
   MOZ_ASSERT(info.numArenasFreeCommitted <= numArenasFreeCommitted);
   --numArenasFreeCommitted;
 }
 
-Arena* Chunk::fetchNextFreeArena(JSRuntime* rt) {
+Arena* TenuredChunk::fetchNextFreeArena(GCRuntime* gc) {
   MOZ_ASSERT(info.numArenasFreeCommitted > 0);
   MOZ_ASSERT(info.numArenasFreeCommitted <= info.numArenasFree);
+  MOZ_ASSERT(info.freeArenasHead);
 
   Arena* arena = info.freeArenasHead;
   info.freeArenasHead = arena->next;
   --info.numArenasFreeCommitted;
   --info.numArenasFree;
-  rt->gc.updateOnFreeArenaAlloc(info);
-
-  return arena;
-}
-
-Arena* Chunk::fetchNextDecommittedArena() {
-  MOZ_ASSERT(info.numArenasFreeCommitted == 0);
-  MOZ_ASSERT(info.numArenasFree > 0);
-
-  unsigned offset = findDecommittedArenaOffset();
-  info.lastDecommittedArenaOffset = offset + 1;
-  --info.numArenasFree;
-  decommittedArenas.unset(offset);
-
-  Arena* arena = &arenas[offset];
-  MarkPagesInUse(arena, ArenaSize);
-  arena->setAsNotAllocated();
+  gc->updateOnFreeArenaAlloc(info);
 
   return arena;
 }
 
 /*
- * Search for and return the next decommitted Arena. Our goal is to keep
- * lastDecommittedArenaOffset "close" to a free arena. We do this by setting
- * it to the most recently freed arena when we free, and forcing it to
+ * Search for and return the next decommitted page. Our goal is to keep
+ * lastDecommittedPageOffset "close" to a free page. We do this by setting
+ * it to the most recently freed page when we free, and forcing it to
  * the last alloc + 1 when we allocate.
  */
-uint32_t Chunk::findDecommittedArenaOffset() {
+uint32_t TenuredChunk::findDecommittedPageOffset() {
   /* Note: lastFreeArenaOffset can be past the end of the list. */
-  for (unsigned i = info.lastDecommittedArenaOffset; i < ArenasPerChunk; i++) {
-    if (decommittedArenas.get(i)) {
+  for (unsigned i = info.lastDecommittedPageOffset; i < PagesPerChunk; i++) {
+    if (decommittedPages[i]) {
       return i;
     }
   }
-  for (unsigned i = 0; i < info.lastDecommittedArenaOffset; i++) {
-    if (decommittedArenas.get(i)) {
+  for (unsigned i = 0; i < info.lastDecommittedPageOffset; i++) {
+    if (decommittedPages[i]) {
       return i;
     }
   }
   MOZ_CRASH("No decommitted arenas found.");
 }
 
-// ///////////  System -> Chunk Allocator  /////////////////////////////////////
+// ///////////  System -> TenuredChunk Allocator  //////////////////////////////
 
-Chunk* GCRuntime::getOrAllocChunk(AutoLockGCBgAlloc& lock) {
-  Chunk* chunk = emptyChunks(lock).pop();
+TenuredChunk* GCRuntime::getOrAllocChunk(AutoLockGCBgAlloc& lock) {
+  TenuredChunk* chunk = emptyChunks(lock).pop();
   if (!chunk) {
-    chunk = Chunk::allocate(rt);
+    chunk = TenuredChunk::allocate(this);
     if (!chunk) {
       return nullptr;
     }
@@ -689,70 +820,71 @@ Chunk* GCRuntime::getOrAllocChunk(AutoLockGCBgAlloc& lock) {
   return chunk;
 }
 
-void GCRuntime::recycleChunk(Chunk* chunk, const AutoLockGC& lock) {
-  AlwaysPoison(&chunk->trailer, JS_FREED_CHUNK_PATTERN, sizeof(ChunkTrailer),
+void GCRuntime::recycleChunk(TenuredChunk* chunk, const AutoLockGC& lock) {
+  AlwaysPoison(chunk, JS_FREED_CHUNK_PATTERN, sizeof(ChunkBase),
                MemCheckKind::MakeNoAccess);
   emptyChunks(lock).push(chunk);
 }
 
-Chunk* GCRuntime::pickChunk(AutoLockGCBgAlloc& lock) {
+TenuredChunk* GCRuntime::pickChunk(AutoLockGCBgAlloc& lock) {
   if (availableChunks(lock).count()) {
     return availableChunks(lock).head();
   }
 
-  Chunk* chunk = getOrAllocChunk(lock);
+  TenuredChunk* chunk = getOrAllocChunk(lock);
   if (!chunk) {
     return nullptr;
   }
 
-  chunk->init(rt);
+  chunk->init(this);
   MOZ_ASSERT(chunk->info.numArenasFreeCommitted == 0);
   MOZ_ASSERT(chunk->unused());
   MOZ_ASSERT(!fullChunks(lock).contains(chunk));
   MOZ_ASSERT(!availableChunks(lock).contains(chunk));
 
-  chunkAllocationSinceLastGC = true;
-
   availableChunks(lock).push(chunk);
 
   return chunk;
 }
 
-BackgroundAllocTask::BackgroundAllocTask(JSRuntime* rt, ChunkPool& pool)
-    : GCParallelTaskHelper(rt),
+BackgroundAllocTask::BackgroundAllocTask(GCRuntime* gc, ChunkPool& pool)
+    : GCParallelTask(gc),
       chunkPool_(pool),
       enabled_(CanUseExtraThreads() && GetCPUCount() >= 2) {}
 
-void BackgroundAllocTask::run() {
+void BackgroundAllocTask::run(AutoLockHelperThreadState& lock) {
+  AutoUnlockHelperThreadState unlock(lock);
+
   TraceLoggerThread* logger = TraceLoggerForCurrentThread();
   AutoTraceLog logAllocation(logger, TraceLogger_GCAllocation);
 
-  AutoLockGC lock(runtime());
-  while (!cancel_ && runtime()->gc.wantBackgroundAllocation(lock)) {
-    Chunk* chunk;
+  AutoLockGC gcLock(gc);
+  while (!isCancelled() && gc->wantBackgroundAllocation(gcLock)) {
+    TenuredChunk* chunk;
     {
-      AutoUnlockGC unlock(lock);
-      chunk = Chunk::allocate(runtime());
+      AutoUnlockGC unlock(gcLock);
+      chunk = TenuredChunk::allocate(gc);
       if (!chunk) {
         break;
       }
-      chunk->init(runtime());
+      chunk->init(gc);
     }
     chunkPool_.ref().push(chunk);
   }
 }
 
 /* static */
-Chunk* Chunk::allocate(JSRuntime* rt) {
-  Chunk* chunk = static_cast<Chunk*>(MapAlignedPages(ChunkSize, ChunkSize));
+TenuredChunk* TenuredChunk::allocate(GCRuntime* gc) {
+  void* chunk = MapAlignedPages(ChunkSize, ChunkSize);
   if (!chunk) {
     return nullptr;
   }
-  rt->gc.stats().count(gcstats::COUNT_NEW_CHUNK);
-  return chunk;
+
+  gc->stats().count(gcstats::COUNT_NEW_CHUNK);
+  return static_cast<TenuredChunk*>(chunk);
 }
 
-void Chunk::init(JSRuntime* rt) {
+void TenuredChunk::init(GCRuntime* gc) {
   /* The chunk may still have some regions marked as no-access. */
   MOZ_MAKE_MEM_UNDEFINED(this, ChunkSize);
 
@@ -763,11 +895,7 @@ void Chunk::init(JSRuntime* rt) {
   Poison(this, JS_FRESH_TENURED_PATTERN, ChunkSize,
          MemCheckKind::MakeUndefined);
 
-  /*
-   * We clear the bitmap to guard against JS::GCThingIsMarkedGray being called
-   * on uninitialized data, which would happen before the first GC cycle.
-   */
-  bitmap.clear();
+  new (this) TenuredChunk(gc->rt);
 
   /*
    * Decommit the arenas. We do this after poisoning so that if the OS does
@@ -775,19 +903,21 @@ void Chunk::init(JSRuntime* rt) {
    */
   decommitAllArenas();
 
-  /* Initialize the chunk info. */
-  info.init();
-  new (&trailer) ChunkTrailer(rt);
+#ifdef DEBUG
+  verify();
+#endif
 
   /* The rest of info fields are initialized in pickChunk. */
 }
 
-void Chunk::decommitAllArenas() {
-  decommittedArenas.clear(true);
-  MarkPagesUnused(&arenas[0], ArenasPerChunk * ArenaSize);
+void TenuredChunk::decommitAllArenas() {
+  decommittedPages.SetAll();
+  if (DecommitEnabled()) {
+    MarkPagesUnusedSoft(&arenas[0], ArenasPerChunk * ArenaSize);
+  }
 
   info.freeArenasHead = nullptr;
-  info.lastDecommittedArenaOffset = 0;
+  info.lastDecommittedPageOffset = 0;
   info.numArenasFree = ArenasPerChunk;
   info.numArenasFreeCommitted = 0;
 }
