@@ -15,6 +15,7 @@
 
 #include "jstypes.h"
 #include "gc/GCRuntime.h"
+#include "gc/IteratorUtils.h"
 #include "gc/Zone.h"
 #include "vm/Compartment.h"
 #include "vm/Runtime.h"
@@ -33,36 +34,25 @@ namespace js {
 // zone, consider using AutoLockAllAtoms.
 enum ZoneSelector { WithAtoms, SkipAtoms };
 
-// Iterate over all zones in the runtime, except those which may be in use by
-// parse threads.
-class ZonesIter {
+// Iterate over all zones in the runtime apart from the atoms zone and those
+// which may be in use by parse threads.
+class NonAtomZonesIter {
   gc::AutoEnterIteration iterMarker;
-  JS::Zone* atomsZone;
   JS::Zone** it;
   JS::Zone** end;
 
  public:
-  ZonesIter(gc::GCRuntime* gc, ZoneSelector selector)
-      : iterMarker(gc),
-        atomsZone(selector == WithAtoms ? gc->atomsZone.ref() : nullptr),
-        it(gc->zones().begin()),
-        end(gc->zones().end()) {
-    if (!atomsZone) {
-      skipHelperThreadZones();
-    }
+  explicit NonAtomZonesIter(gc::GCRuntime* gc)
+      : iterMarker(gc), it(gc->zones().begin()), end(gc->zones().end()) {
+    skipHelperThreadZones();
   }
-  ZonesIter(JSRuntime* rt, ZoneSelector selector)
-      : ZonesIter(&rt->gc, selector) {}
+  explicit NonAtomZonesIter(JSRuntime* rt) : NonAtomZonesIter(&rt->gc) {}
 
-  bool done() const { return !atomsZone && it == end; }
+  bool done() const { return it == end; }
 
   void next() {
     MOZ_ASSERT(!done());
-    if (atomsZone) {
-      atomsZone = nullptr;
-    } else {
-      it++;
-    }
+    it++;
     skipHelperThreadZones();
   }
 
@@ -74,16 +64,56 @@ class ZonesIter {
 
   JS::Zone* get() const {
     MOZ_ASSERT(!done());
-    return atomsZone ? atomsZone : *it;
+    return *it;
   }
 
   operator JS::Zone*() const { return get(); }
   JS::Zone* operator->() const { return get(); }
 };
 
-struct CompartmentsInZoneIter {
-  using ItemType = JS::Compartment;
+// Iterate over all zones in the runtime, except those which may be in use by
+// parse threads.  May or may not include the atoms zone.
+class ZonesIter {
+  JS::Zone* atomsZone;
+  NonAtomZonesIter otherZones;
 
+ public:
+  ZonesIter(gc::GCRuntime* gc, ZoneSelector selector)
+      : atomsZone(selector == WithAtoms ? gc->atomsZone.ref() : nullptr),
+        otherZones(gc) {}
+  ZonesIter(JSRuntime* rt, ZoneSelector selector)
+      : ZonesIter(&rt->gc, selector) {}
+
+  bool done() const { return !atomsZone && otherZones.done(); }
+
+  JS::Zone* get() const {
+    MOZ_ASSERT(!done());
+    return atomsZone ? atomsZone : otherZones.get();
+  }
+
+  void next() {
+    MOZ_ASSERT(!done());
+    if (atomsZone) {
+      atomsZone = nullptr;
+      return;
+    }
+
+    otherZones.next();
+  }
+
+  operator JS::Zone*() const { return get(); }
+  JS::Zone* operator->() const { return get(); }
+};
+
+// Iterate over all zones in the runtime, except those which may be in use by
+// parse threads.
+class AllZonesIter : public ZonesIter {
+ public:
+  explicit AllZonesIter(gc::GCRuntime* gc) : ZonesIter(gc, WithAtoms) {}
+  explicit AllZonesIter(JSRuntime* rt) : AllZonesIter(&rt->gc) {}
+};
+
+struct CompartmentsInZoneIter {
   explicit CompartmentsInZoneIter(JS::Zone* zone) : zone(zone) {
     it = zone->compartments().begin();
   }
@@ -118,6 +148,7 @@ class RealmsInCompartmentIter {
  public:
   explicit RealmsInCompartmentIter(JS::Compartment* comp) : comp(comp) {
     it = comp->realms().begin();
+    MOZ_ASSERT(!done(), "Compartments must have at least one realm");
   }
 
   bool done() const {
@@ -138,94 +169,28 @@ class RealmsInCompartmentIter {
   JS::Realm* operator->() const { return get(); }
 };
 
-class RealmsInZoneIter {
-  CompartmentsInZoneIter comp;
-  mozilla::Maybe<RealmsInCompartmentIter> realm;
-
- public:
-  using ItemType = JS::Realm;
-
-  explicit RealmsInZoneIter(JS::Zone* zone) : comp(zone) {
-    settleOnCompartment();
-  }
-
-  void settleOnCompartment() {
-    if (!comp.done()) {
-      realm.emplace(comp.get());
-      MOZ_ASSERT(!realm->done(), "compartment must have at least one realm");
-    }
-  }
-
-  bool done() const {
-    MOZ_ASSERT(comp.done() == realm.isNothing());
-    return comp.done();
-  }
-  void next() {
-    MOZ_ASSERT(!done());
-
-    realm->next();
-
-    if (realm->done()) {
-      realm.reset();
-      comp.next();
-      settleOnCompartment();
-    }
-  }
-
-  JS::Realm* get() const { return realm->get(); }
-
-  operator JS::Realm*() const { return get(); }
-  JS::Realm* operator->() const { return get(); }
-};
+using RealmsInZoneIter =
+    NestedIterator<CompartmentsInZoneIter, RealmsInCompartmentIter>;
 
 // This iterator iterates over all the compartments or realms in a given set of
 // zones. The set of zones is determined by iterating ZoneIterT. The set of
 // compartments or realms is determined by InnerIterT.
 template <class ZonesIterT, class InnerIterT>
-class CompartmentsOrRealmsIterT {
-  using T = typename InnerIterT::ItemType;
-
+class CompartmentsOrRealmsIterT
+    : public NestedIterator<ZonesIterT, InnerIterT> {
   gc::AutoEnterIteration iterMarker;
-  ZonesIterT zone;
-  mozilla::Maybe<InnerIterT> inner;
 
  public:
   explicit CompartmentsOrRealmsIterT(gc::GCRuntime* gc)
-      : iterMarker(gc), zone(gc, SkipAtoms) {
-    if (!zone.done()) {
-      inner.emplace(zone);
-    }
-  }
+      : NestedIterator<ZonesIterT, InnerIterT>(gc), iterMarker(gc) {}
   explicit CompartmentsOrRealmsIterT(JSRuntime* rt)
       : CompartmentsOrRealmsIterT(&rt->gc) {}
-
-  bool done() const { return zone.done(); }
-
-  void next() {
-    MOZ_ASSERT(!done());
-    MOZ_ASSERT(!inner.ref().done());
-    inner->next();
-    if (inner->done()) {
-      inner.reset();
-      zone.next();
-      if (!zone.done()) {
-        inner.emplace(zone);
-      }
-    }
-  }
-
-  T* get() const {
-    MOZ_ASSERT(!done());
-    return *inner;
-  }
-
-  operator T*() const { return get(); }
-  T* operator->() const { return get(); }
 };
 
 using CompartmentsIter =
-    CompartmentsOrRealmsIterT<ZonesIter, CompartmentsInZoneIter>;
-using RealmsIter = CompartmentsOrRealmsIterT<ZonesIter, RealmsInZoneIter>;
+    CompartmentsOrRealmsIterT<NonAtomZonesIter, CompartmentsInZoneIter>;
+using RealmsIter =
+    CompartmentsOrRealmsIterT<NonAtomZonesIter, RealmsInZoneIter>;
 
 }  // namespace js
 
