@@ -11,101 +11,53 @@
 #include "NamespaceImports.h"
 
 #include "gc/GCInternals.h"
+#include "gc/Marking.h"
 #include "gc/PublicIterators.h"
 #include "gc/Zone.h"
 #include "util/Memory.h"
 #include "util/Text.h"
 #include "vm/BigIntType.h"
+#include "vm/GetterSetter.h"
 #include "vm/JSFunction.h"
 #include "vm/JSScript.h"
+#include "vm/PropMap.h"
 #include "vm/Shape.h"
 #include "vm/SymbolType.h"
 
 #include "gc/GC-inl.h"
 #include "gc/Marking-inl.h"
-#include "vm/ObjectGroup-inl.h"
 #include "vm/Realm-inl.h"
-#include "vm/TypeInference-inl.h"
 
 using namespace js;
 using namespace js::gc;
 using mozilla::DebugOnly;
 
-namespace js {
-template <typename T>
-void CheckTracedThing(JSTracer* trc, T thing);
-}  // namespace js
-
-/*** Callback Tracer Dispatch ***********************************************/
-template <typename T>
-bool DoCallback(JS::CallbackTracer* trc, T** thingp, const char* name) {
-  CheckTracedThing(trc, *thingp);
-  JS::AutoTracingName ctx(trc, name);
-
-  return trc->dispatchToOnEdge(thingp);
-}
-#define INSTANTIATE_ALL_VALID_TRACE_FUNCTIONS(name, type, _, _1) \
-  template bool DoCallback<type>(JS::CallbackTracer*, type**, const char*);
-JS_FOR_EACH_TRACEKIND(INSTANTIATE_ALL_VALID_TRACE_FUNCTIONS);
-#undef INSTANTIATE_ALL_VALID_TRACE_FUNCTIONS
-
-template <typename T>
-bool DoCallback(JS::CallbackTracer* trc, T* thingp, const char* name) {
-  // Return true by default. For some types the lambda below won't be called.
-  bool ret = true;
-  auto thing = MapGCThingTyped(*thingp, [trc, name, &ret](auto t) {
-    if (!DoCallback(trc, &t, name)) {
-      ret = false;
-      return TaggedPtr<T>::empty();
-    }
-    return TaggedPtr<T>::wrap(t);
-  });
-  // Only update *thingp if the value changed, to avoid TSan false positives for
-  // template objects when using DumpHeapTracer or UbiNode tracers while Ion
-  // compiling off-thread.
-  if (thing.isSome() && thing.value() != *thingp) {
-    *thingp = thing.value();
-  }
-  return ret;
-}
-template bool DoCallback<JS::Value>(JS::CallbackTracer*, JS::Value*,
-                                    const char*);
-template bool DoCallback<JS::PropertyKey>(JS::CallbackTracer*, JS::PropertyKey*,
-                                          const char*);
-template bool DoCallback<TaggedProto>(JS::CallbackTracer*, TaggedProto*,
-                                      const char*);
-
-void JS::CallbackTracer::getTracingEdgeName(char* buffer, size_t bufferSize) {
+void JS::TracingContext::getEdgeName(char* buffer, size_t bufferSize) {
   MOZ_ASSERT(bufferSize > 0);
-  if (contextFunctor_) {
-    (*contextFunctor_)(this, buffer, bufferSize);
+  if (functor_) {
+    (*functor_)(this, buffer, bufferSize);
     return;
   }
-  if (contextIndex_ != InvalidIndex) {
-    snprintf(buffer, bufferSize, "%s[%zu]", contextName_, contextIndex_);
+  if (index_ != InvalidIndex) {
+    snprintf(buffer, bufferSize, "%s[%zu]", name_, index_);
     return;
   }
-  snprintf(buffer, bufferSize, "%s", contextName_);
+  snprintf(buffer, bufferSize, "%s", name_);
 }
 
 /*** Public Tracing API *****************************************************/
 
 JS_PUBLIC_API void JS::TraceChildren(JSTracer* trc, GCCellPtr thing) {
-  js::TraceChildren(trc, thing.asCell(), thing.kind());
-}
-
-void js::TraceChildren(JSTracer* trc, void* thing, JS::TraceKind kind) {
-  MOZ_ASSERT(thing);
-  ApplyGCThingTyped(thing, kind, [trc](auto t) {
+  ApplyGCThingTyped(thing.asCell(), thing.kind(), [trc](auto t) {
     MOZ_ASSERT_IF(t->runtimeFromAnyThread() != trc->runtime(),
-                  ThingIsPermanentAtomOrWellKnownSymbol(t) ||
+                  t->isPermanentAndMayBeShared() ||
                       t->zoneFromAnyThread()->isSelfHostingZone());
     t->traceChildren(trc);
   });
 }
 
-JS_PUBLIC_API void JS::TraceIncomingCCWs(
-    JSTracer* trc, const JS::CompartmentSet& compartments) {
+void js::gc::TraceIncomingCCWs(JSTracer* trc,
+                               const JS::CompartmentSet& compartments) {
   for (CompartmentsIter source(trc->runtime()); !source.done(); source.next()) {
     if (compartments.has(source)) {
       continue;
@@ -133,45 +85,12 @@ JS_PUBLIC_API void JS::TraceIncomingCCWs(
 /*** Cycle Collector Helpers ************************************************/
 
 // This function is used by the Cycle Collector (CC) to trace through -- or in
-// CC parlance, traverse -- a Shape tree. The CC does not care about Shapes or
-// BaseShapes, only the JSObjects held live by them. Thus, we walk the Shape
-// lineage, but only report non-Shape things. This effectively makes the entire
-// shape lineage into a single node in the CC, saving tremendous amounts of
-// space and time in its algorithms.
-//
-// The algorithm implemented here uses only bounded stack space. This would be
-// possible to implement outside the engine, but would require much extra
-// infrastructure and many, many more slow GOT lookups. We have implemented it
-// inside SpiderMonkey, despite the lack of general applicability, for the
-// simplicity and performance of FireFox's embedding of this engine.
+// CC parlance, traverse -- a Shape. The CC does not care about Shapes,
+// BaseShapes or PropMaps, only the JSObjects held live by them. Thus, we only
+// report non-Shape things.
 void gc::TraceCycleCollectorChildren(JS::CallbackTracer* trc, Shape* shape) {
-  do {
-    MOZ_ASSERT(shape->base());
-    shape->base()->assertConsistency();
-
-    // Don't trace the propid because the CC doesn't care about jsid.
-
-    if (shape->hasGetterObject()) {
-      JSObject* tmp = shape->getterObject();
-      DoCallback(trc, &tmp, "getter");
-      MOZ_ASSERT(tmp == shape->getterObject());
-    }
-
-    if (shape->hasSetterObject()) {
-      JSObject* tmp = shape->setterObject();
-      DoCallback(trc, &tmp, "setter");
-      MOZ_ASSERT(tmp == shape->setterObject());
-    }
-
-    shape = shape->previous();
-  } while (shape);
-}
-
-void gc::TraceCycleCollectorChildren(JS::CallbackTracer* trc,
-                                     ObjectGroup* group) {
-  MOZ_ASSERT(trc->isCallbackTracer());
-
-  group->traceChildren(trc);
+  shape->base()->traceChildren(trc);
+  // Don't trace the PropMap because the CC doesn't care about PropertyKey.
 }
 
 /*** Traced Edge Printer ****************************************************/
@@ -218,9 +137,8 @@ static const char* StringKindHeader(JSString* str) {
   return "linear: ";
 }
 
-JS_PUBLIC_API void JS_GetTraceThingInfo(char* buf, size_t bufsize,
-                                        JSTracer* trc, void* thing,
-                                        JS::TraceKind kind, bool details) {
+void js::gc::GetTraceThingInfo(char* buf, size_t bufsize, void* thing,
+                               JS::TraceKind kind, bool details) {
   const char* name = nullptr; /* silence uninitialized warning */
   size_t n;
 
@@ -233,6 +151,14 @@ JS_PUBLIC_API void JS_GetTraceThingInfo(char* buf, size_t bufsize,
       name = "base_shape";
       break;
 
+    case JS::TraceKind::GetterSetter:
+      name = "getter_setter";
+      break;
+
+    case JS::TraceKind::PropMap:
+      name = "prop_map";
+      break;
+
     case JS::TraceKind::JitCode:
       name = "jitcode";
       break;
@@ -246,10 +172,6 @@ JS_PUBLIC_API void JS_GetTraceThingInfo(char* buf, size_t bufsize,
       break;
     }
 
-    case JS::TraceKind::ObjectGroup:
-      name = "object_group";
-      break;
-
     case JS::TraceKind::RegExpShared:
       name = "reg_exp_shared";
       break;
@@ -312,7 +234,7 @@ JS_PUBLIC_API void JS_GetTraceThingInfo(char* buf, size_t bufsize,
       }
 
       case JS::TraceKind::Script: {
-        js::BaseScript* script = static_cast<js::BaseScript*>(thing);
+        auto* script = static_cast<js::BaseScript*>(thing);
         snprintf(buf, bufsize, " %s:%u", script->filename(), script->lineno());
         break;
       }
@@ -341,7 +263,7 @@ JS_PUBLIC_API void JS_GetTraceThingInfo(char* buf, size_t bufsize,
       }
 
       case JS::TraceKind::Symbol: {
-        JS::Symbol* sym = static_cast<JS::Symbol*>(thing);
+        auto* sym = static_cast<JS::Symbol*>(thing);
         if (JSAtom* desc = sym->description()) {
           *buf++ = ' ';
           bufsize--;
@@ -353,7 +275,7 @@ JS_PUBLIC_API void JS_GetTraceThingInfo(char* buf, size_t bufsize,
       }
 
       case JS::TraceKind::Scope: {
-        js::Scope* scope = static_cast<js::Scope*>(thing);
+        auto* scope = static_cast<js::Scope*>(thing);
         snprintf(buf, bufsize, " %s", js::ScopeKindString(scope->kind()));
         break;
       }
@@ -365,9 +287,9 @@ JS_PUBLIC_API void JS_GetTraceThingInfo(char* buf, size_t bufsize,
   buf[bufsize - 1] = '\0';
 }
 
-JS::CallbackTracer::CallbackTracer(JSContext* cx,
-                                   WeakMapTraceKind weakTraceKind)
-    : CallbackTracer(cx->runtime(), weakTraceKind) {}
+JS::CallbackTracer::CallbackTracer(JSContext* cx, JS::TracerKind kind,
+                                   JS::TraceOptions options)
+    : CallbackTracer(cx->runtime(), kind, options) {}
 
 uint32_t JSTracer::gcNumberForMarking() const {
   MOZ_ASSERT(isMarkingTracer());
