@@ -1,28 +1,39 @@
 // Test that we don't repeatedly trigger last-ditch GCs.
 
-function allocUntilFail() {
-    gc();
-    let initGCNumber = gcparam("gcNumber");
-    let error;
-    try {
-        let a = [];
-        while (true) {
-            a.push(Symbol()); // Symbols are tenured.
-        }
-    } catch(err) {
-        error = err;
-    }
-    let finalGCNumber = gcparam("gcNumber");
-    gc();
-    assertEq(error, "out of memory");
-    return finalGCNumber - initGCNumber;
-}
-
 // Turn of any zeal which will disrupt GC number checks.
 gczeal(0);
 
-// Set a small heap limit.
-gcparam("maxBytes", 1024 * 1024);
+// Get initial heap size and limit.
+gc();
+const initialSize = gcparam("gcBytes");
+const initialMaxSize = gcparam("maxBytes");
+
+function allocUntilFail() {
+  gc();
+  const initGCNumber = gcparam("majorGCNumber");
+
+  // Set a small heap limit.
+  gcparam("maxBytes", initialSize + 16 * 1024);
+
+  let error;
+  try {
+    let a = [];
+    while (true) {
+      a.push(Symbol()); // Symbols are tenured.
+    }
+  } catch(err) {
+    error = err;
+  }
+
+  const finalGCNumber = gcparam("majorGCNumber");
+
+  // Resetore heap limit.
+  gcparam("maxBytes", initialMaxSize);
+
+  gc();
+  assertEq(error, "out of memory");
+  return finalGCNumber - initGCNumber;
+}
 
 // Set the time limit for skipping last ditch GCs to 5 seconds.
 gcparam("minLastDitchGCPeriod", 5);
