@@ -1,5 +1,3 @@
-// |jit-test| skip-if: !wasmBulkMemSupported() || !wasmReftypesEnabled()
-
 // Declared segments parse and validate
 wasmFullPass(`
 	(module
@@ -11,22 +9,25 @@ wasmFullPass(`
 	)
 `);
 
-// Declared segments cannot be used by bulk-memory operations
-function test(ins) {
-	assertErrorMessage(
-		() => wasmEvalText(`
-			(module
-				(func $f1)
-				(table 1 1 funcref)
-				(elem declare $f1)
-				(func $start ${ins})
-				(start $start)
-			)
-		`),
-		WebAssembly.RuntimeError,
-		'index out of bounds');
-}
-test('(table.init 0 (i32.const 0) (i32.const 0) (i32.const 1))');
+// Declared segments can be used with externref
+wasmFullPass(`
+	(module
+		(elem declare externref (ref.null extern))
+		(func $run)
+		(export "run" (func $run))
+	)
+`);
+
+// Declared segments can be used by bulk-memory operations
+wasmEvalText(`
+	(module
+		(func $f1)
+		(table 1 1 funcref)
+		(elem declare $f1)
+		(func $start (table.init 0 (i32.const 0) (i32.const 0) (i32.const 1)))
+		(start $start)
+	)
+`);
 
 // Declared segments don't cause initialization of a table
 wasmAssert(`
@@ -37,7 +38,7 @@ wasmAssert(`
 		(func $at (param i32) (result i32)
 			local.get 0
 			table.get 0
-			ref.is_null func
+			ref.is_null
 		)
 		(export "at" (func $at))
 	)
