@@ -20,7 +20,7 @@ assertErrorMessage(() => wasmEvalText('(module (type $a (func)) (func (call_indi
 assertErrorMessage(() => wasmEvalText('(module (func (br $a)))'), SyntaxError, /failed to find label/);
 assertErrorMessage(() => wasmEvalText('(module (func (block $a ) (br $a)))'), SyntaxError, /failed to find label/);
 
-assertErrorMessage(() => wasmEvalText(`(module (func (call ${0xffffffff})))`), WebAssembly.CompileError, /callee index out of range/);
+assertErrorMessage(() => wasmEvalText(`(module (func (call ${0xffffffff})))`), WebAssembly.CompileError, /(callee index out of range)|(function index out of bounds)/);
 assertErrorMessage(() => wasmEvalText(`(module (export "func" ${0xffffffff}))`), SyntaxError, parsingError);
 
 wasmEvalText('(module (func (param $a i32)))');
@@ -51,7 +51,6 @@ wasmEvalText('(module (func) (table $t funcref (elem 0 0 0)))');
 const { Table } = WebAssembly;
 const table = new Table({initial:1, element:"funcref"});
 assertErrorMessage(() => wasmEvalText('(module (table $t (import) 1 funcref))'), SyntaxError, parsingError);
-assertErrorMessage(() => wasmEvalText('(module (table $t (import "mod") 1 funcref))'), SyntaxError, parsingError);
 assertErrorMessage(() => wasmEvalText('(module (table $t (import "mod" "field") 1 funcref (elem 1 2 3)))'), SyntaxError, parsingError);
 wasmEvalText('(module (table $t (import "mod" "field") 1 funcref))', {mod: {field: table}});
 
@@ -64,7 +63,6 @@ assertEq(wasmEvalText('(module (func) (table $t (export "tbl") funcref (elem 0 0
 // Functions.
 assertErrorMessage(() => wasmEvalText('(module (func $t import))'), SyntaxError, parsingError);
 assertErrorMessage(() => wasmEvalText('(module (func $t (import)))'), SyntaxError, parsingError);
-assertErrorMessage(() => wasmEvalText('(module (func $t (import "mod")))'), SyntaxError, parsingError);
 assertErrorMessage(() => wasmEvalText('(module (func $t (import "mod" "func" (local i32))))'), SyntaxError, parsingError);
 
 const func = i => 42 + i;
@@ -95,11 +93,10 @@ assertErrorMessage(() => wasmEvalText(`
 // Globals.
 assertErrorMessage(() => wasmEvalText('(module (global $t (export)))'), SyntaxError, parsingError);
 assertErrorMessage(() => wasmEvalText('(module (global $t (export "g")))'), SyntaxError, parsingError);
-assertErrorMessage(() => wasmEvalText('(module (global $t (export "g") i32))'), WebAssembly.CompileError, /unexpected initializer expression/);
+assertErrorMessage(() => wasmEvalText('(module (global $t (export "g") i32))'), WebAssembly.CompileError, /popping value/);
 wasmEvalText('(module (global $t (export "g") i32 (i32.const 42)))');
 
 assertErrorMessage(() => wasmEvalText('(module (global $t (import) i32))'), SyntaxError, parsingError);
-assertErrorMessage(() => wasmEvalText('(module (global $t (import "mod") i32))'), SyntaxError, parsingError);
 assertErrorMessage(() => wasmEvalText('(module (global $t (import "mod" "field")))'), SyntaxError, parsingError);
 assertErrorMessage(() => wasmEvalText('(module (global $t (import "mod" "field")) i32 (i32.const 42))'), SyntaxError, parsingError);
 wasmEvalText('(module (global $t (import "mod" "field") i32))', { mod: {field: 42} });
@@ -119,7 +116,6 @@ wasmEvalText('(module (memory $t (export "g") 0))');
 
 const mem = new WebAssembly.Memory({ initial: 1 });
 assertErrorMessage(() => wasmEvalText('(module (memory $t (import) 1))'), SyntaxError, parsingError);
-assertErrorMessage(() => wasmEvalText('(module (memory $t (import "mod") 1))'), SyntaxError, parsingError);
 assertErrorMessage(() => wasmEvalText('(module (memory $t (import "mod" "field")))'), SyntaxError, parsingError);
 wasmEvalText('(module (memory $t (import "mod" "field") 1))', { mod: {field: mem} });
 
