@@ -9,10 +9,18 @@
 
 // This file declares various analysis passes that operate on MIR.
 
+#include <stddef.h>
+#include <stdint.h>
+
+#include "jit/IonTypes.h"
 #include "jit/JitAllocPolicy.h"
+#include "js/TypeDecls.h"
+#include "js/Utility.h"
+#include "js/Vector.h"
 
 namespace js {
 
+class GenericPrinter;
 class PlainObject;
 
 namespace jit {
@@ -24,50 +32,50 @@ class MIRGenerator;
 class MIRGraph;
 class MTest;
 
-MOZ_MUST_USE bool PruneUnusedBranches(MIRGenerator* mir, MIRGraph& graph);
+[[nodiscard]] bool PruneUnusedBranches(MIRGenerator* mir, MIRGraph& graph);
 
-MOZ_MUST_USE bool FoldTests(MIRGraph& graph);
+[[nodiscard]] bool FoldTests(MIRGraph& graph);
 
-MOZ_MUST_USE bool FoldEmptyBlocks(MIRGraph& graph);
+[[nodiscard]] bool FoldEmptyBlocks(MIRGraph& graph);
 
-MOZ_MUST_USE bool SplitCriticalEdges(MIRGraph& graph);
+[[nodiscard]] bool SplitCriticalEdges(MIRGraph& graph);
 
 bool IsUint32Type(const MDefinition* def);
 
 enum Observability { ConservativeObservability, AggressiveObservability };
 
-MOZ_MUST_USE bool EliminatePhis(MIRGenerator* mir, MIRGraph& graph,
-                                Observability observe);
+[[nodiscard]] bool EliminatePhis(MIRGenerator* mir, MIRGraph& graph,
+                                 Observability observe);
 
 size_t MarkLoopBlocks(MIRGraph& graph, MBasicBlock* header, bool* canOsr);
 
 void UnmarkLoopBlocks(MIRGraph& graph, MBasicBlock* header);
 
-MOZ_MUST_USE bool MakeLoopsContiguous(MIRGraph& graph);
+[[nodiscard]] bool MakeLoopsContiguous(MIRGraph& graph);
 
-MOZ_MUST_USE bool EliminateDeadResumePointOperands(MIRGenerator* mir,
-                                                   MIRGraph& graph);
+[[nodiscard]] bool EliminateDeadResumePointOperands(MIRGenerator* mir,
+                                                    MIRGraph& graph);
 
-MOZ_MUST_USE bool EliminateDeadCode(MIRGenerator* mir, MIRGraph& graph);
+[[nodiscard]] bool EliminateDeadCode(MIRGenerator* mir, MIRGraph& graph);
 
-MOZ_MUST_USE bool FoldLoadsWithUnbox(MIRGenerator* mir, MIRGraph& graph);
+[[nodiscard]] bool FoldLoadsWithUnbox(MIRGenerator* mir, MIRGraph& graph);
 
-MOZ_MUST_USE bool ApplyTypeInformation(MIRGenerator* mir, MIRGraph& graph);
+[[nodiscard]] bool ApplyTypeInformation(MIRGenerator* mir, MIRGraph& graph);
 
 void RenumberBlocks(MIRGraph& graph);
 
-MOZ_MUST_USE bool AccountForCFGChanges(MIRGenerator* mir, MIRGraph& graph,
-                                       bool updateAliasAnalysis,
-                                       bool underValueNumberer = false);
+[[nodiscard]] bool AccountForCFGChanges(MIRGenerator* mir, MIRGraph& graph,
+                                        bool updateAliasAnalysis,
+                                        bool underValueNumberer = false);
 
-MOZ_MUST_USE bool RemoveUnmarkedBlocks(MIRGenerator* mir, MIRGraph& graph,
-                                       uint32_t numMarkedBlocks);
+[[nodiscard]] bool RemoveUnmarkedBlocks(MIRGenerator* mir, MIRGraph& graph,
+                                        uint32_t numMarkedBlocks);
 
 void ClearDominatorTree(MIRGraph& graph);
 
-MOZ_MUST_USE bool BuildDominatorTree(MIRGraph& graph);
+[[nodiscard]] bool BuildDominatorTree(MIRGraph& graph);
 
-MOZ_MUST_USE bool BuildPhiReverseMapping(MIRGraph& graph);
+[[nodiscard]] bool BuildPhiReverseMapping(MIRGraph& graph);
 
 void AssertBasicGraphCoherency(MIRGraph& graph, bool force = false);
 
@@ -77,9 +85,9 @@ void AssertExtendedGraphCoherency(MIRGraph& graph,
                                   bool underValueNumberer = false,
                                   bool force = false);
 
-MOZ_MUST_USE bool EliminateRedundantChecks(MIRGraph& graph);
+[[nodiscard]] bool EliminateRedundantChecks(MIRGraph& graph);
 
-MOZ_MUST_USE bool AddKeepAliveInstructions(MIRGraph& graph);
+[[nodiscard]] bool AddKeepAliveInstructions(MIRGraph& graph);
 
 // Simple linear sum of the form 'n' or 'x + n'.
 struct SimpleLinearSum {
@@ -100,12 +108,14 @@ struct SimpleLinearSum {
 enum class MathSpace { Modulo, Infinite, Unknown };
 
 SimpleLinearSum ExtractLinearSum(MDefinition* ins,
-                                 MathSpace space = MathSpace::Unknown);
+                                 MathSpace space = MathSpace::Unknown,
+                                 int32_t recursionDepth = 0);
 
-MOZ_MUST_USE bool ExtractLinearInequality(MTest* test,
-                                          BranchDirection direction,
-                                          SimpleLinearSum* plhs,
-                                          MDefinition** prhs, bool* plessEqual);
+[[nodiscard]] bool ExtractLinearInequality(MTest* test,
+                                           BranchDirection direction,
+                                           SimpleLinearSum* plhs,
+                                           MDefinition** prhs,
+                                           bool* plessEqual);
 
 struct LinearTerm {
   MDefinition* term;
@@ -129,15 +139,15 @@ class LinearSum {
 
   // These return false on an integer overflow, and afterwards the sum must
   // not be used.
-  MOZ_MUST_USE bool multiply(int32_t scale);
-  MOZ_MUST_USE bool add(const LinearSum& other, int32_t scale = 1);
-  MOZ_MUST_USE bool add(SimpleLinearSum other, int32_t scale = 1);
-  MOZ_MUST_USE bool add(MDefinition* term, int32_t scale);
-  MOZ_MUST_USE bool add(int32_t constant);
+  [[nodiscard]] bool multiply(int32_t scale);
+  [[nodiscard]] bool add(const LinearSum& other, int32_t scale = 1);
+  [[nodiscard]] bool add(SimpleLinearSum other, int32_t scale = 1);
+  [[nodiscard]] bool add(MDefinition* term, int32_t scale);
+  [[nodiscard]] bool add(int32_t constant);
 
   // Unlike the above function, on failure this leaves the sum unchanged and
   // it can still be used.
-  MOZ_MUST_USE bool divide(uint32_t scale);
+  [[nodiscard]] bool divide(uint32_t scale);
 
   int32_t constant() const { return constant_; }
   size_t numTerms() const { return terms_.length(); }
@@ -152,23 +162,10 @@ class LinearSum {
   int32_t constant_;
 };
 
-// Convert all components of a linear sum (except, optionally, the constant)
+// Convert all components of a linear sum (except the constant)
 // and add any new instructions to the end of block.
 MDefinition* ConvertLinearSum(TempAllocator& alloc, MBasicBlock* block,
-                              const LinearSum& sum,
-                              bool convertConstant = false);
-
-// Convert the test 'sum >= 0' to a comparison, adding any necessary
-// instructions to the end of block.
-MCompare* ConvertLinearInequality(TempAllocator& alloc, MBasicBlock* block,
-                                  const LinearSum& sum);
-
-MOZ_MUST_USE bool AnalyzeNewScriptDefiniteProperties(
-    JSContext* cx, DPAConstraintInfo& constraintInfo, HandleFunction fun,
-    ObjectGroup* group, Handle<PlainObject*> baseobj,
-    Vector<TypeNewScriptInitializer>* initializerList);
-
-MOZ_MUST_USE bool AnalyzeArgumentsUsage(JSContext* cx, JSScript* script);
+                              const LinearSum& sum, BailoutKind bailoutKind);
 
 bool DeadIfUnused(const MDefinition* def);
 
