@@ -10,13 +10,13 @@
 #include "mozilla/HashFunctions.h"
 
 #include <algorithm>
+#include <initializer_list>
 #include <stdint.h>
 
-#include "jsfriendapi.h"
 #include "jstypes.h"
 
+#include "js/ScalarType.h"  // js::Scalar::Type
 #include "js/Value.h"
-#include "vm/StringType.h"
 
 namespace js {
 
@@ -41,9 +41,9 @@ class IonCompilationId {
 
 namespace jit {
 
-typedef uint32_t RecoverOffset;
-typedef uint32_t SnapshotOffset;
-typedef uint32_t BailoutId;
+using RecoverOffset = uint32_t;
+using SnapshotOffset = uint32_t;
+using BailoutId = uint32_t;
 
 // The maximum size of any buffer associated with an assembler or code object.
 // This is chosen to not overflow a signed integer, leaving room for an extra
@@ -56,223 +56,179 @@ static const uint32_t SNAPSHOT_MAX_NARGS = 127;
 static const SnapshotOffset INVALID_RECOVER_OFFSET = uint32_t(-1);
 static const SnapshotOffset INVALID_SNAPSHOT_OFFSET = uint32_t(-1);
 
-// Different kinds of bailouts. When extending this enum, make sure to check
-// the bits reserved for bailout kinds in Bailouts.h
-enum BailoutKind {
-  // Normal bailouts, that don't need to be handled specially when restarting
-  // in baseline.
+/*
+ * [SMDOC] Avoiding repeated bailouts / invalidations
+ *
+ * To avoid getting trapped in a "compilation -> bailout -> invalidation ->
+ * recompilation -> bailout -> invalidation -> ..." loop, every snapshot in
+ * Warp code is assigned a BailoutKind. If we bail out at that snapshot,
+ * FinishBailoutToBaseline will examine the BailoutKind and take appropriate
+ * action. In general:
+ *
+ * 1. If the bailing instruction comes from transpiled CacheIR, then when we
+ *    bail out and continue execution in the baseline interpreter, the
+ *    corresponding stub should fail a guard. As a result, we will either
+ *    increment the enteredCount for a subsequent stub or attach a new stub,
+ *    either of which will prevent WarpOracle from transpiling the failing stub
+ *    when we recompile.
+ *
+ *    Note: this means that every CacheIR op that can bail out in Warp must
+ *    have an equivalent guard in the baseline CacheIR implementation.
+ *
+ *    FirstExecution works according to the same principles: we have never hit
+ *    this IC before, but after we bail to baseline we will attach a stub and
+ *    recompile with better CacheIR information.
+ *
+ * 2. If the bailout occurs because an assumption we made in WarpBuilder was
+ *    invalidated, then FinishBailoutToBaseline will set a flag on the script
+ *    to avoid that assumption in the future: for example, UninitializedLexical.
+ *
+ * 3. Similarly, if the bailing instruction is generated or modified by a MIR
+ *    optimization, then FinishBailoutToBaseline will set a flag on the script
+ *    to make that optimization more conservative in the future.  Examples
+ *    include LICM, EagerTruncation, and HoistBoundsCheck.
+ *
+ * 4. Some bailouts can't be handled in Warp, even after a recompile. For
+ *    example, Warp does not support catching exceptions. If this happens
+ *    too often, then the cost of bailing out repeatedly outweighs the
+ *    benefit of Warp compilation, so we invalidate the script and disable
+ *    Warp compilation.
+ *
+ * 5. Some bailouts don't happen in performance-sensitive code: for example,
+ *    the |debugger| statement. We just ignore those.
+ */
+enum class BailoutKind : uint8_t {
+  Unknown,
+
+  // An instruction generated by the transpiler. If this instruction bails out,
+  // attaching a new stub in baseline will invalidate the current Warp script
+  // and avoid a bailout loop.
+  TranspiledCacheIR,
+
+  // An optimistic unbox on the cold path for a non-Value phi failed. If this
+  // instruction bails out, we will invalidate the script and mark the
+  // HadSpeculativePhiBailout flag on the script.
+  SpeculativePhi,
+
+  // A conversion inserted by a type policy. If this instruction bails out,
+  // we expect to throw an error. If this happens too frequently, we will
+  // invalidate the current Warp script and disable recompilation.
+  TypePolicy,
+
+  // An instruction hoisted by LICM.  If this instruction bails out, we will
+  // bail out to baseline to see if we attach a new stub. If we do, then the
+  // more than once, we will invalidate the current Warp script and
+  // mark the hadLICMInvalidation flag on the script.
+  LICM,
+
+  // An instruction moved up by InstructionReordering.  If this
+  // instruction bails out, we will mark the ReorderingBailout flag on
+  // the script. If this happens too frequently, we will invalidate
+  // the script.
+  InstructionReordering,
+
+  // An instruction created or hoisted by tryHoistBoundsCheck.
+  // If this instruction bails out, we will invalidate the current Warp script
+  // and mark the HoistBoundsCheckBailout flag on the script.
+  HoistBoundsCheck,
+
+  // An eager truncation generated by range analysis.
+  // If this instruction bails out, we will invalidate the current Warp script
+  // and mark the EagerTruncationBailout flag on the script.
+  EagerTruncation,
+
+  // A folded unbox instruction generated by FoldLoadsWithUnbox.
+  // If this instruction bails out, we will invalidate the current Warp script
+  // and mark the UnboxFoldingBailout flag on the script.
+  UnboxFolding,
 
   // An inevitable bailout (MBail instruction or type barrier that always bails)
-  Bailout_Inevitable,
+  Inevitable,
 
   // Bailing out during a VM call. Many possible causes that are hard
   // to distinguish statically at snapshot construction time.
   // We just lump them together.
-  Bailout_DuringVMCall,
+  DuringVMCall,
 
-  // Call to a non-JSFunction (problem for |apply|)
-  Bailout_NonJSFunctionCallee,
+  // A spread call or funapply had more than JIT_ARGS_LENGTH_MAX arguments.
+  // We bail out to handle this in the VM. If this happens too frequently,
+  // we will invalidate the current Warp script and disable recompilation.
+  TooManyArguments,
 
-  // Dynamic scope chain lookup produced |undefined|
-  Bailout_DynamicNameNotFound,
-
-  // Input string contains 'arguments' or 'eval'
-  Bailout_StringArgumentsEval,
-
-  // Bailout on overflow, but don't immediately invalidate.
-  // Used for abs, sub and LoadUnboxedScalar (when loading a uint32 that
-  // doesn't fit in an int32).
-  Bailout_Overflow,
-
-  // floor, ceiling and round bail if input is NaN, if output would be -0 or
-  // doesn't fit in int32 range
-  Bailout_Round,
-
-  // Non-primitive value used as input for ToDouble, ToInt32, ToString, etc.
-  // For ToInt32, can also mean that input can't be converted without precision
-  // loss (e.g. 5.5).
-  Bailout_NonPrimitiveInput,
-
-  // For ToInt32, would lose precision when converting (e.g. 5.5).
-  Bailout_PrecisionLoss,
-
-  // We tripped a type barrier (object was not in the expected TypeSet)
-  Bailout_TypeBarrierO,
-  // We tripped a type barrier (value was not in the expected TypeSet)
-  Bailout_TypeBarrierV,
-  // We tripped a type monitor (wrote an unexpected type in a property)
-  Bailout_MonitorTypes,
-
-  // We hit a hole in an array.
-  Bailout_Hole,
-
-  // Array access with negative index
-  Bailout_NegativeIndex,
-
-  // Array access with non integer index
-  Bailout_NonIntegerIndex,
-
-  // Pretty specific case:
-  //  - need a type barrier on a property write
-  //  - all but one of the observed types have property types that reflect
-  //    the value
-  //  - we need to guard that we're not given an object of that one other type
-  //    also used for the unused GuardClass instruction
-  Bailout_ObjectIdentityOrTypeGuard,
-
-  // Unbox expects a given type, bails out if it doesn't get it.
-  Bailout_NonInt32Input,
-  Bailout_NonNumericInput,  // unboxing a double works with int32 too
-  Bailout_NonBooleanInput,
-  Bailout_NonObjectInput,
-  Bailout_NonStringInput,
-  Bailout_NonSymbolInput,
-  Bailout_NonBigIntInput,
-
-  // Atomic operations require shared memory, bail out if the typed array
-  // maps unshared memory.
-  Bailout_NonSharedTypedArrayInput,
-
-  // We hit a |debugger;| statement.
-  Bailout_Debugger,
-
-  // |this| used uninitialized in a derived constructor
-  Bailout_UninitializedThis,
-
-  // Derived constructors must return object or undefined
-  Bailout_BadDerivedConstructorReturn,
+  // We hit an active |debugger;| statement.
+  Debugger,
 
   // We hit this code for the first time.
-  Bailout_FirstExecution,
+  FirstExecution,
 
-  // END Normal bailouts
-
-  // Bailouts caused by invalid assumptions based on Baseline code.
-  // Causes immediate invalidation.
-
-  // Like Bailout_Overflow, but causes immediate invalidation.
-  Bailout_OverflowInvalidate,
-
-  // Used for integer division, multiplication and modulo.
-  // If there's a remainder, bails to return a double.
-  // Can also signal overflow or result of -0.
-  // Can also signal division by 0 (returns inf, a double).
-  Bailout_DoubleOutput,
-
-  // END Invalid assumptions bailouts
-
-  // A bailout at the very start of a function indicates that there may be
-  // a type mismatch in the arguments that necessitates a reflow.
-  Bailout_ArgumentCheck,
-
-  // A bailout triggered by a bounds-check failure.
-  Bailout_BoundsCheck,
-  // A bailout triggered by a typed object whose backing buffer was detached.
-  Bailout_Detached,
-
-  // A shape guard based on TI information failed.
-  // (We saw an object whose shape does not match that / any of those observed
-  // by the baseline IC.)
-  Bailout_ShapeGuard,
-
-  // When we're trying to use an uninitialized lexical.
-  Bailout_UninitializedLexical,
+  // A lexical check failed. We will set lexical checks as unmovable.
+  UninitializedLexical,
 
   // A bailout to baseline from Ion on exception to handle Debugger hooks.
-  Bailout_IonExceptionDebugMode
+  IonExceptionDebugMode,
+
+  // We returned to a stack frame after invalidating its IonScript.
+  OnStackInvalidation,
+
+  // We have executed code that should be unreachable, and need to assert.
+  Unreachable,
+
+  Limit
 };
 
 inline const char* BailoutKindString(BailoutKind kind) {
   switch (kind) {
-    // Normal bailouts.
-    case Bailout_Inevitable:
-      return "Bailout_Inevitable";
-    case Bailout_DuringVMCall:
-      return "Bailout_DuringVMCall";
-    case Bailout_NonJSFunctionCallee:
-      return "Bailout_NonJSFunctionCallee";
-    case Bailout_DynamicNameNotFound:
-      return "Bailout_DynamicNameNotFound";
-    case Bailout_StringArgumentsEval:
-      return "Bailout_StringArgumentsEval";
-    case Bailout_Overflow:
-      return "Bailout_Overflow";
-    case Bailout_Round:
-      return "Bailout_Round";
-    case Bailout_NonPrimitiveInput:
-      return "Bailout_NonPrimitiveInput";
-    case Bailout_PrecisionLoss:
-      return "Bailout_PrecisionLoss";
-    case Bailout_TypeBarrierO:
-      return "Bailout_TypeBarrierO";
-    case Bailout_TypeBarrierV:
-      return "Bailout_TypeBarrierV";
-    case Bailout_MonitorTypes:
-      return "Bailout_MonitorTypes";
-    case Bailout_Hole:
-      return "Bailout_Hole";
-    case Bailout_NegativeIndex:
-      return "Bailout_NegativeIndex";
-    case Bailout_NonIntegerIndex:
-      return "Bailout_NonIntegerIndex";
-    case Bailout_ObjectIdentityOrTypeGuard:
-      return "Bailout_ObjectIdentityOrTypeGuard";
-    case Bailout_NonInt32Input:
-      return "Bailout_NonInt32Input";
-    case Bailout_NonNumericInput:
-      return "Bailout_NonNumericInput";
-    case Bailout_NonBooleanInput:
-      return "Bailout_NonBooleanInput";
-    case Bailout_NonObjectInput:
-      return "Bailout_NonObjectInput";
-    case Bailout_NonStringInput:
-      return "Bailout_NonStringInput";
-    case Bailout_NonSymbolInput:
-      return "Bailout_NonSymbolInput";
-    case Bailout_NonBigIntInput:
-      return "Bailout_NonBigIntInput";
-    case Bailout_NonSharedTypedArrayInput:
-      return "Bailout_NonSharedTypedArrayInput";
-    case Bailout_Debugger:
-      return "Bailout_Debugger";
-    case Bailout_UninitializedThis:
-      return "Bailout_UninitializedThis";
-    case Bailout_BadDerivedConstructorReturn:
-      return "Bailout_BadDerivedConstructorReturn";
-    case Bailout_FirstExecution:
-      return "Bailout_FirstExecution";
+    case BailoutKind::Unknown:
+      return "Unknown";
+    case BailoutKind::TranspiledCacheIR:
+      return "TranspiledCacheIR";
+    case BailoutKind::SpeculativePhi:
+      return "SpeculativePhi";
+    case BailoutKind::TypePolicy:
+      return "TypePolicy";
+    case BailoutKind::LICM:
+      return "LICM";
+    case BailoutKind::InstructionReordering:
+      return "InstructionReordering";
+    case BailoutKind::HoistBoundsCheck:
+      return "HoistBoundsCheck";
+    case BailoutKind::EagerTruncation:
+      return "EagerTruncation";
+    case BailoutKind::UnboxFolding:
+      return "UnboxFolding";
+    case BailoutKind::Inevitable:
+      return "Inevitable";
+    case BailoutKind::DuringVMCall:
+      return "DuringVMCall";
+    case BailoutKind::TooManyArguments:
+      return "TooManyArguments";
+    case BailoutKind::Debugger:
+      return "Debugger";
+    case BailoutKind::FirstExecution:
+      return "FirstExecution";
+    case BailoutKind::UninitializedLexical:
+      return "UninitializedLexical";
+    case BailoutKind::IonExceptionDebugMode:
+      return "IonExceptionDebugMode";
+    case BailoutKind::OnStackInvalidation:
+      return "OnStackInvalidation";
+    case BailoutKind::Unreachable:
+      return "Unreachable";
 
-    // Bailouts caused by invalid assumptions.
-    case Bailout_OverflowInvalidate:
-      return "Bailout_OverflowInvalidate";
-    case Bailout_DoubleOutput:
-      return "Bailout_DoubleOutput";
-
-    // Other bailouts.
-    case Bailout_ArgumentCheck:
-      return "Bailout_ArgumentCheck";
-    case Bailout_BoundsCheck:
-      return "Bailout_BoundsCheck";
-    case Bailout_Detached:
-      return "Bailout_Detached";
-    case Bailout_ShapeGuard:
-      return "Bailout_ShapeGuard";
-    case Bailout_UninitializedLexical:
-      return "Bailout_UninitializedLexical";
-    case Bailout_IonExceptionDebugMode:
-      return "Bailout_IonExceptionDebugMode";
-    default:
-      MOZ_CRASH("Invalid BailoutKind");
+    case BailoutKind::Limit:
+      break;
   }
+
+  MOZ_CRASH("Invalid BailoutKind");
 }
 
 static const uint32_t ELEMENT_TYPE_BITS = 5;
 static const uint32_t ELEMENT_TYPE_SHIFT = 0;
 static const uint32_t ELEMENT_TYPE_MASK = (1 << ELEMENT_TYPE_BITS) - 1;
-static const uint32_t VECTOR_SCALE_BITS = 3;
-static const uint32_t VECTOR_SCALE_SHIFT =
+static const uint32_t VECTOR_TYPE_BITS = 1;
+static const uint32_t VECTOR_TYPE_SHIFT =
     ELEMENT_TYPE_BITS + ELEMENT_TYPE_SHIFT;
-static const uint32_t VECTOR_SCALE_MASK = (1 << VECTOR_SCALE_BITS) - 1;
+static const uint32_t VECTOR_TYPE_MASK = (1 << VECTOR_TYPE_BITS) - 1;
 
 // The integer SIMD types have a lot of operations that do the exact same thing
 // for signed and unsigned integer types. Sometimes it is simpler to treat
@@ -289,12 +245,22 @@ enum class SimdSign {
 
 class SimdConstant {
  public:
-  enum Type { Int8x16, Int16x8, Int32x4, Float32x4, Undefined = -1 };
+  enum Type {
+    Int8x16,
+    Int16x8,
+    Int32x4,
+    Int64x2,
+    Float32x4,
+    Float64x2,
+    Undefined = -1
+  };
 
   typedef int8_t I8x16[16];
   typedef int16_t I16x8[8];
   typedef int32_t I32x4[4];
+  typedef int64_t I64x2[2];
   typedef float F32x4[4];
+  typedef double F64x2[2];
 
  private:
   Type type_;
@@ -302,7 +268,9 @@ class SimdConstant {
     I8x16 i8x16;
     I16x8 i16x8;
     I32x4 i32x4;
+    I64x2 i64x2;
     F32x4 f32x4;
+    F64x2 f64x2;
   } u;
 
   bool defined() const { return type_ != Undefined; }
@@ -347,6 +315,18 @@ class SimdConstant {
     std::fill_n(cst.u.i32x4, 4, v);
     return cst;
   }
+  static SimdConstant CreateX2(const int64_t* array) {
+    SimdConstant cst;
+    cst.type_ = Int64x2;
+    memcpy(cst.u.i64x2, array, sizeof(cst.u));
+    return cst;
+  }
+  static SimdConstant SplatX2(int64_t v) {
+    SimdConstant cst;
+    cst.type_ = Int64x2;
+    std::fill_n(cst.u.i64x2, 2, v);
+    return cst;
+  }
   static SimdConstant CreateX4(const float* array) {
     SimdConstant cst;
     cst.type_ = Float32x4;
@@ -359,6 +339,18 @@ class SimdConstant {
     std::fill_n(cst.u.f32x4, 4, v);
     return cst;
   }
+  static SimdConstant CreateX2(const double* array) {
+    SimdConstant cst;
+    cst.type_ = Float64x2;
+    memcpy(cst.u.f64x2, array, sizeof(cst.u));
+    return cst;
+  }
+  static SimdConstant SplatX2(double v) {
+    SimdConstant cst;
+    cst.type_ = Float64x2;
+    std::fill_n(cst.u.f64x2, 2, v);
+    return cst;
+  }
 
   // Overloads for use by templates.
   static SimdConstant CreateSimd128(const int8_t* array) {
@@ -370,15 +362,31 @@ class SimdConstant {
   static SimdConstant CreateSimd128(const int32_t* array) {
     return CreateX4(array);
   }
+  static SimdConstant CreateSimd128(const int64_t* array) {
+    return CreateX2(array);
+  }
   static SimdConstant CreateSimd128(const float* array) {
     return CreateX4(array);
   }
+  static SimdConstant CreateSimd128(const double* array) {
+    return CreateX2(array);
+  }
 
   Type type() const {
     MOZ_ASSERT(defined());
     return type_;
   }
 
+  bool isFloatingType() const {
+    MOZ_ASSERT(defined());
+    return type_ >= Float32x4;
+  }
+
+  bool isIntegerType() const {
+    MOZ_ASSERT(defined());
+    return type_ <= Int64x2;
+  }
+
   // Get the raw bytes of the constant.
   const void* bytes() const { return u.i8x16; }
 
@@ -397,39 +405,58 @@ class SimdConstant {
     return u.i32x4;
   }
 
+  const I64x2& asInt64x2() const {
+    MOZ_ASSERT(defined() && type_ == Int64x2);
+    return u.i64x2;
+  }
+
   const F32x4& asFloat32x4() const {
     MOZ_ASSERT(defined() && type_ == Float32x4);
     return u.f32x4;
   }
 
-  bool operator==(const SimdConstant& rhs) const {
+  const F64x2& asFloat64x2() const {
+    MOZ_ASSERT(defined() && type_ == Float64x2);
+    return u.f64x2;
+  }
+
+  bool bitwiseEqual(const SimdConstant& rhs) const {
     MOZ_ASSERT(defined() && rhs.defined());
-    if (type() != rhs.type()) {
-      return false;
-    }
-    // Takes negative zero into accuont, as it's a bit comparison.
     return memcmp(&u, &rhs.u, sizeof(u)) == 0;
   }
-  bool operator!=(const SimdConstant& rhs) const { return !operator==(rhs); }
 
-  // SimdConstant is a HashPolicy
-  typedef SimdConstant Lookup;
+  bool isZeroBits() const {
+    MOZ_ASSERT(defined());
+    return u.i64x2[0] == 0 && u.i64x2[1] == 0;
+  }
+
+  bool isOneBits() const {
+    MOZ_ASSERT(defined());
+    return ~u.i64x2[0] == 0 && ~u.i64x2[1] == 0;
+  }
+
+  // SimdConstant is a HashPolicy.  Currently we discriminate by type, but it
+  // may be that we should only be discriminating by int vs float.
+  using Lookup = SimdConstant;
+
   static HashNumber hash(const SimdConstant& val) {
     uint32_t hash = mozilla::HashBytes(&val.u, sizeof(val.u));
     return mozilla::AddToHash(hash, val.type_);
   }
+
   static bool match(const SimdConstant& lhs, const SimdConstant& rhs) {
-    return lhs == rhs;
+    return lhs.type() == rhs.type() && lhs.bitwiseEqual(rhs);
   }
 };
 
 enum class IntConversionBehavior {
   // These two try to convert the input to an int32 using ToNumber and
   // will fail if the resulting int32 isn't strictly equal to the input.
-  Normal,
-  NegativeZeroCheck,
-  // These two will convert the input to an int32 with loss of precision.
+  Normal,             // Succeeds on -0: converts to 0.
+  NegativeZeroCheck,  // Fails on -0.
+  // These three will convert the input to an int32 with loss of precision.
   Truncate,
+  TruncateNoWrap,
   ClampToUint8,
 };
 
@@ -444,46 +471,32 @@ enum class MIRType : uint8_t {
   Boolean,
   Int32,
   Int64,
+  IntPtr,
   Double,
   Float32,
   // Types above have trivial conversion to a number.
   String,
   Symbol,
   BigInt,
+  Simd128,
   // Types above are primitive (including undefined and null).
   Object,
-  MagicOptimizedArguments,    // JS_OPTIMIZED_ARGUMENTS magic value.
   MagicOptimizedOut,          // JS_OPTIMIZED_OUT magic value.
   MagicHole,                  // JS_ELEMENTS_HOLE magic value.
   MagicIsConstructing,        // JS_IS_CONSTRUCTING magic value.
   MagicUninitializedLexical,  // JS_UNINITIALIZED_LEXICAL magic value.
   // Types above are specialized.
   Value,
-  SinCosDouble,  // Optimizing a sin/cos to sincos.
-  ObjectOrNull,
-  None,         // Invalid, used as a placeholder.
-  Slots,        // A slots vector
-  Elements,     // An elements vector
-  Pointer,      // An opaque pointer that receives no special treatment
-  RefOrNull,    // Wasm Ref/AnyRef/NullRef: a raw JSObject* or a raw (void*)0
-  Shape,        // A Shape pointer.
-  ObjectGroup,  // An ObjectGroup pointer.
-  Last = ObjectGroup,
-  // Representing both SIMD.IntBxN and SIMD.UintBxN.
-  Int8x16 = Int32 | (4 << VECTOR_SCALE_SHIFT),
-  Int16x8 = Int32 | (3 << VECTOR_SCALE_SHIFT),
-  Int32x4 = Int32 | (2 << VECTOR_SCALE_SHIFT),
-  Float32x4 = Float32 | (2 << VECTOR_SCALE_SHIFT),
-  Bool8x16 = Boolean | (4 << VECTOR_SCALE_SHIFT),
-  Bool16x8 = Boolean | (3 << VECTOR_SCALE_SHIFT),
-  Bool32x4 = Boolean | (2 << VECTOR_SCALE_SHIFT),
-  Doublex2 = Double | (1 << VECTOR_SCALE_SHIFT)
+  None,          // Invalid, used as a placeholder.
+  Slots,         // A slots vector
+  Elements,      // An elements vector
+  Pointer,       // An opaque pointer that receives no special treatment
+  RefOrNull,     // Wasm Ref/AnyRef/NullRef: a raw JSObject* or a raw (void*)0
+  StackResults,  // Wasm multi-value stack result area, which may contain refs
+  Shape,         // A Shape pointer.
+  Last = Shape
 };
 
-static inline bool IsSimdType(MIRType type) {
-  return ((uint8_t(type) >> VECTOR_SCALE_SHIFT) & VECTOR_SCALE_MASK) != 0;
-}
-
 static inline MIRType MIRTypeFromValueType(JSValueType type) {
   // This function does not deal with magic types. Magic constants should be
   // filtered out in MIRTypeFromValue.
@@ -532,7 +545,6 @@ static inline JSValueType ValueTypeFromMIRType(MIRType type) {
       return JSVAL_TYPE_SYMBOL;
     case MIRType::BigInt:
       return JSVAL_TYPE_BIGINT;
-    case MIRType::MagicOptimizedArguments:
     case MIRType::MagicOptimizedOut:
     case MIRType::MagicHole:
     case MIRType::MagicIsConstructing:
@@ -558,6 +570,8 @@ static inline size_t MIRTypeToSize(MIRType type) {
       return 4;
     case MIRType::Double:
       return 8;
+    case MIRType::Simd128:
+      return 16;
     case MIRType::Pointer:
     case MIRType::RefOrNull:
       return sizeof(uintptr_t);
@@ -578,6 +592,8 @@ static inline const char* StringFromMIRType(MIRType type) {
       return "Int32";
     case MIRType::Int64:
       return "Int64";
+    case MIRType::IntPtr:
+      return "IntPtr";
     case MIRType::Double:
       return "Double";
     case MIRType::Float32:
@@ -590,8 +606,6 @@ static inline const char* StringFromMIRType(MIRType type) {
       return "BigInt";
     case MIRType::Object:
       return "Object";
-    case MIRType::MagicOptimizedArguments:
-      return "MagicOptimizedArguments";
     case MIRType::MagicOptimizedOut:
       return "MagicOptimizedOut";
     case MIRType::MagicHole:
@@ -602,10 +616,6 @@ static inline const char* StringFromMIRType(MIRType type) {
       return "MagicUninitializedLexical";
     case MIRType::Value:
       return "Value";
-    case MIRType::SinCosDouble:
-      return "SinCosDouble";
-    case MIRType::ObjectOrNull:
-      return "ObjectOrNull";
     case MIRType::None:
       return "None";
     case MIRType::Slots:
@@ -616,26 +626,12 @@ static inline const char* StringFromMIRType(MIRType type) {
       return "Pointer";
     case MIRType::RefOrNull:
       return "RefOrNull";
+    case MIRType::StackResults:
+      return "StackResults";
     case MIRType::Shape:
       return "Shape";
-    case MIRType::ObjectGroup:
-      return "ObjectGroup";
-    case MIRType::Int32x4:
-      return "Int32x4";
-    case MIRType::Int16x8:
-      return "Int16x8";
-    case MIRType::Int8x16:
-      return "Int8x16";
-    case MIRType::Float32x4:
-      return "Float32x4";
-    case MIRType::Bool32x4:
-      return "Bool32x4";
-    case MIRType::Bool16x8:
-      return "Bool16x8";
-    case MIRType::Bool8x16:
-      return "Bool8x16";
-    case MIRType::Doublex2:
-      return "Doublex2";
+    case MIRType::Simd128:
+      return "Simd128";
   }
   MOZ_CRASH("Unknown MIRType.");
 }
@@ -673,7 +669,6 @@ static inline bool IsNullOrUndefined(MIRType type) {
 static inline bool IsMagicType(MIRType type) {
   return type == MIRType::MagicHole || type == MIRType::MagicOptimizedOut ||
          type == MIRType::MagicIsConstructing ||
-         type == MIRType::MagicOptimizedArguments ||
          type == MIRType::MagicUninitializedLexical;
 }
 
@@ -696,18 +691,18 @@ static inline MIRType ScalarTypeToMIRType(Scalar::Type type) {
     case Scalar::BigInt64:
     case Scalar::BigUint64:
       MOZ_CRASH("NYI");
+    case Scalar::Simd128:
+      return MIRType::Simd128;
     case Scalar::MaxTypedArrayViewType:
       break;
   }
   MOZ_CRASH("unexpected kind");
 }
 
-static inline const char* PropertyNameToExtraName(PropertyName* name) {
-  JS::AutoCheckCannotGC nogc;
-  if (!name->hasLatin1Chars()) {
-    return nullptr;
-  }
-  return reinterpret_cast<const char*>(name->latin1Chars(nogc));
+static constexpr bool NeedsPostBarrier(MIRType type) {
+  MOZ_ASSERT(type != MIRType::Value);
+  return type == MIRType::Object || type == MIRType::String ||
+         type == MIRType::BigInt;
 }
 
 #ifdef DEBUG
@@ -722,17 +717,41 @@ static inline const char* PropertyNameToExtraName(PropertyName* name) {
 #endif  // DEBUG
 
 enum ABIArgType {
+  // A pointer sized integer
   ArgType_General = 0x1,
-  ArgType_Double = 0x2,
-  ArgType_Float32 = 0x3,
-  ArgType_Int64 = 0x4,
+  // A 32-bit integer
+  ArgType_Int32 = 0x2,
+  // A 64-bit integer
+  ArgType_Int64 = 0x3,
+  // A 32-bit floating point number
+  ArgType_Float32 = 0x4,
+  // A 64-bit floating point number
+  ArgType_Float64 = 0x5,
 
   RetType_Shift = 0x0,
   ArgType_Shift = 0x3,
   ArgType_Mask = 0x7
 };
 
-enum ABIFunctionType {
+namespace detail {
+
+static constexpr int MakeABIFunctionType(
+    ABIArgType ret, std::initializer_list<ABIArgType> args) {
+  int abiType = ret << RetType_Shift;
+  int i = 1;
+  for (auto arg : args) {
+    abiType |= (arg << (ArgType_Shift * i));
+    i++;
+  }
+  return abiType;
+}
+
+}  // namespace detail
+
+enum ABIFunctionType : uint32_t {
+  // The enum must be explicitly typed to avoid UB: some validly constructed
+  // members are larger than any explicitly declared members.
+
   // VM functions that take 0-9 non-double arguments
   // and return a non-double value.
   Args_General0 = ArgType_General << RetType_Shift,
@@ -747,13 +766,13 @@ enum ABIFunctionType {
 
   // int64 f(double)
   Args_Int64_Double =
-      (ArgType_Int64 << RetType_Shift) | (ArgType_Double << ArgType_Shift),
+      (ArgType_Int64 << RetType_Shift) | (ArgType_Float64 << ArgType_Shift),
 
   // double f()
-  Args_Double_None = ArgType_Double << RetType_Shift,
+  Args_Double_None = ArgType_Float64 << RetType_Shift,
 
   // int f(double)
-  Args_Int_Double = Args_General0 | (ArgType_Double << ArgType_Shift),
+  Args_Int_Double = Args_General0 | (ArgType_Float64 << ArgType_Shift),
 
   // int f(float32)
   Args_Int_Float32 = Args_General0 | (ArgType_Float32 << ArgType_Shift),
@@ -768,7 +787,7 @@ enum ABIFunctionType {
                         (ArgType_General << (ArgType_Shift * 2)),
 
   // double f(double)
-  Args_Double_Double = Args_Double_None | (ArgType_Double << ArgType_Shift),
+  Args_Double_Double = Args_Double_None | (ArgType_Float64 << ArgType_Shift),
 
   // double f(int)
   Args_Double_Int = Args_Double_None | (ArgType_General << ArgType_Shift),
@@ -780,11 +799,11 @@ enum ABIFunctionType {
   // double f(double, int)
   Args_Double_DoubleInt = Args_Double_None |
                           (ArgType_General << (ArgType_Shift * 1)) |
-                          (ArgType_Double << (ArgType_Shift * 2)),
+                          (ArgType_Float64 << (ArgType_Shift * 2)),
 
   // double f(double, double)
   Args_Double_DoubleDouble =
-      Args_Double_Double | (ArgType_Double << (ArgType_Shift * 2)),
+      Args_Double_Double | (ArgType_Float64 << (ArgType_Shift * 2)),
 
   // float f(float, float)
   Args_Float32_Float32Float32 =
@@ -792,32 +811,38 @@ enum ABIFunctionType {
 
   // double f(int, double)
   Args_Double_IntDouble = Args_Double_None |
-                          (ArgType_Double << (ArgType_Shift * 1)) |
+                          (ArgType_Float64 << (ArgType_Shift * 1)) |
                           (ArgType_General << (ArgType_Shift * 2)),
 
   // int f(int, double)
-  Args_Int_IntDouble = Args_General0 | (ArgType_Double << (ArgType_Shift * 1)) |
+  Args_Int_IntDouble = Args_General0 |
+                       (ArgType_Float64 << (ArgType_Shift * 1)) |
                        (ArgType_General << (ArgType_Shift * 2)),
 
+  // int f(double, int)
+  Args_Int_DoubleInt = Args_General0 |
+                       (ArgType_General << (ArgType_Shift * 1)) |
+                       (ArgType_Float64 << (ArgType_Shift * 2)),
+
   // double f(double, double, double)
   Args_Double_DoubleDoubleDouble =
-      Args_Double_DoubleDouble | (ArgType_Double << (ArgType_Shift * 3)),
+      Args_Double_DoubleDouble | (ArgType_Float64 << (ArgType_Shift * 3)),
 
   // double f(double, double, double, double)
   Args_Double_DoubleDoubleDoubleDouble =
-      Args_Double_DoubleDoubleDouble | (ArgType_Double << (ArgType_Shift * 4)),
+      Args_Double_DoubleDoubleDouble | (ArgType_Float64 << (ArgType_Shift * 4)),
 
   // int f(double, int, int)
   Args_Int_DoubleIntInt = Args_General0 |
                           (ArgType_General << (ArgType_Shift * 1)) |
                           (ArgType_General << (ArgType_Shift * 2)) |
-                          (ArgType_Double << (ArgType_Shift * 3)),
+                          (ArgType_Float64 << (ArgType_Shift * 3)),
 
   // int f(int, double, int, int)
   Args_Int_IntDoubleIntInt = Args_General0 |
                              (ArgType_General << (ArgType_Shift * 1)) |
                              (ArgType_General << (ArgType_Shift * 2)) |
-                             (ArgType_Double << (ArgType_Shift * 3)) |
+                             (ArgType_Float64 << (ArgType_Shift * 3)) |
                              (ArgType_General << (ArgType_Shift * 4)),
 
   Args_Int_GeneralGeneralGeneralInt64 =
@@ -830,23 +855,57 @@ enum ABIFunctionType {
                                       (ArgType_General << (ArgType_Shift * 1)) |
                                       (ArgType_General << (ArgType_Shift * 2)) |
                                       (ArgType_Int64 << (ArgType_Shift * 3)) |
-                                      (ArgType_Int64 << (ArgType_Shift * 4))
+                                      (ArgType_Int64 << (ArgType_Shift * 4)),
+
+  Args_Int32_General =
+      detail::MakeABIFunctionType(ArgType_Int32, {ArgType_General}),
+  Args_Int32_GeneralInt32 = detail::MakeABIFunctionType(
+      ArgType_Int32, {ArgType_General, ArgType_Int32}),
+  Args_Int32_GeneralInt32Int32 = detail::MakeABIFunctionType(
+      ArgType_Int32, {ArgType_General, ArgType_Int32, ArgType_Int32}),
+  Args_Int32_GeneralInt32Int32Int32Int32 = detail::MakeABIFunctionType(
+      ArgType_Int32, {ArgType_General, ArgType_Int32, ArgType_Int32,
+                      ArgType_Int32, ArgType_Int32}),
+  Args_Int32_GeneralInt32Int32Int32Int32Int32 = detail::MakeABIFunctionType(
+      ArgType_Int32, {ArgType_General, ArgType_Int32, ArgType_Int32,
+                      ArgType_Int32, ArgType_Int32, ArgType_Int32}),
+  Args_Int32_GeneralInt32Int32Int32General = detail::MakeABIFunctionType(
+      ArgType_Int32, {ArgType_General, ArgType_Int32, ArgType_Int32,
+                      ArgType_Int32, ArgType_General}),
+  Args_Int32_GeneralInt32Int32Int64 = detail::MakeABIFunctionType(
+      ArgType_Int32,
+      {ArgType_General, ArgType_Int32, ArgType_Int32, ArgType_Int64}),
+  Args_Int32_GeneralInt32Int32General = detail::MakeABIFunctionType(
+      ArgType_Int32,
+      {ArgType_General, ArgType_Int32, ArgType_Int32, ArgType_General}),
+  Args_Int32_GeneralInt32Int64Int64 = detail::MakeABIFunctionType(
+      ArgType_Int32,
+      {ArgType_General, ArgType_Int32, ArgType_Int64, ArgType_Int64}),
+  Args_Int32_GeneralInt32GeneralInt32 = detail::MakeABIFunctionType(
+      ArgType_Int32,
+      {ArgType_General, ArgType_Int32, ArgType_General, ArgType_Int32}),
+  Args_Int32_GeneralInt32GeneralInt32Int32 = detail::MakeABIFunctionType(
+      ArgType_Int32, {ArgType_General, ArgType_Int32, ArgType_General,
+                      ArgType_Int32, ArgType_Int32}),
+  Args_Int32_GeneralGeneral = detail::MakeABIFunctionType(
+      ArgType_Int32, {ArgType_General, ArgType_General}),
+  Args_Int32_GeneralGeneralGeneral = detail::MakeABIFunctionType(
+      ArgType_Int32, {ArgType_General, ArgType_General, ArgType_General}),
+  Args_Int32_GeneralGeneralInt32Int32 = detail::MakeABIFunctionType(
+      ArgType_Int32,
+      {ArgType_General, ArgType_General, ArgType_Int32, ArgType_Int32}),
+  Args_General_GeneralInt32 = detail::MakeABIFunctionType(
+      ArgType_General, {ArgType_General, ArgType_Int32}),
+  Args_General_GeneralInt32Int32 = detail::MakeABIFunctionType(
+      ArgType_General, {ArgType_General, ArgType_Int32, ArgType_Int32}),
+  Args_General_GeneralInt32General = detail::MakeABIFunctionType(
+      ArgType_General, {ArgType_General, ArgType_Int32, ArgType_General}),
 };
 
-enum class BarrierKind : uint32_t {
-  // No barrier is needed.
-  NoBarrier,
-
-  // The barrier only has to check the value's type tag is in the TypeSet.
-  // Specific object types don't have to be checked.
-  TypeTagOnly,
-
-  // Check if the value is in the TypeSet, including the object type if it's
-  // an object.
-  TypeSet
-};
-
-enum ReprotectCode { Reprotect = true, DontReprotect = false };
+static constexpr ABIFunctionType MakeABIFunctionType(
+    ABIArgType ret, std::initializer_list<ABIArgType> args) {
+  return ABIFunctionType(detail::MakeABIFunctionType(ret, args));
+}
 
 // Rounding modes for round instructions.
 enum class RoundingMode { Down, Up, NearestTiesToEven, TowardsZero };
@@ -857,12 +916,21 @@ enum class RoundingMode { Down, Up, NearestTiesToEven, TowardsZero };
 static const uint32_t MAX_UNCHECKED_LEAF_FRAME_SIZE = 64;
 
 // Truncating conversion modifiers.
-typedef uint32_t TruncFlags;
+using TruncFlags = uint32_t;
 static const TruncFlags TRUNC_UNSIGNED = TruncFlags(1) << 0;
 static const TruncFlags TRUNC_SATURATING = TruncFlags(1) << 1;
 
 enum BranchDirection { FALSE_BRANCH, TRUE_BRANCH };
 
+template <typename T>
+constexpr T SplatByteToUInt(uint8_t val, uint8_t x) {
+  T splatted = val;
+  for (; x > 1; x--) {
+    splatted |= splatted << 8;
+  }
+  return splatted;
+}
+
 }  // namespace jit
 }  // namespace js
 
