@@ -8,28 +8,26 @@
 
 #include "jit/BaselineDebugModeOSR.h"
 #include "jit/BaselineIC.h"
+#include "jit/CalleeToken.h"
+#include "jit/IonScript.h"
 #include "jit/JitcodeMap.h"
 #include "jit/JitFrames.h"
+#include "jit/JitRuntime.h"
+#include "jit/JitScript.h"
+#include "jit/MacroAssembler.h"  // js::jit::Assembler::GetPointer
+#include "jit/SafepointIndex.h"
 #include "jit/Safepoints.h"
+#include "jit/ScriptFromCalleeToken.h"
+#include "jit/VMFunctions.h"
+#include "js/friend/DumpFunctions.h"  // js::DumpObject, js::DumpValue
+
+#include "vm/JSScript-inl.h"
 
 using namespace js;
 using namespace js::jit;
 
 JSJitFrameIter::JSJitFrameIter(const JitActivation* activation)
-    : current_(activation->jsExitFP()),
-      type_(FrameType::Exit),
-      resumePCinCurrentFrame_(nullptr),
-      frameSize_(0),
-      cachedSafepointIndex_(nullptr),
-      activation_(activation) {
-  if (activation_->bailoutData()) {
-    current_ = activation_->bailoutData()->fp();
-    frameSize_ = activation_->bailoutData()->topFrameSize();
-    type_ = FrameType::Bailout;
-  } else {
-    MOZ_ASSERT(!TlsContext.get()->inUnsafeCallWithABI);
-  }
-}
+    : JSJitFrameIter(activation, FrameType::Exit, activation->jsExitFP()) {}
 
 JSJitFrameIter::JSJitFrameIter(const JitActivation* activation,
                                FrameType frameType, uint8_t* fp)
@@ -40,8 +38,13 @@ JSJitFrameIter::JSJitFrameIter(const JitActivation* activation,
       cachedSafepointIndex_(nullptr),
       activation_(activation) {
   MOZ_ASSERT(type_ == FrameType::JSJitToWasm || type_ == FrameType::Exit);
-  MOZ_ASSERT(!activation_->bailoutData());
-  MOZ_ASSERT(!TlsContext.get()->inUnsafeCallWithABI);
+  if (activation_->bailoutData()) {
+    current_ = activation_->bailoutData()->fp();
+    frameSize_ = activation_->bailoutData()->topFrameSize();
+    type_ = FrameType::Bailout;
+  } else {
+    MOZ_ASSERT(!TlsContext.get()->inUnsafeCallWithABI);
+  }
 }
 
 bool JSJitFrameIter::checkInvalidation() const {
@@ -131,14 +134,6 @@ void JSJitFrameIter::baselineScriptAndPc(JSScript** scriptRes,
 
   MOZ_ASSERT(pcRes);
 
-  // Use the frame's override pc, if we have one. This should only happen
-  // when we're in FinishBailoutToBaseline, handling an exception or toggling
-  // debug mode.
-  if (jsbytecode* overridePc = baselineFrame()->maybeOverridePc()) {
-    *pcRes = overridePc;
-    return;
-  }
-
   // The Baseline Interpreter stores the bytecode pc in the frame.
   if (baselineFrame()->runningInInterpreter()) {
     MOZ_ASSERT(baselineFrame()->interpreterScript() == script);
@@ -146,10 +141,10 @@ void JSJitFrameIter::baselineScriptAndPc(JSScript** scriptRes,
     return;
   }
 
-  // Else, there must be a BaselineScript with a VMCallEntry for the current
-  // return address.
+  // There must be a BaselineScript with a RetAddrEntry for the current return
+  // address.
   uint8_t* retAddr = resumePCinCurrentFrame();
-  RetAddrEntry& entry =
+  const RetAddrEntry& entry =
       script->baselineScript()->retAddrEntryFromReturnAddress(retAddr);
   *pcRes = entry.pc(script);
 }
@@ -163,6 +158,22 @@ uint8_t* JSJitFrameIter::prevFp() const {
 void JSJitFrameIter::operator++() {
   MOZ_ASSERT(!isEntry());
 
+  // Compute BaselineFrame size, the size stored in the descriptor excluding
+  // VMFunction arguments pushed for VM calls.
+  //
+  // In debug builds this is equivalent to BaselineFrame::debugFrameSize_. This
+  // is asserted at the end of this method.
+  if (current()->prevType() == FrameType::BaselineJS) {
+    uint32_t frameSize = prevFrameLocalSize();
+    if (isExitFrame() && exitFrame()->isWrapperExit()) {
+      const VMFunctionData* data = exitFrame()->footer()->function();
+      frameSize -= data->explicitStackSlots() * sizeof(void*);
+    }
+    baselineFrameSize_ = mozilla::Some(frameSize);
+  } else {
+    baselineFrameSize_ = mozilla::Nothing();
+  }
+
   frameSize_ = prevFrameLocalSize();
   cachedSafepointIndex_ = nullptr;
 
@@ -176,6 +187,9 @@ void JSJitFrameIter::operator++() {
   type_ = current()->prevType();
   resumePCinCurrentFrame_ = current()->returnAddress();
   current_ = prevFp();
+
+  MOZ_ASSERT_IF(isBaselineJS(),
+                baselineFrame()->debugFrameSize() == *baselineFrameSize_);
 }
 
 uintptr_t* JSJitFrameIter::spillBase() const {
@@ -205,8 +219,7 @@ MachineState JSJitFrameIter::machineState() const {
     machine.setRegisterLocation(*iter, --spill);
   }
 
-  uint8_t* spillAlign =
-      alignDoubleSpillWithOffset(reinterpret_cast<uint8_t*>(spill), 0);
+  uint8_t* spillAlign = alignDoubleSpill(reinterpret_cast<uint8_t*>(spill));
 
   char* floatSpill = reinterpret_cast<char*>(spillAlign);
   FloatRegisterSet fregs = reader.allFloatSpills().set();
@@ -313,16 +326,14 @@ void JSJitFrameIter::dumpBaseline() const {
 
   fprintf(stderr, "  script = %p, pc = %p (offset %u)\n", (void*)script, pc,
           uint32_t(script->pcToOffset(pc)));
-  fprintf(stderr, "  current op: %s\n", CodeName[*pc]);
+  fprintf(stderr, "  current op: %s\n", CodeName(JSOp(*pc)));
 
-  fprintf(stderr, "  actual args: %d\n", numActualArgs());
+  fprintf(stderr, "  actual args: %u\n", numActualArgs());
 
-  BaselineFrame* frame = baselineFrame();
-
-  for (unsigned i = 0; i < frame->numValueSlots(); i++) {
+  for (unsigned i = 0; i < baselineFrameNumValueSlots(); i++) {
     fprintf(stderr, "  slot %u: ", i);
 #if defined(DEBUG) || defined(JS_JITSPEW)
-    Value* v = frame->valueSlot(i);
+    Value* v = baselineFrame()->valueSlot(i);
     DumpValue(*v);
 #else
     fprintf(stderr, "?\n");
@@ -428,11 +439,13 @@ bool JSJitFrameIter::verifyReturnAddressUsingNativeToBytecodeMap() {
   MOZ_ASSERT(depth > 0 && depth != UINT32_MAX);
   MOZ_ASSERT(location.length() == depth);
 
-  JitSpew(JitSpew_Profiling, "Found bytecode location of depth %d:", depth);
+  JitSpew(JitSpew_Profiling, "Found bytecode location of depth %u:", depth);
   for (size_t i = 0; i < location.length(); i++) {
-    JitSpew(JitSpew_Profiling, "   %s:%u - %zu", location[i].script->filename(),
-            location[i].script->lineno(),
-            size_t(location[i].pc - location[i].script->code()));
+    JitSpew(JitSpew_Profiling, "   %s:%u - %zu",
+            location[i].getDebugOnlyScript()->filename(),
+            location[i].getDebugOnlyScript()->lineno(),
+            size_t(location[i].toRawBytecode() -
+                   location[i].getDebugOnlyScript()->code()));
   }
 
   if (type_ == FrameType::IonJS) {
@@ -447,10 +460,12 @@ bool JSJitFrameIter::verifyReturnAddressUsingNativeToBytecodeMap() {
               (int)idx, inlineFrames.script()->filename(),
               inlineFrames.script()->lineno(),
               size_t(inlineFrames.pc() - inlineFrames.script()->code()),
-              location[idx].script->filename(), location[idx].script->lineno(),
-              size_t(location[idx].pc - location[idx].script->code()));
+              location[idx].getDebugOnlyScript()->filename(),
+              location[idx].getDebugOnlyScript()->lineno(),
+              size_t(location[idx].toRawBytecode() -
+                     location[idx].getDebugOnlyScript()->code()));
 
-      MOZ_ASSERT(inlineFrames.script() == location[idx].script);
+      MOZ_ASSERT(inlineFrames.script() == location[idx].getDebugOnlyScript());
 
       if (inlineFrames.more()) {
         ++inlineFrames;
@@ -519,12 +534,16 @@ JSJitProfilingFrameIterator::JSJitProfilingFrameIterator(JSContext* cx,
     }
   }
 
-  MOZ_ASSERT(frameScript()->hasBaselineScript());
-
   // If nothing matches, for now just assume we are at the start of the last
-  // frame's baseline jit code.
+  // frame's baseline jit code or interpreter code.
   type_ = FrameType::BaselineJS;
-  resumePCinCurrentFrame_ = frameScript()->baselineScript()->method()->raw();
+  if (frameScript()->hasBaselineScript()) {
+    resumePCinCurrentFrame_ = frameScript()->baselineScript()->method()->raw();
+  } else {
+    MOZ_ASSERT(IsBaselineInterpreterEnabled());
+    resumePCinCurrentFrame_ =
+        cx->runtime()->jitRuntime()->baselineInterpreter().codeRaw();
+  }
 }
 
 template <typename ReturnType = CommonFrameLayout*>
@@ -574,8 +593,8 @@ bool JSJitProfilingFrameIterator::tryInitWithTable(JitcodeGlobalTable* table,
 
   JSScript* callee = frameScript();
 
-  MOZ_ASSERT(entry->isIon() || entry->isBaseline() || entry->isIonCache() ||
-             entry->isDummy());
+  MOZ_ASSERT(entry->isIon() || entry->isBaseline() ||
+             entry->isBaselineInterpreter() || entry->isDummy());
 
   // Treat dummy lookups as an empty frame sequence.
   if (entry->isDummy()) {
@@ -609,16 +628,8 @@ bool JSJitProfilingFrameIterator::tryInitWithTable(JitcodeGlobalTable* table,
     return true;
   }
 
-  if (entry->isIonCache()) {
-    void* ptr = entry->ionCacheEntry().rejoinAddr();
-    const JitcodeGlobalEntry& ionEntry = table->lookupInfallible(ptr);
-    MOZ_ASSERT(ionEntry.isIon());
-
-    if (ionEntry.ionEntry().getScript(0) != callee) {
-      return false;
-    }
-
-    type_ = FrameType::IonJS;
+  if (entry->isBaselineInterpreter()) {
+    type_ = FrameType::BaselineJS;
     resumePCinCurrentFrame_ = pc;
     return true;
   }
@@ -626,31 +637,28 @@ bool JSJitProfilingFrameIterator::tryInitWithTable(JitcodeGlobalTable* table,
   return false;
 }
 
-void JSJitProfilingFrameIterator::fixBaselineReturnAddress() {
+const char* JSJitProfilingFrameIterator::baselineInterpreterLabel() const {
   MOZ_ASSERT(type_ == FrameType::BaselineJS);
-  BaselineFrame* bl = (BaselineFrame*)(fp_ - BaselineFrame::FramePointerOffset -
-                                       BaselineFrame::Size());
+  return frameScript()->jitScript()->profileString();
+}
 
-  // Debug mode OSR for Baseline uses a "continuation fixer" and stashes the
-  // actual return address in an auxiliary structure.
-  if (BaselineDebugModeOSRInfo* info = bl->getDebugModeOSRInfo()) {
-    resumePCinCurrentFrame_ = info->resumeAddr;
-    return;
-  }
+void JSJitProfilingFrameIterator::baselineInterpreterScriptPC(
+    JSScript** script, jsbytecode** pc, uint64_t* realmID) const {
+  MOZ_ASSERT(type_ == FrameType::BaselineJS);
+  BaselineFrame* blFrame =
+      (BaselineFrame*)(fp_ - BaselineFrame::FramePointerOffset -
+                       BaselineFrame::Size());
+  *script = frameScript();
+  *pc = (*script)->code();
 
-  // Certain exception handling cases such as debug OSR or resuming a generator
-  // with .throw() will use BaselineFrame::setOverridePc() to indicate the
-  // effective |pc|. We translate the effective-pc into a Baseline code
-  // address.
-  if (jsbytecode* override = bl->maybeOverridePc()) {
-    PCMappingSlotInfo slotInfo;
-    JSScript* script = bl->script();
-    resumePCinCurrentFrame_ =
-        script->baselineScript()->nativeCodeForPC(script, override, &slotInfo);
+  if (blFrame->runningInInterpreter() &&
+      blFrame->interpreterScript() == *script) {
+    jsbytecode* interpPC = blFrame->interpreterPC();
+    if ((*script)->containsPC(interpPC)) {
+      *pc = interpPC;
+    }
 
-    // NOTE: The stack may not be synced at this PC. For the purpose of
-    // profiler sampling this is fine.
-    return;
+    *realmID = (*script)->realm()->creationOptions().profilerRealmID();
   }
 }
 
@@ -725,7 +733,6 @@ void JSJitProfilingFrameIterator::moveToNextFrame(CommonFrameLayout* frame) {
     resumePCinCurrentFrame_ = frame->returnAddress();
     fp_ = GetPreviousRawFrame<uint8_t*>(frame);
     type_ = FrameType::BaselineJS;
-    fixBaselineReturnAddress();
     return;
   }
 
@@ -738,7 +745,6 @@ void JSJitProfilingFrameIterator::moveToNextFrame(CommonFrameLayout* frame) {
     fp_ = ((uint8_t*)stubFrame->reverseSavedFramePtr()) +
           jit::BaselineFrame::FramePointerOffset;
     type_ = FrameType::BaselineJS;
-    fixBaselineReturnAddress();
     return;
   }
 
@@ -761,7 +767,6 @@ void JSJitProfilingFrameIterator::moveToNextFrame(CommonFrameLayout* frame) {
       fp_ = ((uint8_t*)stubFrame->reverseSavedFramePtr()) +
             jit::BaselineFrame::FramePointerOffset;
       type_ = FrameType::BaselineJS;
-      fixBaselineReturnAddress();
       return;
     }
 
