@@ -6,7 +6,8 @@
 
 #include "jit/MIRGraph.h"
 
-#include "jit/BytecodeAnalysis.h"
+#include "jit/CompileInfo.h"
+#include "jit/InlineScriptTree.h"
 #include "jit/Ion.h"
 #include "jit/JitSpewer.h"
 #include "jit/MIR.h"
@@ -34,7 +35,6 @@ MIRGenerator::MIRGenerator(CompileRealm* realm,
       needsStaticStackAlignment_(false),
       instrumentedProfiling_(false),
       instrumentedProfilingIsCached_(false),
-      safeForMinorGC_(true),
       stringsCanBeInNursery_(realm ? realm->zone()->canNurseryAllocateStrings()
                                    : false),
       bigIntsCanBeInNursery_(realm ? realm->zone()->canNurseryAllocateBigInts()
@@ -49,12 +49,6 @@ mozilla::GenericErrorResult<AbortReason> MIRGenerator::abort(AbortReason r) {
       case AbortReason::Alloc:
         JitSpew(JitSpew_IonAbort, "AbortReason::Alloc");
         break;
-      case AbortReason::Inlining:
-        JitSpew(JitSpew_IonAbort, "AbortReason::Inlining");
-        break;
-      case AbortReason::PreliminaryObjects:
-        JitSpew(JitSpew_IonAbort, "AbortReason::PreliminaryObjects");
-        break;
       case AbortReason::Disable:
         JitSpew(JitSpew_IonAbort, "AbortReason::Disable");
         break;
@@ -103,95 +97,6 @@ void MIRGraph::insertBlockBefore(MBasicBlock* at, MBasicBlock* block) {
   numBlocks_++;
 }
 
-bool MIRGraph::removeSuccessorBlocks(MBasicBlock* start) {
-  if (!start->hasLastIns()) {
-    return true;
-  }
-
-  start->mark();
-
-  // Mark all successors.
-  Vector<MBasicBlock*, 4, SystemAllocPolicy> blocks;
-  for (size_t i = 0; i < start->numSuccessors(); i++) {
-    if (!start->getSuccessor(i)) {
-      continue;
-    }
-    if (start->getSuccessor(i)->isMarked()) {
-      continue;
-    }
-    if (!blocks.append(start->getSuccessor(i))) {
-      return false;
-    }
-    start->getSuccessor(i)->mark();
-  }
-  for (size_t i = 0; i < blocks.length(); i++) {
-    MBasicBlock* block = blocks[i];
-    if (!block->hasLastIns()) {
-      continue;
-    }
-
-    for (size_t j = 0; j < block->numSuccessors(); j++) {
-      if (!block->getSuccessor(j)) {
-        continue;
-      }
-      if (block->getSuccessor(j)->isMarked()) {
-        continue;
-      }
-      if (!blocks.append(block->getSuccessor(j))) {
-        return false;
-      }
-      block->getSuccessor(j)->mark();
-    }
-  }
-
-  if (osrBlock()) {
-    if (osrBlock()->getSuccessor(0)->isMarked()) {
-      osrBlock()->mark();
-    }
-  }
-
-  // Remove blocks.
-  // If they don't have any predecessor
-  for (size_t i = 0; i < blocks.length(); i++) {
-    MBasicBlock* block = blocks[i];
-    bool allMarked = true;
-    for (size_t i = 0; i < block->numPredecessors(); i++) {
-      if (block->getPredecessor(i)->isMarked()) {
-        continue;
-      }
-      allMarked = false;
-      break;
-    }
-    if (allMarked) {
-      removeBlock(block);
-    } else {
-      MOZ_ASSERT(block != osrBlock());
-      for (size_t j = 0; j < block->numPredecessors();) {
-        if (!block->getPredecessor(j)->isMarked()) {
-          j++;
-          continue;
-        }
-        block->removePredecessor(block->getPredecessor(j));
-      }
-      // This shouldn't have any instructions yet.
-      MOZ_ASSERT(block->begin() == block->end());
-    }
-  }
-
-  if (osrBlock()) {
-    if (osrBlock()->getSuccessor(0)->isDead()) {
-      removeBlock(osrBlock());
-    }
-  }
-
-  for (size_t i = 0; i < blocks.length(); i++) {
-    blocks[i]->unmark();
-  }
-  start->unmark();
-
-  return true;
-}
-
 void MIRGraph::removeBlock(MBasicBlock* block) {
   // Remove a block from the graph. It will also cleanup the block.
 
@@ -219,13 +124,6 @@ void MIRGraph::removeBlock(MBasicBlock* block) {
   }
 }
 
-void MIRGraph::removeBlockIncludingPhis(MBasicBlock* block) {
-  // removeBlock doesn't clear phis because of IonBuilder constraints. Here,
-  // we want to totally clear everything.
-  removeBlock(block);
-  block->discardAllPhis();
-}
-
 void MIRGraph::unmarkBlocks() {
   for (MBasicBlockIterator i(blocks_.begin()); i != blocks_.end(); i++) {
     i->unmark();
@@ -252,6 +150,8 @@ MBasicBlock* MBasicBlock::New(MIRGraph& graph, size_t stackDepth,
 MBasicBlock* MBasicBlock::NewPopN(MIRGraph& graph, const CompileInfo& info,
                                   MBasicBlock* pred, BytecodeSite* site,
                                   Kind kind, uint32_t popped) {
+  MOZ_ASSERT(site->pc() != nullptr);
+
   MBasicBlock* block = new (graph.alloc()) MBasicBlock(graph, info, site, kind);
   if (!block->init()) {
     return nullptr;
@@ -264,31 +164,6 @@ MBasicBlock* MBasicBlock::NewPopN(MIRGraph& graph, const CompileInfo& info,
   return block;
 }
 
-MBasicBlock* MBasicBlock::NewWithResumePoint(MIRGraph& graph,
-                                             const CompileInfo& info,
-                                             MBasicBlock* pred,
-                                             BytecodeSite* site,
-                                             MResumePoint* resumePoint) {
-  MBasicBlock* block =
-      new (graph.alloc()) MBasicBlock(graph, info, site, NORMAL);
-
-  MOZ_ASSERT(!resumePoint->instruction());
-  resumePoint->block()->discardResumePoint(resumePoint, RefType_None);
-  resumePoint->setBlock(block);
-  block->addResumePoint(resumePoint);
-  block->entryResumePoint_ = resumePoint;
-
-  if (!block->init()) {
-    return nullptr;
-  }
-
-  if (!block->inheritResumePoint(pred)) {
-    return nullptr;
-  }
-
-  return block;
-}
-
 MBasicBlock* MBasicBlock::NewPendingLoopHeader(MIRGraph& graph,
                                                const CompileInfo& info,
                                                MBasicBlock* pred,
@@ -318,7 +193,7 @@ MBasicBlock* MBasicBlock::NewSplitEdge(MIRGraph& graph, MBasicBlock* pred,
       return nullptr;
     }
   } else {
-    // The predecessor has a PC, this is an IonBuilder compilation.
+    // The predecessor has a PC, this is a Warp compilation.
     MResumePoint* succEntry = succ->entryResumePoint();
 
     BytecodeSite* site =
@@ -447,18 +322,80 @@ MBasicBlock* MBasicBlock::New(MIRGraph& graph, const CompileInfo& info,
   return block;
 }
 
+// Create an empty and unreachable block which jumps to |header|. Used
+// when the normal entry into a loop is removed (but the loop is still
+// reachable due to OSR) to preserve the invariant that every loop
+// header has two predecessors, which is needed for building the
+// dominator tree. The new block is inserted immediately before the
+// header, which preserves the graph ordering (post-order/RPO). These
+// blocks will all be removed before lowering.
+MBasicBlock* MBasicBlock::NewFakeLoopPredecessor(MIRGraph& graph,
+                                                 MBasicBlock* header) {
+  MOZ_ASSERT(graph.osrBlock());
+
+  MBasicBlock* backedge = header->backedge();
+  MBasicBlock* fake = MBasicBlock::New(graph, header->info(), nullptr,
+                                       MBasicBlock::FAKE_LOOP_PRED);
+  if (!fake) {
+    return nullptr;
+  }
+
+  graph.insertBlockBefore(header, fake);
+  fake->setUnreachable();
+
+  // Create fake defs to use as inputs for any phis in |header|.
+  for (MPhiIterator iter(header->phisBegin()), end(header->phisEnd());
+       iter != end; ++iter) {
+    MPhi* phi = *iter;
+    auto* fakeDef = MUnreachableResult::New(graph.alloc(), phi->type());
+    fake->add(fakeDef);
+    if (!phi->addInputSlow(fakeDef)) {
+      return nullptr;
+    }
+  }
+
+  fake->end(MGoto::New(graph.alloc(), header));
+
+  if (!header->addPredecessorWithoutPhis(fake)) {
+    return nullptr;
+  }
+
+  // The backedge is always the last predecessor, but we have added a
+  // new pred. Restore |backedge| as |header|'s loop backedge.
+  header->clearLoopHeader();
+  header->setLoopHeader(backedge);
+
+  return fake;
+}
+
+void MIRGraph::removeFakeLoopPredecessors() {
+  MOZ_ASSERT(osrBlock());
+  size_t id = 0;
+  for (ReversePostorderIterator it = rpoBegin(); it != rpoEnd();) {
+    MBasicBlock* block = *it++;
+    if (block->isFakeLoopPred()) {
+      MOZ_ASSERT(block->unreachable());
+      MBasicBlock* succ = block->getSingleSuccessor();
+      succ->removePredecessor(block);
+      removeBlock(block);
+    } else {
+      block->setId(id++);
+    }
+  }
+#ifdef DEBUG
+  canBuildDominators_ = false;
+#endif
+}
+
 MBasicBlock::MBasicBlock(MIRGraph& graph, const CompileInfo& info,
                          BytecodeSite* site, Kind kind)
-    : unreachable_(false),
-      specialized_(false),
-      graph_(graph),
+    : graph_(graph),
       info_(info),
       predecessors_(graph.alloc()),
       stackPosition_(info_.firstStackSlot()),
       id_(0),
       domIndex_(0),
       numDominated_(0),
-      pc_(site->pc()),
       lir_(nullptr),
       callerResumePoint_(nullptr),
       entryResumePoint_(nullptr),
@@ -470,15 +407,14 @@ MBasicBlock::MBasicBlock(MIRGraph& graph, const CompileInfo& info,
       mark_(false),
       immediatelyDominated_(graph.alloc()),
       immediateDominator_(nullptr),
-      trackedSite_(site),
-      hitCount_(0),
-      hitState_(HitState::NotDefined)
+      trackedSite_(site)
 #if defined(JS_ION_PERF) || defined(DEBUG)
       ,
       lineno_(0u),
       columnIndex_(0u)
 #endif
 {
+  MOZ_ASSERT(trackedSite_, "trackedSite_ is non-nullptr");
 }
 
 bool MBasicBlock::init() { return slots_.init(graph_.alloc(), info_.nslots()); }
@@ -567,26 +503,6 @@ bool MBasicBlock::inherit(TempAllocator& alloc, size_t stackDepth,
   return true;
 }
 
-bool MBasicBlock::inheritResumePoint(MBasicBlock* pred) {
-  // Copy slots from the resume point.
-  stackPosition_ = entryResumePoint_->stackDepth();
-  for (uint32_t i = 0; i < stackPosition_; i++) {
-    slots_[i] = entryResumePoint_->getOperand(i);
-  }
-
-  MOZ_ASSERT(info_.nslots() >= stackPosition_);
-  MOZ_ASSERT(kind_ != PENDING_LOOP_HEADER);
-  MOZ_ASSERT(pred != nullptr);
-
-  callerResumePoint_ = pred->callerResumePoint();
-
-  if (!predecessors_.append(pred)) {
-    return false;
-  }
-
-  return true;
-}
-
 void MBasicBlock::inheritSlots(MBasicBlock* parent) {
   stackPosition_ = parent->stackPosition_;
   copySlots(parent);
@@ -605,78 +521,6 @@ bool MBasicBlock::initEntrySlots(TempAllocator& alloc) {
   return true;
 }
 
-void MBasicBlock::shimmySlots(int discardDepth) {
-  // Move all slots above the given depth down by one,
-  // overwriting the MDefinition at discardDepth.
-
-  MOZ_ASSERT(discardDepth < 0);
-  MOZ_ASSERT(stackPosition_ + discardDepth >= info_.firstStackSlot());
-
-  for (int i = discardDepth; i < -1; i++) {
-    slots_[stackPosition_ + i] = slots_[stackPosition_ + i + 1];
-  }
-
-  --stackPosition_;
-}
-
-bool MBasicBlock::linkOsrValues(MStart* start) {
-  MResumePoint* res = start->resumePoint();
-
-  for (uint32_t i = 0; i < stackDepth(); i++) {
-    MDefinition* def = slots_[i];
-    MInstruction* cloneRp = nullptr;
-    if (i == info().environmentChainSlot()) {
-      if (def->isOsrEnvironmentChain()) {
-        cloneRp = def->toOsrEnvironmentChain();
-      }
-    } else if (i == info().returnValueSlot()) {
-      if (def->isOsrReturnValue()) {
-        cloneRp = def->toOsrReturnValue();
-      }
-    } else if (info().hasArguments() && i == info().argsObjSlot()) {
-      MOZ_ASSERT(def->isConstant() || def->isOsrArgumentsObject());
-      MOZ_ASSERT_IF(def->isConstant(),
-                    def->toConstant()->type() == MIRType::Undefined);
-      if (def->isOsrArgumentsObject()) {
-        cloneRp = def->toOsrArgumentsObject();
-      }
-    } else {
-      MOZ_ASSERT(def->isOsrValue() || def->isGetArgumentsObjectArg() ||
-                 def->isConstant() || def->isParameter());
-
-      // A constant Undefined can show up here for an argument slot when
-      // the function has an arguments object, but the argument in
-      // question is stored on the scope chain.
-      MOZ_ASSERT_IF(def->isConstant(),
-                    def->toConstant()->type() == MIRType::Undefined);
-
-      if (def->isOsrValue()) {
-        cloneRp = def->toOsrValue();
-      } else if (def->isGetArgumentsObjectArg()) {
-        cloneRp = def->toGetArgumentsObjectArg();
-      } else if (def->isParameter()) {
-        cloneRp = def->toParameter();
-      }
-    }
-
-    if (cloneRp) {
-      MResumePoint* clone = MResumePoint::Copy(graph().alloc(), res);
-      if (!clone) {
-        return false;
-      }
-      cloneRp->setResumePoint(clone);
-    }
-  }
-
-  return true;
-}
-
-void MBasicBlock::rewriteAtDepth(int32_t depth, MDefinition* ins) {
-  MOZ_ASSERT(depth < 0);
-  MOZ_ASSERT(stackPosition_ + depth >= info_.firstStackSlot());
-  rewriteSlot(stackPosition_ + depth, ins);
-}
-
 MDefinition* MBasicBlock::environmentChain() {
   return getSlot(info().environmentChainSlot());
 }
@@ -740,16 +584,6 @@ MConstant* MBasicBlock::optimizedOutConstant(TempAllocator& alloc) {
   return constant;
 }
 
-void MBasicBlock::addFromElsewhere(MInstruction* ins) {
-  MOZ_ASSERT(ins->block() != this);
-
-  // Remove |ins| from its containing block.
-  ins->block()->instructions_.remove(ins);
-
-  // Add it to this block.
-  add(ins);
-}
-
 void MBasicBlock::moveBefore(MInstruction* at, MInstruction* ins) {
   // Remove |ins| from the current block.
   MOZ_ASSERT(ins->block() == this);
@@ -757,9 +591,8 @@ void MBasicBlock::moveBefore(MInstruction* at, MInstruction* ins) {
 
   // Insert into new block, which may be distinct.
   // Uses and operands are untouched.
-  ins->setBlock(at->block());
+  ins->setInstructionBlock(at->block(), at->trackedSite());
   at->block()->instructions_.insertBefore(at, ins);
-  ins->setTrackedSite(at->trackedSite());
 }
 
 MInstruction* MBasicBlock::safeInsertTop(MDefinition* ins, IgnoreTop ignore) {
@@ -863,7 +696,7 @@ void MBasicBlock::discardAllInstructionsStartingAt(MInstructionIterator iter) {
   }
 }
 
-void MBasicBlock::discardAllPhiOperands() {
+void MBasicBlock::discardAllPhis() {
   for (MPhiIterator iter = phisBegin(); iter != phisEnd(); iter++) {
     iter->removeAllOperands();
   }
@@ -872,10 +705,7 @@ void MBasicBlock::discardAllPhiOperands() {
        pred++) {
     (*pred)->clearSuccessorWithPhis();
   }
-}
 
-void MBasicBlock::discardAllPhis() {
-  discardAllPhiOperands();
   phis_.clear();
 }
 
@@ -903,28 +733,21 @@ void MBasicBlock::discardAllResumePoints(bool discardEntry) {
 void MBasicBlock::clear() {
   discardAllInstructions();
   discardAllResumePoints();
-
-  // Note: phis are disconnected from the rest of the graph, but are not
-  // removed entirely. If the block being removed is a loop header then
-  // IonBuilder may need to access these phis to more quickly converge on the
-  // possible types in the graph. See IonBuilder::analyzeNewLoopTypes.
-  discardAllPhiOperands();
+  discardAllPhis();
 }
 
 void MBasicBlock::insertBefore(MInstruction* at, MInstruction* ins) {
   MOZ_ASSERT(at->block() == this);
-  ins->setBlock(this);
+  ins->setInstructionBlock(this, at->trackedSite());
   graph().allocDefinitionId(ins);
   instructions_.insertBefore(at, ins);
-  ins->setTrackedSite(at->trackedSite());
 }
 
 void MBasicBlock::insertAfter(MInstruction* at, MInstruction* ins) {
   MOZ_ASSERT(at->block() == this);
-  ins->setBlock(this);
+  ins->setInstructionBlock(this, at->trackedSite());
   graph().allocDefinitionId(ins);
   instructions_.insertAfter(at, ins);
-  ins->setTrackedSite(at->trackedSite());
 }
 
 void MBasicBlock::insertAtEnd(MInstruction* ins) {
@@ -937,7 +760,7 @@ void MBasicBlock::insertAtEnd(MInstruction* ins) {
 
 void MBasicBlock::addPhi(MPhi* phi) {
   phis_.pushBack(phi);
-  phi->setBlock(this);
+  phi->setPhiBlock(this);
   graph().allocDefinitionId(phi);
 }
 
@@ -975,13 +798,13 @@ void MBasicBlock::flagOperandsOfPrunedBranches(MInstruction* ins) {
   // SplitEdge blocks.  SplitEdge blocks only have a Goto instruction before
   // Range Analysis phase.  In adjustInputs, we are manipulating instructions
   // which have a TypePolicy.  So, as a Goto has no operand and no type
-  // policy, the entry resume point should exists.
+  // policy, the entry resume point should exist.
   MOZ_ASSERT(rp);
 
-  // Flag all operand as being potentially used.
+  // Flag all operands as being potentially used.
   while (rp) {
     for (size_t i = 0, end = rp->numOperands(); i < end; i++) {
-      rp->getOperand(i)->setUseRemovedUnchecked();
+      rp->getOperand(i)->setImplicitlyUsedUnchecked();
     }
     rp = rp->caller();
   }
@@ -1005,22 +828,25 @@ bool MBasicBlock::addPredecessorPopN(TempAllocator& alloc, MBasicBlock* pred,
     MDefinition* other = pred->getSlot(i);
 
     if (mine != other) {
+      MIRType phiType = mine->type();
+      if (phiType != other->type()) {
+        phiType = MIRType::Value;
+      }
+
       // If the current instruction is a phi, and it was created in this
       // basic block, then we have already placed this phi and should
       // instead append to its operands.
       if (mine->isPhi() && mine->block() == this) {
         MOZ_ASSERT(predecessors_.length());
+        MOZ_ASSERT(!mine->hasDefUses(),
+                   "should only change type of newly created phis");
+        mine->setResultType(phiType);
         if (!mine->toPhi()->addInputSlow(other)) {
           return false;
         }
       } else {
         // Otherwise, create a new phi node.
-        MPhi* phi;
-        if (mine->type() == other->type()) {
-          phi = MPhi::New(alloc.fallible(), mine->type());
-        } else {
-          phi = MPhi::New(alloc.fallible());
-        }
+        MPhi* phi = MPhi::New(alloc.fallible(), phiType);
         if (!phi) {
           return false;
         }
@@ -1096,16 +922,7 @@ void MBasicBlock::removeImmediatelyDominatedBlock(MBasicBlock* child) {
   }
 }
 
-void MBasicBlock::assertUsesAreNotWithin(MUseIterator use, MUseIterator end) {
-#ifdef DEBUG
-  for (; use != end; use++) {
-    MOZ_ASSERT_IF(use->consumer()->isDefinition(),
-                  use->consumer()->toDefinition()->block()->id() < id());
-  }
-#endif
-}
-
-AbortReason MBasicBlock::setBackedge(TempAllocator& alloc, MBasicBlock* pred) {
+bool MBasicBlock::setBackedge(MBasicBlock* pred) {
   // Predecessors must be finished, and at the correct stack depth.
   MOZ_ASSERT(hasLastIns());
   MOZ_ASSERT(pred->hasLastIns());
@@ -1114,25 +931,15 @@ AbortReason MBasicBlock::setBackedge(TempAllocator& alloc, MBasicBlock* pred) {
   // We must be a pending loop header
   MOZ_ASSERT(kind_ == PENDING_LOOP_HEADER);
 
-  bool hadTypeChange = false;
-
   // Add exit definitions to each corresponding phi at the entry.
-  if (!inheritPhisFromBackedge(alloc, pred, &hadTypeChange)) {
-    return AbortReason::Alloc;
-  }
-
-  if (hadTypeChange) {
-    return AbortReason::Disable;
+  if (!inheritPhisFromBackedge(pred)) {
+    return false;
   }
 
   // We are now a loop header proper
   kind_ = LOOP_HEADER;
 
-  if (!predecessors_.append(pred)) {
-    return AbortReason::Alloc;
-  }
-
-  return AbortReason::NoAbort;
+  return predecessors_.append(pred);
 }
 
 bool MBasicBlock::setBackedgeWasm(MBasicBlock* pred, size_t paramCount) {
@@ -1318,39 +1125,7 @@ void MBasicBlock::removePredecessor(MBasicBlock* pred) {
   removePredecessorWithoutPhiOperands(pred, predIndex);
 }
 
-void MBasicBlock::inheritPhis(MBasicBlock* header) {
-  MResumePoint* headerRp = header->entryResumePoint();
-  size_t stackDepth = headerRp->stackDepth();
-  for (size_t slot = 0; slot < stackDepth; slot++) {
-    MDefinition* exitDef = getSlot(slot);
-    MDefinition* loopDef = headerRp->getOperand(slot);
-    if (loopDef->block() != header) {
-      MOZ_ASSERT(loopDef->block()->id() < header->id());
-      MOZ_ASSERT(loopDef == exitDef);
-      continue;
-    }
-
-    // Phis are allocated by NewPendingLoopHeader.
-    MPhi* phi = loopDef->toPhi();
-    MOZ_ASSERT(phi->numOperands() == 2);
-
-    // The entry definition is always the leftmost input to the phi.
-    MDefinition* entryDef = phi->getOperand(0);
-
-    if (entryDef != exitDef) {
-      continue;
-    }
-
-    // If the entryDef is the same as exitDef, then we must propagate the
-    // phi down to this successor. This chance was missed as part of
-    // setBackedge() because exits are not captured in resume points.
-    setSlot(slot, phi);
-  }
-}
-
-bool MBasicBlock::inheritPhisFromBackedge(TempAllocator& alloc,
-                                          MBasicBlock* backedge,
-                                          bool* hadTypeChange) {
+bool MBasicBlock::inheritPhisFromBackedge(MBasicBlock* backedge) {
   // We must be a pending loop header
   MOZ_ASSERT(kind_ == PENDING_LOOP_HEADER);
 
@@ -1386,35 +1161,14 @@ bool MBasicBlock::inheritPhisFromBackedge(TempAllocator& alloc,
       exitDef = entryDef->getOperand(0);
     }
 
-    bool typeChange = false;
-
     if (!entryDef->addInputSlow(exitDef)) {
       return false;
     }
-    if (!entryDef->checkForTypeChange(alloc, exitDef, &typeChange)) {
-      return false;
-    }
-    *hadTypeChange |= typeChange;
   }
 
   return true;
 }
 
-bool MBasicBlock::specializePhis(TempAllocator& alloc) {
-  if (specialized_) {
-    return true;
-  }
-
-  specialized_ = true;
-  for (MPhiIterator iter = phisBegin(); iter != phisEnd(); iter++) {
-    MPhi* phi = *iter;
-    if (!phi->specializeType(alloc)) {
-      return false;
-    }
-  }
-  return true;
-}
-
 MTest* MBasicBlock::immediateDominatorBranch(BranchDirection* pdirection) {
   *pdirection = FALSE_BRANCH;
 
@@ -1444,94 +1198,6 @@ MTest* MBasicBlock::immediateDominatorBranch(BranchDirection* pdirection) {
   return nullptr;
 }
 
-MBasicBlock::BackupPoint::BackupPoint(MBasicBlock* current)
-    : current_(current),
-      lastIns_(current->hasAnyIns() ? *current->rbegin() : nullptr),
-      stackPosition_(current->stackDepth()),
-      slots_()
-#ifdef DEBUG
-      ,
-      lastPhi_(!current->phisEmpty() ? *current->phis_.rbegin() : nullptr),
-      predecessorsCheckSum_(computePredecessorsCheckSum(current)),
-      instructionsCheckSum_(computeInstructionsCheckSum(current)),
-      id_(current->id()),
-      callerResumePoint_(current->callerResumePoint()),
-      entryResumePoint_(current->entryResumePoint())
-#endif
-{
-  // The block is not yet jumping into a block of an inlined function yet.
-  MOZ_ASSERT(current->outerResumePoint_ == nullptr);
-}
-
-bool MBasicBlock::BackupPoint::init(TempAllocator& alloc) {
-  if (!slots_.init(alloc, stackPosition_)) {
-    return false;
-  }
-  for (size_t i = 0, e = stackPosition_; i < e; ++i) {
-    slots_[i] = current_->slots_[i];
-  }
-  return true;
-}
-
-#ifdef DEBUG
-uintptr_t MBasicBlock::BackupPoint::computePredecessorsCheckSum(
-    MBasicBlock* block) {
-  uintptr_t hash = 0;
-  for (size_t i = 0; i < block->numPredecessors(); i++) {
-    MBasicBlock* pred = block->getPredecessor(i);
-    uintptr_t data = reinterpret_cast<uintptr_t>(pred);
-    hash = data + (hash << 6) + (hash << 16) - hash;
-  }
-  return hash;
-}
-
-HashNumber MBasicBlock::BackupPoint::computeInstructionsCheckSum(
-    MBasicBlock* block) {
-  HashNumber h = 0;
-  MOZ_ASSERT_IF(lastIns_, lastIns_->block() == block);
-  for (MInstructionIterator ins = block->begin(); ins != block->end(); ++ins) {
-    h += ins->valueHash();
-    h += h << 10;
-    h ^= h >> 6;
-  }
-  return h;
-}
-#endif
-
-MBasicBlock* MBasicBlock::BackupPoint::restore() {
-  // No extra Phi got added.
-  MOZ_ASSERT((!current_->phisEmpty() ? *current_->phis_.rbegin() : nullptr) ==
-             lastPhi_);
-
-  MOZ_ASSERT_IF(lastIns_, lastIns_->block() == current_);
-  MOZ_ASSERT_IF(lastIns_, !lastIns_->isDiscarded());
-
-  if (!current_->graph().removeSuccessorBlocks(current_)) {
-    return nullptr;
-  }
-
-  MInstructionIterator lastIns(lastIns_ ? ++(current_->begin(lastIns_))
-                                        : current_->begin());
-  current_->discardAllInstructionsStartingAt(lastIns);
-  current_->clearOuterResumePoint();
-
-  MOZ_ASSERT(current_->slots_.length() >= stackPosition_);
-  if (current_->stackPosition_ != stackPosition_) {
-    current_->setStackDepth(stackPosition_);
-  }
-  for (size_t i = 0, e = stackPosition_; i < e; ++i) {
-    current_->slots_[i] = slots_[i];
-  }
-
-  MOZ_ASSERT(current_->id() == id_);
-  MOZ_ASSERT(predecessorsCheckSum_ == computePredecessorsCheckSum(current_));
-  MOZ_ASSERT(instructionsCheckSum_ == computeInstructionsCheckSum(current_));
-  MOZ_ASSERT(current_->callerResumePoint() == callerResumePoint_);
-  MOZ_ASSERT(current_->entryResumePoint() == entryResumePoint_);
-
-  return current_;
-}
-
 void MBasicBlock::dumpStack(GenericPrinter& out) {
 #ifdef DEBUG
   out.printf(" %-3s %-16s %-6s %-10s\n", "#", "name", "copyOf", "first/next");
