@@ -7,8 +7,6 @@
 #ifndef jit_ProcessExecutableMemory_h
 #define jit_ProcessExecutableMemory_h
 
-#include "mozilla/Attributes.h"
-
 #include "util/Poison.h"
 
 namespace js {
@@ -70,15 +68,23 @@ enum class ProtectionSetting {
   Executable,
 };
 
-enum class MustFlushICache { No, Yes };
+/// Whether the instruction cache must be flushed:
+//- No means no flushing will happen.
+//- LocalThreadOnly means only the local thread's icache will be flushed.
+//- AllThreads means all the threads' icaches will be flushed; this must be used
+// when the compiling thread and the executing thread might be different.
 
-extern MOZ_MUST_USE bool ReprotectRegion(void* start, size_t size,
-                                         ProtectionSetting protection,
-                                         MustFlushICache flushICache);
+enum class MustFlushICache { No, LocalThreadOnly, AllThreads };
+
+enum class FlushICacheSpec { LocalThreadOnly, AllThreads };
+
+[[nodiscard]] extern bool ReprotectRegion(void* start, size_t size,
+                                          ProtectionSetting protection,
+                                          MustFlushICache flushICache);
 
 // Functions called at process start-up/shutdown to initialize/release the
 // executable memory region.
-extern MOZ_MUST_USE bool InitProcessExecutableMemory();
+[[nodiscard]] extern bool InitProcessExecutableMemory();
 extern void ReleaseProcessExecutableMemory();
 
 // Allocate/deallocate executable pages.
@@ -100,6 +106,9 @@ extern bool CanLikelyAllocateMoreExecutableMemory();
 // the process allocate executable memory.
 extern size_t LikelyAvailableExecutableMemory();
 
+// Returns whether |p| is stored in the executable code buffer.
+extern bool AddressIsInExecutableMemory(const void* p);
+
 }  // namespace jit
 }  // namespace js
 
