@@ -9,6 +9,7 @@
 using namespace js;
 using namespace js::jit;
 
+#ifdef DEBUG
 bool AllocationIntegrityState::record() {
   // Ignore repeated record() calls.
   if (!instructions.empty()) {
@@ -89,15 +90,14 @@ bool AllocationIntegrityState::record() {
   return true;
 }
 
-bool AllocationIntegrityState::check(bool populateSafepoints) {
+bool AllocationIntegrityState::check() {
   MOZ_ASSERT(!instructions.empty());
 
-#ifdef JS_JITSPEW
+#  ifdef JS_JITSPEW
   if (JitSpewEnabled(JitSpew_RegAlloc)) {
     dump();
   }
-#endif
-#ifdef DEBUG
+#  endif
   for (size_t blockIndex = 0; blockIndex < graph.numBlocks(); blockIndex++) {
     LBlock* block = graph.getBlock(blockIndex);
 
@@ -133,7 +133,6 @@ bool AllocationIntegrityState::check(bool populateSafepoints) {
       }
     }
   }
-#endif
 
   // Check that the register assignment and move groups preserve the original
   // semantics of the virtual registers. Each virtual register has a single
@@ -159,20 +158,16 @@ bool AllocationIntegrityState::check(bool populateSafepoints) {
           }
           uint32_t vreg = info.temps[i].virtualRegister();
           LAllocation* alloc = ins->getTemp(i)->output();
-          if (!checkSafepointAllocation(ins, vreg, *alloc,
-                                        populateSafepoints)) {
-            return false;
-          }
+          checkSafepointAllocation(ins, vreg, *alloc);
         }
-        MOZ_ASSERT_IF(ins->isCall() && !populateSafepoints,
-                      safepoint->liveRegs().emptyFloat() &&
-                          safepoint->liveRegs().emptyGeneral());
+        MOZ_ASSERT_IF(ins->isCall(), safepoint->liveRegs().emptyFloat() &&
+                                         safepoint->liveRegs().emptyGeneral());
       }
 
       size_t inputIndex = 0;
       for (LInstruction::InputIterator alloc(*ins); alloc.more();
-           alloc.next()) {
-        LAllocation oldInput = info.inputs[inputIndex++];
+           inputIndex++, alloc.next()) {
+        LAllocation oldInput = info.inputs[inputIndex];
         if (!oldInput.isUse()) {
           continue;
         }
@@ -180,24 +175,43 @@ bool AllocationIntegrityState::check(bool populateSafepoints) {
         uint32_t vreg = oldInput.toUse()->virtualRegister();
 
         if (safepoint && !oldInput.toUse()->usedAtStart()) {
-          if (!checkSafepointAllocation(ins, vreg, **alloc,
-                                        populateSafepoints)) {
-            return false;
+          checkSafepointAllocation(ins, vreg, **alloc);
+        }
+
+        // Temps must never alias inputs (even at-start uses) unless explicitly
+        // requested.
+        for (size_t i = 0; i < ins->numTemps(); i++) {
+          if (ins->getTemp(i)->isBogusTemp()) {
+            continue;
           }
+          LAllocation* tempAlloc = ins->getTemp(i)->output();
+
+          // Fixed uses and fixed temps are allowed to alias.
+          if (oldInput.toUse()->isFixedRegister() && info.temps[i].isFixed()) {
+            continue;
+          }
+
+          // MUST_REUSE_INPUT temps will alias their input.
+          if (info.temps[i].policy() == LDefinition::MUST_REUSE_INPUT &&
+              info.temps[i].getReusedInput() == inputIndex) {
+            continue;
+          }
+
+          MOZ_ASSERT(!tempAlloc->aliases(**alloc));
         }
 
         // Start checking at the previous instruction, in case this
         // instruction reuses its input register for an output.
         LInstructionReverseIterator riter = block->rbegin(ins);
         riter++;
-        if (!checkIntegrity(block, *riter, vreg, **alloc, populateSafepoints)) {
+        if (!checkIntegrity(block, *riter, vreg, **alloc)) {
           return false;
         }
 
         while (!worklist.empty()) {
           IntegrityItem item = worklist.popCopy();
           if (!checkIntegrity(item.block, *item.block->rbegin(), item.vreg,
-                              item.alloc, populateSafepoints)) {
+                              item.alloc)) {
             return false;
           }
         }
@@ -209,8 +223,8 @@ bool AllocationIntegrityState::check(bool populateSafepoints) {
 }
 
 bool AllocationIntegrityState::checkIntegrity(LBlock* block, LInstruction* ins,
-                                              uint32_t vreg, LAllocation alloc,
-                                              bool populateSafepoints) {
+                                              uint32_t vreg,
+                                              LAllocation alloc) {
   for (LInstructionReverseIterator iter(block->rbegin(ins));
        iter != block->rend(); iter++) {
     ins = *iter;
@@ -257,9 +271,7 @@ bool AllocationIntegrityState::checkIntegrity(LBlock* block, LInstruction* ins,
     }
 
     if (ins->safepoint()) {
-      if (!checkSafepointAllocation(ins, vreg, alloc, populateSafepoints)) {
-        return false;
-      }
+      checkSafepointAllocation(ins, vreg, alloc);
     }
   }
 
@@ -294,29 +306,24 @@ bool AllocationIntegrityState::checkIntegrity(LBlock* block, LInstruction* ins,
   return true;
 }
 
-bool AllocationIntegrityState::checkSafepointAllocation(
-    LInstruction* ins, uint32_t vreg, LAllocation alloc,
-    bool populateSafepoints) {
+void AllocationIntegrityState::checkSafepointAllocation(LInstruction* ins,
+                                                        uint32_t vreg,
+                                                        LAllocation alloc) {
   LSafepoint* safepoint = ins->safepoint();
   MOZ_ASSERT(safepoint);
 
   if (ins->isCall() && alloc.isRegister()) {
-    return true;
+    return;
   }
 
   if (alloc.isRegister()) {
-    AnyRegister reg = alloc.toRegister();
-    if (populateSafepoints) {
-      safepoint->addLiveRegister(reg);
-    }
-
-    MOZ_ASSERT(safepoint->liveRegs().has(reg));
+    MOZ_ASSERT(safepoint->liveRegs().has(alloc.toRegister()));
   }
 
   // The |this| argument slot is implicitly included in all safepoints.
   if (alloc.isArgument() &&
       alloc.toArgument()->index() < THIS_FRAME_ARGSLOT + sizeof(Value)) {
-    return true;
+    return;
   }
 
   LDefinition::Type type = virtualRegisters[vreg]
@@ -325,66 +332,32 @@ bool AllocationIntegrityState::checkSafepointAllocation(
 
   switch (type) {
     case LDefinition::OBJECT:
-      if (populateSafepoints) {
-        JitSpew(JitSpew_RegAlloc, "Safepoint object v%u i%u %s", vreg,
-                ins->id(), alloc.toString().get());
-        if (!safepoint->addGcPointer(alloc)) {
-          return false;
-        }
-      }
       MOZ_ASSERT(safepoint->hasGcPointer(alloc));
       break;
+    case LDefinition::STACKRESULTS:
+      MOZ_ASSERT(safepoint->hasAllGcPointersFromStackArea(alloc));
+      break;
     case LDefinition::SLOTS:
-      if (populateSafepoints) {
-        JitSpew(JitSpew_RegAlloc, "Safepoint slots v%u i%u %s", vreg, ins->id(),
-                alloc.toString().get());
-        if (!safepoint->addSlotsOrElementsPointer(alloc)) {
-          return false;
-        }
-      }
       MOZ_ASSERT(safepoint->hasSlotsOrElementsPointer(alloc));
       break;
-#ifdef JS_NUNBOX32
+#  ifdef JS_NUNBOX32
     // Do not assert that safepoint information for nunbox types is complete,
     // as if a vreg for a value's components are copied in multiple places
     // then the safepoint information may not reflect all copies. All copies
     // of payloads must be reflected, however, for generational GC.
     case LDefinition::TYPE:
-      if (populateSafepoints) {
-        JitSpew(JitSpew_RegAlloc, "Safepoint type v%u i%u %s", vreg, ins->id(),
-                alloc.toString().get());
-        if (!safepoint->addNunboxType(vreg, alloc)) {
-          return false;
-        }
-      }
       break;
     case LDefinition::PAYLOAD:
-      if (populateSafepoints) {
-        JitSpew(JitSpew_RegAlloc, "Safepoint payload v%u i%u %s", vreg,
-                ins->id(), alloc.toString().get());
-        if (!safepoint->addNunboxPayload(vreg, alloc)) {
-          return false;
-        }
-      }
       MOZ_ASSERT(safepoint->hasNunboxPayload(alloc));
       break;
-#else
+#  else
     case LDefinition::BOX:
-      if (populateSafepoints) {
-        JitSpew(JitSpew_RegAlloc, "Safepoint boxed value v%u i%u %s", vreg,
-                ins->id(), alloc.toString().get());
-        if (!safepoint->addBoxedValue(alloc)) {
-          return false;
-        }
-      }
       MOZ_ASSERT(safepoint->hasBoxedValue(alloc));
       break;
-#endif
+#  endif
     default:
       break;
   }
-
-  return true;
 }
 
 bool AllocationIntegrityState::addPredecessor(LBlock* block, uint32_t vreg,
@@ -410,18 +383,22 @@ bool AllocationIntegrityState::addPredecessor(LBlock* block, uint32_t vreg,
 }
 
 void AllocationIntegrityState::dump() {
-#ifdef DEBUG
-  fprintf(stderr, "Register Allocation Integrity State:\n");
+#  ifdef JS_JITSPEW
+  JitSpewCont(JitSpew_RegAlloc, "\n");
+  JitSpew(JitSpew_RegAlloc, "Register Allocation Integrity State:");
 
   for (size_t blockIndex = 0; blockIndex < graph.numBlocks(); blockIndex++) {
     LBlock* block = graph.getBlock(blockIndex);
     MBasicBlock* mir = block->mir();
 
-    fprintf(stderr, "\nBlock %lu", static_cast<unsigned long>(blockIndex));
+    JitSpewHeader(JitSpew_RegAlloc);
+    JitSpewCont(JitSpew_RegAlloc, "  Block %lu",
+                static_cast<unsigned long>(blockIndex));
     for (size_t i = 0; i < mir->numSuccessors(); i++) {
-      fprintf(stderr, " [successor %u]", mir->getSuccessor(i)->id());
+      JitSpewCont(JitSpew_RegAlloc, " [successor %u]",
+                  mir->getSuccessor(i)->id());
     }
-    fprintf(stderr, "\n");
+    JitSpewCont(JitSpew_RegAlloc, "\n");
 
     for (size_t i = 0; i < block->numPhis(); i++) {
       const InstructionInfo& info = blocks[blockIndex].phis[i];
@@ -430,12 +407,14 @@ void AllocationIntegrityState::dump() {
       CodePosition output(block->getPhi(block->numPhis() - 1)->id(),
                           CodePosition::OUTPUT);
 
-      fprintf(stderr, "[%u,%u Phi] [def %s] ", input.bits(), output.bits(),
-              phi->getDef(0)->toString().get());
+      JitSpewHeader(JitSpew_RegAlloc);
+      JitSpewCont(JitSpew_RegAlloc, "    [%u,%u Phi] [def %s] ", input.bits(),
+                  output.bits(), phi->getDef(0)->toString().get());
       for (size_t j = 0; j < phi->numOperands(); j++) {
-        fprintf(stderr, " [use %s]", info.inputs[j].toString().get());
+        JitSpewCont(JitSpew_RegAlloc, " [use %s]",
+                    info.inputs[j].toString().get());
       }
-      fprintf(stderr, "\n");
+      JitSpewCont(JitSpew_RegAlloc, "\n");
     }
 
     for (LInstructionIterator iter = block->begin(); iter != block->end();
@@ -446,46 +425,49 @@ void AllocationIntegrityState::dump() {
       CodePosition input(ins->id(), CodePosition::INPUT);
       CodePosition output(ins->id(), CodePosition::OUTPUT);
 
-      fprintf(stderr, "[");
+      JitSpewHeader(JitSpew_RegAlloc);
+      JitSpewCont(JitSpew_RegAlloc, "    [");
       if (input != CodePosition::MIN) {
-        fprintf(stderr, "%u,%u ", input.bits(), output.bits());
+        JitSpewCont(JitSpew_RegAlloc, "%u,%u ", input.bits(), output.bits());
       }
-      fprintf(stderr, "%s]", ins->opName());
+      JitSpewCont(JitSpew_RegAlloc, "%s]", ins->opName());
 
       if (ins->isMoveGroup()) {
         LMoveGroup* group = ins->toMoveGroup();
         for (int i = group->numMoves() - 1; i >= 0; i--) {
-          fprintf(stderr, " [%s -> %s]",
-                  group->getMove(i).from().toString().get(),
-                  group->getMove(i).to().toString().get());
+          JitSpewCont(JitSpew_RegAlloc, " [%s -> %s]",
+                      group->getMove(i).from().toString().get(),
+                      group->getMove(i).to().toString().get());
         }
-        fprintf(stderr, "\n");
+        JitSpewCont(JitSpew_RegAlloc, "\n");
         continue;
       }
 
       for (size_t i = 0; i < ins->numDefs(); i++) {
-        fprintf(stderr, " [def %s]", ins->getDef(i)->toString().get());
+        JitSpewCont(JitSpew_RegAlloc, " [def %s]",
+                    ins->getDef(i)->toString().get());
       }
 
       for (size_t i = 0; i < ins->numTemps(); i++) {
         LDefinition* temp = ins->getTemp(i);
         if (!temp->isBogusTemp()) {
-          fprintf(stderr, " [temp v%u %s]", info.temps[i].virtualRegister(),
-                  temp->toString().get());
+          JitSpewCont(JitSpew_RegAlloc, " [temp v%u %s]",
+                      info.temps[i].virtualRegister(), temp->toString().get());
         }
       }
 
       size_t index = 0;
       for (LInstruction::InputIterator alloc(*ins); alloc.more();
            alloc.next()) {
-        fprintf(stderr, " [use %s", info.inputs[index++].toString().get());
+        JitSpewCont(JitSpew_RegAlloc, " [use %s",
+                    info.inputs[index++].toString().get());
         if (!alloc->isConstant()) {
-          fprintf(stderr, " %s", alloc->toString().get());
+          JitSpewCont(JitSpew_RegAlloc, " %s", alloc->toString().get());
         }
-        fprintf(stderr, "]");
+        JitSpewCont(JitSpew_RegAlloc, "]");
       }
 
-      fprintf(stderr, "\n");
+      JitSpewCont(JitSpew_RegAlloc, "\n");
     }
   }
 
@@ -514,8 +496,9 @@ void AllocationIntegrityState::dump() {
   }
 
   fprintf(stderr, "\n");
-#endif
+#  endif
 }
+#endif  // DEBUG
 
 const CodePosition CodePosition::MAX(UINT_MAX);
 const CodePosition CodePosition::MIN(0);
@@ -592,71 +575,83 @@ LMoveGroup* RegisterAllocator::getMoveGroupAfter(LInstruction* ins) {
 
 void RegisterAllocator::dumpInstructions() {
 #ifdef JS_JITSPEW
-  fprintf(stderr, "Instructions:\n");
+  JitSpewCont(JitSpew_RegAlloc, "\n");
+  JitSpew(JitSpew_RegAlloc, "Instructions:");
 
   for (size_t blockIndex = 0; blockIndex < graph.numBlocks(); blockIndex++) {
     LBlock* block = graph.getBlock(blockIndex);
     MBasicBlock* mir = block->mir();
 
-    fprintf(stderr, "\nBlock %lu", static_cast<unsigned long>(blockIndex));
+    JitSpewHeader(JitSpew_RegAlloc);
+    JitSpewCont(JitSpew_RegAlloc, "  Block %lu",
+                static_cast<unsigned long>(blockIndex));
     for (size_t i = 0; i < mir->numSuccessors(); i++) {
-      fprintf(stderr, " [successor %u]", mir->getSuccessor(i)->id());
+      JitSpewCont(JitSpew_RegAlloc, " [successor %u]",
+                  mir->getSuccessor(i)->id());
     }
-    fprintf(stderr, "\n");
+    JitSpewCont(JitSpew_RegAlloc, "\n");
 
     for (size_t i = 0; i < block->numPhis(); i++) {
       LPhi* phi = block->getPhi(i);
 
-      fprintf(stderr, "[%u,%u Phi] [def %s]", inputOf(phi).bits(),
-              outputOf(phi).bits(), phi->getDef(0)->toString().get());
+      JitSpewHeader(JitSpew_RegAlloc);
+      JitSpewCont(JitSpew_RegAlloc, "    [%u,%u Phi] [def %s]",
+                  inputOf(phi).bits(), outputOf(phi).bits(),
+                  phi->getDef(0)->toString().get());
       for (size_t j = 0; j < phi->numOperands(); j++) {
-        fprintf(stderr, " [use %s]", phi->getOperand(j)->toString().get());
+        JitSpewCont(JitSpew_RegAlloc, " [use %s]",
+                    phi->getOperand(j)->toString().get());
       }
-      fprintf(stderr, "\n");
+      JitSpewCont(JitSpew_RegAlloc, "\n");
     }
 
     for (LInstructionIterator iter = block->begin(); iter != block->end();
          iter++) {
       LInstruction* ins = *iter;
 
-      fprintf(stderr, "[");
+      JitSpewHeader(JitSpew_RegAlloc);
+      JitSpewCont(JitSpew_RegAlloc, "    [");
       if (ins->id() != 0) {
-        fprintf(stderr, "%u,%u ", inputOf(ins).bits(), outputOf(ins).bits());
+        JitSpewCont(JitSpew_RegAlloc, "%u,%u ", inputOf(ins).bits(),
+                    outputOf(ins).bits());
       }
-      fprintf(stderr, "%s]", ins->opName());
+      JitSpewCont(JitSpew_RegAlloc, "%s]", ins->opName());
 
       if (ins->isMoveGroup()) {
         LMoveGroup* group = ins->toMoveGroup();
         for (int i = group->numMoves() - 1; i >= 0; i--) {
           // Use two printfs, as LAllocation::toString is not reentant.
-          fprintf(stderr, " [%s", group->getMove(i).from().toString().get());
-          fprintf(stderr, " -> %s]", group->getMove(i).to().toString().get());
+          JitSpewCont(JitSpew_RegAlloc, " [%s",
+                      group->getMove(i).from().toString().get());
+          JitSpewCont(JitSpew_RegAlloc, " -> %s]",
+                      group->getMove(i).to().toString().get());
         }
-        fprintf(stderr, "\n");
+        JitSpewCont(JitSpew_RegAlloc, "\n");
         continue;
       }
 
       for (size_t i = 0; i < ins->numDefs(); i++) {
-        fprintf(stderr, " [def %s]", ins->getDef(i)->toString().get());
+        JitSpewCont(JitSpew_RegAlloc, " [def %s]",
+                    ins->getDef(i)->toString().get());
       }
 
       for (size_t i = 0; i < ins->numTemps(); i++) {
         LDefinition* temp = ins->getTemp(i);
         if (!temp->isBogusTemp()) {
-          fprintf(stderr, " [temp %s]", temp->toString().get());
+          JitSpewCont(JitSpew_RegAlloc, " [temp %s]", temp->toString().get());
         }
       }
 
       for (LInstruction::InputIterator alloc(*ins); alloc.more();
            alloc.next()) {
         if (!alloc->isBogus()) {
-          fprintf(stderr, " [use %s]", alloc->toString().get());
+          JitSpewCont(JitSpew_RegAlloc, " [use %s]", alloc->toString().get());
         }
       }
 
-      fprintf(stderr, "\n");
+      JitSpewCont(JitSpew_RegAlloc, "\n");
     }
   }
-  fprintf(stderr, "\n");
+  JitSpewCont(JitSpew_RegAlloc, "\n");
 #endif  // JS_JITSPEW
 }
