@@ -7,18 +7,20 @@
 #ifndef jit_arm_Assembler_arm_h
 #define jit_arm_Assembler_arm_h
 
-#include "mozilla/ArrayUtils.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/MathAlgorithms.h"
 
+#include <algorithm>
+#include <iterator>
+
 #include "jit/arm/Architecture-arm.h"
 #include "jit/arm/disasm/Disasm-arm.h"
 #include "jit/CompactBuffer.h"
-#include "jit/IonCode.h"
-#include "jit/JitRealm.h"
+#include "jit/JitCode.h"
 #include "jit/shared/Assembler-shared.h"
 #include "jit/shared/Disassembler-shared.h"
 #include "jit/shared/IonAssemblerBufferWithConstantPools.h"
+#include "wasm/WasmTypes.h"
 
 union PoolHintPun;
 
@@ -78,8 +80,7 @@ static constexpr Register IntArgReg2 = r2;
 static constexpr Register IntArgReg3 = r3;
 static constexpr Register HeapReg = r10;
 static constexpr Register CallTempNonArgRegs[] = {r5, r6, r7, r8};
-static const uint32_t NumCallTempNonArgRegs =
-    mozilla::ArrayLength(CallTempNonArgRegs);
+static const uint32_t NumCallTempNonArgRegs = std::size(CallTempNonArgRegs);
 
 // These register assignments for the 64-bit atomic ops are frequently too
 // constraining, but we have no way of expressing looser constraints to the
@@ -98,6 +99,8 @@ static constexpr Register64 CmpXchgNew64 =
     Register64(CmpXchgNewHi, CmpXchgNewLo);
 static constexpr Register CmpXchgOutLo = IntArgReg0;
 static constexpr Register CmpXchgOutHi = IntArgReg1;
+static constexpr Register64 CmpXchgOut64 =
+    Register64(CmpXchgOutHi, CmpXchgOutLo);
 
 // Exchange: Any two non-equal odd/even pairs would do for `new` and `out`.
 
@@ -116,8 +119,12 @@ static constexpr Register64 FetchOpVal64 =
     Register64(FetchOpValHi, FetchOpValLo);
 static constexpr Register FetchOpTmpLo = IntArgReg2;
 static constexpr Register FetchOpTmpHi = IntArgReg3;
+static constexpr Register64 FetchOpTmp64 =
+    Register64(FetchOpTmpHi, FetchOpTmpLo);
 static constexpr Register FetchOpOutLo = IntArgReg0;
 static constexpr Register FetchOpOutHi = IntArgReg1;
+static constexpr Register64 FetchOpOut64 =
+    Register64(FetchOpOutHi, FetchOpOutLo);
 
 class ABIArgGenerator {
   unsigned intRegIndex_;
@@ -145,6 +152,7 @@ class ABIArgGenerator {
   ABIArg next(MIRType argType);
   ABIArg& current() { return current_; }
   uint32_t stackBytesConsumedSoFar() const { return stackOffset_; }
+  void increaseStackOffset(uint32_t bytes) { stackOffset_ += bytes; }
 };
 
 bool IsUnaligned(const wasm::MemoryAccessDesc& access);
@@ -156,7 +164,7 @@ static constexpr Register ABINonArgReg2 = r6;
 static constexpr Register ABINonArgReg3 = r7;
 
 // This register may be volatile or nonvolatile. Avoid d15 which is the
-// ScratchDoubleReg.
+// ScratchDoubleReg_.
 static constexpr FloatRegister ABINonArgDoubleReg{FloatRegisters::d8,
                                                   VFPRegister::Double};
 
@@ -183,8 +191,19 @@ static constexpr Register WasmTableCallScratchReg1 = ABINonArgReg1;
 static constexpr Register WasmTableCallSigReg = ABINonArgReg2;
 static constexpr Register WasmTableCallIndexReg = ABINonArgReg3;
 
+// Register used as a scratch along the return path in the fast js -> wasm stub
+// code.  This must not overlap ReturnReg, JSReturnOperand, or WasmTlsReg.  It
+// must be a volatile register.
+static constexpr Register WasmJitEntryReturnScratch = r5;
+
+// Register used to store a reference to an exception thrown by Wasm to an
+// exception handling block. Should not overlap with WasmTlsReg.
+static constexpr Register WasmExceptionReg = ABINonArgReg2;
+
 static constexpr Register PreBarrierReg = r1;
 
+static constexpr Register InterpreterPCReg = r9;
+
 static constexpr Register InvalidReg{Registers::invalid_reg};
 static constexpr FloatRegister InvalidFloatReg;
 
@@ -208,23 +227,27 @@ static constexpr FloatRegister ReturnFloat32Reg = {FloatRegisters::d0,
 static constexpr FloatRegister ReturnDoubleReg = {FloatRegisters::d0,
                                                   VFPRegister::Double};
 static constexpr FloatRegister ReturnSimd128Reg = InvalidFloatReg;
-static constexpr FloatRegister ScratchFloat32Reg = {FloatRegisters::s30,
-                                                    VFPRegister::Single};
-static constexpr FloatRegister ScratchDoubleReg = {FloatRegisters::d15,
-                                                   VFPRegister::Double};
+static constexpr FloatRegister ScratchFloat32Reg_ = {FloatRegisters::s30,
+                                                     VFPRegister::Single};
+static constexpr FloatRegister ScratchDoubleReg_ = {FloatRegisters::d15,
+                                                    VFPRegister::Double};
 static constexpr FloatRegister ScratchSimd128Reg = InvalidFloatReg;
 static constexpr FloatRegister ScratchUIntReg = {FloatRegisters::d15,
                                                  VFPRegister::UInt};
 static constexpr FloatRegister ScratchIntReg = {FloatRegisters::d15,
                                                 VFPRegister::Int};
 
+// Do not reference ScratchFloat32Reg_ directly, use ScratchFloat32Scope
+// instead.
 struct ScratchFloat32Scope : public AutoFloatRegisterScope {
   explicit ScratchFloat32Scope(MacroAssembler& masm)
-      : AutoFloatRegisterScope(masm, ScratchFloat32Reg) {}
+      : AutoFloatRegisterScope(masm, ScratchFloat32Reg_) {}
 };
+
+// Do not reference ScratchDoubleReg_ directly, use ScratchDoubleScope instead.
 struct ScratchDoubleScope : public AutoFloatRegisterScope {
   explicit ScratchDoubleScope(MacroAssembler& masm)
-      : AutoFloatRegisterScope(masm, ScratchDoubleReg) {}
+      : AutoFloatRegisterScope(masm, ScratchDoubleReg_) {}
 };
 
 // Registerd used in RegExpMatcher instruction (do not use JSReturnOperand).
@@ -268,11 +291,6 @@ static_assert(JitStackAlignment % sizeof(Value) == 0 &&
                   JitStackValueAlignment >= 1,
               "Stack alignment should be a non-zero multiple of sizeof(Value)");
 
-// This boolean indicates whether we support SIMD instructions flavoured for
-// this architecture or not. Rather than a method in the LIRGenerator, it is
-// here such that it is accessible from the entire codebase. Once full support
-// for SIMD is reached on all tier-1 platforms, this constant can be deleted.
-static constexpr bool SupportsSimd = false;
 static constexpr uint32_t SimdMemoryAlignment = 8;
 
 static_assert(CodeAlignment % SimdMemoryAlignment == 0,
@@ -291,14 +309,10 @@ static_assert(JitStackAlignment % SimdMemoryAlignment == 0,
 static const uint32_t WasmStackAlignment = SimdMemoryAlignment;
 static const uint32_t WasmTrapInstructionLength = 4;
 
-// Does this architecture support SIMD conversions between Uint32x4 and
-// Float32x4?
-static constexpr bool SupportsUint32x4FloatConversions = false;
-
-// Does this architecture support comparisons of unsigned integer vectors?
-static constexpr bool SupportsUint8x16Compares = false;
-static constexpr bool SupportsUint16x8Compares = false;
-static constexpr bool SupportsUint32x4Compares = false;
+// See comments in wasm::GenerateFunctionPrologue.  The difference between these
+// is the size of the largest callable prologue on the platform.
+static constexpr uint32_t WasmCheckedCallEntryOffset = 0u;
+static constexpr uint32_t WasmCheckedTailEntryOffset = 12u;
 
 static const Scale ScalePointer = TimesFour;
 
@@ -1039,8 +1053,6 @@ inline Imm32 Imm64::firstHalf() const { return low(); }
 
 inline Imm32 Imm64::secondHalf() const { return hi(); }
 
-void PatchJump(CodeLocationJump& jump_, CodeLocationLabel label);
-
 class InstructionIterator {
  private:
   Instruction* inst_;
@@ -1193,8 +1205,6 @@ class Assembler : public AssemblerShared {
 
  public:
   void resetCounter();
-  uint32_t actualIndex(uint32_t) const;
-  static uint8_t* PatchableJumpAddress(JitCode* code, uint32_t index);
   static uint32_t NopFill;
   static uint32_t GetNopFill();
   static uint32_t AsmPoolMaxOffset;
@@ -1259,11 +1269,9 @@ class Assembler : public AssemblerShared {
 
   static DoubleCondition InvertCondition(DoubleCondition cond);
 
-  void writeRelocation(BufferOffset src) {
-    jumpRelocations_.writeUnsigned(src.getOffset());
-  }
-
   void writeDataRelocation(BufferOffset offset, ImmGCPtr ptr) {
+    // Raw GC pointer relocations and Value relocations both end up in
+    // Assembler::TraceDataRelocations.
     if (ptr.value) {
       if (gc::IsInsideNursery(ptr.value)) {
         embedsNurseryPointers_ = true;
@@ -1296,7 +1304,7 @@ class Assembler : public AssemblerShared {
 #endif
   }
 
-  static const Register getStackPointer() { return StackPointer; }
+  Register getStackPointer() const { return StackPointer; }
 
  private:
   bool isFinished;
@@ -1384,6 +1392,10 @@ class Assembler : public AssemblerShared {
                       SBit s = LeaveCC, Condition c = Always);
   BufferOffset as_orr(Register dest, Register src1, Operand2 op2,
                       SBit s = LeaveCC, Condition c = Always);
+  // Reverse byte operations:
+  BufferOffset as_rev(Register dest, Register src, Condition c = Always);
+  BufferOffset as_rev16(Register dest, Register src, Condition c = Always);
+  BufferOffset as_revsh(Register dest, Register src, Condition c = Always);
   // Mathematical operations:
   BufferOffset as_adc(Register dest, Register src1, Operand2 op2,
                       SBit s = LeaveCC, Condition c = Always);
@@ -1471,11 +1483,6 @@ class Assembler : public AssemblerShared {
   // Load a 32 bit immediate from a pool into a register.
   BufferOffset as_Imm32Pool(Register dest, uint32_t value,
                             Condition c = Always);
-  // Make a patchable jump that can target the entire 32 bit address space.
-  BufferOffset as_BranchPool(uint32_t value, RepatchLabel* label,
-                             const LabelDoc& documentation,
-                             ARMBuffer::PoolEntry* pe = nullptr,
-                             Condition c = Always);
 
   // Load a 64 bit floating point immediate from a pool into a register.
   BufferOffset as_FImm64Pool(VFPRegister dest, double value,
@@ -1638,6 +1645,12 @@ class Assembler : public AssemblerShared {
   BufferOffset as_vdtm(LoadStore st, Register rn, VFPRegister vd, int length,
                        /* also has update conditions */ Condition c = Always);
 
+  // vldr/vstr variants that handle unaligned accesses.  These encode as NEON
+  // single-element instructions and can only be used if NEON is available.
+  // Here, vd must be tagged as a float or double register.
+  BufferOffset as_vldr_unaligned(VFPRegister vd, Register rn);
+  BufferOffset as_vstr_unaligned(VFPRegister vd, Register rn);
+
   BufferOffset as_vimm(VFPRegister vd, VFPImm imm, Condition c = Always);
 
   BufferOffset as_vmrs(Register r, Condition c = Always);
@@ -1646,7 +1659,6 @@ class Assembler : public AssemblerShared {
   // Label operations.
   bool nextLink(BufferOffset b, BufferOffset* next);
   void bind(Label* label, BufferOffset boff = BufferOffset());
-  void bind(RepatchLabel* label);
   uint32_t currentOffset() { return nextOffset().getOffset(); }
   void retarget(Label* label, Label* target);
   // I'm going to pretend this doesn't exist for now.
@@ -1674,7 +1686,7 @@ class Assembler : public AssemblerShared {
 
   static bool SupportsFloatingPoint() { return HasVFP(); }
   static bool SupportsUnalignedAccesses() { return HasARMv7(); }
-  static bool SupportsSimd() { return js::jit::SupportsSimd; }
+  static bool SupportsFastUnalignedAccesses() { return false; }
 
   static bool HasRoundInstruction(RoundingMode mode) { return false; }
 
@@ -1682,7 +1694,7 @@ class Assembler : public AssemblerShared {
   void addPendingJump(BufferOffset src, ImmPtr target, RelocationKind kind) {
     enoughMemory_ &= jumps_.append(RelativePatch(target.value, kind));
     if (kind == RelocationKind::JITCODE) {
-      writeRelocation(src);
+      jumpRelocations_.writeUnsigned(src.getOffset());
     }
   }
 
@@ -1703,7 +1715,7 @@ class Assembler : public AssemblerShared {
 
   // Copy the assembly code to the given buffer, and perform any pending
   // relocations relying on the target address.
-  void executableCopy(uint8_t* buffer, bool flushICache = true);
+  void executableCopy(uint8_t* buffer);
 
   // Actual assembly emitting functions.
 
@@ -1770,8 +1782,8 @@ class Assembler : public AssemblerShared {
     dtmDelta = dtmDelta ? dtmDelta : 1;
     // The operand for the vstr/vldr instruction is the lowest register in the
     // range.
-    int low = Min(dtmLastReg, vdtmFirstReg);
-    int high = Max(dtmLastReg, vdtmFirstReg);
+    int low = std::min(dtmLastReg, vdtmFirstReg);
+    int high = std::max(dtmLastReg, vdtmFirstReg);
     // Fencepost problem.
     int len = high - low + 1;
     // vdtm can only transfer 16 registers at once.  If we need to transfer
@@ -1783,7 +1795,7 @@ class Assembler : public AssemblerShared {
     int adjustHigh = dtmLoadStore == IsStore ? -1 : 0;
     while (len > 0) {
       // Limit the instruction to 16 registers.
-      int curLen = Min(len, 16);
+      int curLen = std::min(len, 16);
       // If it is a store, we want to start at the high end and move down
       // (e.g. vpush d16-d31; vpush d0-d15).
       int curStart = (dtmLoadStore == IsStore) ? high - curLen + 1 : low;
@@ -1841,14 +1853,6 @@ class Assembler : public AssemblerShared {
   void leaveNoPool();
   void enterNoNops();
   void leaveNoNops();
-  // This should return a BOffImm, but we didn't want to require everyplace
-  // that used the AssemblerBuffer to make that class.
-  static ptrdiff_t GetBranchOffset(const Instruction* i);
-  static void RetargetNearBranch(Instruction* i, int offset, Condition cond,
-                                 bool final = true);
-  static void RetargetNearBranch(Instruction* i, int offset, bool final = true);
-  static void RetargetFarBranch(Instruction* i, uint8_t** slot, uint8_t* dest,
-                                Condition cond);
 
   static void WritePoolHeader(uint8_t* start, Pool* p, bool isNatural);
   static void WritePoolGuard(BufferOffset branch, Instruction* inst,
@@ -1940,7 +1944,7 @@ class Instruction {
 };  // Instruction
 
 // Make sure that it is the right size.
-JS_STATIC_ASSERT(sizeof(Instruction) == 4);
+static_assert(sizeof(Instruction) == 4);
 
 inline void InstructionIterator::advanceRaw(ptrdiff_t instructions) {
   inst_ = inst_ + instructions;
@@ -1961,7 +1965,7 @@ class InstDTR : public Instruction {
   static bool IsTHIS(const Instruction& i);
   static InstDTR* AsTHIS(const Instruction& i);
 };
-JS_STATIC_ASSERT(sizeof(InstDTR) == sizeof(Instruction));
+static_assert(sizeof(InstDTR) == sizeof(Instruction));
 
 class InstLDR : public InstDTR {
  public:
@@ -1990,7 +1994,7 @@ class InstLDR : public InstDTR {
     return (uint32_t*)raw() + offset + 2;
   }
 };
-JS_STATIC_ASSERT(sizeof(InstDTR) == sizeof(InstLDR));
+static_assert(sizeof(InstDTR) == sizeof(InstLDR));
 
 class InstNOP : public Instruction {
  public:
@@ -2022,7 +2026,7 @@ class InstBranchReg : public Instruction {
   // Make sure we are branching to a pre-known register
   bool checkDest(Register dest);
 };
-JS_STATIC_ASSERT(sizeof(InstBranchReg) == sizeof(Instruction));
+static_assert(sizeof(InstBranchReg) == sizeof(Instruction));
 
 // Branching to an immediate offset, or calling an immediate offset
 class InstBranchImm : public Instruction {
@@ -2040,7 +2044,7 @@ class InstBranchImm : public Instruction {
 
   void extractImm(BOffImm* dest);
 };
-JS_STATIC_ASSERT(sizeof(InstBranchImm) == sizeof(Instruction));
+static_assert(sizeof(InstBranchImm) == sizeof(Instruction));
 
 // Very specific branching instructions.
 class InstBXReg : public InstBranchReg {
@@ -2094,7 +2098,7 @@ class InstMovWT : public Instruction {
   static bool IsTHIS(Instruction& i);
   static InstMovWT* AsTHIS(Instruction& i);
 };
-JS_STATIC_ASSERT(sizeof(InstMovWT) == sizeof(Instruction));
+static_assert(sizeof(InstMovWT) == sizeof(Instruction));
 
 class InstMovW : public InstMovWT {
  public:
@@ -2204,16 +2208,6 @@ static inline bool GetTempRegForIntArg(uint32_t usedIntArgs,
   return true;
 }
 
-#if !defined(JS_CODEGEN_ARM_HARDFP) || defined(JS_SIMULATOR_ARM)
-
-static inline uint32_t GetArgStackDisp(uint32_t arg) {
-  MOZ_ASSERT(!UseHardFpABI());
-  MOZ_ASSERT(arg >= NumIntArgRegs);
-  return (arg - NumIntArgRegs) * sizeof(intptr_t);
-}
-
-#endif
-
 #if defined(JS_CODEGEN_ARM_HARDFP) || defined(JS_SIMULATOR_ARM)
 
 static inline bool GetFloat32ArgReg(uint32_t usedIntArgs,
@@ -2237,47 +2231,6 @@ static inline bool GetDoubleArgReg(uint32_t usedIntArgs, uint32_t usedFloatArgs,
   return true;
 }
 
-static inline uint32_t GetIntArgStackDisp(uint32_t usedIntArgs,
-                                          uint32_t usedFloatArgs,
-                                          uint32_t* padding) {
-  MOZ_ASSERT(UseHardFpABI());
-  MOZ_ASSERT(usedIntArgs >= NumIntArgRegs);
-  uint32_t doubleSlots =
-      Max(0, (int32_t)usedFloatArgs - (int32_t)NumFloatArgRegs);
-  doubleSlots *= 2;
-  int intSlots = usedIntArgs - NumIntArgRegs;
-  return (intSlots + doubleSlots + *padding) * sizeof(intptr_t);
-}
-
-static inline uint32_t GetFloat32ArgStackDisp(uint32_t usedIntArgs,
-                                              uint32_t usedFloatArgs,
-                                              uint32_t* padding) {
-  MOZ_ASSERT(UseHardFpABI());
-  MOZ_ASSERT(usedFloatArgs >= NumFloatArgRegs);
-  uint32_t intSlots = 0;
-  if (usedIntArgs > NumIntArgRegs) {
-    intSlots = usedIntArgs - NumIntArgRegs;
-  }
-  uint32_t float32Slots = usedFloatArgs - NumFloatArgRegs;
-  return (intSlots + float32Slots + *padding) * sizeof(intptr_t);
-}
-
-static inline uint32_t GetDoubleArgStackDisp(uint32_t usedIntArgs,
-                                             uint32_t usedFloatArgs,
-                                             uint32_t* padding) {
-  MOZ_ASSERT(UseHardFpABI());
-  MOZ_ASSERT(usedFloatArgs >= NumFloatArgRegs);
-  uint32_t intSlots = 0;
-  if (usedIntArgs > NumIntArgRegs) {
-    intSlots = usedIntArgs - NumIntArgRegs;
-    // Update the amount of padding required.
-    *padding += (*padding + usedIntArgs) % 2;
-  }
-  uint32_t doubleSlots = usedFloatArgs - NumFloatArgRegs;
-  doubleSlots *= 2;
-  return (intSlots + doubleSlots + *padding) * sizeof(intptr_t);
-}
-
 #endif
 
 class DoubleEncoder {
