@@ -7,8 +7,8 @@
 #ifndef jit_arm64_SharedICHelpers_arm64_h
 #define jit_arm64_SharedICHelpers_arm64_h
 
-#include "jit/BaselineFrame.h"
 #include "jit/BaselineIC.h"
+#include "jit/JitFrames.h"
 #include "jit/MacroAssembler.h"
 #include "jit/SharedICRegisters.h"
 
@@ -27,12 +27,8 @@ inline void EmitRepushTailCallReg(MacroAssembler& masm) {
   // No-op on ARM because link register is always holding the return address.
 }
 
-inline void EmitCallIC(MacroAssembler& masm, const ICEntry* entry,
-                       CodeOffset* callOffset) {
-  // Load stub pointer into ICStubReg.
-  masm.loadPtr(AbsoluteAddress(entry).offset(ICEntry::offsetOfFirstStub()),
-               ICStubReg);
-
+inline void EmitCallIC(MacroAssembler& masm, CodeOffset* callOffset) {
+  // The stub pointer must already be in ICStubReg.
   // Load stubcode pointer from the ICStub.
   // R2 won't be active when we call ICs, so we can use r0.
   static_assert(R2 == ValueOperand(r0));
@@ -43,22 +39,6 @@ inline void EmitCallIC(MacroAssembler& masm, const ICEntry* entry,
   *callOffset = CodeOffset(masm.currentOffset());
 }
 
-inline void EmitEnterTypeMonitorIC(
-    MacroAssembler& masm,
-    size_t monitorStubOffset = ICMonitoredStub::offsetOfFirstMonitorStub()) {
-  // This is expected to be called from within an IC, when ICStubReg is
-  // properly initialized to point to the stub.
-  masm.loadPtr(Address(ICStubReg, (uint32_t)monitorStubOffset), ICStubReg);
-
-  // Load stubcode pointer from BaselineStubEntry.
-  // R2 won't be active when we call ICs, so we can use r0.
-  static_assert(R2 == ValueOperand(r0));
-  masm.loadPtr(Address(ICStubReg, ICStub::offsetOfStubCode()), r0);
-
-  // Jump to the stubcode.
-  masm.Br(x0);
-}
-
 inline void EmitReturnFromIC(MacroAssembler& masm) {
   masm.abiret();  // Defaults to lr.
 }
@@ -98,7 +78,7 @@ inline void EmitPreBarrier(MacroAssembler& masm, const AddrType& addr,
 
 inline void EmitStubGuardFailure(MacroAssembler& masm) {
   // Load next stub into ICStubReg.
-  masm.loadPtr(Address(ICStubReg, ICStub::offsetOfNext()), ICStubReg);
+  masm.loadPtr(Address(ICStubReg, ICCacheIRStub::offsetOfNext()), ICStubReg);
 
   // Return address is already loaded, just jump to the next stubcode.
   masm.jump(Address(ICStubReg, ICStub::offsetOfStubCode()));
