@@ -9,11 +9,8 @@
 #include "mozilla/DebugOnly.h"
 #include "mozilla/MathAlgorithms.h"
 
-#include "jsutil.h"
-
 #include "gc/Marking.h"
 #include "jit/ExecutableAllocator.h"
-#include "jit/JitRealm.h"
 #include "vm/Realm.h"
 
 using mozilla::DebugOnly;
@@ -98,13 +95,6 @@ bool AssemblerMIPSShared::swapBuffer(wasm::Bytes& bytes) {
   return true;
 }
 
-uint32_t AssemblerMIPSShared::actualIndex(uint32_t idx_) const { return idx_; }
-
-uint8_t* AssemblerMIPSShared::PatchableJumpAddress(JitCode* code,
-                                                   uint32_t pe_) {
-  return code->raw() + pe_;
-}
-
 void AssemblerMIPSShared::copyJumpRelocationTable(uint8_t* dest) {
   if (jumpRelocations_.length()) {
     memcpy(dest, jumpRelocations_.buffer(), jumpRelocations_.length());
@@ -370,10 +360,16 @@ InstImm AssemblerMIPSShared::getBranchCode(Register s, Condition c) {
 InstImm AssemblerMIPSShared::getBranchCode(FloatTestKind testKind,
                                            FPConditionBit fcc) {
   MOZ_ASSERT(!(fcc && FccMask));
+#ifdef MIPSR6
+  RSField rsField = ((testKind == TestForTrue ? rs_t : rs_f));
+
+  return InstImm(op_cop1, rsField, FloatRegisters::f24 << 16, BOffImm16(0));
+#else
   uint32_t rtField = ((testKind == TestForTrue ? 1 : 0) | (fcc << FccShift))
                      << RTShift;
 
   return InstImm(op_cop1, rs_bc1, rtField, BOffImm16(0));
+#endif
 }
 
 BufferOffset AssemblerMIPSShared::as_j(JOffImm26 off) {
@@ -389,8 +385,13 @@ BufferOffset AssemblerMIPSShared::as_jal(JOffImm26 off) {
 
 BufferOffset AssemblerMIPSShared::as_jr(Register rs) {
   spew("jr     %3s", rs.name());
+#ifdef MIPSR6
+  BufferOffset bo =
+      writeInst(InstReg(op_special, rs, zero, zero, ff_jalr).encode());
+#else
   BufferOffset bo =
       writeInst(InstReg(op_special, rs, zero, zero, ff_jr).encode());
+#endif
   return bo;
 }
 BufferOffset AssemblerMIPSShared::as_jalr(Register rs) {
@@ -464,25 +465,119 @@ BufferOffset AssemblerMIPSShared::as_div(Register rs, Register rt) {
   return writeInst(InstReg(op_special, rs, rt, ff_div).encode());
 }
 
+BufferOffset AssemblerMIPSShared::as_div(Register rd, Register rs,
+                                         Register rt) {
+  spew("div    %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x2, ff_div).encode());
+}
+
 BufferOffset AssemblerMIPSShared::as_divu(Register rs, Register rt) {
   spew("divu   %3s,%3s", rs.name(), rt.name());
   return writeInst(InstReg(op_special, rs, rt, ff_divu).encode());
 }
 
+BufferOffset AssemblerMIPSShared::as_divu(Register rd, Register rs,
+                                          Register rt) {
+  spew("divu    %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x2, ff_divu).encode());
+}
+
+BufferOffset AssemblerMIPSShared::as_mod(Register rd, Register rs,
+                                         Register rt) {
+  spew("mod    %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x3, ff_mod).encode());
+}
+
+BufferOffset AssemblerMIPSShared::as_modu(Register rd, Register rs,
+                                          Register rt) {
+  spew("modu   %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x3, ff_modu).encode());
+}
+
 BufferOffset AssemblerMIPSShared::as_ddiv(Register rs, Register rt) {
   spew("ddiv   %3s,%3s", rs.name(), rt.name());
   return writeInst(InstReg(op_special, rs, rt, ff_ddiv).encode());
 }
 
+BufferOffset AssemblerMIPSShared::as_ddiv(Register rd, Register rs,
+                                          Register rt) {
+  spew("ddiv   %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x2, ff_ddiv).encode());
+}
+
 BufferOffset AssemblerMIPSShared::as_ddivu(Register rs, Register rt) {
   spew("ddivu  %3s,%3s", rs.name(), rt.name());
   return writeInst(InstReg(op_special, rs, rt, ff_ddivu).encode());
 }
 
+BufferOffset AssemblerMIPSShared::as_ddivu(Register rd, Register rs,
+                                           Register rt) {
+  spew("ddivu  %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x2, ff_ddivu).encode());
+}
+
 BufferOffset AssemblerMIPSShared::as_mul(Register rd, Register rs,
                                          Register rt) {
   spew("mul    %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+#ifdef MIPSR6
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x2, ff_mul).encode());
+#else
   return writeInst(InstReg(op_special2, rs, rt, rd, ff_mul).encode());
+#endif
+}
+
+BufferOffset AssemblerMIPSShared::as_muh(Register rd, Register rs,
+                                         Register rt) {
+  spew("muh  %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x3, ff_muh).encode());
+}
+
+BufferOffset AssemblerMIPSShared::as_mulu(Register rd, Register rs,
+                                          Register rt) {
+  spew("mulu %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x2, ff_mulu).encode());
+}
+
+BufferOffset AssemblerMIPSShared::as_muhu(Register rd, Register rs,
+                                          Register rt) {
+  spew("muhu %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x3, ff_muhu).encode());
+}
+
+BufferOffset AssemblerMIPSShared::as_dmul(Register rd, Register rs,
+                                          Register rt) {
+  spew("dmul   %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x2, ff_dmul).encode());
+}
+
+BufferOffset AssemblerMIPSShared::as_dmuh(Register rd, Register rs,
+                                          Register rt) {
+  spew("dmuh   %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x3, ff_dmuh).encode());
+}
+
+BufferOffset AssemblerMIPSShared::as_dmulu(Register rd, Register rt,
+                                           Register rs) {
+  spew("dmulu   %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x2, ff_dmulu).encode());
+}
+
+BufferOffset AssemblerMIPSShared::as_dmuhu(Register rd, Register rt,
+                                           Register rs) {
+  spew("dmuhu   %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x3, ff_dmuhu).encode());
+}
+
+BufferOffset AssemblerMIPSShared::as_dmod(Register rd, Register rs,
+                                          Register rt) {
+  spew("dmod    %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x3, ff_dmod).encode());
+}
+
+BufferOffset AssemblerMIPSShared::as_dmodu(Register rd, Register rs,
+                                           Register rt) {
+  spew("dmodu    %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x3, ff_dmodu).encode());
 }
 
 BufferOffset AssemblerMIPSShared::as_madd(Register rs, Register rt) {
@@ -685,13 +780,21 @@ BufferOffset AssemblerMIPSShared::as_lwr(Register rd, Register rs,
 
 BufferOffset AssemblerMIPSShared::as_ll(Register rd, Register rs, int16_t off) {
   spew("ll     %3s, (0x%x)%2s", rd.name(), off, rs.name());
+#ifdef MIPSR6
+  return writeInst(InstReg(op_special3, rs, rd, ff_ll).encode());
+#else
   return writeInst(InstImm(op_ll, rs, rd, Imm16(off)).encode());
+#endif
 }
 
 BufferOffset AssemblerMIPSShared::as_lld(Register rd, Register rs,
                                          int16_t off) {
   spew("lld     %3s, (0x%x)%2s", rd.name(), off, rs.name());
+#ifdef MIPSR6
+  return writeInst(InstReg(op_special3, rs, rd, ff_lld).encode());
+#else
   return writeInst(InstImm(op_lld, rs, rd, Imm16(off)).encode());
+#endif
 }
 
 BufferOffset AssemblerMIPSShared::as_ld(Register rd, Register rs, int16_t off) {
@@ -740,13 +843,21 @@ BufferOffset AssemblerMIPSShared::as_swr(Register rd, Register rs,
 
 BufferOffset AssemblerMIPSShared::as_sc(Register rd, Register rs, int16_t off) {
   spew("sc     %3s, (0x%x)%2s", rd.name(), off, rs.name());
+#ifdef MIPSR6
+  return writeInst(InstReg(op_special3, rs, rd, ff_sc).encode());
+#else
   return writeInst(InstImm(op_sc, rs, rd, Imm16(off)).encode());
+#endif
 }
 
 BufferOffset AssemblerMIPSShared::as_scd(Register rd, Register rs,
                                          int16_t off) {
+#ifdef MIPSR6
+  return writeInst(InstReg(op_special3, rs, rd, ff_scd).encode());
+#else
   spew("scd     %3s, (0x%x)%2s", rd.name(), off, rs.name());
   return writeInst(InstImm(op_scd, rs, rd, Imm16(off)).encode());
+#endif
 }
 
 BufferOffset AssemblerMIPSShared::as_sd(Register rd, Register rs, int16_t off) {
@@ -766,6 +877,18 @@ BufferOffset AssemblerMIPSShared::as_sdr(Register rd, Register rs,
   return writeInst(InstImm(op_sdr, rs, rd, Imm16(off)).encode());
 }
 
+BufferOffset AssemblerMIPSShared::as_seleqz(Register rd, Register rs,
+                                            Register rt) {
+  spew("seleqz    %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x0, ff_seleqz).encode());
+}
+
+BufferOffset AssemblerMIPSShared::as_selnez(Register rd, Register rs,
+                                            Register rt) {
+  spew("selnez    %3s,%3s,%3s", rd.name(), rs.name(), rt.name());
+  return writeInst(InstReg(op_special, rs, rt, rd, 0x0, ff_selnez).encode());
+}
+
 BufferOffset AssemblerMIPSShared::as_gslbx(Register rd, Register rs,
                                            Register ri, int16_t off) {
   MOZ_ASSERT(Imm8::IsInSignedRange(off));
@@ -905,12 +1028,35 @@ BufferOffset AssemblerMIPSShared::as_movf(Register rd, Register rs,
 // Bit twiddling.
 BufferOffset AssemblerMIPSShared::as_clz(Register rd, Register rs) {
   spew("clz    %3s,%3s", rd.name(), rs.name());
+#ifdef MIPSR6
+  return writeInst(InstReg(op_special, rs, 0x0, rd, 0x1, ff_clz).encode());
+#else
   return writeInst(InstReg(op_special2, rs, rd, rd, ff_clz).encode());
+#endif
 }
 
 BufferOffset AssemblerMIPSShared::as_dclz(Register rd, Register rs) {
   spew("dclz   %3s,%3s", rd.name(), rs.name());
+#ifdef MIPSR6
+  return writeInst(InstReg(op_special, rs, 0x0, rd, 0x1, ff_dclz).encode());
+#else
   return writeInst(InstReg(op_special2, rs, rd, rd, ff_dclz).encode());
+#endif
+}
+
+BufferOffset AssemblerMIPSShared::as_wsbh(Register rd, Register rt) {
+  spew("wsbh   %3s,%3s", rd.name(), rt.name());
+  return writeInst(InstReg(op_special3, zero, rt, rd, 0x2, ff_bshfl).encode());
+}
+
+BufferOffset AssemblerMIPSShared::as_dsbh(Register rd, Register rt) {
+  spew("dsbh   %3s,%3s", rd.name(), rt.name());
+  return writeInst(InstReg(op_special3, zero, rt, rd, 0x2, ff_dbshfl).encode());
+}
+
+BufferOffset AssemblerMIPSShared::as_dshd(Register rd, Register rt) {
+  spew("dshd   %3s,%3s", rd.name(), rt.name());
+  return writeInst(InstReg(op_special3, zero, rt, rd, 0x5, ff_dbshfl).encode());
 }
 
 BufferOffset AssemblerMIPSShared::as_ins(Register rt, Register rs, uint16_t pos,
@@ -1394,12 +1540,24 @@ BufferOffset AssemblerMIPSShared::as_cf(FloatFormat fmt, FloatRegister fs,
                                         FloatRegister ft, FPConditionBit fcc) {
   if (fmt == DoubleFloat) {
     spew("c.f.d  FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_d_r6, ft, fs, FloatRegisters::f24, ff_c_f_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_d, ft, fs, fcc << FccShift, ff_c_f_fmt).encode());
+#endif
   } else {
     spew("c.f.s  FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_s_r6, ft, fs, FloatRegisters::f24, ff_c_f_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_s, ft, fs, fcc << FccShift, ff_c_f_fmt).encode());
+#endif
   }
 }
 
@@ -1407,12 +1565,24 @@ BufferOffset AssemblerMIPSShared::as_cun(FloatFormat fmt, FloatRegister fs,
                                          FloatRegister ft, FPConditionBit fcc) {
   if (fmt == DoubleFloat) {
     spew("c.un.d FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_d_r6, ft, fs, FloatRegisters::f24, ff_c_un_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_d, ft, fs, fcc << FccShift, ff_c_un_fmt).encode());
+#endif
   } else {
     spew("c.un.s FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_s_r6, ft, fs, FloatRegisters::f24, ff_c_un_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_s, ft, fs, fcc << FccShift, ff_c_un_fmt).encode());
+#endif
   }
 }
 
@@ -1420,12 +1590,24 @@ BufferOffset AssemblerMIPSShared::as_ceq(FloatFormat fmt, FloatRegister fs,
                                          FloatRegister ft, FPConditionBit fcc) {
   if (fmt == DoubleFloat) {
     spew("c.eq.d FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_d_r6, ft, fs, FloatRegisters::f24, ff_c_eq_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_d, ft, fs, fcc << FccShift, ff_c_eq_fmt).encode());
+#endif
   } else {
     spew("c.eq.s FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_s_r6, ft, fs, FloatRegisters::f24, ff_c_eq_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_s, ft, fs, fcc << FccShift, ff_c_eq_fmt).encode());
+#endif
   }
 }
 
@@ -1434,12 +1616,24 @@ BufferOffset AssemblerMIPSShared::as_cueq(FloatFormat fmt, FloatRegister fs,
                                           FPConditionBit fcc) {
   if (fmt == DoubleFloat) {
     spew("c.ueq.d FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_d_r6, ft, fs, FloatRegisters::f24, ff_c_ueq_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_d, ft, fs, fcc << FccShift, ff_c_ueq_fmt).encode());
+#endif
   } else {
     spew("c.ueq.s FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_s_r6, ft, fs, FloatRegisters::f24, ff_c_ueq_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_s, ft, fs, fcc << FccShift, ff_c_ueq_fmt).encode());
+#endif
   }
 }
 
@@ -1448,12 +1642,24 @@ BufferOffset AssemblerMIPSShared::as_colt(FloatFormat fmt, FloatRegister fs,
                                           FPConditionBit fcc) {
   if (fmt == DoubleFloat) {
     spew("c.olt.d FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_d_r6, ft, fs, FloatRegisters::f24, ff_c_olt_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_d, ft, fs, fcc << FccShift, ff_c_olt_fmt).encode());
+#endif
   } else {
     spew("c.olt.s FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_s_r6, ft, fs, FloatRegisters::f24, ff_c_olt_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_s, ft, fs, fcc << FccShift, ff_c_olt_fmt).encode());
+#endif
   }
 }
 
@@ -1462,12 +1668,24 @@ BufferOffset AssemblerMIPSShared::as_cult(FloatFormat fmt, FloatRegister fs,
                                           FPConditionBit fcc) {
   if (fmt == DoubleFloat) {
     spew("c.ult.d FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_d_r6, ft, fs, FloatRegisters::f24, ff_c_ult_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_d, ft, fs, fcc << FccShift, ff_c_ult_fmt).encode());
+#endif
   } else {
     spew("c.ult.s FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_s_r6, ft, fs, FloatRegisters::f24, ff_c_ult_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_s, ft, fs, fcc << FccShift, ff_c_ult_fmt).encode());
+#endif
   }
 }
 
@@ -1476,12 +1694,24 @@ BufferOffset AssemblerMIPSShared::as_cole(FloatFormat fmt, FloatRegister fs,
                                           FPConditionBit fcc) {
   if (fmt == DoubleFloat) {
     spew("c.ole.d FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_d_r6, ft, fs, FloatRegisters::f24, ff_c_ole_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_d, ft, fs, fcc << FccShift, ff_c_ole_fmt).encode());
+#endif
   } else {
     spew("c.ole.s FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_s_r6, ft, fs, FloatRegisters::f24, ff_c_ole_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_s, ft, fs, fcc << FccShift, ff_c_ole_fmt).encode());
+#endif
   }
 }
 
@@ -1490,12 +1720,24 @@ BufferOffset AssemblerMIPSShared::as_cule(FloatFormat fmt, FloatRegister fs,
                                           FPConditionBit fcc) {
   if (fmt == DoubleFloat) {
     spew("c.ule.d FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_d_r6, ft, fs, FloatRegisters::f24, ff_c_ule_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_d, ft, fs, fcc << FccShift, ff_c_ule_fmt).encode());
+#endif
   } else {
     spew("c.ule.s FCC%d,%3s,%3s", fcc, fs.name(), ft.name());
+#ifdef MIPSR6
+    return writeInst(
+        InstReg(op_cop1, rs_s_r6, ft, fs, FloatRegisters::f24, ff_c_ule_fmt)
+            .encode());
+#else
     return writeInst(
         InstReg(op_cop1, rs_s, ft, fs, fcc << FccShift, ff_c_ule_fmt).encode());
+#endif
   }
 }
 
@@ -1548,6 +1790,28 @@ BufferOffset AssemblerMIPSShared::as_movn(FloatFormat fmt, FloatRegister fd,
   }
 }
 
+BufferOffset AssemblerMIPSShared::as_max(FloatFormat fmt, FloatRegister fd,
+                                         FloatRegister fs, FloatRegister ft) {
+  if (fmt == DoubleFloat) {
+    spew("max  %3s,%3s,%3s", fd.name(), fs.name(), ft.name());
+    return writeInst(InstReg(op_cop1, rs_d, ft, fs, fd, ff_max).encode());
+  } else {
+    spew("max  %3s,%3s,%3s", fd.name(), fs.name(), ft.name());
+    return writeInst(InstReg(op_cop1, rs_s, ft, fs, fd, ff_max).encode());
+  }
+}
+
+BufferOffset AssemblerMIPSShared::as_min(FloatFormat fmt, FloatRegister fd,
+                                         FloatRegister fs, FloatRegister ft) {
+  if (fmt == DoubleFloat) {
+    spew("min  %3s,%3s,%3s", fd.name(), fs.name(), ft.name());
+    return writeInst(InstReg(op_cop1, rs_d, ft, fs, fd, ff_min).encode());
+  } else {
+    spew("min  %3s,%3s,%3s", fd.name(), fs.name(), ft.name());
+    return writeInst(InstReg(op_cop1, rs_s, ft, fs, fd, ff_min).encode());
+  }
+}
+
 BufferOffset AssemblerMIPSShared::as_tge(Register rs, Register rt,
                                          uint32_t code) {
   MOZ_ASSERT(code <= MAX_BREAK_CODE);
@@ -1693,7 +1957,7 @@ Instruction* Instruction::next() { return this + 1; }
 InstImm AssemblerMIPSShared::invertBranch(InstImm branch,
                                           BOffImm16 skipOffset) {
   uint32_t rt = 0;
-  Opcode op = (Opcode)(branch.extractOpcode() << OpcodeShift);
+  OpcodeField op = (OpcodeField)(branch.extractOpcode() << OpcodeShift);
   switch (op) {
     case op_beq:
       branch.setBOffImm16(skipOffset);
@@ -1747,8 +2011,6 @@ void AssemblerMIPSShared::ToggleToJmp(CodeLocationLabel inst_) {
   MOZ_ASSERT(inst->extractOpcode() == ((uint32_t)op_andi >> OpcodeShift));
   // We converted beq to andi, so now we restore it.
   inst->setOpcode(op_beq);
-
-  AutoFlushICache::flush(uintptr_t(inst), 4);
 }
 
 void AssemblerMIPSShared::ToggleToCmp(CodeLocationLabel inst_) {
@@ -1758,8 +2020,6 @@ void AssemblerMIPSShared::ToggleToCmp(CodeLocationLabel inst_) {
   MOZ_ASSERT(inst->extractOpcode() == ((uint32_t)op_beq >> OpcodeShift));
   // Replace "beq $zero, $zero, offset" with "andi $zero, $zero, offset"
   inst->setOpcode(op_andi);
-
-  AutoFlushICache::flush(uintptr_t(inst), 4);
 }
 
 void AssemblerMIPSShared::UpdateLuiOriValue(Instruction* inst0,
@@ -1774,7 +2034,7 @@ void AssemblerMIPSShared::UpdateLuiOriValue(Instruction* inst0,
 
 #ifdef JS_JITSPEW
 void AssemblerMIPSShared::decodeBranchInstAndSpew(InstImm branch) {
-  Opcode op = (Opcode)(branch.extractOpcode() << OpcodeShift);
+  OpcodeField op = (OpcodeField)(branch.extractOpcode() << OpcodeShift);
   uint32_t rt_id;
   uint32_t rs_id;
   uint32_t immi = branch.extractImm16Value();
