@@ -9,10 +9,10 @@
 
 #include "mozilla/EndianUtils.h"
 
-#include "jit/JitFrames.h"
 #include "jit/mips-shared/MacroAssembler-mips-shared.h"
 #include "jit/MoveResolver.h"
 #include "vm/BytecodeUtil.h"
+#include "wasm/WasmTypes.h"
 
 namespace js {
 namespace jit {
@@ -72,7 +72,7 @@ class MacroAssemblerMIPS : public MacroAssemblerMIPSShared {
   using MacroAssemblerMIPSShared::ma_sd;
   using MacroAssemblerMIPSShared::ma_ss;
   using MacroAssemblerMIPSShared::ma_store;
-  using MacroAssemblerMIPSShared::ma_subTestOverflow;
+  using MacroAssemblerMIPSShared::ma_sub32TestOverflow;
 
   void ma_li(Register dest, CodeLabel* label);
 
@@ -90,13 +90,50 @@ class MacroAssemblerMIPS : public MacroAssemblerMIPSShared {
 
   // arithmetic based ops
   // add
-  void ma_addTestOverflow(Register rd, Register rs, Register rt,
-                          Label* overflow);
-  void ma_addTestOverflow(Register rd, Register rs, Imm32 imm, Label* overflow);
+  void ma_add32TestOverflow(Register rd, Register rs, Register rt,
+                            Label* overflow);
+  void ma_add32TestOverflow(Register rd, Register rs, Imm32 imm,
+                            Label* overflow);
+
+  void ma_addPtrTestOverflow(Register rd, Register rs, Register rt,
+                             Label* overflow) {
+    ma_add32TestOverflow(rd, rs, rt, overflow);
+  }
+
+  void ma_addPtrTestOverflow(Register rd, Register rs, Imm32 imm,
+                             Label* overflow) {
+    ma_add32TestOverflow(rd, rs, imm, overflow);
+  }
+
+  void ma_addPtrTestCarry(Condition cond, Register rd, Register rs, Register rt,
+                          Label* overflow) {
+    ma_add32TestCarry(cond, rd, rs, rt, overflow);
+  }
+
+  void ma_addPtrTestCarry(Condition cond, Register rd, Register rs, Imm32 imm,
+                          Label* overflow) {
+    ma_add32TestCarry(cond, rd, rs, imm, overflow);
+  }
 
   // subtract
-  void ma_subTestOverflow(Register rd, Register rs, Register rt,
-                          Label* overflow);
+  void ma_sub32TestOverflow(Register rd, Register rs, Register rt,
+                            Label* overflow);
+
+  void ma_subPtrTestOverflow(Register rd, Register rs, Register rt,
+                             Label* overflow) {
+    ma_sub32TestOverflow(rd, rs, rt, overflow);
+  }
+
+  void ma_subPtrTestOverflow(Register rd, Register rs, Imm32 imm,
+                             Label* overflow) {
+    ma_li(ScratchRegister, imm);
+    ma_sub32TestOverflow(rd, rs, ScratchRegister, overflow);
+  }
+
+  void ma_mulPtrTestOverflow(Register rd, Register rs, Register rt,
+                             Label* overflow) {
+    ma_mul32TestOverflow(rd, rs, rt, overflow);
+  }
 
   // memory
   // shortcut for when we know we're transferring 32 bits of data
@@ -202,6 +239,8 @@ class MacroAssemblerMIPSCompat : public MacroAssemblerMIPS {
   void convertDoubleToFloat32(FloatRegister src, FloatRegister dest);
   void convertDoubleToInt32(FloatRegister src, Register dest, Label* fail,
                             bool negativeZeroCheck = true);
+  void convertDoubleToPtr(FloatRegister src, Register dest, Label* fail,
+                          bool negativeZeroCheck = true);
   void convertFloat32ToInt32(FloatRegister src, Register dest, Label* fail,
                              bool negativeZeroCheck = true);
 
@@ -370,31 +409,32 @@ class MacroAssemblerMIPSCompat : public MacroAssemblerMIPS {
   // Extended unboxing API. If the payload is already in a register, returns
   // that register. Otherwise, provides a move to the given scratch register,
   // and returns that.
-  MOZ_MUST_USE Register extractObject(const Address& address, Register scratch);
-  MOZ_MUST_USE Register extractObject(const ValueOperand& value,
-                                      Register scratch) {
-    return value.payloadReg();
-  }
-  MOZ_MUST_USE Register extractString(const ValueOperand& value,
-                                      Register scratch) {
-    return value.payloadReg();
-  }
-  MOZ_MUST_USE Register extractSymbol(const ValueOperand& value,
-                                      Register scratch) {
-    return value.payloadReg();
-  }
-  MOZ_MUST_USE Register extractInt32(const ValueOperand& value,
-                                     Register scratch) {
-    return value.payloadReg();
-  }
-  MOZ_MUST_USE Register extractBoolean(const ValueOperand& value,
+  [[nodiscard]] Register extractObject(const Address& address,
+                                       Register scratch);
+  [[nodiscard]] Register extractObject(const ValueOperand& value,
                                        Register scratch) {
     return value.payloadReg();
   }
-  MOZ_MUST_USE Register extractTag(const Address& address, Register scratch);
-  MOZ_MUST_USE Register extractTag(const BaseIndex& address, Register scratch);
-  MOZ_MUST_USE Register extractTag(const ValueOperand& value,
-                                   Register scratch) {
+  [[nodiscard]] Register extractString(const ValueOperand& value,
+                                       Register scratch) {
+    return value.payloadReg();
+  }
+  [[nodiscard]] Register extractSymbol(const ValueOperand& value,
+                                       Register scratch) {
+    return value.payloadReg();
+  }
+  [[nodiscard]] Register extractInt32(const ValueOperand& value,
+                                      Register scratch) {
+    return value.payloadReg();
+  }
+  [[nodiscard]] Register extractBoolean(const ValueOperand& value,
+                                        Register scratch) {
+    return value.payloadReg();
+  }
+  [[nodiscard]] Register extractTag(const Address& address, Register scratch);
+  [[nodiscard]] Register extractTag(const BaseIndex& address, Register scratch);
+  [[nodiscard]] Register extractTag(const ValueOperand& value,
+                                    Register scratch) {
     return value.typeReg();
   }
 
@@ -568,7 +608,7 @@ class MacroAssemblerMIPSCompat : public MacroAssemblerMIPS {
   void storeTypeTag(ImmTag tag, Address dest);
   void storeTypeTag(ImmTag tag, const BaseIndex& dest);
 
-  void handleFailureWithHandlerTail(void* handler, Label* profilerExitTail);
+  void handleFailureWithHandlerTail(Label* profilerExitTail);
 
   template <typename T>
   void wasmAtomicStore64(const wasm::MemoryAccessDesc& access, const T& mem,
