@@ -21,6 +21,10 @@ static const uint32_t SimdMemoryAlignment =
 static const uint32_t WasmStackAlignment = 8;
 static const uint32_t WasmTrapInstructionLength = 0;
 
+// See comments in wasm::GenerateFunctionPrologue.
+static constexpr uint32_t WasmCheckedCallEntryOffset = 0u;
+static constexpr uint32_t WasmCheckedTailEntryOffset = 1u;
+
 class Registers {
  public:
   enum RegisterID {
