@@ -18,7 +18,8 @@ namespace jit {
 // ===============================================================
 
 void MacroAssembler::move64(Imm64 imm, Register64 dest) {
-  movq(ImmWord(imm.value), dest.reg);
+  // Use mov instead of movq because it has special optimizations for imm == 0.
+  mov(ImmWord(imm.value), dest.reg);
 }
 
 void MacroAssembler::move64(Register64 src, Register64 dest) {
@@ -53,6 +54,10 @@ void MacroAssembler::move32To64SignExtend(Register src, Register64 dest) {
   movslq(src, dest.reg);
 }
 
+void MacroAssembler::move32SignExtendToPtr(Register src, Register dest) {
+  movslq(src, dest);
+}
+
 void MacroAssembler::move32ZeroExtendToPtr(Register src, Register dest) {
   movl(src, dest);
 }
@@ -64,6 +69,11 @@ void MacroAssembler::load32SignExtendToPtr(const Address& src, Register dest) {
   movslq(Operand(src), dest);
 }
 
+// ===============================================================
+// Logical instructions
+
+void MacroAssembler::notPtr(Register reg) { notq(reg); }
+
 void MacroAssembler::andPtr(Register src, Register dest) { andq(src, dest); }
 
 void MacroAssembler::andPtr(Imm32 imm, Register dest) { andq(imm, dest); }
@@ -224,6 +234,10 @@ void MacroAssembler::sub64(Imm64 imm, Register64 dest) {
   subPtr(ImmWord(imm.value), dest.reg);
 }
 
+void MacroAssembler::mulPtr(Register rhs, Register srcDest) {
+  imulq(rhs, srcDest);
+}
+
 void MacroAssembler::mul64(Imm64 imm, const Register64& dest,
                            const Register temp) {
   MOZ_ASSERT(temp == InvalidReg);
@@ -231,8 +245,12 @@ void MacroAssembler::mul64(Imm64 imm, const Register64& dest,
 }
 
 void MacroAssembler::mul64(Imm64 imm, const Register64& dest) {
-  movq(ImmWord(uintptr_t(imm.value)), ScratchReg);
-  imulq(ScratchReg, dest.reg);
+  if (INT32_MIN <= int64_t(imm.value) && int64_t(imm.value) <= INT32_MAX) {
+    imulq(Imm32((int32_t)imm.value), dest.reg, dest.reg);
+  } else {
+    movq(ImmWord(uintptr_t(imm.value)), ScratchReg);
+    imulq(ScratchReg, dest.reg);
+  }
 }
 
 void MacroAssembler::mul64(const Register64& src, const Register64& dest,
@@ -283,12 +301,25 @@ void MacroAssembler::lshiftPtr(Imm32 imm, Register dest) {
   shlq(imm, dest);
 }
 
+void MacroAssembler::lshiftPtr(Register shift, Register srcDest) {
+  if (Assembler::HasBMI2()) {
+    shlxq(srcDest, shift, srcDest);
+    return;
+  }
+  MOZ_ASSERT(shift == rcx);
+  shlq_cl(srcDest);
+}
+
 void MacroAssembler::lshift64(Imm32 imm, Register64 dest) {
   MOZ_ASSERT(0 <= imm.value && imm.value < 64);
   lshiftPtr(imm, dest.reg);
 }
 
 void MacroAssembler::lshift64(Register shift, Register64 srcDest) {
+  if (Assembler::HasBMI2()) {
+    shlxq(srcDest.reg, shift, srcDest.reg);
+    return;
+  }
   MOZ_ASSERT(shift == rcx);
   shlq_cl(srcDest.reg);
 }
@@ -298,11 +329,24 @@ void MacroAssembler::rshiftPtr(Imm32 imm, Register dest) {
   shrq(imm, dest);
 }
 
+void MacroAssembler::rshiftPtr(Register shift, Register srcDest) {
+  if (Assembler::HasBMI2()) {
+    shrxq(srcDest, shift, srcDest);
+    return;
+  }
+  MOZ_ASSERT(shift == rcx);
+  shrq_cl(srcDest);
+}
+
 void MacroAssembler::rshift64(Imm32 imm, Register64 dest) {
   rshiftPtr(imm, dest.reg);
 }
 
 void MacroAssembler::rshift64(Register shift, Register64 srcDest) {
+  if (Assembler::HasBMI2()) {
+    shrxq(srcDest.reg, shift, srcDest.reg);
+    return;
+  }
   MOZ_ASSERT(shift == rcx);
   shrq_cl(srcDest.reg);
 }
@@ -318,6 +362,10 @@ void MacroAssembler::rshift64Arithmetic(Imm32 imm, Register64 dest) {
 }
 
 void MacroAssembler::rshift64Arithmetic(Register shift, Register64 srcDest) {
+  if (Assembler::HasBMI2()) {
+    sarxq(srcDest.reg, shift, srcDest.reg);
+    return;
+  }
   MOZ_ASSERT(shift == rcx);
   sarq_cl(srcDest.reg);
 }
@@ -390,8 +438,10 @@ void MacroAssembler::cmpPtrSet(Condition cond, T1 lhs, T2 rhs, Register dest) {
 // Bit counting functions
 
 void MacroAssembler::clz64(Register64 src, Register dest) {
-  // On very recent chips (Haswell and newer) there is actually an
-  // LZCNT instruction that does all of this.
+  if (AssemblerX86Shared::HasLZCNT()) {
+    lzcntq(src.reg, dest);
+    return;
+  }
 
   Label nonzero;
   bsrq(src.reg, dest);
@@ -402,6 +452,11 @@ void MacroAssembler::clz64(Register64 src, Register dest) {
 }
 
 void MacroAssembler::ctz64(Register64 src, Register dest) {
+  if (AssemblerX86Shared::HasBMI1()) {
+    tzcntq(src.reg, dest);
+    return;
+  }
+
   Label nonzero;
   bsfq(src.reg, dest);
   j(Assembler::NonZero, &nonzero);
@@ -670,6 +725,12 @@ void MacroAssembler::branchTestMagic(Condition cond, const Address& valaddr,
   j(cond, label);
 }
 
+void MacroAssembler::branchTestValue(Condition cond, const BaseIndex& lhs,
+                                     const ValueOperand& rhs, Label* label) {
+  MOZ_ASSERT(cond == Assembler::Equal || cond == Assembler::NotEqual);
+  branchPtr(cond, lhs, rhs.valueReg(), label);
+}
+
 void MacroAssembler::branchToComputedAddress(const BaseIndex& address) {
   jmp(Operand(address));
 }
@@ -680,6 +741,13 @@ void MacroAssembler::cmpPtrMovePtr(Condition cond, Register lhs, Register rhs,
   cmovCCq(cond, src, dest);
 }
 
+void MacroAssembler::cmpPtrMovePtr(Condition cond, Register lhs,
+                                   const Address& rhs, Register src,
+                                   Register dest) {
+  cmpPtr(lhs, Operand(rhs));
+  cmovCCq(cond, src, dest);
+}
+
 void MacroAssembler::cmp32MovePtr(Condition cond, Register lhs, Imm32 rhs,
                                   Register src, Register dest) {
   cmp32(lhs, rhs);
@@ -757,6 +825,52 @@ void MacroAssembler::spectreBoundsCheck32(Register index, const Address& length,
   }
 }
 
+void MacroAssembler::spectreBoundsCheckPtr(Register index, Register length,
+                                           Register maybeScratch,
+                                           Label* failure) {
+  MOZ_ASSERT(length != maybeScratch);
+  MOZ_ASSERT(index != maybeScratch);
+
+  ScratchRegisterScope scratch(*this);
+  MOZ_ASSERT(index != scratch);
+  MOZ_ASSERT(length != scratch);
+
+  if (JitOptions.spectreIndexMasking) {
+    movePtr(ImmWord(0), scratch);
+  }
+
+  cmpPtr(index, length);
+  j(Assembler::AboveOrEqual, failure);
+
+  if (JitOptions.spectreIndexMasking) {
+    cmovCCq(Assembler::AboveOrEqual, scratch, index);
+  }
+}
+
+void MacroAssembler::spectreBoundsCheckPtr(Register index,
+                                           const Address& length,
+                                           Register maybeScratch,
+                                           Label* failure) {
+  MOZ_ASSERT(index != length.base);
+  MOZ_ASSERT(length.base != maybeScratch);
+  MOZ_ASSERT(index != maybeScratch);
+
+  ScratchRegisterScope scratch(*this);
+  MOZ_ASSERT(index != scratch);
+  MOZ_ASSERT(length.base != scratch);
+
+  if (JitOptions.spectreIndexMasking) {
+    movePtr(ImmWord(0), scratch);
+  }
+
+  cmpPtr(index, Operand(length));
+  j(Assembler::AboveOrEqual, failure);
+
+  if (JitOptions.spectreIndexMasking) {
+    cmovCCq(Assembler::AboveOrEqual, scratch, index);
+  }
+}
+
 // ========================================================================
 // SIMD.
 //
@@ -773,42 +887,6 @@ void MacroAssembler::anyTrueSimd128(FloatRegister src, Register dest) {
   cmovCCl(NonZero, one, dest);
 }
 
-// Integer Multiply
-
-void MacroAssembler::mulInt64x2(FloatRegister rhs, FloatRegister lhsDest,
-                                Register64 temp) {
-  ScratchRegisterScope t1(*this);
-  Register t2 = temp.reg;
-  vpextrq(0, lhsDest, t1);
-  vpextrq(0, rhs, t2);
-  imulq(t2, t1);
-  vpinsrq(0, t1, lhsDest, lhsDest);
-  vpextrq(1, lhsDest, t1);
-  vpextrq(1, rhs, t2);
-  imulq(t2, t1);
-  vpinsrq(1, t1, lhsDest, lhsDest);
-}
-
-// Right shift by scalar
-
-void MacroAssembler::rightShiftInt64x2(Register rhs, FloatRegister lhsDest) {
-  ScratchRegisterScope scratch(*this);
-
-  MOZ_ASSERT(rhs == rcx);  // We need CL
-
-  vpextrq(0, lhsDest, scratch);
-  sarq_cl(scratch);
-  vpinsrq(0, scratch, lhsDest, lhsDest);
-  vpextrq(1, lhsDest, scratch);
-  sarq_cl(scratch);
-  vpinsrq(1, scratch, lhsDest, lhsDest);
-}
-
-void MacroAssembler::rightShiftInt64x2(Imm32 count, FloatRegister src,
-                                       FloatRegister dest) {
-  MacroAssemblerX64::rightShiftInt64x2(count, src, dest);
-}
-
 // Extract lane as scalar
 
 void MacroAssembler::extractLaneInt64x2(uint32_t lane, FloatRegister src,
