@@ -27,14 +27,21 @@ class MacroAssemblerX86Shared : public Assembler {
   const MacroAssembler& asMasm() const;
 
  public:
-  typedef Vector<CodeOffset, 0, SystemAllocPolicy> UsesVector;
+#ifdef JS_CODEGEN_X64
+  typedef X86Encoding::JmpSrc UsesItem;
+#else
+  typedef CodeOffset UsesItem;
+#endif
+
+  typedef Vector<UsesItem, 0, SystemAllocPolicy> UsesVector;
+  static_assert(sizeof(UsesItem) == 4);
 
  protected:
   // For Double, Float and SimdData, make the move ctors explicit so that MSVC
   // knows what to use instead of copying these data structures.
   template <class T>
   struct Constant {
-    typedef T Pod;
+    using Pod = T;
 
     T value;
     UsesVector uses;
@@ -83,12 +90,16 @@ class MacroAssemblerX86Shared : public Assembler {
  public:
   using Assembler::call;
 
-  MacroAssemblerX86Shared() {}
+  MacroAssemblerX86Shared() = default;
 
   bool appendRawCode(const uint8_t* code, size_t numBytes) {
     return masm.appendRawCode(code, numBytes);
   }
 
+  void addToPCRel4(uint32_t offset, int32_t bias) {
+    return masm.addToPCRel4(offset, bias);
+  }
+
   // Evaluate srcDest = minmax<isMax>{Float32,Double}(srcDest, second).
   // Checks for NaN if canBeNaN is true.
   void minMaxDouble(FloatRegister srcDest, FloatRegister second, bool canBeNaN,
@@ -139,9 +150,7 @@ class MacroAssemblerX86Shared : public Assembler {
   void cmp32(const Operand& lhs, Imm32 rhs) { cmpl(rhs, lhs); }
   void cmp32(const Operand& lhs, Register rhs) { cmpl(rhs, lhs); }
   void cmp32(Register lhs, const Operand& rhs) { cmpl(rhs, lhs); }
-  CodeOffset cmp32WithPatch(Register lhs, Imm32 rhs) {
-    return cmplWithPatch(rhs, lhs);
-  }
+
   void atomic_inc32(const Operand& addr) { lock_incl(addr); }
   void atomic_dec32(const Operand& addr) { lock_decl(addr); }
 
@@ -166,7 +175,7 @@ class MacroAssemblerX86Shared : public Assembler {
   void jump(Label* label) { jmp(label); }
   void jump(JitCode* code) { jmp(code); }
   void jump(TrampolinePtr code) { jmp(ImmPtr(code.value)); }
-  void jump(RepatchLabel* label) { jmp(label); }
+  void jump(ImmPtr ptr) { jmp(ptr); }
   void jump(Register reg) { jmp(Operand(reg)); }
   void jump(const Address& addr) { jmp(Operand(addr)); }
 
@@ -283,10 +292,18 @@ class MacroAssemblerX86Shared : public Assembler {
   void load16ZeroExtend(const BaseIndex& src, Register dest) {
     movzwl(Operand(src), dest);
   }
+  template <typename S>
+  void load16UnalignedZeroExtend(const S& src, Register dest) {
+    load16ZeroExtend(src, dest);
+  }
   template <typename S, typename T>
   void store16(const S& src, const T& dest) {
     movw(src, Operand(dest));
   }
+  template <typename S, typename T>
+  void store16Unaligned(const S& src, const T& dest) {
+    store16(src, dest);
+  }
   void load16SignExtend(const Operand& src, Register dest) {
     movswl(src, dest);
   }
@@ -296,11 +313,19 @@ class MacroAssemblerX86Shared : public Assembler {
   void load16SignExtend(const BaseIndex& src, Register dest) {
     movswl(Operand(src), dest);
   }
+  template <typename S>
+  void load16UnalignedSignExtend(const S& src, Register dest) {
+    load16SignExtend(src, dest);
+  }
   void load32(const Address& address, Register dest) {
     movl(Operand(address), dest);
   }
   void load32(const BaseIndex& src, Register dest) { movl(Operand(src), dest); }
   void load32(const Operand& src, Register dest) { movl(src, dest); }
+  template <typename S>
+  void load32Unaligned(const S& src, Register dest) {
+    load32(src, dest);
+  }
   template <typename S, typename T>
   void store32(const S& src, const T& dest) {
     movl(src, Operand(dest));
@@ -309,6 +334,10 @@ class MacroAssemblerX86Shared : public Assembler {
   void store32_NoSecondScratch(const S& src, const T& dest) {
     store32(src, dest);
   }
+  template <typename S, typename T>
+  void store32Unaligned(const S& src, const T& dest) {
+    store32(src, dest);
+  }
   void loadDouble(const Address& src, FloatRegister dest) { vmovsd(src, dest); }
   void loadDouble(const BaseIndex& src, FloatRegister dest) {
     vmovsd(src, dest);
@@ -362,226 +391,162 @@ class MacroAssemblerX86Shared : public Assembler {
     vcvtdq2ps(src, dest);
   }
 
-  // SIMD methods, defined in MacroAssembler-x86-shared-SIMD.cpp.
-  void checkedConvertFloat32x4ToInt32x4(FloatRegister src, FloatRegister dest,
-                                        Register temp, Label* oolCheck,
-                                        Label* rejoin);
-  void oolConvertFloat32x4ToInt32x4(FloatRegister src, Register temp,
-                                    Label* rejoin, Label* onConversionError);
-  void checkedConvertFloat32x4ToUint32x4(FloatRegister src, FloatRegister dest,
-                                         Register temp, FloatRegister tempF,
-                                         Label* failed);
+  void binarySimd128(const SimdConstant& rhs, FloatRegister lhsDest,
+                     void (MacroAssembler::*regOp)(const Operand&,
+                                                   FloatRegister,
+                                                   FloatRegister),
+                     void (MacroAssembler::*constOp)(const SimdConstant&,
+                                                     FloatRegister));
+  void binarySimd128(const SimdConstant& rhs, FloatRegister lhsDest,
+                     void (MacroAssembler::*regOp)(const Operand&,
+                                                   FloatRegister),
+                     void (MacroAssembler::*constOp)(const SimdConstant&,
+                                                     FloatRegister));
 
-  void createInt32x4(Register lane0, Register lane1, Register lane2,
-                     Register lane3, FloatRegister dest);
-  void createFloat32x4(FloatRegister lane0, FloatRegister lane1,
-                       FloatRegister lane2, FloatRegister lane3,
-                       FloatRegister temp, FloatRegister output);
+  // SIMD methods, defined in MacroAssembler-x86-shared-SIMD.cpp.
+
+  void unsignedConvertInt32x4ToFloat32x4(FloatRegister src, FloatRegister dest);
+  void unsignedConvertInt32x4ToFloat64x2(FloatRegister src, FloatRegister dest);
+  void bitwiseTestSimd128(const SimdConstant& rhs, FloatRegister lhs);
+
+  void truncSatFloat32x4ToInt32x4(FloatRegister src, FloatRegister dest);
+  void unsignedTruncSatFloat32x4ToInt32x4(FloatRegister src, FloatRegister temp,
+                                          FloatRegister dest);
+  void truncSatFloat64x2ToInt32x4(FloatRegister src, FloatRegister temp,
+                                  FloatRegister dest);
+  void unsignedTruncSatFloat64x2ToInt32x4(FloatRegister src, FloatRegister temp,
+                                          FloatRegister dest);
 
   void splatX16(Register input, FloatRegister output);
   void splatX8(Register input, FloatRegister output);
   void splatX4(Register input, FloatRegister output);
   void splatX4(FloatRegister input, FloatRegister output);
-
-  void reinterpretSimd(bool isIntegerLaneType, FloatRegister input,
-                       FloatRegister output);
+  void splatX2(FloatRegister input, FloatRegister output);
 
   void extractLaneInt32x4(FloatRegister input, Register output, unsigned lane);
   void extractLaneFloat32x4(FloatRegister input, FloatRegister output,
-                            unsigned lane, bool canonicalize);
+                            unsigned lane);
+  void extractLaneFloat64x2(FloatRegister input, FloatRegister output,
+                            unsigned lane);
   void extractLaneInt16x8(FloatRegister input, Register output, unsigned lane,
                           SimdSign sign);
   void extractLaneInt8x16(FloatRegister input, Register output, unsigned lane,
                           SimdSign sign);
-  void extractLaneSimdBool(FloatRegister input, Register output,
-                           unsigned numLanes, unsigned lane);
 
-  void insertLaneSimdInt(FloatRegister input, Register value,
-                         FloatRegister output, unsigned lane,
-                         unsigned numLanes);
-  void insertLaneFloat32x4(FloatRegister input, FloatRegister value,
-                           FloatRegister output, unsigned lane);
+  void replaceLaneFloat32x4(FloatRegister rhs, FloatRegister lhsDest,
+                            unsigned lane);
+  void replaceLaneFloat64x2(FloatRegister rhs, FloatRegister lhsDest,
+                            unsigned lane);
 
-  void allTrueSimdBool(FloatRegister input, Register output);
-  void anyTrueSimdBool(FloatRegister input, Register output);
-
-  void swizzleInt32x4(FloatRegister input, FloatRegister output,
-                      unsigned lanes[4]);
-  void swizzleFloat32x4(FloatRegister input, FloatRegister output,
-                        unsigned lanes[4]);
-  void swizzleInt8x16(FloatRegister input, FloatRegister output,
-                      const mozilla::Maybe<Register>& temp, int8_t lanes[16]);
-
-  void shuffleX4(FloatRegister lhs, Operand rhs, FloatRegister out,
-                 const mozilla::Maybe<FloatRegister>& maybeTemp,
-                 unsigned lanes[4]);
   void shuffleInt8x16(FloatRegister lhs, FloatRegister rhs,
-                      FloatRegister output,
-                      const mozilla::Maybe<FloatRegister>& maybeFloatTemp,
-                      const mozilla::Maybe<Register>& maybeTemp,
-                      uint8_t lanes[16]);
+                      FloatRegister output, const uint8_t lanes[16]);
+  void blendInt8x16(FloatRegister lhs, FloatRegister rhs, FloatRegister output,
+                    FloatRegister temp, const uint8_t lanes[16]);
+  void blendInt16x8(FloatRegister lhs, FloatRegister rhs, FloatRegister output,
+                    const uint16_t lanes[8]);
 
   void compareInt8x16(FloatRegister lhs, Operand rhs, Assembler::Condition cond,
                       FloatRegister output);
+  void compareInt8x16(Assembler::Condition cond, const SimdConstant& rhs,
+                      FloatRegister lhsDest);
   void compareInt16x8(FloatRegister lhs, Operand rhs, Assembler::Condition cond,
                       FloatRegister output);
+  void compareInt16x8(Assembler::Condition cond, const SimdConstant& rhs,
+                      FloatRegister lhsDest);
   void compareInt32x4(FloatRegister lhs, Operand rhs, Assembler::Condition cond,
                       FloatRegister output);
+  void compareInt32x4(Assembler::Condition cond, const SimdConstant& rhs,
+                      FloatRegister lhsDest);
+  void compareForEqualityInt64x2(FloatRegister lhs, Operand rhs,
+                                 Assembler::Condition cond,
+                                 FloatRegister output);
+  void compareForOrderingInt64x2(FloatRegister lhs, Operand rhs,
+                                 Assembler::Condition cond, FloatRegister temp1,
+                                 FloatRegister temp2, FloatRegister output);
   void compareFloat32x4(FloatRegister lhs, Operand rhs,
                         Assembler::Condition cond, FloatRegister output);
+  void compareFloat32x4(Assembler::Condition cond, const SimdConstant& rhs,
+                        FloatRegister lhsDest);
+  void compareFloat64x2(FloatRegister lhs, Operand rhs,
+                        Assembler::Condition cond, FloatRegister output);
+  void compareFloat64x2(Assembler::Condition cond, const SimdConstant& rhs,
+                        FloatRegister lhsDest);
 
-  void addInt8x16(FloatRegister lhs, Operand rhs, FloatRegister output) {
-    vpaddb(rhs, lhs, output);
-  }
-  void addInt16x8(FloatRegister lhs, Operand rhs, FloatRegister output) {
-    vpaddw(rhs, lhs, output);
-  }
-  void addInt32x4(FloatRegister lhs, Operand rhs, FloatRegister output) {
-    vpaddd(rhs, lhs, output);
-  }
-  void addFloat32x4(FloatRegister lhs, Operand rhs, FloatRegister output) {
-    vaddps(rhs, lhs, output);
-  }
-
-  void addSatInt8x16(FloatRegister lhs, Operand rhs, SimdSign sign,
-                     FloatRegister output) {
-    if (sign == SimdSign::Signed) {
-      vpaddsb(rhs, lhs, output);
-    } else {
-      vpaddusb(rhs, lhs, output);
-    }
-  }
-  void addSatInt16x8(FloatRegister lhs, Operand rhs, SimdSign sign,
-                     FloatRegister output) {
-    if (sign == SimdSign::Signed) {
-      vpaddsw(rhs, lhs, output);
-    } else {
-      vpaddusw(rhs, lhs, output);
-    }
-  }
-
-  void subInt8x16(FloatRegister lhs, Operand rhs, FloatRegister output) {
-    vpsubb(rhs, lhs, output);
-  }
-  void subInt16x8(FloatRegister lhs, Operand rhs, FloatRegister output) {
-    vpsubw(rhs, lhs, output);
-  }
-  void subInt32x4(FloatRegister lhs, Operand rhs, FloatRegister output) {
-    vpsubd(rhs, lhs, output);
-  }
-  void subFloat32x4(FloatRegister lhs, Operand rhs, FloatRegister output) {
-    vsubps(rhs, lhs, output);
-  }
-
-  void subSatInt8x16(FloatRegister lhs, Operand rhs, SimdSign sign,
-                     FloatRegister output) {
-    if (sign == SimdSign::Signed) {
-      vpsubsb(rhs, lhs, output);
-    } else {
-      vpsubusb(rhs, lhs, output);
-    }
-  }
-  void subSatInt16x8(FloatRegister lhs, Operand rhs, SimdSign sign,
-                     FloatRegister output) {
-    if (sign == SimdSign::Signed) {
-      vpsubsw(rhs, lhs, output);
-    } else {
-      vpsubusw(rhs, lhs, output);
-    }
-  }
-
-  void mulInt16x8(FloatRegister lhs, Operand rhs, FloatRegister output) {
-    vpmullw(rhs, lhs, output);
-  }
-  void mulInt32x4(FloatRegister lhs, Operand rhs,
-                  const mozilla::Maybe<FloatRegister>& temp,
-                  FloatRegister output);
-  void mulFloat32x4(FloatRegister lhs, Operand rhs, FloatRegister output) {
-    vmulps(rhs, lhs, output);
-  }
-
-  void negInt8x16(Operand in, FloatRegister out) {
-    zeroSimd128Int(out);
-    packedSubInt8(in, out);
-  }
-  void negInt16x8(Operand in, FloatRegister out) {
-    zeroSimd128Int(out);
-    packedSubInt16(in, out);
-  }
-  void negInt32x4(Operand in, FloatRegister out) {
-    zeroSimd128Int(out);
-    packedSubInt32(in, out);
-  }
-  void negFloat32x4(Operand in, FloatRegister out);
-
-  void notInt8x16(Operand in, FloatRegister out);
-  void notInt16x8(Operand in, FloatRegister out);
-  void notInt32x4(Operand in, FloatRegister out);
-  void notFloat32x4(Operand in, FloatRegister out);
-
-  void divFloat32x4(FloatRegister lhs, Operand rhs, FloatRegister output) {
-    vdivps(rhs, lhs, output);
-  }
-  void minFloat32x4(FloatRegister lhs, Operand rhs, FloatRegister output);
-  void maxFloat32x4(FloatRegister lhs, Operand rhs, FloatRegister temp,
-                    FloatRegister output);
-  void minNumFloat32x4(FloatRegister lhs, Operand rhs, FloatRegister temp,
+  void minMaxFloat32x4(bool isMin, FloatRegister lhs, Operand rhs,
+                       FloatRegister temp1, FloatRegister temp2,
                        FloatRegister output);
-  void maxNumFloat32x4(FloatRegister lhs, Operand rhs, FloatRegister temp,
+  void minMaxFloat64x2(bool isMin, FloatRegister lhs, Operand rhs,
+                       FloatRegister temp1, FloatRegister temp2,
                        FloatRegister output);
+  void minFloat32x4(FloatRegister lhs, Operand rhs, FloatRegister temp1,
+                    FloatRegister temp2, FloatRegister output);
+  void maxFloat32x4(FloatRegister lhs, Operand rhs, FloatRegister temp1,
+                    FloatRegister temp2, FloatRegister output);
 
-  void absFloat32x4(Operand in, FloatRegister out);
+  void minFloat64x2(FloatRegister lhs, Operand rhs, FloatRegister temp1,
+                    FloatRegister temp2, FloatRegister output);
+  void maxFloat64x2(FloatRegister lhs, Operand rhs, FloatRegister temp1,
+                    FloatRegister temp2, FloatRegister output);
 
-  void bitwiseAndFloat32x4(FloatRegister lhs, const Operand& rhs,
-                           FloatRegister dest) {
-    vandps(rhs, lhs, dest);
-  }
-  void bitwiseAndSimdInt(FloatRegister lhs, const Operand& rhs,
-                         FloatRegister dest) {
-    vpand(rhs, lhs, dest);
-  }
+  void packedShiftByScalarInt8x16(
+      FloatRegister in, Register count, Register temp, FloatRegister xtmp,
+      FloatRegister dest,
+      void (MacroAssemblerX86Shared::*shift)(FloatRegister, FloatRegister,
+                                             FloatRegister),
+      void (MacroAssemblerX86Shared::*extend)(const Operand&, FloatRegister));
 
-  void bitwiseOrFloat32x4(FloatRegister lhs, const Operand& rhs,
-                          FloatRegister dest) {
-    vorps(rhs, lhs, dest);
-  }
-  void bitwiseOrSimdInt(FloatRegister lhs, const Operand& rhs,
-                        FloatRegister dest) {
-    vpor(rhs, lhs, dest);
-  }
+  void packedLeftShiftByScalarInt8x16(FloatRegister in, Register count,
+                                      Register temp, FloatRegister xtmp,
+                                      FloatRegister dest);
+  void packedLeftShiftByScalarInt8x16(Imm32 count, FloatRegister src,
+                                      FloatRegister dest);
+  void packedRightShiftByScalarInt8x16(FloatRegister in, Register count,
+                                       Register temp, FloatRegister xtmp,
+                                       FloatRegister dest);
+  void packedRightShiftByScalarInt8x16(Imm32 count, FloatRegister src,
+                                       FloatRegister dest);
+  void packedUnsignedRightShiftByScalarInt8x16(FloatRegister in, Register count,
+                                               Register temp,
+                                               FloatRegister xtmp,
+                                               FloatRegister dest);
+  void packedUnsignedRightShiftByScalarInt8x16(Imm32 count, FloatRegister src,
+                                               FloatRegister dest);
 
-  void bitwiseXorFloat32x4(FloatRegister lhs, const Operand& rhs,
-                           FloatRegister dest) {
-    vxorps(rhs, lhs, dest);
-  }
-  void bitwiseXorSimdInt(FloatRegister lhs, const Operand& rhs,
-                         FloatRegister dest) {
-    vpxor(rhs, lhs, dest);
-  }
-
-  void bitwiseAndNotFloat32x4(FloatRegister lhs, const Operand& rhs,
-                              FloatRegister dest) {
-    vandnps(rhs, lhs, dest);
-  }
-  void bitwiseAndNotSimdInt(FloatRegister lhs, const Operand& rhs,
-                            FloatRegister dest) {
-    vpandn(rhs, lhs, dest);
-  }
-
-  void zeroSimd128Float(FloatRegister dest) { vxorps(dest, dest, dest); }
-  void zeroSimd128Int(FloatRegister dest) { vpxor(dest, dest, dest); }
+  void packedLeftShiftByScalarInt16x8(FloatRegister in, Register count,
+                                      Register temp, FloatRegister dest);
+  void packedRightShiftByScalarInt16x8(FloatRegister in, Register count,
+                                       Register temp, FloatRegister dest);
+  void packedUnsignedRightShiftByScalarInt16x8(FloatRegister in, Register count,
+                                               Register temp,
+                                               FloatRegister dest);
 
+  void packedLeftShiftByScalarInt32x4(FloatRegister in, Register count,
+                                      Register temp, FloatRegister dest);
+  void packedRightShiftByScalarInt32x4(FloatRegister in, Register count,
+                                       Register temp, FloatRegister dest);
+  void packedUnsignedRightShiftByScalarInt32x4(FloatRegister in, Register count,
+                                               Register temp,
+                                               FloatRegister dest);
+  void packedLeftShiftByScalarInt64x2(FloatRegister in, Register count,
+                                      Register temp, FloatRegister dest);
+  void packedRightShiftByScalarInt64x2(FloatRegister in, Register count,
+                                       Register temp1, FloatRegister temp2,
+                                       FloatRegister dest);
+  void packedRightShiftByScalarInt64x2(Imm32 count, FloatRegister src,
+                                       FloatRegister dest);
+  void packedUnsignedRightShiftByScalarInt64x2(FloatRegister in, Register count,
+                                               Register temp,
+                                               FloatRegister dest);
   void selectSimd128(FloatRegister mask, FloatRegister onTrue,
                      FloatRegister onFalse, FloatRegister temp,
                      FloatRegister output);
-  void selectX4(FloatRegister mask, FloatRegister onTrue, FloatRegister onFalse,
-                FloatRegister temp, FloatRegister output) {
-    if (AssemblerX86Shared::HasAVX()) {
-      vblendvps(mask, onTrue, onFalse, output);
-    } else {
-      selectSimd128(mask, onTrue, onFalse, temp, output);
-    }
-  }
+  void popcntInt8x16(FloatRegister src, FloatRegister temp,
+                     FloatRegister output);
+
+  // SIMD inline methods private to the implementation, that appear to be used.
+
+  void zeroSimd128Float(FloatRegister dest) { vxorps(dest, dest, dest); }
+  void zeroSimd128Int(FloatRegister dest) { vpxor(dest, dest, dest); }
 
   template <class T, class Reg>
   inline void loadScalar(const Operand& src, Reg dest);
@@ -602,23 +567,18 @@ class MacroAssemblerX86Shared : public Assembler {
     vmovdqa(src, Operand(dest));
   }
   void moveSimd128Int(FloatRegister src, FloatRegister dest) {
-    vmovdqa(src, dest);
+    if (src != dest) {
+      vmovdqa(src, dest);
+    }
   }
   FloatRegister reusedInputInt32x4(FloatRegister src, FloatRegister dest) {
+    MOZ_ASSERT(src.isSimd128() && dest.isSimd128());
     if (HasAVX()) {
       return src;
     }
     moveSimd128Int(src, dest);
     return dest;
   }
-  FloatRegister reusedInputAlignedInt32x4(const Operand& src,
-                                          FloatRegister dest) {
-    if (HasAVX() && src.kind() == Operand::FPREG) {
-      return FloatRegister::FromCode(src.fpu());
-    }
-    loadAlignedSimd128Int(src, dest);
-    return dest;
-  }
   void loadUnalignedSimd128Int(const Address& src, FloatRegister dest) {
     vmovdqu(Operand(src), dest);
   }
@@ -628,7 +588,6 @@ class MacroAssemblerX86Shared : public Assembler {
   void loadUnalignedSimd128Int(const Operand& src, FloatRegister dest) {
     vmovdqu(src, dest);
   }
-
   void storeUnalignedSimd128Int(FloatRegister src, const Address& dest) {
     vmovdqu(src, Operand(dest));
   }
@@ -638,11 +597,151 @@ class MacroAssemblerX86Shared : public Assembler {
   void storeUnalignedSimd128Int(FloatRegister src, const Operand& dest) {
     vmovdqu(src, dest);
   }
-  void packedEqualInt32x4(const Operand& src, FloatRegister dest) {
-    vpcmpeqd(src, dest, dest);
+  void packedLeftShiftByScalarInt16x8(Imm32 count, FloatRegister dest) {
+    count.value &= 15;
+    vpsllw(count, dest, dest);
   }
-  void packedGreaterThanInt32x4(const Operand& src, FloatRegister dest) {
-    vpcmpgtd(src, dest, dest);
+  void packedRightShiftByScalarInt16x8(Imm32 count, FloatRegister dest) {
+    count.value &= 15;
+    vpsraw(count, dest, dest);
+  }
+  void packedUnsignedRightShiftByScalarInt16x8(Imm32 count,
+                                               FloatRegister dest) {
+    count.value &= 15;
+    vpsrlw(count, dest, dest);
+  }
+  void packedLeftShiftByScalarInt32x4(Imm32 count, FloatRegister dest) {
+    count.value &= 31;
+    vpslld(count, dest, dest);
+  }
+  void packedRightShiftByScalarInt32x4(Imm32 count, FloatRegister dest) {
+    count.value &= 31;
+    vpsrad(count, dest, dest);
+  }
+  void packedUnsignedRightShiftByScalarInt32x4(Imm32 count,
+                                               FloatRegister dest) {
+    count.value &= 31;
+    vpsrld(count, dest, dest);
+  }
+  void loadAlignedSimd128Float(const Address& src, FloatRegister dest) {
+    vmovaps(Operand(src), dest);
+  }
+  void loadAlignedSimd128Float(const Operand& src, FloatRegister dest) {
+    vmovaps(src, dest);
+  }
+  void storeAlignedSimd128Float(FloatRegister src, const Address& dest) {
+    vmovaps(src, Operand(dest));
+  }
+  void moveSimd128Float(FloatRegister src, FloatRegister dest) {
+    if (src != dest) {
+      vmovaps(src, dest);
+    }
+  }
+  FloatRegister reusedInputSimd128Float(FloatRegister src, FloatRegister dest) {
+    MOZ_ASSERT(src.isSimd128() && dest.isSimd128());
+    if (HasAVX()) {
+      return src;
+    }
+    moveSimd128Float(src, dest);
+    return dest;
+  }
+  void loadUnalignedSimd128(const Operand& src, FloatRegister dest) {
+    vmovups(src, dest);
+  }
+  void storeUnalignedSimd128(FloatRegister src, const Operand& dest) {
+    vmovups(src, dest);
+  }
+
+  static uint32_t ComputeShuffleMask(uint32_t x = 0, uint32_t y = 1,
+                                     uint32_t z = 2, uint32_t w = 3) {
+    MOZ_ASSERT(x < 4 && y < 4 && z < 4 && w < 4);
+    uint32_t r = (w << 6) | (z << 4) | (y << 2) | (x << 0);
+    MOZ_ASSERT(r < 256);
+    return r;
+  }
+
+  void shuffleInt32(uint32_t mask, FloatRegister src, FloatRegister dest) {
+    vpshufd(mask, src, dest);
+  }
+  void moveLowInt32(FloatRegister src, Register dest) { vmovd(src, dest); }
+
+  void moveHighPairToLowPairFloat32(FloatRegister src, FloatRegister dest) {
+    vmovhlps(src, dest, dest);
+  }
+  void shuffleFloat32(uint32_t mask, FloatRegister src, FloatRegister dest) {
+    // The shuffle instruction on x86 is such that it moves 2 words from
+    // the dest and 2 words from the src operands. To simplify things, just
+    // clobber the output with the input and apply the instruction
+    // afterwards.
+    // Note: this is useAtStart-safe because src isn't read afterwards.
+    FloatRegister srcCopy = reusedInputSimd128Float(src, dest);
+    vshufps(mask, srcCopy, srcCopy, dest);
+  }
+
+  // Unused SIMD methods, defined in MacroAssemble-x86-shared-SIMD-unused.cpp.
+  // Don't use these without moving them out of that file and moving the
+  // declaration into the list above.
+
+  void checkedConvertFloat32x4ToInt32x4(FloatRegister src, FloatRegister dest,
+                                        Register temp, Label* oolCheck,
+                                        Label* rejoin);
+  void oolConvertFloat32x4ToInt32x4(FloatRegister src, Register temp,
+                                    Label* rejoin, Label* onConversionError);
+  void checkedConvertFloat32x4ToUint32x4(FloatRegister src, FloatRegister dest,
+                                         Register temp, FloatRegister tempF,
+                                         Label* failed);
+  void createInt32x4(Register lane0, Register lane1, Register lane2,
+                     Register lane3, FloatRegister dest);
+  void createFloat32x4(FloatRegister lane0, FloatRegister lane1,
+                       FloatRegister lane2, FloatRegister lane3,
+                       FloatRegister temp, FloatRegister output);
+  void reinterpretSimd(bool isIntegerLaneType, FloatRegister input,
+                       FloatRegister output);
+  void extractLaneSimdBool(FloatRegister input, Register output,
+                           unsigned numLanes, unsigned lane);
+  void allTrueSimdBool(FloatRegister input, Register output);
+  void anyTrueSimdBool(FloatRegister input, Register output);
+  void swizzleInt32x4(FloatRegister input, FloatRegister output,
+                      unsigned lanes[4]);
+  void swizzleFloat32x4(FloatRegister input, FloatRegister output,
+                        unsigned lanes[4]);
+  void oldSwizzleInt8x16(FloatRegister input, FloatRegister output,
+                         const mozilla::Maybe<Register>& temp,
+                         int8_t lanes[16]);
+  void shuffleX4(FloatRegister lhs, Operand rhs, FloatRegister out,
+                 const mozilla::Maybe<FloatRegister>& maybeTemp,
+                 unsigned lanes[4]);
+  void minNumFloat32x4(FloatRegister lhs, Operand rhs, FloatRegister temp,
+                       FloatRegister output);
+  void maxNumFloat32x4(FloatRegister lhs, Operand rhs, FloatRegister temp,
+                       FloatRegister output);
+
+  // Unused inline methods ditto.
+
+  void bitwiseAndSimdInt(FloatRegister lhs, const Operand& rhs,
+                         FloatRegister dest) {
+    vpand(rhs, lhs, dest);
+  }
+  void bitwiseOrSimdInt(FloatRegister lhs, const Operand& rhs,
+                        FloatRegister dest) {
+    vpor(rhs, lhs, dest);
+  }
+  void bitwiseOrFloat32x4(FloatRegister lhs, const Operand& rhs,
+                          FloatRegister dest) {
+    vorps(rhs, lhs, dest);
+  }
+  void bitwiseAndNotFloat32x4(FloatRegister lhs, const Operand& rhs,
+                              FloatRegister dest) {
+    vandnps(rhs, lhs, dest);
+  }
+  FloatRegister reusedInputAlignedInt32x4(const Operand& src,
+                                          FloatRegister dest) {
+    MOZ_ASSERT(dest.isSimd128());
+    if (HasAVX() && src.kind() == Operand::FPREG) {
+      return FloatRegister::FromCode(src.fpu());
+    }
+    loadAlignedSimd128Int(src, dest);
+    return dest;
   }
   void packedAddInt8(const Operand& src, FloatRegister dest) {
     vpaddb(src, dest, dest);
@@ -673,101 +772,15 @@ class MacroAssemblerX86Shared : public Assembler {
     // TODO See comment above. See also bug 1068028.
     vrsqrtps(src, dest);
   }
-  void packedSqrtFloat32x4(const Operand& src, FloatRegister dest) {
-    vsqrtps(src, dest);
-  }
-
- public:
-  void packedLeftShiftByScalarInt16x8(FloatRegister in, Register count,
-                                      Register temp, FloatRegister dest);
-  void packedRightShiftByScalarInt16x8(FloatRegister in, Register count,
-                                       Register temp, FloatRegister dest);
-  void packedUnsignedRightShiftByScalarInt16x8(FloatRegister in, Register count,
-                                               Register temp,
-                                               FloatRegister dest);
-
-  void packedLeftShiftByScalarInt16x8(Imm32 count, FloatRegister dest) {
-    count.value &= 15;
-    vpsllw(count, dest, dest);
-  }
-  void packedRightShiftByScalarInt16x8(Imm32 count, FloatRegister dest) {
-    count.value &= 15;
-    vpsraw(count, dest, dest);
-  }
-  void packedUnsignedRightShiftByScalarInt16x8(Imm32 count,
+  FloatRegister reusedInputAlignedSimd128Float(const Operand& src,
                                                FloatRegister dest) {
-    count.value &= 15;
-    vpsrlw(count, dest, dest);
-  }
-
-  void packedLeftShiftByScalarInt32x4(FloatRegister in, Register count,
-                                      Register temp, FloatRegister dest);
-  void packedRightShiftByScalarInt32x4(FloatRegister in, Register count,
-                                       Register temp, FloatRegister dest);
-  void packedUnsignedRightShiftByScalarInt32x4(FloatRegister in, Register count,
-                                               Register temp,
-                                               FloatRegister dest);
-
-  void packedLeftShiftByScalarInt32x4(Imm32 count, FloatRegister dest) {
-    count.value &= 31;
-    vpslld(count, dest, dest);
-  }
-  void packedRightShiftByScalarInt32x4(Imm32 count, FloatRegister dest) {
-    count.value &= 31;
-    vpsrad(count, dest, dest);
-  }
-  void packedUnsignedRightShiftByScalarInt32x4(Imm32 count,
-                                               FloatRegister dest) {
-    count.value &= 31;
-    vpsrld(count, dest, dest);
-  }
-
-  void loadAlignedSimd128Float(const Address& src, FloatRegister dest) {
-    vmovaps(Operand(src), dest);
-  }
-  void loadAlignedSimd128Float(const Operand& src, FloatRegister dest) {
-    vmovaps(src, dest);
-  }
-
-  void storeAlignedSimd128Float(FloatRegister src, const Address& dest) {
-    vmovaps(src, Operand(dest));
-  }
-  void moveSimd128Float(FloatRegister src, FloatRegister dest) {
-    vmovaps(src, dest);
-  }
-  FloatRegister reusedInputFloat32x4(FloatRegister src, FloatRegister dest) {
-    if (HasAVX()) {
-      return src;
-    }
-    moveSimd128Float(src, dest);
-    return dest;
-  }
-  FloatRegister reusedInputAlignedFloat32x4(const Operand& src,
-                                            FloatRegister dest) {
+    MOZ_ASSERT(dest.isSimd128());
     if (HasAVX() && src.kind() == Operand::FPREG) {
       return FloatRegister::FromCode(src.fpu());
     }
     loadAlignedSimd128Float(src, dest);
     return dest;
   }
-  void loadUnalignedSimd128Float(const Address& src, FloatRegister dest) {
-    vmovups(Operand(src), dest);
-  }
-  void loadUnalignedSimd128Float(const BaseIndex& src, FloatRegister dest) {
-    vmovdqu(Operand(src), dest);
-  }
-  void loadUnalignedSimd128Float(const Operand& src, FloatRegister dest) {
-    vmovups(src, dest);
-  }
-  void storeUnalignedSimd128Float(FloatRegister src, const Address& dest) {
-    vmovups(src, Operand(dest));
-  }
-  void storeUnalignedSimd128Float(FloatRegister src, const BaseIndex& dest) {
-    vmovups(src, Operand(dest));
-  }
-  void storeUnalignedSimd128Float(FloatRegister src, const Operand& dest) {
-    vmovups(src, dest);
-  }
   void packedAddFloat32(const Operand& src, FloatRegister dest) {
     vaddps(src, dest, dest);
   }
@@ -780,37 +793,14 @@ class MacroAssemblerX86Shared : public Assembler {
   void packedDivFloat32(const Operand& src, FloatRegister dest) {
     vdivps(src, dest, dest);
   }
-
-  static uint32_t ComputeShuffleMask(uint32_t x = 0, uint32_t y = 1,
-                                     uint32_t z = 2, uint32_t w = 3) {
-    MOZ_ASSERT(x < 4 && y < 4 && z < 4 && w < 4);
-    uint32_t r = (w << 6) | (z << 4) | (y << 2) | (x << 0);
-    MOZ_ASSERT(r < 256);
-    return r;
-  }
-
-  void shuffleInt32(uint32_t mask, FloatRegister src, FloatRegister dest) {
-    vpshufd(mask, src, dest);
-  }
-  void moveLowInt32(FloatRegister src, Register dest) { vmovd(src, dest); }
-
-  void moveHighPairToLowPairFloat32(FloatRegister src, FloatRegister dest) {
-    vmovhlps(src, dest, dest);
-  }
-  void shuffleFloat32(uint32_t mask, FloatRegister src, FloatRegister dest) {
-    // The shuffle instruction on x86 is such that it moves 2 words from
-    // the dest and 2 words from the src operands. To simplify things, just
-    // clobber the output with the input and apply the instruction
-    // afterwards.
-    // Note: this is useAtStart-safe because src isn't read afterwards.
-    FloatRegister srcCopy = reusedInputFloat32x4(src, dest);
-    vshufps(mask, srcCopy, srcCopy, dest);
-  }
   void shuffleMix(uint32_t mask, const Operand& src, FloatRegister dest) {
     // Note this uses vshufps, which is a cross-domain penalty on CPU where it
     // applies, but that's the way clang and gcc do it.
     vshufps(mask, src, dest, dest);
   }
+  void selectX4(FloatRegister mask, FloatRegister onTrue, FloatRegister onFalse,
+                FloatRegister temp, FloatRegister output);
+  // End unused SIMD.
 
   void moveFloatAsDouble(Register src, FloatRegister dest) {
     vmovd(src, dest);
@@ -887,6 +877,22 @@ class MacroAssemblerX86Shared : public Assembler {
     j(Assembler::NotEqual, fail);
   }
 
+  void truncateDoubleToInt32(FloatRegister src, Register dest, Label* fail) {
+    // vcvttsd2si returns 0x80000000 on failure. Test for it by
+    // subtracting 1 and testing overflow. The other possibility is to test
+    // equality for INT_MIN after a comparison, but 1 costs fewer bytes to
+    // materialize.
+    vcvttsd2si(src, dest);
+    cmp32(dest, Imm32(1));
+    j(Assembler::Overflow, fail);
+  }
+  void truncateFloat32ToInt32(FloatRegister src, Register dest, Label* fail) {
+    // Same trick as explained in the above comment.
+    vcvttss2si(src, dest);
+    cmp32(dest, Imm32(1));
+    j(Assembler::Overflow, fail);
+  }
+
   inline void clampIntToUint8(Register reg);
 
   bool maybeInlineDouble(double d, FloatRegister dest) {
@@ -917,13 +923,11 @@ class MacroAssemblerX86Shared : public Assembler {
   }
 
   bool maybeInlineSimd128Int(const SimdConstant& v, const FloatRegister& dest) {
-    static const SimdConstant zero = SimdConstant::SplatX4(0);
-    static const SimdConstant minusOne = SimdConstant::SplatX4(-1);
-    if (v == zero) {
+    if (v.isZeroBits()) {
       zeroSimd128Int(dest);
       return true;
     }
-    if (v == minusOne) {
+    if (v.isOneBits()) {
       vpcmpeqw(Operand(dest), dest, dest);
       return true;
     }
@@ -931,10 +935,7 @@ class MacroAssemblerX86Shared : public Assembler {
   }
   bool maybeInlineSimd128Float(const SimdConstant& v,
                                const FloatRegister& dest) {
-    static const SimdConstant zero = SimdConstant::SplatX4(0.f);
-    if (v == zero) {
-      // This won't get inlined if the SimdConstant v contains -0 in any
-      // lane, as operator== here does a memcmp.
+    if (v.isZeroBits()) {
       zeroSimd128Float(dest);
       return true;
     }
@@ -984,6 +985,21 @@ class MacroAssemblerX86Shared : public Assembler {
     }
   }
 
+  void emitSetRegisterIfZero(Register dest) {
+    if (AllocatableGeneralRegisterSet(Registers::SingleByteRegs).has(dest)) {
+      // If the register we're defining is a single byte register,
+      // take advantage of the setCC instruction
+      setCC(AssemblerX86Shared::Zero, dest);
+      movzbl(dest, dest);
+    } else {
+      Label end;
+      movl(Imm32(1), dest);
+      j(AssemblerX86Shared::Zero, &end);
+      mov(ImmWord(0), dest);
+      bind(&end);
+    }
+  }
+
   // Emit a JMP that can be toggled to a CMP. See ToggleToJmp(), ToggleToCmp().
   CodeOffset toggledJump(Label* label) {
     CodeOffset offset(size());
@@ -1000,8 +1016,6 @@ class MacroAssemblerX86Shared : public Assembler {
     // Exists for ARM compatibility.
   }
 
-  CodeOffset labelForPatch() { return CodeOffset(size()); }
-
   void abiret() { ret(); }
 
  protected:
