@@ -7,6 +7,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "js/friend/WindowProxy.h"  // js::SetWindowProxyClass
 #include "js/Wrapper.h"
 #include "jsapi-tests/tests.h"
 #include "vm/JSObject.h"
@@ -46,8 +47,7 @@ BEGIN_TEST(testBug604087) {
   js::WrapperOptions options;
   options.setClass(&OuterWrapperClass);
   JS::RootedObject outerObj(
-      cx,
-      js::Wrapper::NewSingleton(cx, global, &js::Wrapper::singleton, options));
+      cx, js::Wrapper::New(cx, global, &js::Wrapper::singleton, options));
   JS::RealmOptions globalOptions;
   JS::RootedObject compartment2(
       cx, JS_NewGlobalObject(cx, getGlobalClass(), nullptr,
@@ -78,8 +78,7 @@ BEGIN_TEST(testBug604087) {
   JS::RootedObject next(cx);
   {
     JSAutoRealm ar(cx, compartment2);
-    next = js::Wrapper::NewSingleton(cx, compartment2, &js::Wrapper::singleton,
-                                     options);
+    next = js::Wrapper::New(cx, compartment2, &js::Wrapper::singleton, options);
     CHECK(next);
   }
 
