@@ -5,8 +5,7 @@
 #include "js/CompilationAndEvaluation.h"
 #include "js/SourceText.h"
 #include "jsapi-tests/tests.h"
-
-using mozilla::ArrayLength;
+#include "util/Text.h"
 
 BEGIN_TEST(testJSEvaluateScript) {
   JS::RootedObject obj(cx, JS_NewPlainObject(cx));
@@ -20,8 +19,7 @@ BEGIN_TEST(testJSEvaluateScript) {
   CHECK(scopeChain.append(obj));
 
   JS::SourceText<char16_t> srcBuf;
-  CHECK(srcBuf.init(cx, src, ArrayLength(src) - 1,
-                    JS::SourceOwnership::Borrowed));
+  CHECK(srcBuf.init(cx, src, js_strlen(src), JS::SourceOwnership::Borrowed));
 
   CHECK(JS::Evaluate(cx, scopeChain, opts.setFileAndLine(__FILE__, __LINE__),
                      srcBuf, &retval));
