@@ -14,6 +14,7 @@
 #include "js/Value.h"
 
 #include "jsapi-tests/tests.h"
+#include "jsapi-tests/testsJit.h"
 
 #include "jit/MacroAssembler-inl.h"
 
@@ -25,46 +26,10 @@ using mozilla::PositiveInfinity;
 
 #if defined(JS_CODEGEN_X86) || defined(JS_CODEGEN_X64)
 
-typedef void (*EnterTest)();
-
-static bool Prepare(MacroAssembler& masm) {
-  AllocatableRegisterSet regs(RegisterSet::All());
-  LiveRegisterSet save(regs.asLiveSet());
-  masm.PushRegsInMask(save);
-  return true;
-}
-
-static bool Execute(JSContext* cx, MacroAssembler& masm) {
-  AllocatableRegisterSet regs(RegisterSet::All());
-  LiveRegisterSet save(regs.asLiveSet());
-  masm.PopRegsInMask(save);
-  masm.ret();  // Add return statement to be sure.
-
-  if (masm.oom()) {
-    return false;
-  }
-
-  Linker linker(masm, "Test");
-  JitCode* code = linker.newCode(cx, CodeKind::Other);
-  if (!code) {
-    return false;
-  }
-  if (!ExecutableAllocator::makeExecutable(code->raw(), code->bufferSize())) {
-    return false;
-  }
-
-  JS::AutoSuppressGCAnalysis suppress;
-  EnterTest test = code->as<EnterTest>();
-  test();
-  return true;
-}
-
 BEGIN_TEST(testJitMacroAssembler_flexibleDivMod) {
   StackMacroAssembler masm(cx);
 
-  if (!Prepare(masm)) {
-    return false;
-  }
+  PrepareJit(masm);
 
   // Test case divides 9/2;
   const uintptr_t quotient_result = 4;
@@ -113,16 +78,14 @@ BEGIN_TEST(testJitMacroAssembler_flexibleDivMod) {
     }
   }
 
-  return Execute(cx, masm);
+  return ExecuteJit(cx, masm);
 }
 END_TEST(testJitMacroAssembler_flexibleDivMod)
 
 BEGIN_TEST(testJitMacroAssembler_flexibleRemainder) {
   StackMacroAssembler masm(cx);
 
-  if (!Prepare(masm)) {
-    return false;
-  }
+  PrepareJit(masm);
 
   // Test case divides 9/2;
   const uintptr_t dividend = 9;
@@ -162,16 +125,14 @@ BEGIN_TEST(testJitMacroAssembler_flexibleRemainder) {
     }
   }
 
-  return Execute(cx, masm);
+  return ExecuteJit(cx, masm);
 }
 END_TEST(testJitMacroAssembler_flexibleRemainder)
 
 BEGIN_TEST(testJitMacroAssembler_flexibleQuotient) {
   StackMacroAssembler masm(cx);
 
-  if (!Prepare(masm)) {
-    return false;
-  }
+  PrepareJit(masm);
 
   // Test case divides 9/2;
   const uintptr_t dividend = 9;
@@ -211,7 +172,7 @@ BEGIN_TEST(testJitMacroAssembler_flexibleQuotient) {
     }
   }
 
-  return Execute(cx, masm);
+  return ExecuteJit(cx, masm);
 }
 END_TEST(testJitMacroAssembler_flexibleQuotient)
 
@@ -225,9 +186,7 @@ bool shiftTest(JSContext* cx, const char* name,
                const uintptr_t* result) {
   StackMacroAssembler masm(cx);
 
-  if (!Prepare(masm)) {
-    return false;
-  }
+  PrepareJit(masm);
 
   JS::AutoSuppressGCAnalysis suppress;
   AllocatableGeneralRegisterSet leftOutputHandSides(GeneralRegisterSet::All());
@@ -240,7 +199,7 @@ bool shiftTest(JSContext* cx, const char* name,
       Register rhs = rightHandSides.takeAny();
 
       // You can only use shift as the same reg if the values are the same
-      if (lhsOutput == rhs && lhsInput != rhsInput) {
+      if (lhsOutput == rhs && *lhsInput != *rhsInput) {
         continue;
       }
 
@@ -255,9 +214,11 @@ bool shiftTest(JSContext* cx, const char* name,
       masm.branch32(Assembler::NotEqual, AbsoluteAddress(result), lhsOutput,
                     &outputFail);
 
-      // Ensure RHS was not clobbered
-      masm.branch32(Assembler::NotEqual, AbsoluteAddress(rhsInput), rhs,
-                    &clobberRhs);
+      // Ensure RHS was not clobbered, unless it's also the output register.
+      if (lhsOutput != rhs) {
+        masm.branch32(Assembler::NotEqual, AbsoluteAddress(rhsInput), rhs,
+                      &clobberRhs);
+      }
 
       if (lhsOutput != ecx && rhs != ecx) {
         // If neither lhsOutput nor rhs is ecx, make sure ecx has been
@@ -294,7 +255,7 @@ bool shiftTest(JSContext* cx, const char* name,
     }
   }
 
-  return Execute(cx, masm);
+  return ExecuteJit(cx, masm);
 }
 
 BEGIN_TEST(testJitMacroAssembler_flexibleRshift) {
@@ -418,9 +379,7 @@ END_TEST(testJitMacroAssembler_flexibleLshift)
 BEGIN_TEST(testJitMacroAssembler_truncateDoubleToInt64) {
   StackMacroAssembler masm(cx);
 
-  if (!Prepare(masm)) {
-    return false;
-  }
+  PrepareJit(masm);
 
   AllocatableGeneralRegisterSet allRegs(GeneralRegisterSet::All());
   AllocatableFloatRegisterSet allFloatRegs(FloatRegisterSet::All());
@@ -459,16 +418,14 @@ BEGIN_TEST(testJitMacroAssembler_truncateDoubleToInt64) {
 
   masm.freeStack(sizeof(int32_t));
 
-  return Execute(cx, masm);
+  return ExecuteJit(cx, masm);
 }
 END_TEST(testJitMacroAssembler_truncateDoubleToInt64)
 
 BEGIN_TEST(testJitMacroAssembler_truncateDoubleToUInt64) {
   StackMacroAssembler masm(cx);
 
-  if (!Prepare(masm)) {
-    return false;
-  }
+  PrepareJit(masm);
 
   AllocatableGeneralRegisterSet allRegs(GeneralRegisterSet::All());
   AllocatableFloatRegisterSet allFloatRegs(FloatRegisterSet::All());
@@ -512,16 +469,14 @@ BEGIN_TEST(testJitMacroAssembler_truncateDoubleToUInt64) {
 
   masm.freeStack(sizeof(int32_t));
 
-  return Execute(cx, masm);
+  return ExecuteJit(cx, masm);
 }
 END_TEST(testJitMacroAssembler_truncateDoubleToUInt64)
 
 BEGIN_TEST(testJitMacroAssembler_branchDoubleNotInInt64Range) {
   StackMacroAssembler masm(cx);
 
-  if (!Prepare(masm)) {
-    return false;
-  }
+  PrepareJit(masm);
 
   AllocatableGeneralRegisterSet allRegs(GeneralRegisterSet::All());
   AllocatableFloatRegisterSet allFloatRegs(FloatRegisterSet::All());
@@ -566,16 +521,14 @@ BEGIN_TEST(testJitMacroAssembler_branchDoubleNotInInt64Range) {
 
   masm.freeStack(sizeof(int32_t));
 
-  return Execute(cx, masm);
+  return ExecuteJit(cx, masm);
 }
 END_TEST(testJitMacroAssembler_branchDoubleNotInInt64Range)
 
 BEGIN_TEST(testJitMacroAssembler_branchDoubleNotInUInt64Range) {
   StackMacroAssembler masm(cx);
 
-  if (!Prepare(masm)) {
-    return false;
-  }
+  PrepareJit(masm);
 
   AllocatableGeneralRegisterSet allRegs(GeneralRegisterSet::All());
   AllocatableFloatRegisterSet allFloatRegs(FloatRegisterSet::All());
@@ -623,16 +576,14 @@ BEGIN_TEST(testJitMacroAssembler_branchDoubleNotInUInt64Range) {
 
   masm.freeStack(sizeof(int32_t));
 
-  return Execute(cx, masm);
+  return ExecuteJit(cx, masm);
 }
 END_TEST(testJitMacroAssembler_branchDoubleNotInUInt64Range)
 
 BEGIN_TEST(testJitMacroAssembler_lshift64) {
   StackMacroAssembler masm(cx);
 
-  if (!Prepare(masm)) {
-    return false;
-  }
+  PrepareJit(masm);
 
   AllocatableGeneralRegisterSet allRegs(GeneralRegisterSet::All());
   AllocatableFloatRegisterSet allFloatRegs(FloatRegisterSet::All());
@@ -692,16 +643,14 @@ BEGIN_TEST(testJitMacroAssembler_lshift64) {
 
   masm.freeStack(sizeof(int32_t));
 
-  return Execute(cx, masm);
+  return ExecuteJit(cx, masm);
 }
 END_TEST(testJitMacroAssembler_lshift64)
 
 BEGIN_TEST(testJitMacroAssembler_rshift64Arithmetic) {
   StackMacroAssembler masm(cx);
 
-  if (!Prepare(masm)) {
-    return false;
-  }
+  PrepareJit(masm);
 
   AllocatableGeneralRegisterSet allRegs(GeneralRegisterSet::All());
   AllocatableFloatRegisterSet allFloatRegs(FloatRegisterSet::All());
@@ -761,16 +710,14 @@ BEGIN_TEST(testJitMacroAssembler_rshift64Arithmetic) {
 
   masm.freeStack(sizeof(int32_t));
 
-  return Execute(cx, masm);
+  return ExecuteJit(cx, masm);
 }
 END_TEST(testJitMacroAssembler_rshift64Arithmetic)
 
 BEGIN_TEST(testJitMacroAssembler_rshift64) {
   StackMacroAssembler masm(cx);
 
-  if (!Prepare(masm)) {
-    return false;
-  }
+  PrepareJit(masm);
 
   AllocatableGeneralRegisterSet allRegs(GeneralRegisterSet::All());
   AllocatableFloatRegisterSet allFloatRegs(FloatRegisterSet::All());
@@ -829,7 +776,7 @@ BEGIN_TEST(testJitMacroAssembler_rshift64) {
 
   masm.freeStack(sizeof(int32_t));
 
-  return Execute(cx, masm);
+  return ExecuteJit(cx, masm);
 }
 END_TEST(testJitMacroAssembler_rshift64)
 
