@@ -5,13 +5,14 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "js/Object.h"  // JS::GetClass
 #include "jsapi-tests/tests.h"
 
 BEGIN_TEST(testSetProperty_InheritedGlobalSetter) {
   // This is a JSAPI test because jsapi-test globals can be set up to not have
   // a resolve hook and therefore can use the property cache in some cases
   // where the shell can't.
-  MOZ_RELEASE_ASSERT(!JS_GetClass(global)->getResolve());
+  MOZ_RELEASE_ASSERT(!JS::GetClass(global)->getResolve());
 
   CHECK(JS::InitRealmStandardClasses(cx));
 
