@@ -4,12 +4,13 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "mozilla/ScopeExit.h"
-
 #include "gc/PublicIterators.h"
+#include "js/friend/WindowProxy.h"  // js::IsWindow, js::IsWindowProxy
 #include "js/Wrapper.h"
 #include "proxy/DeadObjectProxy.h"
+#include "proxy/DOMProxy.h"
 #include "vm/Iteration.h"
+#include "vm/Runtime.h"
 #include "vm/WrapperObject.h"
 
 #include "gc/Nursery-inl.h"
@@ -45,7 +46,7 @@ static bool MarkAtoms(JSContext* cx, HandleIdVector ids) {
 
 bool CrossCompartmentWrapper::getOwnPropertyDescriptor(
     JSContext* cx, HandleObject wrapper, HandleId id,
-    MutableHandle<PropertyDescriptor> desc) const {
+    MutableHandle<mozilla::Maybe<PropertyDescriptor>> desc) const {
   PIERCE(cx, wrapper, MarkAtoms(cx, id),
          Wrapper::getOwnPropertyDescriptor(cx, wrapper, id, desc),
          cx->compartment()->wrap(cx, desc));
@@ -81,11 +82,6 @@ bool CrossCompartmentWrapper::getPrototype(JSContext* cx, HandleObject wrapper,
     if (!GetPrototype(cx, wrapped, protop)) {
       return false;
     }
-    if (protop) {
-      if (!JSObject::setDelegate(cx, protop)) {
-        return false;
-      }
-    }
   }
 
   return cx->compartment()->wrap(cx, protop);
@@ -112,12 +108,6 @@ bool CrossCompartmentWrapper::getPrototypeIfOrdinary(
     if (!*isOrdinary) {
       return true;
     }
-
-    if (protop) {
-      if (!JSObject::setDelegate(cx, protop)) {
-        return false;
-      }
-    }
   }
 
   return cx->compartment()->wrap(cx, protop);
@@ -372,18 +362,7 @@ bool CrossCompartmentWrapper::boxedValue_unbox(JSContext* cx,
 
 const CrossCompartmentWrapper CrossCompartmentWrapper::singleton(0u);
 
-static void NukeRemovedCrossCompartmentWrapper(JSContext* cx,
-                                               JSObject* wrapper) {
-  MOZ_ASSERT(wrapper->is<CrossCompartmentWrapperObject>());
-
-  NotifyGCNukeWrapper(wrapper);
-
-  wrapper->as<ProxyObject>().nuke();
-
-  MOZ_ASSERT(IsDeadProxyObject(wrapper));
-}
-
-JS_FRIEND_API void js::NukeCrossCompartmentWrapper(JSContext* cx,
+JS_PUBLIC_API void js::NukeCrossCompartmentWrapper(JSContext* cx,
                                                    JSObject* wrapper) {
   JS::Compartment* comp = wrapper->compartment();
   auto ptr = comp->lookupWrapper(Wrapper::wrappedObject(wrapper));
@@ -393,13 +372,13 @@ JS_FRIEND_API void js::NukeCrossCompartmentWrapper(JSContext* cx,
   NukeRemovedCrossCompartmentWrapper(cx, wrapper);
 }
 
-JS_FRIEND_API void js::NukeCrossCompartmentWrapperIfExists(
+JS_PUBLIC_API void js::NukeCrossCompartmentWrapperIfExists(
     JSContext* cx, JS::Compartment* source, JSObject* target) {
   MOZ_ASSERT(source != target->compartment());
   MOZ_ASSERT(!target->is<CrossCompartmentWrapperObject>());
   auto ptr = source->lookupWrapper(target);
   if (ptr) {
-    JSObject* wrapper = &ptr->value().get().toObject();
+    JSObject* wrapper = ptr->value().get();
     NukeCrossCompartmentWrapper(cx, wrapper);
   }
 }
@@ -422,7 +401,7 @@ static bool NukedAllRealms(JS::Compartment* comp) {
  * that on tab close (outer window destruction).  Thus the option of how to
  * handle the global object.
  */
-JS_FRIEND_API bool js::NukeCrossCompartmentWrappers(
+JS_PUBLIC_API bool js::NukeCrossCompartmentWrappers(
     JSContext* cx, const CompartmentFilter& sourceFilter, JS::Realm* target,
     js::NukeReferencesToWindow nukeReferencesToWindow,
     js::NukeReferencesFromTarget nukeReferencesFromTarget) {
@@ -447,11 +426,9 @@ JS_FRIEND_API bool js::NukeCrossCompartmentWrappers(
          target->compartment() == c.get() && NukedAllRealms(c.get()));
 
     // Iterate only the wrappers that have target compartment matched unless
-    // |nukeAll| is true. The string wrappers that we're not interested in
-    // won't be iterated, we can exclude them easily because they have
-    // compartment nullptr. Use Maybe to avoid copying from conditionally
-    // initializing NonStringWrapperEnum.
-    mozilla::Maybe<Compartment::NonStringWrapperEnum> e;
+    // |nukeAll| is true. Use Maybe to avoid copying from conditionally
+    // initializing ObjectWrapperEnum.
+    mozilla::Maybe<Compartment::ObjectWrapperEnum> e;
     if (MOZ_LIKELY(!nukeAll)) {
       e.emplace(c, target->compartment());
     } else {
@@ -459,19 +436,13 @@ JS_FRIEND_API bool js::NukeCrossCompartmentWrappers(
       c.get()->nukedOutgoingWrappers = true;
     }
     for (; !e->empty(); e->popFront()) {
-      // Skip debugger references because NukeCrossCompartmentWrapper()
-      // doesn't know how to nuke them yet, see bug 1084626 for more
-      // information.
-      const CrossCompartmentKey& k = e->front().key();
-      if (!k.is<JSObject*>()) {
-        continue;
-      }
+      JSObject* key = e->front().key();
 
       AutoWrapperRooter wobj(cx, WrapperValue(*e));
 
-      // Unwrap from the wrapped object in CrossCompartmentKey instead of
-      // the wrapper, this could save us a bit of time.
-      JSObject* wrapped = UncheckedUnwrap(k.as<JSObject*>());
+      // Unwrap from the wrapped object in key instead of the wrapper, this
+      // could save us a bit of time.
+      JSObject* wrapped = UncheckedUnwrap(key);
 
       // Don't nuke wrappers for objects in other realms in the target
       // compartment unless nukeAll is set because in that case we want to nuke
@@ -503,7 +474,7 @@ JS_FRIEND_API bool js::NukeCrossCompartmentWrappers(
   return true;
 }
 
-JS_FRIEND_API bool js::AllowNewWrapper(JS::Compartment* target, JSObject* obj) {
+JS_PUBLIC_API bool js::AllowNewWrapper(JS::Compartment* target, JSObject* obj) {
   // Disallow creating new wrappers if we nuked the object realm or target
   // compartment. However, we always need to provide live wrappers for
   // ScriptSourceObjects, since they're used for cross-compartment cloned
@@ -524,7 +495,7 @@ JS_FRIEND_API bool js::AllowNewWrapper(JS::Compartment* target, JSObject* obj) {
   return true;
 }
 
-JS_FRIEND_API bool js::NukedObjectRealm(JSObject* obj) {
+JS_PUBLIC_API bool js::NukedObjectRealm(JSObject* obj) {
   return obj->nonCCWRealm()->nukedIncomingWrappers;
 }
 
@@ -544,9 +515,6 @@ void js::RemapWrapper(JSContext* cx, JSObject* wobjArg,
   MOZ_ASSERT(!newTarget->is<CrossCompartmentWrapperObject>());
   JSObject* origTarget = Wrapper::wrappedObject(wobj);
   MOZ_ASSERT(origTarget);
-  MOZ_ASSERT(!JS_IsDeadWrapper(origTarget),
-             "We don't want a dead proxy in the wrapper map");
-  Value origv = ObjectValue(*origTarget);
   JS::Compartment* wcompartment = wobj->compartment();
   MOZ_ASSERT(wcompartment != newTarget->compartment());
 
@@ -556,33 +524,54 @@ void js::RemapWrapper(JSContext* cx, JSObject* wobjArg,
   // for the same target), we must not have an existing wrapper for the new
   // target, otherwise this will break.
   MOZ_ASSERT_IF(origTarget != newTarget,
-                !wcompartment->lookupWrapper(ObjectValue(*newTarget)));
+                !wcompartment->lookupWrapper(newTarget));
 
   // The old value should still be in the cross-compartment wrapper map, and
   // the lookup should return wobj.
-  WrapperMap::Ptr p = wcompartment->lookupWrapper(origv);
-  MOZ_ASSERT(&p->value().unsafeGet()->toObject() == wobj);
+  ObjectWrapperMap::Ptr p = wcompartment->lookupWrapper(origTarget);
+  MOZ_ASSERT(*p->value().unsafeGet() == wobj);
   wcompartment->removeWrapper(p);
 
   // When we remove origv from the wrapper map, its wrapper, wobj, must
   // immediately cease to be a cross-compartment wrapper. Nuke it.
   NukeCrossCompartmentWrapper(cx, wobj);
 
-  // wobj is no longer a cross-compartment wrapper after nuking it, so we can
-  // now use nonCCWRealm.
+  // If the target is a dead wrapper, and we're just fixing wrappers for
+  // it, then we're done now that the CCW is a dead wrapper.
+  if (JS_IsDeadWrapper(origTarget)) {
+    MOZ_RELEASE_ASSERT(origTarget == newTarget);
+    return;
+  }
+
+  js::RemapDeadWrapper(cx, wobj, newTarget);
+}
+
+// Given a dead proxy object |wobj|, turn it into a cross-compartment wrapper
+// pointing at |newTarget|.
+// This operation crashes on failure rather than leaving the heap in an
+// inconsistent state.
+void js::RemapDeadWrapper(JSContext* cx, HandleObject wobj,
+                          HandleObject newTarget) {
+  MOZ_ASSERT(IsDeadProxyObject(wobj));
+  MOZ_ASSERT(!newTarget->is<CrossCompartmentWrapperObject>());
+
+  AutoDisableProxyCheck adpc;
+
+  // wobj is not a cross-compartment wrapper, so we can use nonCCWRealm.
   Realm* wrealm = wobj->nonCCWRealm();
 
   // First, we wrap it in the new compartment. We try to use the existing
-  // wrapper, |wobj|, since it's been nuked anyway. The wrap() function has
+  // wrapper, |wobj|, since it's been nuked anyway. The rewrap() function has
   // the choice to reuse |wobj| or not.
   RootedObject tobj(cx, newTarget);
   AutoRealmUnchecked ar(cx, wrealm);
   AutoEnterOOMUnsafeRegion oomUnsafe;
+  JS::Compartment* wcompartment = wobj->compartment();
   if (!wcompartment->rewrap(cx, &tobj, wobj)) {
     oomUnsafe.crash("js::RemapWrapper");
   }
 
-  // If wrap() reused |wobj|, it will have overwritten it and returned with
+  // If rewrap() reused |wobj|, it will have overwritten it and returned with
   // |tobj == wobj|. Otherwise, |tobj| will point to a new wrapper and |wobj|
   // will still be nuked. In the latter case, we replace |wobj| with the
   // contents of the new wrapper in |tobj|.
@@ -590,37 +579,40 @@ void js::RemapWrapper(JSContext* cx, JSObject* wobjArg,
     // Now, because we need to maintain object identity, we do a brain
     // transplant on the old object so that it contains the contents of the
     // new one.
-    JSObject::swap(cx, wobj, tobj);
+    JSObject::swap(cx, wobj, tobj, oomUnsafe);
   }
 
-  // Before swapping, this wrapper came out of wrap(), which enforces the
+  if (!wobj->is<WrapperObject>()) {
+    MOZ_ASSERT(js::IsDOMRemoteProxyObject(wobj));
+    return;
+  }
+
+  // Before swapping, this wrapper came out of rewrap(), which enforces the
   // invariant that the wrapper in the map points directly to the key.
   MOZ_ASSERT(Wrapper::wrappedObject(wobj) == newTarget);
 
+  // Update the incremental weakmap marking state.
+  wobj->zone()->afterAddDelegate(wobj);
+
   // Update the entry in the compartment's wrapper map to point to the old
   // wrapper, which has now been updated (via reuse or swap).
-  MOZ_ASSERT(wobj->is<WrapperObject>());
-  if (!wcompartment->putWrapper(cx, CrossCompartmentKey(newTarget),
-                                ObjectValue(*wobj))) {
+  if (!wcompartment->putWrapper(cx, newTarget, wobj)) {
     oomUnsafe.crash("js::RemapWrapper");
   }
 }
 
 // Remap all cross-compartment wrappers pointing to |oldTarget| to point to
 // |newTarget|. All wrappers are recomputed.
-JS_FRIEND_API bool js::RemapAllWrappersForObject(JSContext* cx,
-                                                 JSObject* oldTargetArg,
-                                                 JSObject* newTargetArg) {
-  MOZ_ASSERT(!IsInsideNursery(oldTargetArg));
-  MOZ_ASSERT(!IsInsideNursery(newTargetArg));
-
-  RootedValue origv(cx, ObjectValue(*oldTargetArg));
-  RootedObject newTarget(cx, newTargetArg);
+JS_PUBLIC_API bool js::RemapAllWrappersForObject(JSContext* cx,
+                                                 HandleObject oldTarget,
+                                                 HandleObject newTarget) {
+  MOZ_ASSERT(!IsInsideNursery(oldTarget));
+  MOZ_ASSERT(!IsInsideNursery(newTarget));
 
   AutoWrapperVector toTransplant(cx);
 
   for (CompartmentsIter c(cx->runtime()); !c.done(); c.next()) {
-    if (WrapperMap::Ptr wp = c->lookupWrapper(origv)) {
+    if (ObjectWrapperMap::Ptr wp = c->lookupWrapper(oldTarget)) {
       // We found a wrapper. Remember and root it.
       if (!toTransplant.append(WrapperValue(wp))) {
         return false;
@@ -629,13 +621,13 @@ JS_FRIEND_API bool js::RemapAllWrappersForObject(JSContext* cx,
   }
 
   for (const WrapperValue& v : toTransplant) {
-    RemapWrapper(cx, &v.toObject(), newTarget);
+    RemapWrapper(cx, v, newTarget);
   }
 
   return true;
 }
 
-JS_FRIEND_API bool js::RecomputeWrappers(
+JS_PUBLIC_API bool js::RecomputeWrappers(
     JSContext* cx, const CompartmentFilter& sourceFilter,
     const CompartmentFilter& targetFilter) {
   bool evictedNursery = false;
@@ -647,21 +639,16 @@ JS_FRIEND_API bool js::RecomputeWrappers(
       continue;
     }
 
-    if (!evictedNursery && c->hasNurseryAllocatedWrapperEntries(targetFilter)) {
+    if (!evictedNursery &&
+        c->hasNurseryAllocatedObjectWrapperEntries(targetFilter)) {
       cx->runtime()->gc.evictNursery();
       evictedNursery = true;
     }
 
-    // Iterate over the wrappers, filtering appropriately.
-    for (Compartment::NonStringWrapperEnum e(c, targetFilter); !e.empty();
+    // Iterate over object wrappers, filtering appropriately.
+    for (Compartment::ObjectWrapperEnum e(c, targetFilter); !e.empty();
          e.popFront()) {
-      // Filter out non-objects.
-      CrossCompartmentKey& k = e.front().mutableKey();
-      if (!k.is<JSObject*>()) {
-        continue;
-      }
-
-      // Add it to the list.
+      // Add the wrapper to the list.
       if (!toRecompute.append(WrapperValue(e))) {
         return false;
       }
@@ -669,8 +656,7 @@ JS_FRIEND_API bool js::RecomputeWrappers(
   }
 
   // Recompute all the wrappers in the list.
-  for (const WrapperValue& v : toRecompute) {
-    JSObject* wrapper = &v.toObject();
+  for (const WrapperValue& wrapper : toRecompute) {
     JSObject* wrapped = Wrapper::wrappedObject(wrapper);
     RemapWrapper(cx, wrapper, wrapped);
   }
