@@ -8,6 +8,9 @@
 
 #include "jsexn.h"
 
+#include "js/friend/ErrorMessages.h"  // js::GetErrorMessage, JSMSG_*
+#include "js/friend/WindowProxy.h"    // js::IsWindowProxy
+#include "js/Object.h"                // JS::GetBuiltinClass
 #include "js/Proxy.h"
 #include "vm/ErrorObject.h"
 #include "vm/JSContext.h"
@@ -44,7 +47,7 @@ bool Wrapper::finalizeInBackground(const Value& priv) const {
 
 bool ForwardingProxyHandler::getOwnPropertyDescriptor(
     JSContext* cx, HandleObject proxy, HandleId id,
-    MutableHandle<PropertyDescriptor> desc) const {
+    MutableHandle<mozilla::Maybe<PropertyDescriptor>> desc) const {
   assertEnteredPolicy(cx, proxy, id, GET | SET | GET_PROPERTY_DESCRIPTOR);
   RootedObject target(cx, proxy->as<ProxyObject>().target());
   return GetOwnPropertyDescriptor(cx, target, id, desc);
@@ -224,7 +227,7 @@ bool ForwardingProxyHandler::hasInstance(JSContext* cx, HandleObject proxy,
 bool ForwardingProxyHandler::getBuiltinClass(JSContext* cx, HandleObject proxy,
                                              ESClass* cls) const {
   RootedObject target(cx, proxy->as<ProxyObject>().target());
-  return GetBuiltinClass(cx, target, cls);
+  return JS::GetBuiltinClass(cx, target, cls);
 }
 
 bool ForwardingProxyHandler::isArray(JSContext* cx, HandleObject proxy,
@@ -282,19 +285,6 @@ JSObject* Wrapper::New(JSContext* cx, JSObject* obj, const Wrapper* handler,
   return NewProxyObject(cx, handler, priv, options.proto(), options);
 }
 
-JSObject* Wrapper::NewSingleton(JSContext* cx, JSObject* obj,
-                                const Wrapper* handler,
-                                const WrapperOptions& options) {
-  // If this is a cross-compartment wrapper allocate it in the compartment's
-  // first global. See Compartment::globalForNewCCW.
-  mozilla::Maybe<AutoRealm> ar;
-  if (handler->isCrossCompartmentWrapper()) {
-    ar.emplace(cx, &cx->compartment()->globalForNewCCW());
-  }
-  RootedValue priv(cx, ObjectValue(*obj));
-  return NewSingletonProxyObject(cx, handler, priv, options.proto(), options);
-}
-
 JSObject* Wrapper::Renew(JSObject* existing, JSObject* obj,
                          const Wrapper* handler) {
   existing->as<ProxyObject>().renew(handler, ObjectValue(*obj));
@@ -329,7 +319,7 @@ JSObject* Wrapper::wrappedObject(JSObject* wrapper) {
   return target;
 }
 
-JS_FRIEND_API JSObject* js::UncheckedUnwrapWithoutExpose(JSObject* wrapped) {
+JS_PUBLIC_API JSObject* js::UncheckedUnwrapWithoutExpose(JSObject* wrapped) {
   while (true) {
     if (!wrapped->is<WrapperObject>() || MOZ_UNLIKELY(IsWindowProxy(wrapped))) {
       break;
@@ -345,7 +335,7 @@ JS_FRIEND_API JSObject* js::UncheckedUnwrapWithoutExpose(JSObject* wrapped) {
   return wrapped;
 }
 
-JS_FRIEND_API JSObject* js::UncheckedUnwrap(JSObject* wrapped,
+JS_PUBLIC_API JSObject* js::UncheckedUnwrap(JSObject* wrapped,
                                             bool stopAtWindowProxy,
                                             unsigned* flagsp) {
   MOZ_ASSERT(!JS::RuntimeHeapIsCollecting());
@@ -366,7 +356,7 @@ JS_FRIEND_API JSObject* js::UncheckedUnwrap(JSObject* wrapped,
   return wrapped;
 }
 
-JS_FRIEND_API JSObject* js::CheckedUnwrapStatic(JSObject* obj) {
+JS_PUBLIC_API JSObject* js::CheckedUnwrapStatic(JSObject* obj) {
   while (true) {
     JSObject* wrapper = obj;
     obj = UnwrapOneCheckedStatic(obj);
@@ -376,7 +366,7 @@ JS_FRIEND_API JSObject* js::CheckedUnwrapStatic(JSObject* obj) {
   }
 }
 
-JS_FRIEND_API JSObject* js::UnwrapOneCheckedStatic(JSObject* obj) {
+JS_PUBLIC_API JSObject* js::UnwrapOneCheckedStatic(JSObject* obj) {
   MOZ_ASSERT(!JS::RuntimeHeapIsCollecting());
   MOZ_ASSERT(CurrentThreadCanAccessRuntime(obj->runtimeFromAnyThread()));
 
@@ -392,7 +382,7 @@ JS_FRIEND_API JSObject* js::UnwrapOneCheckedStatic(JSObject* obj) {
   return handler->hasSecurityPolicy() ? nullptr : Wrapper::wrappedObject(obj);
 }
 
-JS_FRIEND_API JSObject* js::CheckedUnwrapDynamic(JSObject* obj, JSContext* cx,
+JS_PUBLIC_API JSObject* js::CheckedUnwrapDynamic(JSObject* obj, JSContext* cx,
                                                  bool stopAtWindowProxy) {
   RootedObject wrapper(cx, obj);
   while (true) {
@@ -405,7 +395,7 @@ JS_FRIEND_API JSObject* js::CheckedUnwrapDynamic(JSObject* obj, JSContext* cx,
   }
 }
 
-JS_FRIEND_API JSObject* js::UnwrapOneCheckedDynamic(HandleObject obj,
+JS_PUBLIC_API JSObject* js::UnwrapOneCheckedDynamic(HandleObject obj,
                                                     JSContext* cx,
                                                     bool stopAtWindowProxy) {
   MOZ_ASSERT(!JS::RuntimeHeapIsCollecting());
