@@ -3,10 +3,14 @@
 
 /*---
 esid: sec-array.isarray
-es5id: 15.4.3.2-1-9
-description: Array.isArray applied to Date object
+es5id: 15.4.3.2-0-7
+description: Array.isArray returns false if its argument is not an Array
 ---*/
 
-assert.sameValue(Array.isArray(new Date()), false, 'Array.isArray(new Date())');
+var o = new Object();
+o[12] = 13;
+var b = Array.isArray(o);
+
+assert.sameValue(b, false, 'b');
 
 reportCompare(0, 0);
