@@ -3,10 +3,9 @@
 
 /*---
 esid: sec-array.prototype.filter
-es5id: 15.4.4.20-9-c-iii-1-4
 description: >
     Array.prototype.filter - value of returned array element can be
-    changed or deleted
+    overwritten
 ---*/
 
 function callbackfn(val, idx, obj) {
@@ -21,9 +20,8 @@ var obj = {
 var newArr = Array.prototype.filter.call(obj, callbackfn);
 
 var tempVal = newArr[1];
-delete newArr[1];
+newArr[1] += 1;
 
-assert.notSameValue(tempVal, undefined, 'tempVal');
-assert.sameValue(newArr[1], undefined, 'newArr[1]');
+assert.notSameValue(newArr[1], tempVal, 'newArr[1]');
 
 reportCompare(0, 0);
