@@ -3,10 +3,10 @@
 
 /*---
 info: |
-    The Boolean prototype object is itself a Boolean object
-    (its [[Class]] is "Boolean") whose value is false
-esid: sec-properties-of-the-boolean-prototype-object
-description: Checking type and value of Boolean.prototype
+    The initial value of Boolean.prototype is the Boolean
+    prototype object
+esid: sec-boolean.prototype
+description: Checking Boolean.prototype property
 ---*/
 
 //CHECK#1
