@@ -2,37 +2,34 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.7-6-a-87
+es5id: 15.2.3.7-6-a-83
 description: >
     Object.defineProperties throws TypeError when P.configurable is
-    false, both properties.[[Set]] and P.[[Set]] are two objects which
-    refer to different objects (8.12.9 step 11.a.i)
+    false, P.writalbe is false, properties.value and P.value are two
+    booleans with different values (8.12.9 step 10.a.ii.1)
 includes: [propertyHelper.js]
 ---*/
 
 
 var obj = {};
 
-function set_func1(value) {
-  obj.setVerifyHelpProp = value;
-}
-
 Object.defineProperty(obj, "foo", {
-  set: set_func1,
+  value: false,
+  writable: false,
   configurable: false
 });
 
-function set_func2() {}
-
 try {
   Object.defineProperties(obj, {
     foo: {
-      set: set_func2
+      value: true
     }
   });
   $ERROR("Expected an exception.");
 } catch (e) {
-  verifyWritable(obj, "foo", "setVerifyHelpProp");
+  verifyEqualTo(obj, "foo", false);
+
+  verifyNotWritable(obj, "foo");
 
   verifyNotEnumerable(obj, "foo");
 
