@@ -2,30 +2,33 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.9-2-a-2
+es5id: 15.2.3.9-2-a-5
 description: >
-    Object.freeze - 'P' is own data property that overrides an
+    Object.freeze - 'P' is own accessor property that overrides an
     inherited data property
 includes: [propertyHelper.js]
 ---*/
 
 
-var proto = {
-  foo: 0
-}; // default [[Configurable]] attribute value of foo: true
+var proto = {};
+
+proto.foo = 0; // default [[Configurable]] attribute value of foo: true
 
 var Con = function() {};
 Con.prototype = proto;
 
 var child = new Con();
 
-child.foo = 10; // default [[Configurable]] attribute value of foo: true
+Object.defineProperty(child, "foo", {
+  get: function() {
+    return 10;
+  },
+  configurable: true
+});
 
 Object.freeze(child);
 
-verifyNotWritable(child, "foo");
 verifyNotConfigurable(child, "foo");
-
 assert.sameValue(child.foo, 10);
 
 reportCompare(0, 0);
