@@ -2,15 +2,15 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.3-4-143
+es5id: 15.2.3.3-4-165
 description: >
     Object.getOwnPropertyDescriptor returns data desc for functions on
-    built-ins (Date.prototype.setSeconds)
+    built-ins (RegExp.prototype.exec)
 ---*/
 
-var desc = Object.getOwnPropertyDescriptor(Date.prototype, "setSeconds");
+var desc = Object.getOwnPropertyDescriptor(RegExp.prototype, "exec");
 
-assert.sameValue(desc.value, Date.prototype.setSeconds, 'desc.value');
+assert.sameValue(desc.value, RegExp.prototype.exec, 'desc.value');
 assert.sameValue(desc.writable, true, 'desc.writable');
 assert.sameValue(desc.enumerable, false, 'desc.enumerable');
 assert.sameValue(desc.configurable, true, 'desc.configurable');
