@@ -25,6 +25,6 @@ p.finally(function() {
   sequence.push(2);
   assert.sameValue(reason, thrown, 'onFinally can override the rejection reason by throwing');
 }).then(function() {
+  assert.sameValue(sequence.length, 2);
   checkSequence(sequence, "All expected callbacks called in correct order");
-  $DONE();
-}).catch($ERROR);
+}).then($DONE, $DONE);
