@@ -3,35 +3,35 @@
 // See LICENSE for details.
 
 /*---
-es6id: S25.4.4.3_A7.1_T1
+es6id: S25.4.4.3_A6.2_T1
 author: Sam Mikes
-description: Promise.race([p1, p2]) settles when first settles
+description: Promise.race([p1]) settles immediately
 includes: [promiseHelper.js]
 flags: [async]
 ---*/
 
 var sequence = [];
 
-var p1 = Promise.resolve(1),
-  p2 = Promise.resolve(2),
-  p = Promise.race([p1, p2]);
+var p1 = Promise.reject(1),
+  p = Promise.race([p1]);
 
 sequence.push(1);
 
-p.then(function(arg) {
-  if (arg !== 1) {
-    $ERROR("Expected promise to be fulfilled with 1, got " + arg);
-  }
-
+p.then(function() {
+  $ERROR("Should not fulfill.");
+}, function() {
   sequence.push(4);
-  checkSequence(sequence, "This happens second");
+  assert.sameValue(sequence.length, 4);
+checkSequence(sequence, "This happens second");
 }).catch($DONE);
 
 Promise.resolve().then(function() {
   sequence.push(3);
+  assert.sameValue(sequence.length, 3);
   checkSequence(sequence, "This happens first");
 }).then(function() {
   sequence.push(5);
+  assert.sameValue(sequence.length, 5);
   checkSequence(sequence, "This happens third");
 }).then($DONE, $DONE);
 
