@@ -1,7 +1,7 @@
 // Copyright (C) 2015 the V8 project authors. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 /*---
-es6id: 9.5.14
+esid: sec-proxy-object-internal-methods-and-internal-slots-construct-argumentslist-newtarget
 description: >
     Throws a TypeError if trap result is not an Object: Symbol
 info: |
@@ -11,10 +11,11 @@ info: |
 features: [Proxy, Symbol]
 ---*/
 
-function Target() {
-  this.attr = "done";
-};
-var P = new Proxy(Target, {
+function Target() {}
+
+var P = new Proxy(function() {
+  throw new Test262Error('target should not be called');
+}, {
   construct: function() {
     return Symbol();
   }
