@@ -2,12 +2,12 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.5.4.20-4-39
+es5id: 15.5.4.20-4-10
 description: >
     String.prototype.trim handles whitepace and lineterminators
-    (ab\u0085c)
+    (\uFEFFabc)
 ---*/
 
-assert.sameValue("ab\u0085c".trim(), "ab\u0085c", '"ab\u0085c".trim()');
+assert.sameValue("\uFEFFabc".trim(), "abc", '"\uFEFFabc".trim()');
 
 reportCompare(0, 0);
