@@ -1,4 +1,3 @@
-// |reftest| skip -- Intl.DateTimeFormat-formatRange is not supported
 // Copyright 2019 Google, Inc.  All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 
@@ -8,12 +7,17 @@ description: >
 info: |
   Intl.DateTimeFormat.prototype.formatRangeToParts ( startDate , endDate )
 
-  1. Let dtf be this value.
-  2. If Type(dtf) is not Object, throw a TypeError exception.
-  3. If dtf does not have an [[InitializedDateTimeFormat]] internal slot, throw a TypeError exception.
-  5. Let x be ? ToNumber(startDate).
-  6. Let y be ? ToNumber(endDate).
-  7. If x is greater than y, throw a RangeError exception.
+  4. Let x be ? ToNumber(startDate).
+  5. Let y be ? ToNumber(endDate).
+  6. Return ? FormatDateTimeRangeToParts(dtf, x, y).
+
+  PartitionDateTimeRangePattern ( dateTimeFormat, x, y )
+
+  1. Let x be TimeClip(x).
+  2. If x is NaN, throw a RangeError exception.
+  3. Let y be TimeClip(y).
+  4. If y is NaN, throw a RangeError exception.
+  5. If x is greater than y, throw a RangeError exception.
 
 features: [Intl.DateTimeFormat-formatRange]
 ---*/
