@@ -2,18 +2,16 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 14.1-4-s
-description: >
-    'use strict' directive - not recognized if contains Line
-    Continuation
+es5id: 14.1-14-s
+description: semicolon insertion may come before 'use strict' directive
 flags: [noStrict]
 ---*/
 
   function foo()
   {
-    'use str\
-ict';
-     return (this !== undefined);
+    "another directive"
+    "use strict" ;
+    return (this === undefined);
   }
 
 assert(foo.call(undefined));
