@@ -2,25 +2,24 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-info: Operator x @= y returns x @ y
-es5id: 11.13.2_A3.2_T6
+info: Operator x @= y uses PutValue(x, x @ y)
+es5id: 11.13.2_A3.1_T6
 description: Checking Expression and Variable statements for x <<= y
 ---*/
 
 //CHECK#1
 var x = 1;
-var x1 = (x <<= 1);
-if (x1 !== 2) {
-  $ERROR('#1: var x = 1; var x1 = (x <<= 1); x1 === 2. Actual: ' + (x1));
+x <<= 1; 
+if (x !== 2) {
+  $ERROR('#1: var x = 1; x <<= 1; x === 2. Actual: ' + (x));
 }
 
 //CHECK#2
 var y;
-var y1;
 y = 1;
-y1 = (y <<= 1);
-if (y1 !== 2) {
-  $ERROR('#2: y = 1; y1 = (y <<= 1); y1 === 2. Actual: ' + (y1));
+y <<= 1;
+if (y !== 2) {
+  $ERROR('#2: y = 1; y <<= 1; y === 2. Actual: ' + (y));
 }
 
 reportCompare(0, 0);
