@@ -2,16 +2,15 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-info: If the property doesn't have the DontDelete attribute, return true
-es5id: 11.4.1_A3.2_T1
-description: Checking declared variable
+info: If GetBase(x) doesn't have a property GetPropertyName(x), return true
+esid: sec-delete-operator-runtime-semantics-evaluation
+description: Checking undeclared variable case
 flags: [noStrict]
 ---*/
 
 //CHECK#1
-x = 1;
 if (delete x !== true) {
-  $ERROR('#1: x = 1; delete x === true');
+  $ERROR('#1: delete x === true');
 }
 
 reportCompare(0, 0);
