@@ -2,14 +2,14 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 10.4.3-1-73-s
+es5id: 10.4.3-1-72-s
 description: >
     checking 'this' (strict function declaration called by
-    Function.prototype.call(undefined))
+    Function.prototype.call(null))
 ---*/
 
-function f() { "use strict"; return this===undefined;};
+function f() { "use strict"; return this===null;};
 
-assert(f.call(undefined), 'f.call(undefined) !== true');
+assert(f.call(null), 'f.call(null) !== true');
 
 reportCompare(0, 0);
