@@ -5,11 +5,10 @@
 info: |
     The with statement adds a computed object to the front of the
     scope chain of the current execution context
-es5id: 12.10_A1.2_T1
+es5id: 12.10_A1.12_T1
 description: >
-    Calling a function without "with" statement when the statement
-    itself is declared within the function declaration, leading to
-    normal completion
+    Calling a function without "with" statement declared within the
+    statement, leading to normal completion
 flags: [noStrict]
 ---*/
 
@@ -42,8 +41,8 @@ var st_parseFloat = "parseFloat";
 var st_isNaN = "isNaN";
 var st_isFinite = "isFinite";
 
-var f = function(){
-  with(myObj){
+with(myObj){
+  var f = function(){
     st_p1 = p1;
     st_p2 = p2;
     st_p3 = p3;
@@ -76,10 +75,11 @@ if(!(p3 === 3)){
   $ERROR('#3: p3 === 3. Actual:  p3 ==='+ p3  );
 }
 
-try {
+try{
   p4;
-  $ERROR('#4: p4 is not defined');
-} catch(e) {    
+  $ERROR('#4: p4 doesn\'t exists');
+}
+catch(e){
 }
 
 if(!(p5 === "x5")){
@@ -134,15 +134,14 @@ if(!(st_isFinite !== isFinite)){
   $ERROR('#17: myObj.isFinite !== isFinite');
 }
 
-try{
+try {
   value;
   $ERROR('#18: value is not defined');
-}
-catch(e){
+} catch(e) {    
 }
 
-if(!(myObj.value === "value")){
-  $ERROR('#19: myObj.value === "value". Actual:  myObj.value ==='+ myObj.value  );
+if(!(myObj.value === "myObj_value")){
+  $ERROR('#19: myObj.value === "myObj_value". Actual:  myObj.value ==='+ myObj.value  );
 }
 
 reportCompare(0, 0);
