@@ -6,14 +6,13 @@
 
 #include "vm/DateTime.h"
 
-#include "mozilla/ArrayUtils.h"
 #include "mozilla/ScopeExit.h"
 #include "mozilla/TextUtils.h"
-#include "mozilla/Unused.h"
 
 #include <algorithm>
 #include <cstdlib>
 #include <cstring>
+#include <iterator>
 #include <time.h>
 
 #if !defined(XP_WIN)
@@ -22,6 +21,7 @@
 #endif /* !defined(XP_WIN) */
 
 #include "js/Date.h"
+#include "js/GCAPI.h"
 #include "threading/ExclusiveData.h"
 
 #if JS_HAS_INTL_API && !MOZ_SYSTEM_ICU
@@ -594,6 +594,36 @@ static inline const char* TZContainsAbsolutePath(const char* tzVar) {
   return nullptr;
 }
 
+/**
+ * Reject the input if it doesn't match the time zone id pattern or legacy time
+ * zone names.
+ *
+ * See <https://github.com/eggert/tz/blob/master/theory.html>.
+ */
+static icu::UnicodeString MaybeTimeZoneId(const char* timeZone) {
+  size_t timeZoneLen = std::strlen(timeZone);
+
+  for (size_t i = 0; i < timeZoneLen; i++) {
+    char c = timeZone[i];
+
+    // According to theory.html, '.' is allowed in time zone ids, but the
+    // accompanying zic.c file doesn't allow it. Assume the source file is
+    // correct and disallow '.' here, too.
+    if (mozilla::IsAsciiAlphanumeric(c) || c == '_' || c == '-' || c == '+') {
+      continue;
+    }
+
+    // Reject leading, trailing, or consecutive '/' characters.
+    if (c == '/' && i > 0 && i + 1 < timeZoneLen && timeZone[i + 1] != '/') {
+      continue;
+    }
+
+    return icu::UnicodeString();
+  }
+
+  return icu::UnicodeString(timeZone, timeZoneLen, US_INV);
+}
+
 /**
  * Given a presumptive path |tz| to a zoneinfo time zone file
  * (e.g. /etc/localtime), attempt to compute the time zone encoded by that
@@ -609,8 +639,7 @@ static icu::UnicodeString ReadTimeZoneLink(const char* tz) {
   // The resolved link name can have different paths depending on the OS.
   // Follow ICU and only search for "/zoneinfo/"; see $ICU/common/putil.cpp.
   static constexpr char ZoneInfoPath[] = "/zoneinfo/";
-  constexpr size_t ZoneInfoPathLength =
-      mozilla::ArrayLength(ZoneInfoPath) - 1;  // exclude NUL
+  constexpr size_t ZoneInfoPathLength = js_strlen(ZoneInfoPath);
 
   // Stop following symlinks after a fixed depth, because some common time
   // zones are stored in files whose name doesn't match an Olson time zone
@@ -629,7 +658,7 @@ static icu::UnicodeString ReadTimeZoneLink(const char* tz) {
 
   char linkName[PathMax];
   constexpr size_t linkNameLen =
-      mozilla::ArrayLength(linkName) - 1;  // -1 to null-terminate.
+      std::size(linkName) - 1;  // -1 to null-terminate.
 
   // Return if the TZ value is too large.
   if (std::strlen(tz) > linkNameLen) {
@@ -640,7 +669,7 @@ static icu::UnicodeString ReadTimeZoneLink(const char* tz) {
 
   char linkTarget[PathMax];
   constexpr size_t linkTargetLen =
-      mozilla::ArrayLength(linkTarget) - 1;  // -1 to null-terminate.
+      std::size(linkTarget) - 1;  // -1 to null-terminate.
 
   uint32_t depth = 0;
 
@@ -693,30 +722,7 @@ static icu::UnicodeString ReadTimeZoneLink(const char* tz) {
   }
 
   const char* timeZone = timeZoneWithZoneInfo + ZoneInfoPathLength;
-  size_t timeZoneLen = std::strlen(timeZone);
-
-  // Reject the result if it doesn't match the time zone id pattern or
-  // legacy time zone names.
-  // See <https://github.com/eggert/tz/blob/master/theory.html>.
-  for (size_t i = 0; i < timeZoneLen; i++) {
-    char c = timeZone[i];
-
-    // According to theory.html, '.' is allowed in time zone ids, but the
-    // accompanying zic.c file doesn't allow it. Assume the source file is
-    // correct and disallow '.' here, too.
-    if (mozilla::IsAsciiAlphanumeric(c) || c == '_' || c == '-' || c == '+') {
-      continue;
-    }
-
-    // Reject leading, trailing, or consecutive '/' characters.
-    if (c == '/' && i > 0 && i + 1 < timeZoneLen && timeZone[i + 1] != '/') {
-      continue;
-    }
-
-    return icu::UnicodeString();
-  }
-
-  return icu::UnicodeString(timeZone, timeZoneLen, US_INV);
+  return MaybeTimeZoneId(timeZone);
 }
 #endif /* JS_HAS_INTL_API && !MOZ_SYSTEM_ICU */
 
@@ -748,6 +754,14 @@ void js::DateTimeInfo::internalResyncICUDefaultTimeZone() {
     if (const char* tzlink = TZContainsAbsolutePath(tz)) {
       tzid.setTo(ReadTimeZoneLink(tzlink));
     }
+
+#    ifdef ANDROID
+    // ICU ignores the TZ environment variable on Android. If it doesn't contain
+    // an absolute path, try to parse it as a time zone name.
+    else {
+      tzid.setTo(MaybeTimeZoneId(tz));
+    }
+#    endif
 #  endif /* defined(XP_WIN) */
 
     if (!tzid.isEmpty()) {
