@@ -7,6 +7,7 @@
 #ifndef vm_Runtime_h
 #define vm_Runtime_h
 
+#include "mozilla/Assertions.h"  // MOZ_ASSERT
 #include "mozilla/Atomics.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/DoublyLinkedList.h"
@@ -17,9 +18,9 @@
 #include "mozilla/ThreadLocal.h"
 #include "mozilla/TimeStamp.h"
 #include "mozilla/Vector.h"
+#include "mozilla/XorShift128PlusRNG.h"
 
 #include <algorithm>
-#include <setjmp.h>
 
 #include "jsapi.h"
 
@@ -27,25 +28,24 @@
 #ifdef JS_HAS_INTL_API
 #  include "builtin/intl/SharedIntlData.h"
 #endif
-#include "frontend/BinASTRuntimeSupport.h"
 #include "frontend/NameCollections.h"
 #include "gc/GCRuntime.h"
 #include "gc/Tracer.h"
-#ifndef ENABLE_NEW_REGEXP
-#  include "irregexp/RegExpStack.h"
-#endif
 #include "js/AllocationRecording.h"
 #include "js/BuildId.h"  // JS::BuildIdOp
-#include "js/CompilationAndEvaluation.h"
 #include "js/Debug.h"
+#include "js/experimental/CTypes.h"      // JS::CTypesActivityCallback
 #include "js/experimental/SourceHook.h"  // js::SourceHook
+#include "js/friend/StackLimits.h"       // js::ReportOverRecursed
+#include "js/friend/UsageStatistics.h"   // JSAccumulateTelemetryDataCallback
 #include "js/GCVector.h"
 #include "js/HashTable.h"
+#include "js/Initialization.h"
 #include "js/Modules.h"  // JS::Module{DynamicImport,Metadata,Resolve}Hook
 #ifdef DEBUG
 #  include "js/Proxy.h"  // For AutoEnterPolicy
 #endif
-#include "js/Stream.h"
+#include "js/Stream.h"  // JS::AbortSignalIsAborted
 #include "js/Symbol.h"
 #include "js/UniquePtr.h"
 #include "js/Utility.h"
@@ -57,18 +57,21 @@
 #include "vm/CommonPropertyNames.h"
 #include "vm/GeckoProfiler.h"
 #include "vm/JSAtom.h"
+#include "vm/JSAtomState.h"
 #include "vm/JSScript.h"
 #include "vm/OffThreadPromiseRuntimeState.h"  // js::OffThreadPromiseRuntimeState
 #include "vm/Scope.h"
 #include "vm/SharedImmutableStringsCache.h"
+#include "vm/SharedStencil.h"  // js::SharedImmutableScriptDataTable
 #include "vm/Stack.h"
 #include "vm/SymbolType.h"
 #include "wasm/WasmTypes.h"
 
+struct JSClass;
+
 namespace js {
 
 class AutoAssertNoContentJS;
-class AutoKeepAtoms;
 class EnterDebuggeeNoExecute;
 #ifdef JS_TRACE_LOGGING
 class TraceLoggerThread;
@@ -89,14 +92,12 @@ namespace js {
 
 extern MOZ_COLD void ReportOutOfMemory(JSContext* cx);
 
-/* Different signature because the return type has MOZ_MUST_USE_TYPE. */
-extern MOZ_COLD mozilla::GenericErrorResult<OOM&> ReportOutOfMemoryResult(
+/* Different signature because the return type has [[nodiscard]]_TYPE. */
+extern MOZ_COLD mozilla::GenericErrorResult<OOM> ReportOutOfMemoryResult(
     JSContext* cx);
 
 extern MOZ_COLD void ReportAllocationOverflow(JSContext* maybecx);
 
-extern MOZ_COLD void ReportOverRecursed(JSContext* cx);
-
 class Activation;
 class ActivationIterator;
 
@@ -113,6 +114,10 @@ class Simulator;
 #endif
 }  // namespace jit
 
+namespace frontend {
+class WellKnownParserAtoms;
+}  // namespace frontend
+
 // [SMDOC] JS Engine Threading
 //
 // Threads interacting with a runtime are divided into two categories:
@@ -134,34 +139,6 @@ namespace JS {
 struct RuntimeSizes;
 }  // namespace JS
 
-/* Various built-in or commonly-used names pinned on first context. */
-struct JSAtomState {
-#define PROPERTYNAME_FIELD(idpart, id, text) js::ImmutablePropertyNamePtr id;
-  FOR_EACH_COMMON_PROPERTYNAME(PROPERTYNAME_FIELD)
-#undef PROPERTYNAME_FIELD
-#define PROPERTYNAME_FIELD(name, clasp) js::ImmutablePropertyNamePtr name;
-  JS_FOR_EACH_PROTOTYPE(PROPERTYNAME_FIELD)
-#undef PROPERTYNAME_FIELD
-#define PROPERTYNAME_FIELD(name) js::ImmutablePropertyNamePtr name;
-  JS_FOR_EACH_WELL_KNOWN_SYMBOL(PROPERTYNAME_FIELD)
-#undef PROPERTYNAME_FIELD
-#define PROPERTYNAME_FIELD(name) js::ImmutablePropertyNamePtr Symbol_##name;
-  JS_FOR_EACH_WELL_KNOWN_SYMBOL(PROPERTYNAME_FIELD)
-#undef PROPERTYNAME_FIELD
-
-  js::ImmutablePropertyNamePtr* wellKnownSymbolNames() {
-#define FIRST_PROPERTYNAME_FIELD(name) return &name;
-    JS_FOR_EACH_WELL_KNOWN_SYMBOL(FIRST_PROPERTYNAME_FIELD)
-#undef FIRST_PROPERTYNAME_FIELD
-  }
-
-  js::ImmutablePropertyNamePtr* wellKnownSymbolDescriptions() {
-#define FIRST_PROPERTYNAME_FIELD(name) return &Symbol_##name;
-    JS_FOR_EACH_WELL_KNOWN_SYMBOL(FIRST_PROPERTYNAME_FIELD)
-#undef FIRST_PROPERTYNAME_FIELD
-  }
-};
-
 namespace js {
 
 /*
@@ -194,14 +171,6 @@ struct WellKnownSymbols {
   WellKnownSymbols& operator=(const WellKnownSymbols&) = delete;
 };
 
-#define NAME_OFFSET(name) offsetof(JSAtomState, name)
-
-inline HandlePropertyName AtomStateOffsetToName(const JSAtomState& atomState,
-                                                size_t offset) {
-  return *reinterpret_cast<js::ImmutablePropertyNamePtr*>((char*)&atomState +
-                                                          offset);
-}
-
 // There are several coarse locks in the enum below. These may be either
 // per-runtime or per-process. When acquiring more than one of these locks,
 // the acquisition must be done in the order below to avoid deadlocks.
@@ -228,13 +197,22 @@ struct SelfHostedLazyScript {
   // in BaseScript::jitCodeRaw_.
   uint8_t* jitCodeRaw_ = nullptr;
 
+  // Warm-up count of zero. This field is stored at the same offset as
+  // BaseScript::warmUpData_.
+  ScriptWarmUpData warmUpData_ = {};
+
   static constexpr size_t offsetOfJitCodeRaw() {
     return offsetof(SelfHostedLazyScript, jitCodeRaw_);
   }
+  static constexpr size_t offsetOfWarmUpData() {
+    return offsetof(SelfHostedLazyScript, warmUpData_);
+  }
 };
 
 }  // namespace js
 
+struct JSTelemetrySender;
+
 struct JSRuntime {
  private:
   friend class js::Activation;
@@ -254,6 +232,8 @@ struct JSRuntime {
    */
   JSRuntime* const parentRuntime;
 
+  bool isMainRuntime() const { return !parentRuntime; }
+
 #ifdef DEBUG
   /* The number of child runtimes that have this runtime as their parent. */
   mozilla::Atomic<size_t> childRuntimeCount;
@@ -323,13 +303,14 @@ struct JSRuntime {
     profilerSampleBufferRangeStart_ = rangeStart;
   }
 
-  /* Call this to accumulate telemetry data. */
-  js::MainThreadData<JSAccumulateTelemetryDataCallback> telemetryCallback;
+  /* Call this to accumulate telemetry data. May be called from any thread; the
+   * embedder is responsible for locking. */
+  JSAccumulateTelemetryDataCallback telemetryCallback;
 
   /* Call this to accumulate use counter data. */
   js::MainThreadData<JSSetUseCounterCallback> useCounterCallback;
 
-  js::MainThreadData<JSGetElementCallback> getElementCallback;
+  js::MainThreadData<JSSourceElementCallback> sourceElementCallback;
 
  public:
   // Accumulates data for Firefox telemetry. |id| is the ID of a JS_TELEMETRY_*
@@ -337,10 +318,13 @@ struct JSRuntime {
   // |sample| is the data to add to the histogram.
   void addTelemetry(int id, uint32_t sample, const char* key = nullptr);
 
+  JSTelemetrySender getTelemetrySender() const;
+
   void setTelemetryCallback(JSRuntime* rt,
                             JSAccumulateTelemetryDataCallback callback);
 
-  void setElementCallback(JSRuntime* rt, JSGetElementCallback callback);
+  void setSourceElementCallback(JSRuntime* rt,
+                                JSSourceElementCallback callback);
 
   // Sets the use counter for a specific feature, measuring the presence or
   // absence of usage of a feature on a specific web page and document which
@@ -417,7 +401,7 @@ struct JSRuntime {
   // Heap GC roots for PersistentRooted pointers.
   js::MainThreadData<mozilla::EnumeratedArray<
       JS::RootKind, JS::RootKind::Limit,
-      mozilla::LinkedList<JS::PersistentRooted<void*>>>>
+      mozilla::LinkedList<JS::PersistentRooted<JS::detail::RootListEntry*>>>>
       heapRoots;
 
   void tracePersistentRoots(JSTracer* trc);
@@ -434,10 +418,11 @@ struct JSRuntime {
 
   js::MainThreadData<const JSWrapObjectCallbacks*> wrapObjectCallbacks;
   js::MainThreadData<js::PreserveWrapperCallback> preserveWrapperCallback;
+  js::MainThreadData<js::HasReleasedWrapperCallback> hasReleasedWrapperCallback;
 
   js::MainThreadData<js::ScriptEnvironmentPreparer*> scriptEnvironmentPreparer;
 
-  js::MainThreadData<js::CTypesActivityCallback> ctypesActivityCallback;
+  js::MainThreadData<JS::CTypesActivityCallback> ctypesActivityCallback;
 
  private:
   js::WriteOnceData<const JSClass*> windowProxyClass_;
@@ -446,6 +431,30 @@ struct JSRuntime {
   const JSClass* maybeWindowProxyClass() const { return windowProxyClass_; }
   void setWindowProxyClass(const JSClass* clasp) { windowProxyClass_ = clasp; }
 
+ private:
+  js::WriteOnceData<const JSClass*> abortSignalClass_;
+  js::WriteOnceData<JS::AbortSignalIsAborted> abortSignalIsAborted_;
+
+ public:
+  void initPipeToHandling(const JSClass* abortSignalClass,
+                          JS::AbortSignalIsAborted isAborted) {
+    MOZ_ASSERT(abortSignalClass != nullptr,
+               "doesn't make sense for an embedder to provide a null class "
+               "when specifying pipeTo handling");
+    MOZ_ASSERT(isAborted != nullptr, "must pass a valid function pointer");
+
+    abortSignalClass_ = abortSignalClass;
+    abortSignalIsAborted_ = isAborted;
+  }
+
+  const JSClass* maybeAbortSignalClass() const { return abortSignalClass_; }
+
+  bool abortSignalIsAborted(JSObject* obj) {
+    MOZ_ASSERT(abortSignalIsAborted_ != nullptr,
+               "must call initPipeToHandling first");
+    return abortSignalIsAborted_(obj);
+  }
+
  private:
   // List of non-ephemeron weak containers to sweep during
   // beginSweepingSweepGroup.
@@ -467,22 +476,42 @@ struct JSRuntime {
     }
   };
 
-  using WatchersList =
+  template <typename T>
+  struct GarbageCollectionWatchersLinkAccess {
+    static mozilla::DoublyLinkedListElement<T>& Get(T* aThis) {
+      return aThis->onGarbageCollectionWatchersLink;
+    }
+  };
+
+  using OnNewGlobalWatchersList =
       mozilla::DoublyLinkedList<js::Debugger,
                                 GlobalObjectWatchersLinkAccess<js::Debugger>>;
+  using OnGarbageCollectionWatchersList = mozilla::DoublyLinkedList<
+      js::Debugger, GarbageCollectionWatchersLinkAccess<js::Debugger>>;
 
  private:
   /*
    * List of all enabled Debuggers that have onNewGlobalObject handler
    * methods established.
    */
-  js::MainThreadData<WatchersList> onNewGlobalObjectWatchers_;
+  js::MainThreadData<OnNewGlobalWatchersList> onNewGlobalObjectWatchers_;
+
+  /*
+   * List of all enabled Debuggers that have onGarbageCollection handler
+   * methods established.
+   */
+  js::MainThreadData<OnGarbageCollectionWatchersList>
+      onGarbageCollectionWatchers_;
 
  public:
-  WatchersList& onNewGlobalObjectWatchers() {
+  OnNewGlobalWatchersList& onNewGlobalObjectWatchers() {
     return onNewGlobalObjectWatchers_.ref();
   }
 
+  OnGarbageCollectionWatchersList& onGarbageCollectionWatchers() {
+    return onGarbageCollectionWatchers_.ref();
+  }
+
  private:
   /* Linked list of all Debugger objects in the runtime. */
   js::MainThreadData<mozilla::LinkedList<js::Debugger>> debuggerList_;
@@ -505,6 +534,11 @@ struct JSRuntime {
   bool activeThreadHasScriptDataAccess;
 #endif
 
+  // Number of off-thread ParseTasks that are using this runtime. This is only
+  // updated on main-thread. If this is non-zero we must use `scriptDataLock` to
+  // protect access to the bytecode table;
+  mozilla::Atomic<size_t, mozilla::SequentiallyConsistent> numParseTasks;
+
   // Number of zones which may be operated on by helper threads.
   mozilla::Atomic<size_t, mozilla::SequentiallyConsistent>
       numActiveHelperThreadZones;
@@ -515,16 +549,21 @@ struct JSRuntime {
   void setUsedByHelperThread(JS::Zone* zone);
   void clearUsedByHelperThread(JS::Zone* zone);
 
+  bool hasParseTasks() const { return numParseTasks > 0; }
   bool hasHelperThreadZones() const { return numActiveHelperThreadZones > 0; }
 
+  void addParseTaskRef() { numParseTasks++; }
+  void decParseTaskRef() { numParseTasks--; }
+
 #ifdef DEBUG
-  bool currentThreadHasScriptDataAccess() const {
-    if (!hasHelperThreadZones()) {
-      return js::CurrentThreadCanAccessRuntime(this) &&
-             activeThreadHasScriptDataAccess;
+  void assertCurrentThreadHasScriptDataAccess() const {
+    if (!hasParseTasks()) {
+      MOZ_ASSERT(js::CurrentThreadCanAccessRuntime(this) &&
+                 activeThreadHasScriptDataAccess);
+      return;
     }
 
-    return scriptDataLock.ownedByCurrentThread();
+    scriptDataLock.assertOwnedByCurrentThread();
   }
 
   bool currentThreadHasAtomsTableAccess() const {
@@ -598,12 +637,10 @@ struct JSRuntime {
   static js::GlobalObject* createSelfHostingGlobal(JSContext* cx);
 
  public:
-  bool getUnclonedSelfHostedValue(JSContext* cx, js::HandlePropertyName name,
-                                  js::MutableHandleValue vp);
-  JSFunction* getUnclonedSelfHostedFunction(JSContext* cx,
-                                            js::HandlePropertyName name);
+  void getUnclonedSelfHostedValue(js::PropertyName* name, JS::Value* vp);
+  JSFunction* getUnclonedSelfHostedFunction(js::PropertyName* name);
 
-  MOZ_MUST_USE bool createJitRuntime(JSContext* cx);
+  [[nodiscard]] bool createJitRuntime(JSContext* cx);
   js::jit::JitRuntime* jitRuntime() const { return jitRuntime_.ref(); }
   bool hasJitRuntime() const { return !!jitRuntime_; }
 
@@ -628,16 +665,17 @@ struct JSRuntime {
 
   bool hasInitializedSelfHosting() const { return selfHostingGlobal_; }
 
-  bool initSelfHosting(JSContext* cx);
+  bool initSelfHosting(JSContext* cx, JS::SelfHostedCache xdrCache = nullptr,
+                       JS::SelfHostedWriter xdrWriter = nullptr);
   void finishSelfHosting();
   void traceSelfHostingGlobal(JSTracer* trc);
   bool isSelfHostingGlobal(JSObject* global) {
     return global == selfHostingGlobal_;
   }
+  js::GeneratorKind getSelfHostedFunctionGeneratorKind(JSAtom* name);
   bool createLazySelfHostedFunctionClone(JSContext* cx,
                                          js::HandlePropertyName selfHostedName,
                                          js::HandleAtom name, unsigned nargs,
-                                         js::HandleObject proto,
                                          js::NewObjectKind newKind,
                                          js::MutableHandleFunction fun);
   bool cloneSelfHostedFunctionScript(JSContext* cx,
@@ -750,7 +788,9 @@ struct JSRuntime {
 
  public:
   bool initializeAtoms(JSContext* cx);
+  bool initializeParserAtoms(JSContext* cx);
   void finishAtoms();
+  void finishParserAtoms();
   bool atomsAreFinished() const {
     return !atoms_ && !permanentAtomsDuringInit_;
   }
@@ -791,6 +831,7 @@ struct JSRuntime {
 
   // Cached pointers to various permanent property names.
   js::WriteOnceData<JSAtomState*> commonNames;
+  js::WriteOnceData<js::frontend::WellKnownParserAtoms*> commonParserNames;
 
   // All permanent atoms in the runtime, other than those in staticStrings.
   // Access to this does not require a lock because it is frozen and thus
@@ -828,10 +869,10 @@ struct JSRuntime {
   // within the runtime. This may be modified by threads using
   // AutoLockScriptData.
  private:
-  js::ScriptDataLockData<js::RuntimeScriptDataTable> scriptDataTable_;
+  js::ScriptDataLockData<js::SharedImmutableScriptDataTable> scriptDataTable_;
 
  public:
-  js::RuntimeScriptDataTable& scriptDataTable(
+  js::SharedImmutableScriptDataTable& scriptDataTable(
       const js::AutoLockScriptData& lock) {
     return scriptDataTable_.ref();
   }
@@ -885,13 +926,13 @@ struct JSRuntime {
    *
    * The function must be called outside the GC lock.
    */
-  JS_FRIEND_API void* onOutOfMemory(js::AllocFunction allocator,
+  JS_PUBLIC_API void* onOutOfMemory(js::AllocFunction allocator,
                                     arena_id_t arena, size_t nbytes,
                                     void* reallocPtr = nullptr,
                                     JSContext* maybecx = nullptr);
 
   /*  onOutOfMemory but can call OnLargeAllocationFailure. */
-  JS_FRIEND_API void* onOutOfMemoryCanGC(js::AllocFunction allocator,
+  JS_PUBLIC_API void* onOutOfMemoryCanGC(js::AllocFunction allocator,
                                          arena_id_t arena, size_t nbytes,
                                          void* reallocPtr = nullptr);
 
@@ -1030,14 +1071,6 @@ struct JSRuntime {
     }
   }
 
- public:
-#if defined(JS_BUILD_BINAST)
-  js::BinaryASTSupport& binast() { return binast_; }
-
- private:
-  js::BinaryASTSupport binast_;
-#endif  // defined(JS_BUILD_BINAST)
-
  public:
 #if defined(NIGHTLY_BUILD)
   // Support for informing the embedding of any error thrown.
@@ -1060,6 +1093,34 @@ struct JSRuntime {
 #endif  // defined(NIGHTLY_BUILD)
 };
 
+// Context for sending telemetry to the embedder from any thread, main or
+// helper.  Obtain a |JSTelemetrySender| by calling |getTelemetrySender()| on
+// the |JSRuntime|.
+struct JSTelemetrySender {
+ private:
+  friend struct JSRuntime;
+
+  JSAccumulateTelemetryDataCallback callback_;
+
+  explicit JSTelemetrySender(JSAccumulateTelemetryDataCallback callback)
+      : callback_(callback) {}
+
+ public:
+  JSTelemetrySender() : callback_(nullptr) {}
+  JSTelemetrySender(const JSTelemetrySender& other) = default;
+  explicit JSTelemetrySender(JSRuntime* runtime)
+      : JSTelemetrySender(runtime->getTelemetrySender()) {}
+
+  // Accumulates data for Firefox telemetry. |id| is the ID of a JS_TELEMETRY_*
+  // histogram. |key| provides an additional key to identify the histogram.
+  // |sample| is the data to add to the histogram.
+  void addTelemetry(int id, uint32_t sample, const char* key = nullptr) {
+    if (callback_) {
+      callback_(id, sample, key);
+    }
+  }
+};
+
 namespace js {
 
 static MOZ_ALWAYS_INLINE void MakeRangeGCSafe(Value* vec, size_t len) {
@@ -1122,10 +1183,6 @@ extern mozilla::Atomic<JS::BuildIdOp> GetBuildId;
 
 extern JS::FilenameValidationCallback gFilenameValidationCallback;
 
-// This callback is set by js::SetHelperThreadTaskCallback and may be null.
-// See comment in jsapi.h.
-extern void (*HelperThreadTaskCallback)(js::UniquePtr<js::RunnableTask>);
-
 } /* namespace js */
 
 #endif /* vm_Runtime_h */
