@@ -11,18 +11,17 @@
 namespace js {
 
 SharedImmutableString::SharedImmutableString(
-    ExclusiveData<SharedImmutableStringsCache::Inner>::Guard& locked,
     SharedImmutableStringsCache::StringBox* box)
-    : cache_(locked), box_(box) {
+    : box_(box) {
   MOZ_ASSERT(box);
   box->refcount++;
 }
 
 SharedImmutableString::SharedImmutableString(SharedImmutableString&& rhs)
-    : cache_(std::move(rhs.cache_)), box_(rhs.box_) {
+    : box_(rhs.box_) {
   MOZ_ASSERT(this != &rhs, "self move not allowed");
-  MOZ_ASSERT(rhs.box_);
-  MOZ_ASSERT(rhs.box_->refcount > 0);
+
+  MOZ_ASSERT_IF(rhs.box_, rhs.box_->refcount > 0);
 
   rhs.box_ = nullptr;
 }
@@ -39,9 +38,8 @@ SharedImmutableTwoByteString::SharedImmutableTwoByteString(
     : string_(std::move(string)) {}
 
 SharedImmutableTwoByteString::SharedImmutableTwoByteString(
-    ExclusiveData<SharedImmutableStringsCache::Inner>::Guard& locked,
     SharedImmutableStringsCache::StringBox* box)
-    : string_(locked, box) {
+    : string_(box) {
   MOZ_ASSERT(box->length() % sizeof(char16_t) == 0);
 }
 
@@ -63,7 +61,7 @@ SharedImmutableString::~SharedImmutableString() {
     return;
   }
 
-  auto locked = cache_.inner_->lock();
+  auto locked = box_->cache_->lock();
 
   MOZ_ASSERT(box_->refcount > 0);
 
@@ -74,30 +72,30 @@ SharedImmutableString::~SharedImmutableString() {
 }
 
 SharedImmutableString SharedImmutableString::clone() const {
-  auto locked = cache_.inner_->lock();
+  auto locked = box_->cache_->lock();
   MOZ_ASSERT(box_);
   MOZ_ASSERT(box_->refcount > 0);
-  return SharedImmutableString(locked, box_);
+  return SharedImmutableString(box_);
 }
 
 SharedImmutableTwoByteString SharedImmutableTwoByteString::clone() const {
   return SharedImmutableTwoByteString(string_.clone());
 }
 
-MOZ_MUST_USE mozilla::Maybe<SharedImmutableString>
-SharedImmutableStringsCache::getOrCreate(OwnedChars&& chars, size_t length) {
+[[nodiscard]] SharedImmutableString SharedImmutableStringsCache::getOrCreate(
+    OwnedChars&& chars, size_t length) {
   OwnedChars owned(std::move(chars));
   MOZ_ASSERT(owned);
   return getOrCreate(owned.get(), length, [&]() { return std::move(owned); });
 }
 
-MOZ_MUST_USE mozilla::Maybe<SharedImmutableString>
-SharedImmutableStringsCache::getOrCreate(const char* chars, size_t length) {
+[[nodiscard]] SharedImmutableString SharedImmutableStringsCache::getOrCreate(
+    const char* chars, size_t length) {
   return getOrCreate(chars, length,
                      [&]() { return DuplicateString(chars, length); });
 }
 
-MOZ_MUST_USE mozilla::Maybe<SharedImmutableTwoByteString>
+[[nodiscard]] SharedImmutableTwoByteString
 SharedImmutableStringsCache::getOrCreate(OwnedTwoByteChars&& chars,
                                          size_t length) {
   OwnedTwoByteChars owned(std::move(chars));
@@ -105,7 +103,7 @@ SharedImmutableStringsCache::getOrCreate(OwnedTwoByteChars&& chars,
   return getOrCreate(owned.get(), length, [&]() { return std::move(owned); });
 }
 
-MOZ_MUST_USE mozilla::Maybe<SharedImmutableTwoByteString>
+[[nodiscard]] SharedImmutableTwoByteString
 SharedImmutableStringsCache::getOrCreate(const char16_t* chars, size_t length) {
   return getOrCreate(chars, length,
                      [&]() { return DuplicateString(chars, length); });
