@@ -12,14 +12,19 @@
 #include "mozilla/Maybe.h"
 #include "mozilla/PodOperations.h"
 
+#include "builtin/Array.h"  // js::NewDenseEmptyArray
+#include "builtin/ModuleObject.h"
 #include "jit/BaselineFrame.h"
 #include "jit/RematerializedFrame.h"
 #include "js/Debug.h"
+#include "js/friend/StackLimits.h"  // js::ReportOverRecursed
 #include "vm/EnvironmentObject.h"
+#include "vm/FrameIter.h"  // js::FrameIter
 #include "vm/JSContext.h"
 #include "vm/JSScript.h"
 
 #include "jit/BaselineFrame-inl.h"
+#include "jit/RematerializedFrame-inl.h"  // js::jit::RematerializedFrame::unsetIsDebuggee
 #include "vm/JSObject-inl.h"
 #include "vm/JSScript-inl.h"
 #include "vm/NativeObject-inl.h"
@@ -34,11 +39,7 @@ inline GlobalObject& InterpreterFrame::global() const {
   return script()->global();
 }
 
-inline JSObject& InterpreterFrame::varObj() const {
-  return GetVariablesObject(environmentChain());
-}
-
-inline LexicalEnvironmentObject&
+inline ExtensibleLexicalEnvironmentObject&
 InterpreterFrame::extensibleLexicalEnvironment() const {
   return NearestEnclosingExtensibleLexicalEnvironment(environmentChain());
 }
@@ -49,7 +50,7 @@ inline void InterpreterFrame::initCallFrame(InterpreterFrame* prev,
                                             JSScript* script, Value* argv,
                                             uint32_t nactual,
                                             MaybeConstruct constructing) {
-  MOZ_ASSERT(callee.nonLazyScript() == script);
+  MOZ_ASSERT(callee.baseScript() == script);
 
   /* Initialize stack frame members. */
   flags_ = 0;
@@ -145,6 +146,22 @@ inline EnvironmentObject& InterpreterFrame::aliasedEnvironment(
   return env->as<EnvironmentObject>();
 }
 
+inline EnvironmentObject& InterpreterFrame::aliasedEnvironmentMaybeDebug(
+    EnvironmentCoordinate ec) const {
+  JSObject* env = environmentChain();
+  for (unsigned i = ec.hops(); i; i--) {
+    if (env->is<EnvironmentObject>()) {
+      env = &env->as<EnvironmentObject>().enclosingEnvironment();
+    } else {
+      MOZ_ASSERT(env->is<DebugEnvironmentProxy>());
+      env = &env->as<DebugEnvironmentProxy>().enclosingEnvironment();
+    }
+  }
+  return env->is<EnvironmentObject>()
+             ? env->as<EnvironmentObject>()
+             : env->as<DebugEnvironmentProxy>().environment();
+}
+
 template <typename SpecificEnvironment>
 inline void InterpreterFrame::pushOnEnvironmentChain(SpecificEnvironment& env) {
   MOZ_ASSERT(*environmentChain() == env.enclosingEnvironment());
@@ -161,9 +178,10 @@ inline void InterpreterFrame::popOffEnvironmentChain() {
 }
 
 inline void InterpreterFrame::replaceInnermostEnvironment(
-    EnvironmentObject& env) {
-  MOZ_ASSERT(env.enclosingEnvironment() ==
-             envChain_->as<EnvironmentObject>().enclosingEnvironment());
+    BlockLexicalEnvironmentObject& env) {
+  MOZ_ASSERT(
+      env.enclosingEnvironment() ==
+      envChain_->as<BlockLexicalEnvironmentObject>().enclosingEnvironment());
   envChain_ = &env;
 }
 
@@ -187,6 +205,19 @@ inline void InterpreterFrame::unsetIsDebuggee() {
   flags_ &= ~DEBUGGEE;
 }
 
+inline bool InterpreterFrame::saveGeneratorSlots(JSContext* cx, unsigned nslots,
+                                                 ArrayObject* dest) const {
+  return dest->initDenseElementsFromRange(cx, slots(), slots() + nslots);
+}
+
+inline void InterpreterFrame::restoreGeneratorSlots(ArrayObject* src) {
+  MOZ_ASSERT(script()->nfixed() <= src->length());
+  MOZ_ASSERT(src->length() <= script()->nslots());
+  MOZ_ASSERT(src->getDenseInitializedLength() == src->length());
+  const Value* srcElements = src->getDenseElements();
+  mozilla::PodCopy(slots(), srcElements, src->length());
+}
+
 /*****************************************************************************/
 
 inline void InterpreterStack::purge(JSRuntime* rt) {
@@ -221,7 +252,7 @@ MOZ_ALWAYS_INLINE InterpreterFrame* InterpreterStack::getCallFrame(
     MaybeConstruct constructing, Value** pargv) {
   JSFunction* fun = &args.callee().as<JSFunction>();
 
-  MOZ_ASSERT(fun->nonLazyScript() == script);
+  MOZ_ASSERT(fun->baseScript() == script);
   unsigned nformal = fun->nargs();
   unsigned nvals = script->nslots();
 
@@ -263,9 +294,7 @@ MOZ_ALWAYS_INLINE bool InterpreterStack::pushInlineFrame(
     HandleScript script, MaybeConstruct constructing) {
   RootedFunction callee(cx, &args.callee().as<JSFunction>());
   MOZ_ASSERT(regs.sp == args.end());
-  MOZ_ASSERT(callee->nonLazyScript() == script);
-
-  script->ensureNonLazyCanonicalFunction();
+  MOZ_ASSERT(callee->baseScript() == script);
 
   InterpreterFrame* prev = regs.fp();
   jsbytecode* prevpc = regs.pc;
@@ -300,8 +329,6 @@ MOZ_ALWAYS_INLINE bool InterpreterStack::resumeGeneratorCallFrame(
   Value* prevsp = regs.sp;
   MOZ_ASSERT(prev);
 
-  script->ensureNonLazyCanonicalFunction();
-
   LifoAlloc::Mark mark = allocator_.mark();
 
   // (Async) generators and async functions are not constructors.
@@ -340,40 +367,6 @@ MOZ_ALWAYS_INLINE void InterpreterStack::popInlineFrame(InterpreterRegs& regs) {
   MOZ_ASSERT(regs.fp());
 }
 
-template <class Op>
-inline void FrameIter::unaliasedForEachActual(JSContext* cx, Op op) {
-  switch (data_.state_) {
-    case DONE:
-      break;
-    case INTERP:
-      interpFrame()->unaliasedForEachActual(op);
-      return;
-    case JIT:
-      MOZ_ASSERT(isJSJit());
-      if (jsJitFrame().isIonJS()) {
-        jit::MaybeReadFallback recover(cx, activation()->asJit(),
-                                       &jsJitFrame());
-        ionInlineFrames_.unaliasedForEachActual(cx, op, jit::ReadFrame_Actuals,
-                                                recover);
-      } else if (jsJitFrame().isBailoutJS()) {
-        // :TODO: (Bug 1070962) If we are introspecting the frame which is
-        // being bailed, then we might be in the middle of recovering
-        // instructions. Stacking computeInstructionResults implies that we
-        // might be recovering result twice. In the mean time, to avoid
-        // that, we just return Undefined values for instruction results
-        // which are not yet recovered.
-        jit::MaybeReadFallback fallback;
-        ionInlineFrames_.unaliasedForEachActual(cx, op, jit::ReadFrame_Actuals,
-                                                fallback);
-      } else {
-        MOZ_ASSERT(jsJitFrame().isBaselineJS());
-        jsJitFrame().unaliasedForEachActual(op, jit::ReadFrame_Actuals);
-      }
-      return;
-  }
-  MOZ_CRASH("Unexpected state");
-}
-
 inline HandleValue AbstractFramePtr::returnValue() const {
   if (isInterpreterFrame()) {
     return asInterpreterFrame()->returnValue();
@@ -693,33 +686,21 @@ inline bool AbstractFramePtr::isFunctionFrame() const {
 }
 
 inline bool AbstractFramePtr::isGeneratorFrame() const {
-  if (!isFunctionFrame()) {
+  if (!isFunctionFrame() && !isModuleFrame()) {
     return false;
   }
   JSScript* s = script();
   return s->isGenerator() || s->isAsync();
 }
 
-inline bool AbstractFramePtr::isNonStrictDirectEvalFrame() const {
+inline bool AbstractFramePtr::saveGeneratorSlots(JSContext* cx, unsigned nslots,
+                                                 ArrayObject* dest) const {
+  MOZ_ASSERT(isGeneratorFrame());
   if (isInterpreterFrame()) {
-    return asInterpreterFrame()->isNonStrictDirectEvalFrame();
+    return asInterpreterFrame()->saveGeneratorSlots(cx, nslots, dest);
   }
-  if (isBaselineFrame()) {
-    return asBaselineFrame()->isNonStrictDirectEvalFrame();
-  }
-  MOZ_ASSERT(isRematerializedFrame());
-  return false;
-}
-
-inline bool AbstractFramePtr::isStrictEvalFrame() const {
-  if (isInterpreterFrame()) {
-    return asInterpreterFrame()->isStrictEvalFrame();
-  }
-  if (isBaselineFrame()) {
-    return asBaselineFrame()->isStrictEvalFrame();
-  }
-  MOZ_ASSERT(isRematerializedFrame());
-  return false;
+  MOZ_ASSERT(isBaselineFrame(), "unexpected generator frame in Ion");
+  return asBaselineFrame()->saveGeneratorSlots(cx, nslots, dest);
 }
 
 inline Value* AbstractFramePtr::argv() const {
@@ -832,85 +813,6 @@ inline bool AbstractFramePtr::debuggerNeedsCheckPrimitiveReturn() const {
   return script()->isDerivedClassConstructor();
 }
 
-ActivationEntryMonitor::ActivationEntryMonitor(JSContext* cx)
-    : cx_(cx), entryMonitor_(cx->entryMonitor) {
-  cx->entryMonitor = nullptr;
-}
-
-ActivationEntryMonitor::ActivationEntryMonitor(JSContext* cx,
-                                               InterpreterFrame* entryFrame)
-    : ActivationEntryMonitor(cx) {
-  if (MOZ_UNLIKELY(entryMonitor_)) {
-    init(cx, entryFrame);
-  }
-}
-
-ActivationEntryMonitor::ActivationEntryMonitor(JSContext* cx,
-                                               jit::CalleeToken entryToken)
-    : ActivationEntryMonitor(cx) {
-  if (MOZ_UNLIKELY(entryMonitor_)) {
-    init(cx, entryToken);
-  }
-}
-
-ActivationEntryMonitor::~ActivationEntryMonitor() {
-  if (entryMonitor_) {
-    entryMonitor_->Exit(cx_);
-  }
-
-  cx_->entryMonitor = entryMonitor_;
-}
-
-Activation::Activation(JSContext* cx, Kind kind)
-    : cx_(cx),
-      compartment_(cx->compartment()),
-      prev_(cx->activation_),
-      prevProfiling_(prev_ ? prev_->mostRecentProfiling() : nullptr),
-      hideScriptedCallerCount_(0),
-      frameCache_(cx),
-      asyncStack_(cx, cx->asyncStackForNewActivations()),
-      asyncCause_(cx->asyncCauseForNewActivations),
-      asyncCallIsExplicit_(cx->asyncCallIsExplicit),
-      kind_(kind) {
-  cx->asyncStackForNewActivations() = nullptr;
-  cx->asyncCauseForNewActivations = nullptr;
-  cx->asyncCallIsExplicit = false;
-  cx->activation_ = this;
-}
-
-Activation::~Activation() {
-  MOZ_ASSERT_IF(isProfiling(), this != cx_->profilingActivation_);
-  MOZ_ASSERT(cx_->activation_ == this);
-  MOZ_ASSERT(hideScriptedCallerCount_ == 0);
-  cx_->activation_ = prev_;
-  cx_->asyncCauseForNewActivations = asyncCause_;
-  cx_->asyncStackForNewActivations() = asyncStack_;
-  cx_->asyncCallIsExplicit = asyncCallIsExplicit_;
-}
-
-bool Activation::isProfiling() const {
-  if (isInterpreter()) {
-    return asInterpreter()->isProfiling();
-  }
-
-  MOZ_ASSERT(isJit());
-  return asJit()->isProfiling();
-}
-
-Activation* Activation::mostRecentProfiling() {
-  if (isProfiling()) {
-    return this;
-  }
-  return prevProfiling_;
-}
-
-inline LiveSavedFrameCache* Activation::getLiveSavedFrameCache(JSContext* cx) {
-  if (!frameCache_.get().initialized() && !frameCache_.get().init(cx)) {
-    return nullptr;
-  }
-  return frameCache_.address();
-}
-
 InterpreterActivation::InterpreterActivation(RunState& state, JSContext* cx,
                                              InterpreterFrame* entryFrame)
     : Activation(cx, Interpreter),
@@ -969,89 +871,6 @@ inline bool InterpreterActivation::resumeGeneratorFrame(HandleFunction callee,
   return true;
 }
 
-/* static */ inline mozilla::Maybe<LiveSavedFrameCache::FramePtr>
-LiveSavedFrameCache::FramePtr::create(const FrameIter& iter) {
-  if (iter.done()) {
-    return mozilla::Nothing();
-  }
-
-  if (iter.isPhysicalJitFrame()) {
-    return mozilla::Some(FramePtr(iter.physicalJitFrame()));
-  }
-
-  if (!iter.hasUsableAbstractFramePtr()) {
-    return mozilla::Nothing();
-  }
-
-  auto afp = iter.abstractFramePtr();
-
-  if (afp.isInterpreterFrame()) {
-    return mozilla::Some(FramePtr(afp.asInterpreterFrame()));
-  }
-  if (afp.isWasmDebugFrame()) {
-    return mozilla::Some(FramePtr(afp.asWasmDebugFrame()));
-  }
-  if (afp.isRematerializedFrame()) {
-    return mozilla::Some(FramePtr(afp.asRematerializedFrame()));
-  }
-
-  MOZ_CRASH("unexpected frame type");
-}
-
-/* static */ inline LiveSavedFrameCache::FramePtr
-LiveSavedFrameCache::FramePtr::create(AbstractFramePtr afp) {
-  MOZ_ASSERT(afp);
-
-  if (afp.isBaselineFrame()) {
-    js::jit::CommonFrameLayout* common = afp.asBaselineFrame()->framePrefix();
-    return FramePtr(common);
-  }
-  if (afp.isInterpreterFrame()) {
-    return FramePtr(afp.asInterpreterFrame());
-  }
-  if (afp.isWasmDebugFrame()) {
-    return FramePtr(afp.asWasmDebugFrame());
-  }
-  if (afp.isRematerializedFrame()) {
-    return FramePtr(afp.asRematerializedFrame());
-  }
-
-  MOZ_CRASH("unexpected frame type");
-}
-
-struct LiveSavedFrameCache::FramePtr::HasCachedMatcher {
-  template <typename Frame>
-  bool operator()(Frame* f) const {
-    return f->hasCachedSavedFrame();
-  }
-};
-
-inline bool LiveSavedFrameCache::FramePtr::hasCachedSavedFrame() const {
-  return ptr.match(HasCachedMatcher());
-}
-
-struct LiveSavedFrameCache::FramePtr::SetHasCachedMatcher {
-  template <typename Frame>
-  void operator()(Frame* f) {
-    f->setHasCachedSavedFrame();
-  }
-};
-
-inline void LiveSavedFrameCache::FramePtr::setHasCachedSavedFrame() {
-  ptr.match(SetHasCachedMatcher());
-}
-
-struct LiveSavedFrameCache::FramePtr::ClearHasCachedMatcher {
-  template <typename Frame>
-  void operator()(Frame* f) {
-    f->clearHasCachedSavedFrame();
-  }
-};
-
-inline void LiveSavedFrameCache::FramePtr::clearHasCachedSavedFrame() {
-  ptr.match(ClearHasCachedMatcher());
-}
-
 } /* namespace js */
 
 #endif /* vm_Stack_inl_h */
