@@ -18,11 +18,23 @@
 
 #include "wasm/WasmTypes.h"
 
+#include "mozilla/FloatingPoint.h"
+
+#include <algorithm>
+
+#include "jsmath.h"
+#include "jit/JitFrames.h"
+#include "js/friend/ErrorMessages.h"  // JSMSG_*
 #include "js/Printf.h"
+#include "util/Memory.h"
 #include "vm/ArrayBufferObject.h"
+#include "vm/Warnings.h"  // js:WarnNumberASCII
+#include "wasm/TypedObject.h"
 #include "wasm/WasmBaselineCompile.h"
 #include "wasm/WasmInstance.h"
+#include "wasm/WasmJS.h"
 #include "wasm/WasmSerialize.h"
+#include "wasm/WasmStubs.h"
 
 #include "vm/JSObject-inl.h"
 #include "vm/NativeObject-inl.h"
@@ -34,26 +46,14 @@ using namespace js::wasm;
 using mozilla::IsPowerOfTwo;
 using mozilla::MakeEnumeratedRange;
 
-// We have only tested x64 with WASM_HUGE_MEMORY.
+// We have only tested huge memory on x64 and arm64.
 
-#if defined(JS_CODEGEN_X64) && !defined(WASM_HUGE_MEMORY)
-#  error "Not an expected configuration"
-#endif
-
-// We have only tested WASM_HUGE_MEMORY on x64 and arm64.
-
-#if defined(WASM_HUGE_MEMORY)
+#if defined(WASM_SUPPORTS_HUGE_MEMORY)
 #  if !(defined(JS_CODEGEN_X64) || defined(JS_CODEGEN_ARM64))
 #    error "Not an expected configuration"
 #  endif
 #endif
 
-// More sanity checks.
-
-static_assert(MaxMemoryInitialPages <=
-                  ArrayBufferObject::MaxBufferByteLength / PageSize,
-              "Memory sizing constraint");
-
 // All plausible targets must be able to do at least IEEE754 double
 // loads/stores, hence the lower limit of 8.  Some Intel processors support
 // AVX-512 loads/stores, hence the upper limit of 64.
@@ -62,198 +62,35 @@ static_assert(MaxMemoryAccessSize <= 64, "MaxMemoryAccessSize too high");
 static_assert((MaxMemoryAccessSize & (MaxMemoryAccessSize - 1)) == 0,
               "MaxMemoryAccessSize is not a power of two");
 
-Val::Val(const LitVal& val) {
-  type_ = val.type();
-  switch (type_.code()) {
-    case ValType::I32:
-      u.i32_ = val.i32();
-      return;
-    case ValType::F32:
-      u.f32_ = val.f32();
-      return;
-    case ValType::I64:
-      u.i64_ = val.i64();
-      return;
-    case ValType::F64:
-      u.f64_ = val.f64();
-      return;
-    case ValType::Ref:
-    case ValType::FuncRef:
-    case ValType::AnyRef:
-      u.ref_ = val.ref();
-      return;
-    case ValType::NullRef:
-      break;
-  }
-  MOZ_CRASH();
-}
+#if defined(WASM_SUPPORTS_HUGE_MEMORY)
+// TODO: We want this static_assert back, but it reqires MaxMemory32Bytes to be
+// a constant or constexpr function, not a regular function as now.
+//
+// The assert is also present in WasmMemoryObject::isHuge and
+// WasmMemoryObject::grow, so it's OK to comment out here for now.
 
-void Val::trace(JSTracer* trc) {
-  if (type_.isValid() && type_.isReference() && !u.ref_.isNull()) {
-    // TODO/AnyRef-boxing: With boxed immediates and strings, the write
-    // barrier is going to have to be more complicated.
-    ASSERT_ANYREF_IS_JSOBJECT;
-    TraceManuallyBarrieredEdge(trc, u.ref_.asJSObjectAddress(),
-                               "wasm reference-typed global");
-  }
-}
+// static_assert(MaxMemory32Bytes < HugeMappedSize(),
+//               "Normal array buffer could be confused with huge memory");
+#endif
 
-void AnyRef::trace(JSTracer* trc) {
-  if (value_) {
-    TraceManuallyBarrieredEdge(trc, &value_, "wasm anyref referent");
-  }
-}
+const JSClass WasmJSExceptionObject::class_ = {
+    "WasmJSExnRefObject", JSCLASS_HAS_RESERVED_SLOTS(RESERVED_SLOTS)};
 
-class WasmValueBox : public NativeObject {
-  static const unsigned VALUE_SLOT = 0;
+WasmJSExceptionObject* WasmJSExceptionObject::create(JSContext* cx,
+                                                     MutableHandleValue value) {
+  WasmJSExceptionObject* obj =
+      NewObjectWithGivenProto<WasmJSExceptionObject>(cx, nullptr);
 
- public:
-  static const unsigned RESERVED_SLOTS = 1;
-  static const Class class_;
-
-  static WasmValueBox* create(JSContext* cx, HandleValue val);
-  Value value() const { return getFixedSlot(VALUE_SLOT); }
-};
-
-const Class WasmValueBox::class_ = {"WasmValueBox",
-                                    JSCLASS_HAS_RESERVED_SLOTS(RESERVED_SLOTS)};
-
-WasmValueBox* WasmValueBox::create(JSContext* cx, HandleValue val) {
-  WasmValueBox* obj = (WasmValueBox*)NewObjectWithGivenProto(
-      cx, &WasmValueBox::class_, nullptr);
   if (!obj) {
     return nullptr;
   }
-  obj->setFixedSlot(VALUE_SLOT, val);
+
+  obj->setFixedSlot(VALUE_SLOT, value);
+
   return obj;
 }
 
-bool wasm::BoxAnyRef(JSContext* cx, HandleValue val, MutableHandleAnyRef addr) {
-  if (val.isNull()) {
-    addr.set(AnyRef::null());
-    return true;
-  }
-
-  if (val.isObject()) {
-    JSObject* obj = &val.toObject();
-    MOZ_ASSERT(!obj->is<WasmValueBox>());
-    MOZ_ASSERT(obj->compartment() == cx->compartment());
-    addr.set(AnyRef::fromJSObject(obj));
-    return true;
-  }
-
-  WasmValueBox* box = WasmValueBox::create(cx, val);
-  if (!box) return false;
-  addr.set(AnyRef::fromJSObject(box));
-  return true;
-}
-
-Value wasm::UnboxAnyRef(AnyRef val) {
-  // If UnboxAnyRef needs to allocate then we need a more complicated API, and
-  // we need to root the value in the callers, see comments in callExport().
-  JSObject* obj = val.asJSObject();
-  Value result;
-  if (obj == nullptr) {
-    result.setNull();
-  } else if (obj->is<WasmValueBox>()) {
-    result = obj->as<WasmValueBox>().value();
-  } else {
-    result.setObjectOrNull(obj);
-  }
-  return result;
-}
-
-bool js::IsBoxedWasmAnyRef(JSContext* cx, unsigned argc, Value* vp) {
-  CallArgs args = CallArgsFromVp(argc, vp);
-  MOZ_ASSERT(args.length() == 1);
-  args.rval().setBoolean(args[0].isObject() &&
-                         args[0].toObject().is<WasmValueBox>());
-  return true;
-}
-
-bool js::IsBoxableWasmAnyRef(JSContext* cx, unsigned argc, Value* vp) {
-  CallArgs args = CallArgsFromVp(argc, vp);
-  MOZ_ASSERT(args.length() == 1);
-  args.rval().setBoolean(!(args[0].isObject() || args[0].isNull()));
-  return true;
-}
-
-bool js::BoxWasmAnyRef(JSContext* cx, unsigned argc, Value* vp) {
-  CallArgs args = CallArgsFromVp(argc, vp);
-  MOZ_ASSERT(args.length() == 1);
-  WasmValueBox* box = WasmValueBox::create(cx, args[0]);
-  if (!box) return false;
-  args.rval().setObject(*box);
-  return true;
-}
-
-bool js::UnboxBoxedWasmAnyRef(JSContext* cx, unsigned argc, Value* vp) {
-  CallArgs args = CallArgsFromVp(argc, vp);
-  MOZ_ASSERT(args.length() == 1);
-  WasmValueBox* box = &args[0].toObject().as<WasmValueBox>();
-  args.rval().set(box->value());
-  return true;
-}
-
-bool wasm::IsRoundingFunction(SymbolicAddress callee, jit::RoundingMode* mode) {
-  switch (callee) {
-    case SymbolicAddress::FloorD:
-    case SymbolicAddress::FloorF:
-      *mode = jit::RoundingMode::Down;
-      return true;
-    case SymbolicAddress::CeilD:
-    case SymbolicAddress::CeilF:
-      *mode = jit::RoundingMode::Up;
-      return true;
-    case SymbolicAddress::TruncD:
-    case SymbolicAddress::TruncF:
-      *mode = jit::RoundingMode::TowardsZero;
-      return true;
-    case SymbolicAddress::NearbyIntD:
-    case SymbolicAddress::NearbyIntF:
-      *mode = jit::RoundingMode::NearestTiesToEven;
-      return true;
-    default:
-      return false;
-  }
-}
-
-size_t FuncType::serializedSize() const {
-  return sizeof(ret_) + SerializedPodVectorSize(args_);
-}
-
-uint8_t* FuncType::serialize(uint8_t* cursor) const {
-  cursor = WriteScalar<ExprType>(cursor, ret_);
-  cursor = SerializePodVector(cursor, args_);
-  return cursor;
-}
-
-namespace js {
-namespace wasm {
-
-// ExprType is not POD while ReadScalar requires POD, so specialize.
-template <>
-inline const uint8_t* ReadScalar<ExprType>(const uint8_t* src, ExprType* dst) {
-  static_assert(sizeof(PackedTypeCode) == sizeof(ExprType),
-                "ExprType must carry only a PackedTypeCode");
-  memcpy(dst->packedPtr(), src, sizeof(PackedTypeCode));
-  return src + sizeof(*dst);
-}
-
-}  // namespace wasm
-}  // namespace js
-
-const uint8_t* FuncType::deserialize(const uint8_t* cursor) {
-  (cursor = ReadScalar<ExprType>(cursor, &ret_)) &&
-      (cursor = DeserializePodVector(cursor, &args_));
-  return cursor;
-}
-
-size_t FuncType::sizeOfExcludingThis(MallocSizeOf mallocSizeOf) const {
-  return args_.sizeOfExcludingThis(mallocSizeOf);
-}
-
-typedef uint32_t ImmediateType;  // for 32/64 consistency
+using ImmediateType = uint32_t;  // for 32/64 consistency
 static const unsigned sTotalBits = sizeof(ImmediateType) * 8;
 static const unsigned sTagBits = 1;
 static const unsigned sReturnBit = 1;
@@ -263,16 +100,24 @@ static const unsigned sMaxTypes =
     (sTotalBits - sTagBits - sReturnBit - sLengthBits) / sTypeBits;
 
 static bool IsImmediateType(ValType vt) {
-  switch (vt.code()) {
+  switch (vt.kind()) {
     case ValType::I32:
     case ValType::I64:
     case ValType::F32:
     case ValType::F64:
-    case ValType::FuncRef:
-    case ValType::AnyRef:
+    case ValType::V128:
       return true;
-    case ValType::NullRef:
     case ValType::Ref:
+      switch (vt.refTypeKind()) {
+        case RefType::Func:
+        case RefType::Extern:
+        case RefType::Eq:
+          return true;
+        case RefType::TypeIndex:
+          return false;
+      }
+      break;
+    case ValType::Rtt:
       return false;
   }
   MOZ_CRASH("bad ValType");
@@ -280,7 +125,7 @@ static bool IsImmediateType(ValType vt) {
 
 static unsigned EncodeImmediateType(ValType vt) {
   static_assert(4 < (1 << sTypeBits), "fits");
-  switch (vt.code()) {
+  switch (vt.kind()) {
     case ValType::I32:
       return 0;
     case ValType::I64:
@@ -289,31 +134,49 @@ static unsigned EncodeImmediateType(ValType vt) {
       return 2;
     case ValType::F64:
       return 3;
-    case ValType::FuncRef:
+    case ValType::V128:
       return 4;
-    case ValType::AnyRef:
-      return 5;
-    case ValType::NullRef:
     case ValType::Ref:
+      switch (vt.refTypeKind()) {
+        case RefType::Func:
+          return 5;
+        case RefType::Extern:
+          return 6;
+        case RefType::Eq:
+          return 7;
+        case RefType::TypeIndex:
+          break;
+      }
+      break;
+    case ValType::Rtt:
       break;
   }
   MOZ_CRASH("bad ValType");
 }
 
 /* static */
-bool FuncTypeIdDesc::isGlobal(const FuncType& funcType) {
-  unsigned numTypes =
-      (funcType.ret() == ExprType::Void ? 0 : 1) + (funcType.args().length());
-  if (numTypes > sMaxTypes) {
+bool TypeIdDesc::isGlobal(const TypeDef& type) {
+  if (!type.isFuncType()) {
+    return true;
+  }
+  const FuncType& funcType = type.funcType();
+  const ValTypeVector& results = funcType.results();
+  const ValTypeVector& args = funcType.args();
+  if (results.length() + args.length() > sMaxTypes) {
     return true;
   }
 
-  if (funcType.ret() != ExprType::Void &&
-      !IsImmediateType(NonVoidToValType(funcType.ret()))) {
+  if (results.length() > 1) {
     return true;
   }
 
-  for (ValType v : funcType.args()) {
+  for (ValType v : results) {
+    if (!IsImmediateType(v)) {
+      return true;
+    }
+  }
+
+  for (ValType v : args) {
     if (!IsImmediateType(v)) {
       return true;
     }
@@ -323,10 +186,9 @@ bool FuncTypeIdDesc::isGlobal(const FuncType& funcType) {
 }
 
 /* static */
-FuncTypeIdDesc FuncTypeIdDesc::global(const FuncType& funcType,
-                                      uint32_t globalDataOffset) {
-  MOZ_ASSERT(isGlobal(funcType));
-  return FuncTypeIdDesc(FuncTypeIdDescKind::Global, globalDataOffset);
+TypeIdDesc TypeIdDesc::global(const TypeDef& type, uint32_t globalDataOffset) {
+  MOZ_ASSERT(isGlobal(type));
+  return TypeIdDesc(TypeIdDescKind::Global, globalDataOffset);
 }
 
 static ImmediateType LengthToBits(uint32_t length) {
@@ -336,15 +198,18 @@ static ImmediateType LengthToBits(uint32_t length) {
 }
 
 /* static */
-FuncTypeIdDesc FuncTypeIdDesc::immediate(const FuncType& funcType) {
+TypeIdDesc TypeIdDesc::immediate(const TypeDef& type) {
+  const FuncType& funcType = type.funcType();
+
   ImmediateType immediate = ImmediateBit;
   uint32_t shift = sTagBits;
 
-  if (funcType.ret() != ExprType::Void) {
+  if (funcType.results().length() > 0) {
+    MOZ_ASSERT(funcType.results().length() == 1);
     immediate |= (1 << shift);
     shift += sReturnBit;
 
-    immediate |= EncodeImmediateType(NonVoidToValType(funcType.ret())) << shift;
+    immediate |= EncodeImmediateType(funcType.results()[0]) << shift;
     shift += sTypeBits;
   } else {
     shift += sReturnBit;
@@ -359,62 +224,33 @@ FuncTypeIdDesc FuncTypeIdDesc::immediate(const FuncType& funcType) {
   }
 
   MOZ_ASSERT(shift <= sTotalBits);
-  return FuncTypeIdDesc(FuncTypeIdDescKind::Immediate, immediate);
+  return TypeIdDesc(TypeIdDescKind::Immediate, immediate);
 }
 
-size_t FuncTypeWithId::serializedSize() const {
-  return FuncType::serializedSize() + sizeof(id);
+size_t TypeDefWithId::serializedSize() const {
+  return TypeDef::serializedSize() + sizeof(TypeIdDesc);
 }
 
-uint8_t* FuncTypeWithId::serialize(uint8_t* cursor) const {
-  cursor = FuncType::serialize(cursor);
+uint8_t* TypeDefWithId::serialize(uint8_t* cursor) const {
+  cursor = TypeDef::serialize(cursor);
   cursor = WriteBytes(cursor, &id, sizeof(id));
   return cursor;
 }
 
-const uint8_t* FuncTypeWithId::deserialize(const uint8_t* cursor) {
-  (cursor = FuncType::deserialize(cursor)) &&
-      (cursor = ReadBytes(cursor, &id, sizeof(id)));
+const uint8_t* TypeDefWithId::deserialize(const uint8_t* cursor) {
+  cursor = TypeDef::deserialize(cursor);
+  cursor = ReadBytes(cursor, &id, sizeof(id));
   return cursor;
 }
 
-size_t FuncTypeWithId::sizeOfExcludingThis(MallocSizeOf mallocSizeOf) const {
-  return FuncType::sizeOfExcludingThis(mallocSizeOf);
+size_t TypeDefWithId::sizeOfExcludingThis(MallocSizeOf mallocSizeOf) const {
+  return TypeDef::sizeOfExcludingThis(mallocSizeOf);
 }
 
-// A simple notion of prefix: types and mutability must match exactly.
-
-bool StructType::hasPrefix(const StructType& other) const {
-  if (fields_.length() < other.fields_.length()) {
-    return false;
-  }
-  uint32_t limit = other.fields_.length();
-  for (uint32_t i = 0; i < limit; i++) {
-    if (fields_[i].type != other.fields_[i].type ||
-        fields_[i].isMutable != other.fields_[i].isMutable) {
-      return false;
-    }
-  }
-  return true;
-}
-
-size_t StructType::serializedSize() const {
-  return SerializedPodVectorSize(fields_);
-}
-
-uint8_t* StructType::serialize(uint8_t* cursor) const {
-  cursor = SerializePodVector(cursor, fields_);
-  return cursor;
-}
-
-const uint8_t* StructType::deserialize(const uint8_t* cursor) {
-  (cursor = DeserializePodVector(cursor, &fields_));
-  return cursor;
-}
-
-size_t StructType::sizeOfExcludingThis(MallocSizeOf mallocSizeOf) const {
-  return fields_.sizeOfExcludingThis(mallocSizeOf);
-}
+ArgTypeVector::ArgTypeVector(const FuncType& funcType)
+    : args_(funcType.args()),
+      hasStackResults_(ABIResultIter::HasStackResults(
+          ResultType::Vector(funcType.results()))) {}
 
 size_t Import::serializedSize() const {
   return module.serializedSize() + field.serializedSize() + sizeof(kind);
@@ -461,6 +297,13 @@ uint32_t Export::globalIndex() const {
   return pod.index_;
 }
 
+#ifdef ENABLE_WASM_EXCEPTIONS
+uint32_t Export::eventIndex() const {
+  MOZ_ASSERT(pod.kind_ == DefinitionKind::Event);
+  return pod.index_;
+}
+#endif
+
 uint32_t Export::tableIndex() const {
   MOZ_ASSERT(pod.kind_ == DefinitionKind::Table);
   return pod.index_;
@@ -486,47 +329,134 @@ size_t Export::sizeOfExcludingThis(MallocSizeOf mallocSizeOf) const {
   return fieldName_.sizeOfExcludingThis(mallocSizeOf);
 }
 
+size_t GlobalDesc::serializedSize() const {
+  size_t size = sizeof(kind_);
+  switch (kind_) {
+    case GlobalKind::Import:
+      size += initial_.serializedSize() + sizeof(offset_) + sizeof(isMutable_) +
+              sizeof(isWasm_) + sizeof(isExport_) + sizeof(importIndex_);
+      break;
+    case GlobalKind::Variable:
+      size += initial_.serializedSize() + sizeof(offset_) + sizeof(isMutable_) +
+              sizeof(isWasm_) + sizeof(isExport_);
+      break;
+    case GlobalKind::Constant:
+      size += initial_.serializedSize();
+      break;
+    default:
+      MOZ_CRASH();
+  }
+  return size;
+}
+
+uint8_t* GlobalDesc::serialize(uint8_t* cursor) const {
+  cursor = WriteBytes(cursor, &kind_, sizeof(kind_));
+  switch (kind_) {
+    case GlobalKind::Import:
+      cursor = initial_.serialize(cursor);
+      cursor = WriteBytes(cursor, &offset_, sizeof(offset_));
+      cursor = WriteBytes(cursor, &isMutable_, sizeof(isMutable_));
+      cursor = WriteBytes(cursor, &isWasm_, sizeof(isWasm_));
+      cursor = WriteBytes(cursor, &isExport_, sizeof(isExport_));
+      cursor = WriteBytes(cursor, &importIndex_, sizeof(importIndex_));
+      break;
+    case GlobalKind::Variable:
+      cursor = initial_.serialize(cursor);
+      cursor = WriteBytes(cursor, &offset_, sizeof(offset_));
+      cursor = WriteBytes(cursor, &isMutable_, sizeof(isMutable_));
+      cursor = WriteBytes(cursor, &isWasm_, sizeof(isWasm_));
+      cursor = WriteBytes(cursor, &isExport_, sizeof(isExport_));
+      break;
+    case GlobalKind::Constant:
+      cursor = initial_.serialize(cursor);
+      break;
+    default:
+      MOZ_CRASH();
+  }
+  return cursor;
+}
+
+const uint8_t* GlobalDesc::deserialize(const uint8_t* cursor) {
+  if (!(cursor = ReadBytes(cursor, &kind_, sizeof(kind_)))) {
+    return nullptr;
+  }
+  switch (kind_) {
+    case GlobalKind::Import:
+      (cursor = initial_.deserialize(cursor)) &&
+          (cursor = ReadBytes(cursor, &offset_, sizeof(offset_))) &&
+          (cursor = ReadBytes(cursor, &isMutable_, sizeof(isMutable_))) &&
+          (cursor = ReadBytes(cursor, &isWasm_, sizeof(isWasm_))) &&
+          (cursor = ReadBytes(cursor, &isExport_, sizeof(isExport_))) &&
+          (cursor = ReadBytes(cursor, &importIndex_, sizeof(importIndex_)));
+      break;
+    case GlobalKind::Variable:
+      (cursor = initial_.deserialize(cursor)) &&
+          (cursor = ReadBytes(cursor, &offset_, sizeof(offset_))) &&
+          (cursor = ReadBytes(cursor, &isMutable_, sizeof(isMutable_))) &&
+          (cursor = ReadBytes(cursor, &isWasm_, sizeof(isWasm_))) &&
+          (cursor = ReadBytes(cursor, &isExport_, sizeof(isExport_)));
+      break;
+    case GlobalKind::Constant:
+      cursor = initial_.deserialize(cursor);
+      break;
+    default:
+      MOZ_CRASH();
+  }
+  return cursor;
+}
+
+size_t GlobalDesc::sizeOfExcludingThis(MallocSizeOf mallocSizeOf) const {
+  return initial_.sizeOfExcludingThis(mallocSizeOf);
+}
+
 size_t ElemSegment::serializedSize() const {
-  return sizeof(tableIndex) + sizeof(offsetIfActive) +
+  return sizeof(kind) + sizeof(tableIndex) + sizeof(elemType) +
+         SerializedMaybeSize(offsetIfActive) +
          SerializedPodVectorSize(elemFuncIndices);
 }
 
 uint8_t* ElemSegment::serialize(uint8_t* cursor) const {
+  cursor = WriteBytes(cursor, &kind, sizeof(kind));
   cursor = WriteBytes(cursor, &tableIndex, sizeof(tableIndex));
-  cursor = WriteBytes(cursor, &offsetIfActive, sizeof(offsetIfActive));
+  cursor = WriteBytes(cursor, &elemType, sizeof(elemType));
+  cursor = SerializeMaybe(cursor, offsetIfActive);
   cursor = SerializePodVector(cursor, elemFuncIndices);
   return cursor;
 }
 
 const uint8_t* ElemSegment::deserialize(const uint8_t* cursor) {
-  (cursor = ReadBytes(cursor, &tableIndex, sizeof(tableIndex))) &&
-      (cursor = ReadBytes(cursor, &offsetIfActive, sizeof(offsetIfActive))) &&
+  (cursor = ReadBytes(cursor, &kind, sizeof(kind))) &&
+      (cursor = ReadBytes(cursor, &tableIndex, sizeof(tableIndex))) &&
+      (cursor = ReadBytes(cursor, &elemType, sizeof(elemType))) &&
+      (cursor = DeserializeMaybe(cursor, &offsetIfActive)) &&
       (cursor = DeserializePodVector(cursor, &elemFuncIndices));
   return cursor;
 }
 
 size_t ElemSegment::sizeOfExcludingThis(MallocSizeOf mallocSizeOf) const {
-  return elemFuncIndices.sizeOfExcludingThis(mallocSizeOf);
+  return SizeOfMaybeExcludingThis(offsetIfActive, mallocSizeOf) +
+         elemFuncIndices.sizeOfExcludingThis(mallocSizeOf);
 }
 
 size_t DataSegment::serializedSize() const {
-  return sizeof(offsetIfActive) + SerializedPodVectorSize(bytes);
+  return SerializedMaybeSize(offsetIfActive) + SerializedPodVectorSize(bytes);
 }
 
 uint8_t* DataSegment::serialize(uint8_t* cursor) const {
-  cursor = WriteBytes(cursor, &offsetIfActive, sizeof(offsetIfActive));
+  cursor = SerializeMaybe(cursor, offsetIfActive);
   cursor = SerializePodVector(cursor, bytes);
   return cursor;
 }
 
 const uint8_t* DataSegment::deserialize(const uint8_t* cursor) {
-  (cursor = ReadBytes(cursor, &offsetIfActive, sizeof(offsetIfActive))) &&
+  (cursor = DeserializeMaybe(cursor, &offsetIfActive)) &&
       (cursor = DeserializePodVector(cursor, &bytes));
   return cursor;
 }
 
 size_t DataSegment::sizeOfExcludingThis(MallocSizeOf mallocSizeOf) const {
-  return bytes.sizeOfExcludingThis(mallocSizeOf);
+  return SizeOfMaybeExcludingThis(offsetIfActive, mallocSizeOf) +
+         bytes.sizeOfExcludingThis(mallocSizeOf);
 }
 
 size_t CustomSection::serializedSize() const {
@@ -577,8 +507,10 @@ bool wasm::IsValidARMImmediate(uint32_t i) {
   return valid;
 }
 
-uint32_t wasm::RoundUpToNextValidARMImmediate(uint32_t i) {
-  MOZ_ASSERT(i <= 0xff000000);
+uint64_t wasm::RoundUpToNextValidARMImmediate(uint64_t i) {
+  MOZ_ASSERT(i <= HighestValidARMImmediate);
+  static_assert(HighestValidARMImmediate == 0xff000000,
+                "algorithm relies on specific constant");
 
   if (i <= 16 * 1024 * 1024) {
     i = i ? mozilla::RoundUpPow2(i) : 0;
@@ -591,28 +523,27 @@ uint32_t wasm::RoundUpToNextValidARMImmediate(uint32_t i) {
   return i;
 }
 
-#ifndef WASM_HUGE_MEMORY
-
 bool wasm::IsValidBoundsCheckImmediate(uint32_t i) {
-#  ifdef JS_CODEGEN_ARM
+#ifdef JS_CODEGEN_ARM
   return IsValidARMImmediate(i);
-#  else
+#else
   return true;
-#  endif
+#endif
 }
 
-size_t wasm::ComputeMappedSize(uint32_t maxSize) {
-  MOZ_ASSERT(maxSize % PageSize == 0);
+size_t wasm::ComputeMappedSize(wasm::Pages maxPages) {
+  // TODO: memory64 maximum size may overflow size_t
+  size_t maxSize = maxPages.byteLength();
 
   // It is the bounds-check limit, not the mapped size, that gets baked into
   // code. Thus round up the maxSize to the next valid immediate value
   // *before* adding in the guard page.
 
-#  ifdef JS_CODEGEN_ARM
-  uint32_t boundsCheckLimit = RoundUpToNextValidARMImmediate(maxSize);
-#  else
-  uint32_t boundsCheckLimit = maxSize;
-#  endif
+#ifdef JS_CODEGEN_ARM
+  uint64_t boundsCheckLimit = RoundUpToNextValidARMImmediate(maxSize);
+#else
+  uint64_t boundsCheckLimit = maxSize;
+#endif
   MOZ_ASSERT(IsValidBoundsCheckImmediate(boundsCheckLimit));
 
   MOZ_ASSERT(boundsCheckLimit % gc::SystemPageSize() == 0);
@@ -620,143 +551,6 @@ size_t wasm::ComputeMappedSize(uint32_t maxSize) {
   return boundsCheckLimit + GuardSize;
 }
 
-#endif  // WASM_HUGE_MEMORY
-
-/* static */
-DebugFrame* DebugFrame::from(Frame* fp) {
-  MOZ_ASSERT(fp->tls->instance->code().metadata().debugEnabled);
-  auto* df =
-      reinterpret_cast<DebugFrame*>((uint8_t*)fp - DebugFrame::offsetOfFrame());
-  MOZ_ASSERT(fp->instance() == df->instance());
-  return df;
-}
-
-void DebugFrame::alignmentStaticAsserts() {
-  // VS2017 doesn't consider offsetOfFrame() to be a constexpr, so we have
-  // to use offsetof directly. These asserts can't be at class-level
-  // because the type is incomplete.
-
-  static_assert(WasmStackAlignment >= Alignment,
-                "Aligned by ABI before pushing DebugFrame");
-  static_assert((offsetof(DebugFrame, frame_) + sizeof(Frame)) % Alignment == 0,
-                "Aligned after pushing DebugFrame");
-#ifdef JS_CODEGEN_ARM64
-  // This constraint may or may not be necessary.  If you hit this because
-  // you've changed the frame size then feel free to remove it, but be extra
-  // aware of possible problems.
-  static_assert(sizeof(DebugFrame) % 16 == 0, "ARM64 SP alignment");
-#endif
-}
-
-GlobalObject* DebugFrame::global() const {
-  return &instance()->object()->global();
-}
-
-bool DebugFrame::hasGlobal(const GlobalObject* global) const {
-  return global == &instance()->objectUnbarriered()->global();
-}
-
-JSObject* DebugFrame::environmentChain() const {
-  return &global()->lexicalEnvironment();
-}
-
-bool DebugFrame::getLocal(uint32_t localIndex, MutableHandleValue vp) {
-  ValTypeVector locals;
-  size_t argsLength;
-  if (!instance()->debug().debugGetLocalTypes(funcIndex(), &locals,
-                                              &argsLength)) {
-    return false;
-  }
-
-  BaseLocalIter iter(locals, argsLength, /* debugEnabled = */ true);
-  while (!iter.done() && iter.index() < localIndex) {
-    iter++;
-  }
-  MOZ_ALWAYS_TRUE(!iter.done());
-
-  uint8_t* frame = static_cast<uint8_t*>((void*)this) + offsetOfFrame();
-  void* dataPtr = frame - iter.frameOffset();
-  switch (iter.mirType()) {
-    case jit::MIRType::Int32:
-      vp.set(Int32Value(*static_cast<int32_t*>(dataPtr)));
-      break;
-    case jit::MIRType::Int64:
-      // Just display as a Number; it's ok if we lose some precision
-      vp.set(NumberValue((double)*static_cast<int64_t*>(dataPtr)));
-      break;
-    case jit::MIRType::Float32:
-      vp.set(NumberValue(JS::CanonicalizeNaN(*static_cast<float*>(dataPtr))));
-      break;
-    case jit::MIRType::Double:
-      vp.set(NumberValue(JS::CanonicalizeNaN(*static_cast<double*>(dataPtr))));
-      break;
-    case jit::MIRType::RefOrNull:
-      vp.set(ObjectOrNullValue(*(JSObject**)dataPtr));
-      break;
-    default:
-      MOZ_CRASH("local type");
-  }
-  return true;
-}
-
-void DebugFrame::updateReturnJSValue() {
-  hasCachedReturnJSValue_ = true;
-  ExprType returnType = instance()->debug().debugGetResultType(funcIndex());
-  switch (returnType.code()) {
-    case ExprType::Void:
-      cachedReturnJSValue_.setUndefined();
-      break;
-    case ExprType::I32:
-      cachedReturnJSValue_.setInt32(resultI32_);
-      break;
-    case ExprType::I64:
-      // Just display as a Number; it's ok if we lose some precision
-      cachedReturnJSValue_.setDouble((double)resultI64_);
-      break;
-    case ExprType::F32:
-      cachedReturnJSValue_.setDouble(JS::CanonicalizeNaN(resultF32_));
-      break;
-    case ExprType::F64:
-      cachedReturnJSValue_.setDouble(JS::CanonicalizeNaN(resultF64_));
-      break;
-    case ExprType::Ref:
-      cachedReturnJSValue_ = ObjectOrNullValue((JSObject*)resultRef_);
-      break;
-    case ExprType::FuncRef:
-    case ExprType::AnyRef:
-      cachedReturnJSValue_ = UnboxAnyRef(resultAnyRef_);
-      break;
-    default:
-      MOZ_CRASH("result type");
-  }
-}
-
-HandleValue DebugFrame::returnValue() const {
-  MOZ_ASSERT(hasCachedReturnJSValue_);
-  return HandleValue::fromMarkedLocation(&cachedReturnJSValue_);
-}
-
-void DebugFrame::clearReturnJSValue() {
-  hasCachedReturnJSValue_ = true;
-  cachedReturnJSValue_.setUndefined();
-}
-
-void DebugFrame::observe(JSContext* cx) {
-  if (!observing_) {
-    instance()->debug().adjustEnterAndLeaveFrameTrapsState(
-        cx, /* enabled = */ true);
-    observing_ = true;
-  }
-}
-
-void DebugFrame::leave(JSContext* cx) {
-  if (observing_) {
-    instance()->debug().adjustEnterAndLeaveFrameTrapsState(
-        cx, /* enabled = */ false);
-    observing_ = false;
-  }
-}
-
 bool TrapSiteVectorArray::empty() const {
   for (Trap trap : MakeEnumeratedRange(Trap::Limit)) {
     if (!(*this)[trap].empty()) {
@@ -779,9 +573,9 @@ void TrapSiteVectorArray::swap(TrapSiteVectorArray& rhs) {
   }
 }
 
-void TrapSiteVectorArray::podResizeToFit() {
+void TrapSiteVectorArray::shrinkStorageToFit() {
   for (Trap trap : MakeEnumeratedRange(Trap::Limit)) {
-    (*this)[trap].podResizeToFit();
+    (*this)[trap].shrinkStorageToFit();
   }
 }
 
@@ -839,7 +633,7 @@ CodeRange::CodeRange(Kind kind, uint32_t funcIndex, Offsets offsets)
     : begin_(offsets.begin), ret_(0), end_(offsets.end), kind_(kind) {
   u.funcIndex_ = funcIndex;
   u.func.lineOrBytecode_ = 0;
-  u.func.beginToNormalEntry_ = 0;
+  u.func.beginToUncheckedCallEntry_ = 0;
   u.func.beginToTierEntry_ = 0;
   MOZ_ASSERT(isEntry());
   MOZ_ASSERT(begin_ <= end_);
@@ -868,7 +662,7 @@ CodeRange::CodeRange(Kind kind, uint32_t funcIndex, CallableOffsets offsets)
   MOZ_ASSERT(ret_ < end_);
   u.funcIndex_ = funcIndex;
   u.func.lineOrBytecode_ = 0;
-  u.func.beginToNormalEntry_ = 0;
+  u.func.beginToUncheckedCallEntry_ = 0;
   u.func.beginToTierEntry_ = 0;
 }
 
@@ -895,11 +689,11 @@ CodeRange::CodeRange(uint32_t funcIndex, uint32_t funcLineOrBytecode,
       kind_(Function) {
   MOZ_ASSERT(begin_ < ret_);
   MOZ_ASSERT(ret_ < end_);
-  MOZ_ASSERT(offsets.normalEntry - begin_ <= UINT8_MAX);
+  MOZ_ASSERT(offsets.uncheckedCallEntry - begin_ <= UINT8_MAX);
   MOZ_ASSERT(offsets.tierEntry - begin_ <= UINT8_MAX);
   u.funcIndex_ = funcIndex;
   u.func.lineOrBytecode_ = funcLineOrBytecode;
-  u.func.beginToNormalEntry_ = offsets.normalEntry - begin_;
+  u.func.beginToUncheckedCallEntry_ = offsets.uncheckedCallEntry - begin_;
   u.func.beginToTierEntry_ = offsets.tierEntry - begin_;
 }
 
@@ -916,34 +710,6 @@ const CodeRange* wasm::LookupInSorted(const CodeRangeVector& codeRanges,
   return &codeRanges[match];
 }
 
-UniqueTlsData wasm::CreateTlsData(uint32_t globalDataLength) {
-  void* allocatedBase = js_calloc(TlsDataAlign + offsetof(TlsData, globalArea) +
-                                  globalDataLength);
-  if (!allocatedBase) {
-    return nullptr;
-  }
-
-  auto* tlsData = reinterpret_cast<TlsData*>(
-      AlignBytes(uintptr_t(allocatedBase), TlsDataAlign));
-  tlsData->allocatedBase = allocatedBase;
-
-  return UniqueTlsData(tlsData);
-}
-
-void TlsData::setInterrupt() {
-  interrupt = true;
-  stackLimit = UINTPTR_MAX;
-}
-
-bool TlsData::isInterrupted() const {
-  return interrupt || stackLimit == UINTPTR_MAX;
-}
-
-void TlsData::resetInterrupt(JSContext* cx) {
-  interrupt = false;
-  stackLimit = cx->stackLimitForJitCode(JS::StackForUntrustedScript);
-}
-
 void wasm::Log(JSContext* cx, const char* fmt, ...) {
   MOZ_ASSERT(!cx->isExceptionPending());
 
@@ -955,8 +721,7 @@ void wasm::Log(JSContext* cx, const char* fmt, ...) {
   va_start(args, fmt);
 
   if (UniqueChars chars = JS_vsmprintf(fmt, args)) {
-    JS_ReportErrorFlagsAndNumberASCII(cx, JSREPORT_WARNING, GetErrorMessage,
-                                      nullptr, JSMSG_WASM_VERBOSE, chars.get());
+    WarnNumberASCII(cx, JSMSG_WASM_VERBOSE, chars.get());
     if (cx->isExceptionPending()) {
       cx->clearPendingException();
     }
@@ -988,3 +753,24 @@ void wasm::DebugCodegen(DebugChannel channel, const char* fmt, ...) {
   va_end(ap);
 #endif
 }
+
+#ifdef ENABLE_WASM_SIMD_WORMHOLE
+static const int8_t WormholeTrigger[] = {31, 0, 30, 2,  29, 4,  28, 6,
+                                         27, 8, 26, 10, 25, 12, 24};
+static_assert(sizeof(WormholeTrigger) == 15);
+
+static const int8_t WormholeSignatureBytes[16] = {0xD, 0xE, 0xA, 0xD, 0xD, 0x0,
+                                                  0x0, 0xD, 0xC, 0xA, 0xF, 0xE,
+                                                  0xB, 0xA, 0xB, 0xE};
+static_assert(sizeof(WormholeSignatureBytes) == 16);
+
+bool wasm::IsWormholeTrigger(const V128& shuffleMask) {
+  return memcmp(shuffleMask.bytes, WormholeTrigger, sizeof(WormholeTrigger)) ==
+         0;
+}
+
+jit::SimdConstant wasm::WormholeSignature() {
+  return jit::SimdConstant::CreateX16(WormholeSignatureBytes);
+}
+
+#endif
