@@ -9,7 +9,7 @@ import os
 def read_file(path, base_path=None):
     if base_path is not None:
         path = os.path.join(base_path, path)
-        path = path.replace('\\', '/')
+        path = path.replace("\\", "/")
     with open(path) as fptr:
         return fptr.read()
 
@@ -17,7 +17,7 @@ def read_file(path, base_path=None):
 def write_file(path, text, base_path=None, append=False):
     if base_path is not None:
         path = os.path.join(base_path, path)
-        path = path.replace('\\', '/')
+        path = path.replace("\\", "/")
     mode = "a" if append and os.path.exists(path) else "w"
     with open(path, mode) as text_file:
         text_file.write(text)
