@@ -14,12 +14,11 @@ from mozbuild.base import MachCommandBase
 
 @CommandProvider
 class ConditionsProvider(MachCommandBase):
-    @Command('cmd_foo', category='testing')
-    def run_foo(self):
+    @Command("cmd_foo", category="testing")
+    def run_foo(self, command_context):
         pass
 
-    @Command('cmd_bar', category='testing')
-    @CommandArgument('--baz', action="store_true",
-                     help='Run with baz')
-    def run_bar(self, baz=None):
+    @Command("cmd_bar", category="testing")
+    @CommandArgument("--baz", action="store_true", help="Run with baz")
+    def run_bar(self, command_context, baz=None):
         pass
