@@ -1,12 +1,11 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from ..utilities import flat
+from mozperftest.metrics.notebook.utilities import flat
 
 
 class SingleJsonRetriever:
-    """Transforms perfherder data into the standardized data format.
-    """
+    """Transforms perfherder data into the standardized data format."""
 
     entry_number = 0
 
