@@ -8,20 +8,20 @@ import os
 
 
 def normsep(path):
-    '''
+    """
     Normalize path separators, by using forward slashes instead of whatever
     :py:const:`os.sep` is.
-    '''
-    if os.sep != '/':
+    """
+    if os.sep != "/":
         # Python 2 is happy to do things like byte_string.replace(u'foo',
         # u'bar'), but not Python 3.
         if isinstance(path, bytes):
-            path = path.replace(os.sep.encode('ascii'), b'/')
+            path = path.replace(os.sep.encode("ascii"), b"/")
         else:
-            path = path.replace(os.sep, '/')
-    if os.altsep and os.altsep != '/':
+            path = path.replace(os.sep, "/")
+    if os.altsep and os.altsep != "/":
         if isinstance(path, bytes):
-            path = path.replace(os.altsep.encode('ascii'), b'/')
+            path = path.replace(os.altsep.encode("ascii"), b"/")
         else:
-            path = path.replace(os.altsep, '/')
+            path = path.replace(os.altsep, "/")
     return path
