@@ -25,9 +25,8 @@ else:
     mozharness_dir = path_join(mozbase_dir, "..", "mozharness")
 
 
-def get_playback(config, android_device=None):
-    """ Returns an instance of the right Playback class
-    """
+def get_playback(config):
+    """Returns an instance of the right Playback class"""
     sys.path.insert(0, mozharness_dir)
     sys.path.insert(0, mozproxy_dir)
     sys.path.insert(0, mozproxy_src_dir)
@@ -40,10 +39,7 @@ def get_playback(config, android_device=None):
         LOG.critical("playback_tool name not found in config")
         return None
     try:
-        if android_device is None:
-            return get_backend(tool_name, config)
-        else:
-            return get_backend(tool_name, config, android_device)
+        return get_backend(tool_name, config)
     except KeyError:
         LOG.critical("specified playback tool is unsupported: %s" % tool_name)
         return None
