@@ -26,64 +26,25 @@ test(() => {
 
 test(() => {
 
-  assert_throws_js(RangeError, () => new ReadableStream({ type: null }),
+  assert_throws_js(TypeError, () => new ReadableStream({ type: null }),
     'constructor should throw when the type is null');
-  assert_throws_js(RangeError, () => new ReadableStream({ type: '' }),
+  assert_throws_js(TypeError, () => new ReadableStream({ type: '' }),
     'constructor should throw when the type is empty string');
-  assert_throws_js(RangeError, () => new ReadableStream({ type: 'asdf' }),
+  assert_throws_js(TypeError, () => new ReadableStream({ type: 'asdf' }),
     'constructor should throw when the type is asdf');
-  assert_throws_exactly(error1, () => new ReadableStream({ type: { get toString() {throw error1;} } }), 'constructor should throw when ToString() throws');
-  assert_throws_exactly(error1, () => new ReadableStream({ type: { toString() {throw error1;} } }), 'constructor should throw when ToString() throws');
+  assert_throws_exactly(
+    error1,
+    () => new ReadableStream({ type: { get toString() { throw error1; } } }),
+    'constructor should throw when ToString() throws'
+  );
+  assert_throws_exactly(
+    error1,
+    () => new ReadableStream({ type: { toString() { throw error1; } } }),
+    'constructor should throw when ToString() throws'
+  );
 
 }, 'ReadableStream can\'t be constructed with an invalid type');
 
-test(() => {
-
-  const methods = ['cancel', 'constructor', 'getReader', 'pipeThrough', 'pipeTo', 'tee', 'getIterator'];
-  const properties = methods.concat(['locked']).sort();
-  const symbols = [Symbol.asyncIterator];
-
-  const rs = new ReadableStream();
-  const proto = Object.getPrototypeOf(rs);
-
-  assert_array_equals(Object.getOwnPropertyNames(proto).sort(), properties, 'should have all the correct properties');
-  assert_array_equals(Object.getOwnPropertySymbols(proto).sort(), symbols, 'should have all the correct symbols');
-
-  for (const m of methods) {
-    const propDesc = Object.getOwnPropertyDescriptor(proto, m);
-    assert_false(propDesc.enumerable, 'method should be non-enumerable');
-    assert_true(propDesc.configurable, 'method should be configurable');
-    assert_true(propDesc.writable, 'method should be writable');
-    assert_equals(typeof rs[m], 'function', 'method should be a function');
-    const expectedName = m === 'constructor' ? 'ReadableStream' : m;
-    assert_equals(rs[m].name, expectedName, 'method should have the correct name');
-  }
-
-  const lockedPropDesc = Object.getOwnPropertyDescriptor(proto, 'locked');
-  assert_false(lockedPropDesc.enumerable, 'locked should be non-enumerable');
-  assert_equals(lockedPropDesc.writable, undefined, 'locked should not be a data property');
-  assert_equals(typeof lockedPropDesc.get, 'function', 'locked should have a getter');
-  assert_equals(lockedPropDesc.set, undefined, 'locked should not have a setter');
-  assert_true(lockedPropDesc.configurable, 'locked should be configurable');
-
-  assert_equals(rs.cancel.length, 1, 'cancel should have 1 parameter');
-  assert_equals(rs.constructor.length, 0, 'constructor should have no parameters');
-  assert_equals(rs.getReader.length, 0, 'getReader should have no parameters');
-  assert_equals(rs.pipeThrough.length, 1, 'pipeThrough should have 1 parameters');
-  assert_equals(rs.pipeTo.length, 1, 'pipeTo should have 1 parameter');
-  assert_equals(rs.tee.length, 0, 'tee should have no parameters');
-  assert_equals(rs.getIterator.length, 0, 'getIterator should have no required parameters');
-  assert_equals(rs[Symbol.asyncIterator].length, 0, '@@asyncIterator should have no required parameters');
-
-  const asyncIteratorPropDesc = Object.getOwnPropertyDescriptor(proto, Symbol.asyncIterator);
-  assert_false(asyncIteratorPropDesc.enumerable, '@@asyncIterator should be non-enumerable');
-  assert_true(asyncIteratorPropDesc.configurable, '@@asyncIterator should be configurable');
-  assert_true(asyncIteratorPropDesc.writable, '@@asyncIterator should be writable');
-  assert_equals(typeof rs[Symbol.asyncIterator], 'function', '@@asyncIterator should be a function');
-  assert_equals(rs[Symbol.asyncIterator].name, 'getIterator', '@@asyncIterator should have the correct name');
-
-}, 'ReadableStream instances should have the correct list of properties');
-
 test(() => {
 
   assert_throws_js(TypeError, () => {
@@ -109,38 +70,8 @@ test(() => {
   let startCalled = false;
 
   const source = {
-    start(controller) {
+    start() {
       assert_equals(this, source, 'source is this during start');
-
-      const methods = ['close', 'enqueue', 'error', 'constructor'];
-      const properties = ['desiredSize'].concat(methods).sort();
-      const proto = Object.getPrototypeOf(controller);
-
-      assert_array_equals(Object.getOwnPropertyNames(proto).sort(), properties,
-        'the controller should have the right properties');
-
-      for (const m of methods) {
-        const propDesc = Object.getOwnPropertyDescriptor(proto, m);
-        assert_equals(typeof controller[m], 'function', `should have a ${m} method`);
-        assert_false(propDesc.enumerable, m + ' should be non-enumerable');
-        assert_true(propDesc.configurable, m + ' should be configurable');
-        assert_true(propDesc.writable, m + ' should be writable');
-        const expectedName = m === 'constructor' ? 'ReadableStreamDefaultController' : m;
-        assert_equals(controller[m].name, expectedName, 'method should have the correct name');
-      }
-
-      const desiredSizePropDesc = Object.getOwnPropertyDescriptor(proto, 'desiredSize');
-      assert_false(desiredSizePropDesc.enumerable, 'desiredSize should be non-enumerable');
-      assert_equals(desiredSizePropDesc.writable, undefined, 'desiredSize should not be a data property');
-      assert_equals(typeof desiredSizePropDesc.get, 'function', 'desiredSize should have a getter');
-      assert_equals(desiredSizePropDesc.set, undefined, 'desiredSize should not have a setter');
-      assert_true(desiredSizePropDesc.configurable, 'desiredSize should be configurable');
-
-      assert_equals(controller.close.length, 0, 'close should have no parameters');
-      assert_equals(controller.constructor.length, 0, 'constructor should have no parameters');
-      assert_equals(controller.enqueue.length, 1, 'enqueue should have 1 parameter');
-      assert_equals(controller.error.length, 1, 'error should have 1 parameter');
-
       startCalled = true;
     }
   };
@@ -148,7 +79,7 @@ test(() => {
   new ReadableStream(source);
   assert_true(startCalled);
 
-}, 'ReadableStream start should be called with the proper parameters');
+}, 'ReadableStream start should be called with the proper thisArg');
 
 test(() => {
 
@@ -178,7 +109,7 @@ test(() => {
   (new ReadableStream()).getReader(undefined);
   (new ReadableStream()).getReader({});
   (new ReadableStream()).getReader({ mode: undefined, notmode: 'ignored' });
-  assert_throws_js(RangeError, () => (new ReadableStream()).getReader({ mode: 'potato' }));
+  assert_throws_js(TypeError, () => (new ReadableStream()).getReader({ mode: 'potato' }));
 }, 'default ReadableStream getReader() should only accept mode:undefined');
 
 promise_test(() => {
@@ -279,14 +210,14 @@ promise_test(() => {
       assert_unreached('closed should be rejected');
     }, e => {
       closed = true;
-      assert_true(read);
+      assert_false(read);
       assert_equals(e, error, 'closed should be rejected with the thrown error');
     }),
     reader.read().then(() => {
       assert_unreached('read() should be rejected');
     }, e => {
       read = true;
-      assert_false(closed);
+      assert_true(closed);
       assert_equals(e, error, 'read() should be rejected with the thrown error');
     })
   ]);
@@ -296,18 +227,14 @@ promise_test(() => {
 promise_test(() => {
 
   let pullCount = 0;
-  const startPromise = Promise.resolve();
 
   new ReadableStream({
-    start() {
-      return startPromise;
-    },
     pull() {
       pullCount++;
     }
   });
 
-  return startPromise.then(() => {
+  return flushAsyncEvents().then(() => {
     assert_equals(pullCount, 1, 'pull should be called once start finishes');
     return delay(10);
   }).then(() => {
@@ -319,12 +246,8 @@ promise_test(() => {
 promise_test(() => {
 
   let pullCount = 0;
-  const startPromise = Promise.resolve();
 
   const rs = new ReadableStream({
-    start() {
-      return startPromise;
-    },
     pull(c) {
       // Don't enqueue immediately after start. We want the stream to be empty when we call .read() on it.
       if (pullCount > 0) {
@@ -334,7 +257,7 @@ promise_test(() => {
     }
   });
 
-  return startPromise.then(() => {
+  return flushAsyncEvents().then(() => {
     assert_equals(pullCount, 1, 'pull should be called once start finishes');
   }).then(() => {
     const reader = rs.getReader();
@@ -351,12 +274,10 @@ promise_test(() => {
 promise_test(() => {
 
   let pullCount = 0;
-  const startPromise = Promise.resolve();
 
   const rs = new ReadableStream({
     start(c) {
       c.enqueue('a');
-      return startPromise;
     },
     pull() {
       pullCount++;
@@ -366,7 +287,7 @@ promise_test(() => {
   const read = rs.getReader().read();
   assert_equals(pullCount, 0, 'calling read() should not cause pull to be called yet');
 
-  return startPromise.then(() => {
+  return flushAsyncEvents().then(() => {
     assert_equals(pullCount, 1, 'pull should be called once start finishes');
     return read;
   }).then(r => {
@@ -387,14 +308,13 @@ promise_test(() => {
   const rs = new ReadableStream({
     start(c) {
       c.enqueue('a');
-      return startPromise;
     },
     pull() {
       pullCount++;
     }
   });
 
-  return startPromise.then(() => {
+  return flushAsyncEvents().then(() => {
     assert_equals(pullCount, 0, 'pull should not be called once start finishes, since the queue is full');
 
     const read = rs.getReader().read();
@@ -413,12 +333,10 @@ promise_test(() => {
 
   let pullCount = 0;
   let controller;
-  const startPromise = Promise.resolve();
 
   const rs = new ReadableStream({
     start(c) {
       controller = c;
-      return startPromise;
     },
     pull() {
       ++pullCount;
@@ -426,7 +344,7 @@ promise_test(() => {
   });
 
   const reader = rs.getReader();
-  return startPromise.then(() => {
+  return flushAsyncEvents().then(() => {
     assert_equals(pullCount, 1, 'pull should have been called once by the time the stream starts');
 
     controller.enqueue('a');
@@ -446,12 +364,10 @@ promise_test(() => {
 
   let pullCount = 0;
   let controller;
-  const startPromise = Promise.resolve();
 
   const rs = new ReadableStream({
     start(c) {
       controller = c;
-      return startPromise;
     },
     pull() {
       ++pullCount;
@@ -460,7 +376,7 @@ promise_test(() => {
 
   const reader = rs.getReader();
 
-  return startPromise.then(() => {
+  return flushAsyncEvents().then(() => {
     assert_equals(pullCount, 1, 'pull should have been called once by the time the stream starts');
 
     controller.enqueue('a');
@@ -484,12 +400,8 @@ promise_test(() => {
   let resolve;
   let returnedPromise;
   let timesCalled = 0;
-  const startPromise = Promise.resolve();
 
   const rs = new ReadableStream({
-    start() {
-      return startPromise;
-    },
     pull(c) {
       c.enqueue(++timesCalled);
       returnedPromise = new Promise(r => resolve = r);
@@ -498,9 +410,8 @@ promise_test(() => {
   });
   const reader = rs.getReader();
 
-  return startPromise.then(() => {
-    return reader.read();
-  }).then(result1 => {
+  return reader.read()
+  .then(result1 => {
     assert_equals(timesCalled, 1,
       'pull should have been called once after start, but not yet have been called a second time');
     assert_object_equals(result1, { value: 1, done: false }, 'read() should fulfill with the enqueued value');
@@ -521,7 +432,6 @@ promise_test(() => {
 promise_test(() => {
 
   let timesCalled = 0;
-  const startPromise = Promise.resolve();
 
   const rs = new ReadableStream(
     {
@@ -529,7 +439,6 @@ promise_test(() => {
         c.enqueue('a');
         c.enqueue('b');
         c.enqueue('c');
-        return startPromise;
       },
       pull() {
         ++timesCalled;
@@ -544,7 +453,7 @@ promise_test(() => {
   );
   const reader = rs.getReader();
 
-  return startPromise.then(() => {
+  return flushAsyncEvents().then(() => {
     return reader.read();
   }).then(result1 => {
     assert_object_equals(result1, { value: 'a', done: false }, 'first chunk should be as expected');
